/* Generated by Cython 0.29.24 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "define_macros": [
            [
                "CYTHON_TRACE",
                "1"
            ]
        ],
        "depends": [],
        "name": "ttygif.tty.parser",
        "sources": [
            "./ttygif/tty/parser.pyx"
        ]
    },
    "module_name": "ttygif.tty.parser"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_24"
#define CYTHON_HEX_VERSION 0x001D18F0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__ttygif__tty__parser
#define __PYX_HAVE_API__ttygif__tty__parser
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "pythread.h"
#include <stdint.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "ttygif/tty/parser.pyx",
  "stringsource",
  "array.pxd",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
  "ttygif/tty/image.pxd",
  "ttygif/tty/font.pxd",
  "ttygif/tty/theme.pxd",
  "ttygif/tty/display_state.pxd",
  "ttygif/tty/terminal_graphics.pxd",
};

/*--- Type declarations ---*/
#ifndef _ARRAYARRAY_H
struct arrayobject;
typedef struct arrayobject arrayobject;
#endif
struct __pyx_obj_6ttygif_3tty_5image_bounds;
struct __pyx_obj_6ttygif_3tty_5image_point;
struct __pyx_obj_6ttygif_3tty_5image_rect;
struct __pyx_obj_6ttygif_3tty_5image_image;
struct __pyx_obj_6ttygif_3tty_4font_font;
struct __pyx_obj_6ttygif_3tty_5theme_layer;
struct __pyx_obj_6ttygif_3tty_5theme_theme;
struct __pyx_obj_6ttygif_3tty_13display_state_display_state;
struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics;
struct __pyx_obj_6ttygif_3tty_6parser_term_parser;
struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence;
struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr;
struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr;
struct __pyx_opt_args_6ttygif_3tty_6parser_11term_parser_add_command_sequence;

/* "ttygif/tty/parser.pxd":71
 *     cdef add_event(self,event)
 *     cdef add_text_sequence(self,text,timestamp,delay)
 *     cdef add_command_sequence(self,esc_type,command,params,groups,name,timestamp,delay,text=?)             # <<<<<<<<<<<<<<
 *     cdef debug_sequence(self)
 *     cdef debug_event(self,event,index)
 */
struct __pyx_opt_args_6ttygif_3tty_6parser_11term_parser_add_command_sequence {
  int __pyx_n;
  PyObject *text;
};

/* "image.pxd":10
 * 
 * # sets bounding paramaters for image transformations
 * cdef class bounds:             # <<<<<<<<<<<<<<
 *     cdef public int width
 *     cdef public int height
 */
struct __pyx_obj_6ttygif_3tty_5image_bounds {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ttygif_3tty_5image_bounds *__pyx_vtab;
  int width;
  int height;
  int stride;
  int length;
  int bytes_per_pixel;
};


/* "image.pxd":18
 *     cdef debug(self)
 * 
 * cdef class point:             # <<<<<<<<<<<<<<
 *     cdef int left
 *     cdef int top
 */
struct __pyx_obj_6ttygif_3tty_5image_point {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ttygif_3tty_5image_point *__pyx_vtab;
  int left;
  int top;
};


/* "image.pxd":24
 * 
 * 
 * cdef class rect:             # <<<<<<<<<<<<<<
 *     cdef int left
 *     cdef int top
 */
struct __pyx_obj_6ttygif_3tty_5image_rect {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ttygif_3tty_5image_rect *__pyx_vtab;
  int left;
  int top;
  int right;
  int bottom;
  int width;
  int height;
};


/* "image.pxd":40
 *     cdef debug(self)
 * 
 * cdef class image:             # <<<<<<<<<<<<<<
 *     cdef int         transparent
 *     cdef array.array data
 */
struct __pyx_obj_6ttygif_3tty_5image_image {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ttygif_3tty_5image_image *__pyx_vtab;
  int transparent;
  arrayobject *data;
  struct __pyx_obj_6ttygif_3tty_5image_bounds *dimentions;
  arrayobject *palette;
  int length;
};


/* "font.pxd":8
 * 
 * 
 * cdef class font:             # <<<<<<<<<<<<<<
 *     cdef str          name
 *     cdef int          pointsize
 */
struct __pyx_obj_6ttygif_3tty_4font_font {
  PyObject_HEAD
  PyObject *name;
  int pointsize;
  int height;
  int width;
  int ascent;
  int inleading;
  int exleading;
  int charset;
  arrayobject *graphic;
  PyObject *offset;
};


/* "theme.pxd":7
 * from image cimport rect,image,point
 * 
 * cdef class layer:             # <<<<<<<<<<<<<<
 *     cdef int         z_index
 *     cdef str         file
 */
struct __pyx_obj_6ttygif_3tty_5theme_layer {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ttygif_3tty_5theme_layer *__pyx_vtab;
  int z_index;
  PyObject *file;
  PyObject *mode;
  PyObject *name;
  struct __pyx_obj_6ttygif_3tty_5image_rect *outer;
  struct __pyx_obj_6ttygif_3tty_5image_rect *inner;
  struct __pyx_obj_6ttygif_3tty_5image_rect *bounds;
  struct __pyx_obj_6ttygif_3tty_5image_rect *dst;
  PyObject *center;
  PyObject *copy_mode;
  int transparent;
  struct __pyx_obj_6ttygif_3tty_5image_image *image;
};


/* "theme.pxd":27
 * 
 * 
 * cdef class theme:             # <<<<<<<<<<<<<<
 *     cdef str         name
 *     cdef str         path
 */
struct __pyx_obj_6ttygif_3tty_5theme_theme {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ttygif_3tty_5theme_theme *__pyx_vtab;
  PyObject *name;
  PyObject *path;
  PyObject *font;
  int width;
  int height;
  int background;
  int foreground;
  int default_background;
  int default_foreground;
  int transparent;
  int colors;
  arrayobject *palette;
  struct __pyx_obj_6ttygif_3tty_5image_rect *padding;
  struct __pyx_obj_6ttygif_3tty_5theme_layer *layer1;
  struct __pyx_obj_6ttygif_3tty_5theme_layer *layer2;
  struct __pyx_obj_6ttygif_3tty_5theme_layer *layer3;
  struct __pyx_obj_6ttygif_3tty_5theme_layer *layer4;
  struct __pyx_obj_6ttygif_3tty_5theme_layer *layer5;
};


/* "display_state.pxd":6
 * from libc.stdint cimport uint32_t, int64_t,uint16_t,uint8_t,int32_t
 * 
 * cdef class display_state:             # <<<<<<<<<<<<<<
 *     cdef uint16_t        width
 *     cdef uint16_t        height
 */
struct __pyx_obj_6ttygif_3tty_13display_state_display_state {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *__pyx_vtab;
  uint16_t width;
  uint16_t height;
  int cursor_x;
  int cursor_y;
  int saved_cursor_x;
  int saved_cursor_y;
  PyObject *mode;
  uint8_t default_foreground;
  uint8_t default_background;
  uint8_t foreground;
  uint8_t background;
  int scroll;
  int scroll_top;
  int scroll_bottom;
  PyObject *reverse_video;
  PyObject *bold;
  PyObject *text_mode;
  PyObject *autowrap;
  PyObject *pending_wrap;
  PyObject *display_cursor;
};


/* "terminal_graphics.pxd":13
 * 
 * 
 * cdef class terminal_graphics:             # <<<<<<<<<<<<<<
 *     cdef array.array    data
 *     cdef font           font
 */
struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *__pyx_vtab;
  arrayobject *data;
  struct __pyx_obj_6ttygif_3tty_4font_font *font;
  struct __pyx_obj_6ttygif_3tty_5image_image *viewport;
  struct __pyx_obj_6ttygif_3tty_5image_image *screen;
  struct __pyx_obj_6ttygif_3tty_5image_image *alt_screen;
  struct __pyx_obj_6ttygif_3tty_13display_state_display_state *state;
  struct __pyx_obj_6ttygif_3tty_13display_state_display_state *alt_state;
  PyObject *display_alt_screen;
  struct __pyx_obj_6ttygif_3tty_5theme_theme *theme;
};


/* "ttygif/tty/parser.pxd":11
 * 
 * 
 * cdef class term_parser:             # <<<<<<<<<<<<<<
 *     cdef object            debug_mode
 *     cdef object            sequence
 */
struct __pyx_obj_6ttygif_3tty_6parser_term_parser {
  PyObject_HEAD
  struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *__pyx_vtab;
  PyObject *debug_mode;
  PyObject *sequence;
  int sequence_pos;
  PyObject *extra_text;
  PyObject *no_codes;
  PyObject *bracketed_paste;
  PyObject *current_sequence_position;
  struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *g;
  double last_timestamp;
};


/* "ttygif/tty/parser.pyx":494
 *         self.g.state.cursor_absolute(0,position)
 * 
 *     cdef stream_2_sequence(self,text,timestamp,delay):             # <<<<<<<<<<<<<<
 *         # patterns for filtering out commands from the stream
 *         ANSI_SINGLE   ='[\033]([cDEHMZ78>=])'
 */
struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence {
  PyObject_HEAD
  PyObject *__pyx_v_paramstring;
};


/* "ttygif/tty/parser.pyx":547
 *                 command=groups[10]
 *                 if command in 'Hf':
 *                     params = tuple(int(p) if len(p) != 0 else 1 for p in paramstring.split(';'))             # <<<<<<<<<<<<<<
 *                     while len(params) < 2:
 *                         params = params + (1,)
 */
struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr {
  PyObject_HEAD
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *__pyx_outer_scope;
  PyObject *__pyx_v_p;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};


/* "ttygif/tty/parser.pyx":557
 *                 else:
 *                     try:
 *                         params = tuple(int(p) for p in paramstring.split(';') if len(p) != 0)             # <<<<<<<<<<<<<<
 *                         if len(params) == 0:
 *                             if command in 'JKm':
 */
struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr {
  PyObject_HEAD
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *__pyx_outer_scope;
  PyObject *__pyx_v_p;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  PyObject *(*__pyx_t_2)(PyObject *);
};



/* "image.pxd":10
 * 
 * # sets bounding paramaters for image transformations
 * cdef class bounds:             # <<<<<<<<<<<<<<
 *     cdef public int width
 *     cdef public int height
 */

struct __pyx_vtabstruct_6ttygif_3tty_5image_bounds {
  PyObject *(*debug)(struct __pyx_obj_6ttygif_3tty_5image_bounds *);
};
static struct __pyx_vtabstruct_6ttygif_3tty_5image_bounds *__pyx_vtabptr_6ttygif_3tty_5image_bounds;


/* "image.pxd":18
 *     cdef debug(self)
 * 
 * cdef class point:             # <<<<<<<<<<<<<<
 *     cdef int left
 *     cdef int top
 */

struct __pyx_vtabstruct_6ttygif_3tty_5image_point {
  PyObject *(*debug)(struct __pyx_obj_6ttygif_3tty_5image_point *);
};
static struct __pyx_vtabstruct_6ttygif_3tty_5image_point *__pyx_vtabptr_6ttygif_3tty_5image_point;


/* "image.pxd":24
 * 
 * 
 * cdef class rect:             # <<<<<<<<<<<<<<
 *     cdef int left
 *     cdef int top
 */

struct __pyx_vtabstruct_6ttygif_3tty_5image_rect {
  PyObject *(*update)(struct __pyx_obj_6ttygif_3tty_5image_rect *);
  PyObject *(*percent_x)(struct __pyx_obj_6ttygif_3tty_5image_rect *, float);
  PyObject *(*percent_y)(struct __pyx_obj_6ttygif_3tty_5image_rect *, float);
  PyObject *(*get_x_percent)(struct __pyx_obj_6ttygif_3tty_5image_rect *, float);
  PyObject *(*get_y_percent)(struct __pyx_obj_6ttygif_3tty_5image_rect *, float);
  PyObject *(*point1)(struct __pyx_obj_6ttygif_3tty_5image_rect *);
  PyObject *(*point2)(struct __pyx_obj_6ttygif_3tty_5image_rect *);
  PyObject *(*debug)(struct __pyx_obj_6ttygif_3tty_5image_rect *);
};
static struct __pyx_vtabstruct_6ttygif_3tty_5image_rect *__pyx_vtabptr_6ttygif_3tty_5image_rect;


/* "image.pxd":40
 *     cdef debug(self)
 * 
 * cdef class image:             # <<<<<<<<<<<<<<
 *     cdef int         transparent
 *     cdef array.array data
 */

struct __pyx_vtabstruct_6ttygif_3tty_5image_image {
  struct __pyx_obj_6ttygif_3tty_5image_rect *(*get_rect)(struct __pyx_obj_6ttygif_3tty_5image_image *);
  arrayobject *(*create_buffer)(struct __pyx_obj_6ttygif_3tty_5image_image *, PyObject *, PyObject *);
  uint32_t (*get_position)(struct __pyx_obj_6ttygif_3tty_5image_image *, int, int);
  PyObject *(*get_pixel)(struct __pyx_obj_6ttygif_3tty_5image_image *, int, int);
  uint8_t (*get_pixel_1byte)(struct __pyx_obj_6ttygif_3tty_5image_image *, int, int);
  void (*get_pixel_3byte)(struct __pyx_obj_6ttygif_3tty_5image_image *, int, int, uint8_t *);
  void (*put_pixel)(struct __pyx_obj_6ttygif_3tty_5image_image *, int, int, PyObject *);
  void (*put_pixel_1byte)(struct __pyx_obj_6ttygif_3tty_5image_image *, uint16_t, uint16_t, uint8_t);
  void (*put_pixel_3byte)(struct __pyx_obj_6ttygif_3tty_5image_image *, int, int, uint8_t *);
  PyObject *(*put_pixel_rgb)(struct __pyx_obj_6ttygif_3tty_5image_image *, int, int, int, int, int);
  PyObject *(*clear)(struct __pyx_obj_6ttygif_3tty_5image_image *, uint8_t *);
  PyObject *(*remap_image)(struct __pyx_obj_6ttygif_3tty_5image_image *, arrayobject *, uint8_t);
  PyObject *(*match_color_index)(struct __pyx_obj_6ttygif_3tty_5image_image *, int, int, int);
  PyObject *(*copy)(struct __pyx_obj_6ttygif_3tty_5image_image *, struct __pyx_obj_6ttygif_3tty_5image_image *, struct __pyx_obj_6ttygif_3tty_5image_rect *, struct __pyx_obj_6ttygif_3tty_5image_rect *, uint8_t);
  PyObject *(*copy_remap)(struct __pyx_obj_6ttygif_3tty_5image_image *, struct __pyx_obj_6ttygif_3tty_5image_image *, struct __pyx_obj_6ttygif_3tty_5image_rect *, struct __pyx_obj_6ttygif_3tty_5image_point *, uint8_t);
  PyObject *(*copy_scale)(struct __pyx_obj_6ttygif_3tty_5image_image *, struct __pyx_obj_6ttygif_3tty_5image_image *, struct __pyx_obj_6ttygif_3tty_5image_rect *, struct __pyx_obj_6ttygif_3tty_5image_rect *, uint8_t);
  PyObject *(*copy_tile)(struct __pyx_obj_6ttygif_3tty_5image_image *, struct __pyx_obj_6ttygif_3tty_5image_image *, struct __pyx_obj_6ttygif_3tty_5image_rect *, struct __pyx_obj_6ttygif_3tty_5image_rect *, uint8_t);
  PyObject *(*copy_9slice)(struct __pyx_obj_6ttygif_3tty_5image_image *, struct __pyx_obj_6ttygif_3tty_5image_image *, struct __pyx_obj_6ttygif_3tty_5image_rect *, struct __pyx_obj_6ttygif_3tty_5image_rect *, struct __pyx_obj_6ttygif_3tty_5image_rect *, uint8_t, PyObject *);
  PyObject *(*copy_3slice)(struct __pyx_obj_6ttygif_3tty_5image_image *, struct __pyx_obj_6ttygif_3tty_5image_image *, struct __pyx_obj_6ttygif_3tty_5image_rect *, struct __pyx_obj_6ttygif_3tty_5image_rect *, struct __pyx_obj_6ttygif_3tty_5image_rect *, uint8_t, PyObject *);
  PyObject *(*debug)(struct __pyx_obj_6ttygif_3tty_5image_image *);
};
static struct __pyx_vtabstruct_6ttygif_3tty_5image_image *__pyx_vtabptr_6ttygif_3tty_5image_image;


/* "theme.pxd":7
 * from image cimport rect,image,point
 * 
 * cdef class layer:             # <<<<<<<<<<<<<<
 *     cdef int         z_index
 *     cdef str         file
 */

struct __pyx_vtabstruct_6ttygif_3tty_5theme_layer {
  PyObject *(*debug)(struct __pyx_obj_6ttygif_3tty_5theme_layer *);
  PyObject *(*load_file)(struct __pyx_obj_6ttygif_3tty_5theme_layer *, PyObject *, arrayobject *, int, int);
};
static struct __pyx_vtabstruct_6ttygif_3tty_5theme_layer *__pyx_vtabptr_6ttygif_3tty_5theme_layer;


/* "theme.pxd":27
 * 
 * 
 * cdef class theme:             # <<<<<<<<<<<<<<
 *     cdef str         name
 *     cdef str         path
 */

struct __pyx_vtabstruct_6ttygif_3tty_5theme_theme {
  PyObject *(*init)(struct __pyx_obj_6ttygif_3tty_5theme_theme *);
  PyObject *(*update_layer)(struct __pyx_obj_6ttygif_3tty_5theme_theme *, struct __pyx_obj_6ttygif_3tty_5theme_layer *);
  PyObject *(*__pyx_auto)(struct __pyx_obj_6ttygif_3tty_5theme_theme *);
};
static struct __pyx_vtabstruct_6ttygif_3tty_5theme_theme *__pyx_vtabptr_6ttygif_3tty_5theme_theme;


/* "display_state.pxd":6
 * from libc.stdint cimport uint32_t, int64_t,uint16_t,uint8_t,int32_t
 * 
 * cdef class display_state:             # <<<<<<<<<<<<<<
 *     cdef uint16_t        width
 *     cdef uint16_t        height
 */

struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state {
  PyObject *(*show_cursor)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *);
  PyObject *(*hide_cursor)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *);
  PyObject *(*text_mode_on)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *);
  PyObject *(*text_mode_off)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *);
  PyObject *(*autowrap_on)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *);
  PyObject *(*autowrap_off)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *);
  PyObject *(*set_scroll_region)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *, PyObject *, PyObject *);
  PyObject *(*check_bounds)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *);
  PyObject *(*cursor_up)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *, int);
  PyObject *(*cursor_down)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *, int);
  PyObject *(*cursor_left)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *, int);
  PyObject *(*cursor_right)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *, int);
  PyObject *(*cursor_absolute_x)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *, PyObject *);
  PyObject *(*cursor_absolute_y)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *, PyObject *);
  PyObject *(*cursor_save_position)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *);
  PyObject *(*cursor_restore_position)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *);
  PyObject *(*cursor_get_position)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *);
  PyObject *(*cursor_absolute)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *, PyObject *, PyObject *);
  PyObject *(*set_background)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *, int);
  PyObject *(*set_foreground)(struct __pyx_obj_6ttygif_3tty_13display_state_display_state *, int);
};
static struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *__pyx_vtabptr_6ttygif_3tty_13display_state_display_state;


/* "terminal_graphics.pxd":13
 * 
 * 
 * cdef class terminal_graphics:             # <<<<<<<<<<<<<<
 *     cdef array.array    data
 *     cdef font           font
 */

struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics {
  PyObject *(*alternate_screen_on)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *);
  PyObject *(*alternate_screen_off)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *);
  PyObject *(*write)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *, uint8_t);
  PyObject *(*draw_string)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *, PyObject *, PyObject *, PyObject *);
  PyObject *(*scroll_buffer)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *);
  PyObject *(*draw_character)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *, int, int, uint8_t *);
  PyObject *(*get_text)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *);
  PyObject *(*foreground_from_rgb)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *, PyObject *, PyObject *, PyObject *);
  PyObject *(*background_from_rgb)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *, PyObject *, PyObject *, PyObject *);
  PyObject *(*set_foreground)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *, PyObject *);
  PyObject *(*set_background)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *, PyObject *);
  PyObject *(*copy)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *, struct __pyx_obj_6ttygif_3tty_5theme_layer *);
  PyObject *(*render)(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *);
};
static struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *__pyx_vtabptr_6ttygif_3tty_17terminal_graphics_terminal_graphics;


/* "ttygif/tty/parser.pyx":13
 * # http://man7.org/linux/man-pages/man4/console_codes.4.html
 * 
 * cdef class term_parser:             # <<<<<<<<<<<<<<
 *     def __init__(self,terminal_graphics terminal_graphics,debug_mode=None):
 *         self.debug_mode=debug_mode
 */

struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser {
  PyObject *(*ascii_safe)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*info)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*clear_sequence)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *);
  PyObject *(*rgb_to_palette)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *, PyObject *);
  PyObject *(*remap_character)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*render_to_buffer)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *);
  PyObject *(*procces_OSC)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*process_SINGLE)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*process_CHAR_SET)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*process_G0)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*process_G1)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*process_CSI)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *);
  PyObject *(*cmd_DECSET)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, int);
  PyObject *(*cmd_DECRST)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, int);
  PyObject *(*cmd_bracketed_paste_on)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *);
  PyObject *(*cmd_bracketed_paste_off)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *);
  PyObject *(*cmd_BRACKETED_PASTE)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_set_mode)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_reset_mode)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*set_foreground)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*set_background)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_process_colors)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_render_text)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_DECSTBM)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, int, int);
  PyObject *(*cmd_CUU)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_CUD)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_CUB)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_CUF)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_CPL)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_CNL)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_CHA)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_CUP)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *);
  PyObject *(*cmd_ED)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_EL)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_ECH)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_DCH)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_HVP)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *);
  PyObject *(*cmd_HPA)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*cmd_SCP)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *);
  PyObject *(*cmd_RCP)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *);
  PyObject *(*cmd_VPA)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*stream_2_sequence)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *, PyObject *);
  PyObject *(*last_frame)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *);
  PyObject *(*has_escape)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*add_event)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *);
  PyObject *(*add_text_sequence)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *, PyObject *);
  PyObject *(*add_command_sequence)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, struct __pyx_opt_args_6ttygif_3tty_6parser_11term_parser_add_command_sequence *__pyx_optional_args);
  PyObject *(*debug_sequence)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *);
  PyObject *(*debug_event)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *);
};
static struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *__pyx_vtabptr_6ttygif_3tty_6parser_term_parser;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* Profile.proto */
#ifndef CYTHON_PROFILE
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
  #define CYTHON_PROFILE 0
#else
  #define CYTHON_PROFILE 1
#endif
#endif
#ifndef CYTHON_TRACE_NOGIL
  #define CYTHON_TRACE_NOGIL 0
#else
  #if CYTHON_TRACE_NOGIL && !defined(CYTHON_TRACE)
    #define CYTHON_TRACE 1
  #endif
#endif
#ifndef CYTHON_TRACE
  #define CYTHON_TRACE 0
#endif
#if CYTHON_TRACE
  #undef CYTHON_PROFILE_REUSE_FRAME
#endif
#ifndef CYTHON_PROFILE_REUSE_FRAME
  #define CYTHON_PROFILE_REUSE_FRAME 0
#endif
#if CYTHON_PROFILE || CYTHON_TRACE
  #include "compile.h"
  #include "frameobject.h"
  #include "traceback.h"
  #if CYTHON_PROFILE_REUSE_FRAME
    #define CYTHON_FRAME_MODIFIER static
    #define CYTHON_FRAME_DEL(frame)
  #else
    #define CYTHON_FRAME_MODIFIER
    #define CYTHON_FRAME_DEL(frame) Py_CLEAR(frame)
  #endif
  #define __Pyx_TraceDeclarations\
      static PyCodeObject *__pyx_frame_code = NULL;\
      CYTHON_FRAME_MODIFIER PyFrameObject *__pyx_frame = NULL;\
      int __Pyx_use_tracing = 0;
  #define __Pyx_TraceFrameInit(codeobj)\
      if (codeobj) __pyx_frame_code = (PyCodeObject*) codeobj;
#if PY_VERSION_HEX >= 0x030a00b1
  #define __Pyx_IsTracing(tstate, check_tracing, check_funcs)\
     (unlikely((tstate)->cframe->use_tracing) &&\
         (!(check_tracing) || !(tstate)->tracing) &&\
         (!(check_funcs) || (tstate)->c_profilefunc || (CYTHON_TRACE && (tstate)->c_tracefunc)))
  #define __Pyx_SetTracing(tstate, enable)\
      (tstate)->cframe->use_tracing = (enable)
#else
  #define __Pyx_IsTracing(tstate, check_tracing, check_funcs)\
     (unlikely((tstate)->use_tracing) &&\
         (!(check_tracing) || !(tstate)->tracing) &&\
         (!(check_funcs) || (tstate)->c_profilefunc || (CYTHON_TRACE && (tstate)->c_tracefunc)))
  #define __Pyx_SetTracing(tstate, enable)\
      (tstate)->use_tracing = (enable)
#endif
  #ifdef WITH_THREAD
  #define __Pyx_TraceCall(funcname, srcfile, firstlineno, nogil, goto_error)\
  if (nogil) {\
      if (CYTHON_TRACE_NOGIL) {\
          PyThreadState *tstate;\
          PyGILState_STATE state = PyGILState_Ensure();\
          tstate = __Pyx_PyThreadState_Current;\
          if (__Pyx_IsTracing(tstate, 1, 1)) {\
              __Pyx_use_tracing = __Pyx_TraceSetupAndCall(&__pyx_frame_code, &__pyx_frame, tstate, funcname, srcfile, firstlineno);\
          }\
          PyGILState_Release(state);\
          if (unlikely(__Pyx_use_tracing < 0)) goto_error;\
      }\
  } else {\
      PyThreadState* tstate = PyThreadState_GET();\
      if (__Pyx_IsTracing(tstate, 1, 1)) {\
          __Pyx_use_tracing = __Pyx_TraceSetupAndCall(&__pyx_frame_code, &__pyx_frame, tstate, funcname, srcfile, firstlineno);\
          if (unlikely(__Pyx_use_tracing < 0)) goto_error;\
      }\
  }
  #else
  #define __Pyx_TraceCall(funcname, srcfile, firstlineno, nogil, goto_error)\
  {   PyThreadState* tstate = PyThreadState_GET();\
      if (__Pyx_IsTracing(tstate, 1, 1)) {\
          __Pyx_use_tracing = __Pyx_TraceSetupAndCall(&__pyx_frame_code, &__pyx_frame, tstate, funcname, srcfile, firstlineno);\
          if (unlikely(__Pyx_use_tracing < 0)) goto_error;\
      }\
  }
  #endif
  #define __Pyx_TraceException()\
  if (likely(!__Pyx_use_tracing)); else {\
      PyThreadState* tstate = __Pyx_PyThreadState_Current;\
      if (__Pyx_IsTracing(tstate, 0, 1)) {\
          tstate->tracing++;\
          __Pyx_SetTracing(tstate, 0);\
          PyObject *exc_info = __Pyx_GetExceptionTuple(tstate);\
          if (exc_info) {\
              if (CYTHON_TRACE && tstate->c_tracefunc)\
                  tstate->c_tracefunc(\
                      tstate->c_traceobj, __pyx_frame, PyTrace_EXCEPTION, exc_info);\
              tstate->c_profilefunc(\
                  tstate->c_profileobj, __pyx_frame, PyTrace_EXCEPTION, exc_info);\
              Py_DECREF(exc_info);\
          }\
          __Pyx_SetTracing(tstate, 1);\
          tstate->tracing--;\
      }\
  }
  static void __Pyx_call_return_trace_func(PyThreadState *tstate, PyFrameObject *frame, PyObject *result) {
      PyObject *type, *value, *traceback;
      __Pyx_ErrFetchInState(tstate, &type, &value, &traceback);
      tstate->tracing++;
      __Pyx_SetTracing(tstate, 0);
      if (CYTHON_TRACE && tstate->c_tracefunc)
          tstate->c_tracefunc(tstate->c_traceobj, frame, PyTrace_RETURN, result);
      if (tstate->c_profilefunc)
          tstate->c_profilefunc(tstate->c_profileobj, frame, PyTrace_RETURN, result);
      CYTHON_FRAME_DEL(frame);
      __Pyx_SetTracing(tstate, 1);
      tstate->tracing--;
      __Pyx_ErrRestoreInState(tstate, type, value, traceback);
  }
  #ifdef WITH_THREAD
  #define __Pyx_TraceReturn(result, nogil)\
  if (likely(!__Pyx_use_tracing)); else {\
      if (nogil) {\
          if (CYTHON_TRACE_NOGIL) {\
              PyThreadState *tstate;\
              PyGILState_STATE state = PyGILState_Ensure();\
              tstate = __Pyx_PyThreadState_Current;\
              if (__Pyx_IsTracing(tstate, 0, 0)) {\
                  __Pyx_call_return_trace_func(tstate, __pyx_frame, (PyObject*)result);\
              }\
              PyGILState_Release(state);\
          }\
      } else {\
          PyThreadState* tstate = __Pyx_PyThreadState_Current;\
          if (__Pyx_IsTracing(tstate, 0, 0)) {\
              __Pyx_call_return_trace_func(tstate, __pyx_frame, (PyObject*)result);\
          }\
      }\
  }
  #else
  #define __Pyx_TraceReturn(result, nogil)\
  if (likely(!__Pyx_use_tracing)); else {\
      PyThreadState* tstate = __Pyx_PyThreadState_Current;\
      if (__Pyx_IsTracing(tstate, 0, 0)) {\
          __Pyx_call_return_trace_func(tstate, __pyx_frame, (PyObject*)result);\
      }\
  }
  #endif
  static PyCodeObject *__Pyx_createFrameCodeObject(const char *funcname, const char *srcfile, int firstlineno);
  static int __Pyx_TraceSetupAndCall(PyCodeObject** code, PyFrameObject** frame, PyThreadState* tstate, const char *funcname, const char *srcfile, int firstlineno);
#else
  #define __Pyx_TraceDeclarations
  #define __Pyx_TraceFrameInit(codeobj)
  #define __Pyx_TraceCall(funcname, srcfile, firstlineno, nogil, goto_error)   if ((1)); else goto_error;
  #define __Pyx_TraceException()
  #define __Pyx_TraceReturn(result, nogil)
#endif
#if CYTHON_TRACE
  static int __Pyx_call_line_trace_func(PyThreadState *tstate, PyFrameObject *frame, int lineno) {
      int ret;
      PyObject *type, *value, *traceback;
      __Pyx_ErrFetchInState(tstate, &type, &value, &traceback);
      __Pyx_PyFrame_SetLineNumber(frame, lineno);
      tstate->tracing++;
      __Pyx_SetTracing(tstate, 0);
      ret = tstate->c_tracefunc(tstate->c_traceobj, frame, PyTrace_LINE, NULL);
      __Pyx_SetTracing(tstate, 1);
      tstate->tracing--;
      if (likely(!ret)) {
          __Pyx_ErrRestoreInState(tstate, type, value, traceback);
      } else {
          Py_XDECREF(type);
          Py_XDECREF(value);
          Py_XDECREF(traceback);
      }
      return ret;
  }
  #ifdef WITH_THREAD
  #define __Pyx_TraceLine(lineno, nogil, goto_error)\
  if (likely(!__Pyx_use_tracing)); else {\
      if (nogil) {\
          if (CYTHON_TRACE_NOGIL) {\
              int ret = 0;\
              PyThreadState *tstate;\
              PyGILState_STATE state = PyGILState_Ensure();\
              tstate = __Pyx_PyThreadState_Current;\
              if (__Pyx_IsTracing(tstate, 0, 0) && tstate->c_tracefunc && __pyx_frame->f_trace) {\
                  ret = __Pyx_call_line_trace_func(tstate, __pyx_frame, lineno);\
              }\
              PyGILState_Release(state);\
              if (unlikely(ret)) goto_error;\
          }\
      } else {\
          PyThreadState* tstate = __Pyx_PyThreadState_Current;\
          if (__Pyx_IsTracing(tstate, 0, 0) && tstate->c_tracefunc && __pyx_frame->f_trace) {\
              int ret = __Pyx_call_line_trace_func(tstate, __pyx_frame, lineno);\
              if (unlikely(ret)) goto_error;\
          }\
      }\
  }
  #else
  #define __Pyx_TraceLine(lineno, nogil, goto_error)\
  if (likely(!__Pyx_use_tracing)); else {\
      PyThreadState* tstate = __Pyx_PyThreadState_Current;\
      if (__Pyx_IsTracing(tstate, 0, 0) && tstate->c_tracefunc && __pyx_frame->f_trace) {\
          int ret = __Pyx_call_line_trace_func(tstate, __pyx_frame, lineno);\
          if (unlikely(ret)) goto_error;\
      }\
  }
  #endif
#else
  #define __Pyx_TraceLine(lineno, nogil, goto_error)   if ((1)); else goto_error;
#endif

/* StringJoin.proto */
#if PY_MAJOR_VERSION < 3
#define __Pyx_PyString_Join __Pyx_PyBytes_Join
#define __Pyx_PyBaseString_Join(s, v) (PyUnicode_CheckExact(s) ? PyUnicode_Join(s, v) : __Pyx_PyBytes_Join(s, v))
#else
#define __Pyx_PyString_Join PyUnicode_Join
#define __Pyx_PyBaseString_Join PyUnicode_Join
#endif
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION < 3
    #define __Pyx_PyBytes_Join _PyString_Join
    #else
    #define __Pyx_PyBytes_Join _PyBytes_Join
    #endif
#else
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values);
#endif

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* UnicodeAsUCS4.proto */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject*);

/* object_ord.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyObject_Ord(c)\
    (likely(PyUnicode_Check(c)) ? (long)__Pyx_PyUnicode_AsPy_UCS4(c) : __Pyx__PyObject_Ord(c))
#else
#define __Pyx_PyObject_Ord(c) __Pyx__PyObject_Ord(c)
#endif
static long __Pyx__PyObject_Ord(PyObject* c);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_SubtractObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* Print.proto */
static int __Pyx_Print(PyObject*, PyObject *, int);
#if CYTHON_COMPILING_IN_PYPY || PY_MAJOR_VERSION >= 3
static PyObject* __pyx_print = 0;
static PyObject* __pyx_print_kwargs = 0;
#endif

/* ArrayAPI.proto */
#ifndef _ARRAYARRAY_H
#define _ARRAYARRAY_H
typedef struct arraydescr {
    int typecode;
    int itemsize;
    PyObject * (*getitem)(struct arrayobject *, Py_ssize_t);
    int (*setitem)(struct arrayobject *, Py_ssize_t, PyObject *);
#if PY_MAJOR_VERSION >= 3
    char *formats;
#endif
} arraydescr;
struct arrayobject {
    PyObject_HEAD
    Py_ssize_t ob_size;
    union {
        char *ob_item;
        float *as_floats;
        double *as_doubles;
        int *as_ints;
        unsigned int *as_uints;
        unsigned char *as_uchars;
        signed char *as_schars;
        char *as_chars;
        unsigned long *as_ulongs;
        long *as_longs;
#if PY_MAJOR_VERSION >= 3
        unsigned long long *as_ulonglongs;
        long long *as_longlongs;
#endif
        short *as_shorts;
        unsigned short *as_ushorts;
        Py_UNICODE *as_pyunicodes;
        void *as_voidptr;
    } data;
    Py_ssize_t allocated;
    struct arraydescr *ob_descr;
    PyObject *weakreflist;
#if PY_MAJOR_VERSION >= 3
        int ob_exports;
#endif
};
#ifndef NO_NEWARRAY_INLINE
static CYTHON_INLINE PyObject * newarrayobject(PyTypeObject *type, Py_ssize_t size,
    struct arraydescr *descr) {
    arrayobject *op;
    size_t nbytes;
    if (size < 0) {
        PyErr_BadInternalCall();
        return NULL;
    }
    nbytes = size * descr->itemsize;
    if (nbytes / descr->itemsize != (size_t)size) {
        return PyErr_NoMemory();
    }
    op = (arrayobject *) type->tp_alloc(type, 0);
    if (op == NULL) {
        return NULL;
    }
    op->ob_descr = descr;
    op->allocated = size;
    op->weakreflist = NULL;
    __Pyx_SET_SIZE(op, size);
    if (size <= 0) {
        op->data.ob_item = NULL;
    }
    else {
        op->data.ob_item = PyMem_NEW(char, nbytes);
        if (op->data.ob_item == NULL) {
            Py_DECREF(op);
            return PyErr_NoMemory();
        }
    }
    return (PyObject *) op;
}
#else
PyObject* newarrayobject(PyTypeObject *type, Py_ssize_t size,
    struct arraydescr *descr);
#endif
static CYTHON_INLINE int resize(arrayobject *self, Py_ssize_t n) {
    void *items = (void*) self->data.ob_item;
    PyMem_Resize(items, char, (size_t)(n * self->ob_descr->itemsize));
    if (items == NULL) {
        PyErr_NoMemory();
        return -1;
    }
    self->data.ob_item = (char*) items;
    __Pyx_SET_SIZE(self, n);
    self->allocated = n;
    return 0;
}
static CYTHON_INLINE int resize_smart(arrayobject *self, Py_ssize_t n) {
    void *items = (void*) self->data.ob_item;
    Py_ssize_t newsize;
    if (n < self->allocated && n*4 > self->allocated) {
        __Pyx_SET_SIZE(self, n);
        return 0;
    }
    newsize = n + (n / 2) + 1;
    if (newsize <= n) {
        PyErr_NoMemory();
        return -1;
    }
    PyMem_Resize(items, char, (size_t)(newsize * self->ob_descr->itemsize));
    if (items == NULL) {
        PyErr_NoMemory();
        return -1;
    }
    self->data.ob_item = (char*) items;
    __Pyx_SET_SIZE(self, n);
    self->allocated = newsize;
    return 0;
}
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8_t(uint8_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16_t(uint16_t value);

/* PrintOne.proto */
static int __Pyx_PrintOne(PyObject* stream, PyObject *o);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* CoroutineBase.proto */
typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_ascii_safe(CYTHON_UNUSED struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_text); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_info(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_text); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_clear_sequence(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_rgb_to_palette(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_r, PyObject *__pyx_v_g, PyObject *__pyx_v_b); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_remap_character(CYTHON_UNUSED struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_character); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_render_to_buffer(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_procces_OSC(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_groups); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_process_SINGLE(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_groups); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_process_CHAR_SET(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_groups); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_process_G0(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_groups); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_process_G1(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_groups); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_process_CSI(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_command, PyObject *__pyx_v_params); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_DECSET(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, int __pyx_v_code); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_DECRST(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, int __pyx_v_code); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_bracketed_paste_off(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_bracketed_paste_on(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_BRACKETED_PASTE(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_set_mode(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_cmd); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_reset_mode(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_cmd); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_set_foreground(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_color); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_set_background(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_color); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_process_colors(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_params); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_render_text(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_event); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_DECSTBM(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, int __pyx_v_top, int __pyx_v_bottom); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUU(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUD(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUB(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUF(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CPL(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CNL(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CHA(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_x); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUP(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_ED(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_mode); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_EL(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_mode); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_DCH(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_ECH(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_HVP(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_HPA(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_x); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_SCP(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_RCP(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_VPA(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_position); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_stream_2_sequence(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_text, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_delay); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_last_frame(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_has_escape(CYTHON_UNUSED struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_text); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_add_event(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_event); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_add_text_sequence(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_text, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_delay); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_add_command_sequence(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_esc_type, PyObject *__pyx_v_command, PyObject *__pyx_v_params, PyObject *__pyx_v_groups, PyObject *__pyx_v_name, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_delay, struct __pyx_opt_args_6ttygif_3tty_6parser_11term_parser_add_command_sequence *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_debug_sequence(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_debug_event(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_event, PyObject *__pyx_v_index); /* proto*/

/* Module declarations from 'cpython.version' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.module' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.mapping' */

/* Module declarations from 'cpython.iterator' */

/* Module declarations from 'cpython.number' */

/* Module declarations from 'cpython.int' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.bool' */
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;

/* Module declarations from 'cpython.long' */

/* Module declarations from 'cpython.float' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.complex' */
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;

/* Module declarations from 'cpython.string' */

/* Module declarations from 'cpython.unicode' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.instance' */

/* Module declarations from 'cpython.function' */

/* Module declarations from 'cpython.method' */

/* Module declarations from 'cpython.weakref' */

/* Module declarations from 'cpython.getargs' */

/* Module declarations from 'cpython.pythread' */

/* Module declarations from 'cpython.pystate' */

/* Module declarations from 'cpython.cobject' */

/* Module declarations from 'cpython.oldbuffer' */

/* Module declarations from 'cpython.set' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'cpython.pycapsule' */

/* Module declarations from 'cpython' */

/* Module declarations from 'array' */

/* Module declarations from 'cpython.array' */
static PyTypeObject *__pyx_ptype_7cpython_5array_array = 0;
static CYTHON_INLINE int __pyx_f_7cpython_5array_extend_buffer(arrayobject *, char *, Py_ssize_t); /*proto*/

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'ttygif.tty.image' */
static PyTypeObject *__pyx_ptype_6ttygif_3tty_5image_bounds = 0;
static PyTypeObject *__pyx_ptype_6ttygif_3tty_5image_point = 0;
static PyTypeObject *__pyx_ptype_6ttygif_3tty_5image_rect = 0;
static PyTypeObject *__pyx_ptype_6ttygif_3tty_5image_image = 0;

/* Module declarations from 'ttygif.tty.font' */
static PyTypeObject *__pyx_ptype_6ttygif_3tty_4font_font = 0;

/* Module declarations from 'ttygif.tty.theme' */
static PyTypeObject *__pyx_ptype_6ttygif_3tty_5theme_layer = 0;
static PyTypeObject *__pyx_ptype_6ttygif_3tty_5theme_theme = 0;

/* Module declarations from 'ttygif.tty.display_state' */
static PyTypeObject *__pyx_ptype_6ttygif_3tty_13display_state_display_state = 0;

/* Module declarations from 'ttygif.tty.terminal_graphics' */
static PyTypeObject *__pyx_ptype_6ttygif_3tty_17terminal_graphics_terminal_graphics = 0;

/* Module declarations from 'ttygif.tty.parser' */
static PyTypeObject *__pyx_ptype_6ttygif_3tty_6parser_term_parser = 0;
static PyTypeObject *__pyx_ptype_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence = 0;
static PyTypeObject *__pyx_ptype_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr = 0;
static PyTypeObject *__pyx_ptype_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr = 0;
static PyObject *__pyx_f_6ttygif_3tty_6parser___pyx_unpickle_term_parser__set_state(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "ttygif.tty.parser"
extern int __pyx_module_is_main_ttygif__tty__parser;
int __pyx_module_is_main_ttygif__tty__parser = 0;

/* Implementation of 'ttygif.tty.parser' */
static PyObject *__pyx_builtin_xrange;
static PyObject *__pyx_builtin_chr;
static PyObject *__pyx_builtin_round;
static PyObject *__pyx_builtin_MemoryError;
static const char __pyx_k_[] = "";
static const char __pyx_k_0[] = "{0}";
static const char __pyx_k_A[] = "A";
static const char __pyx_k_B[] = "B";
static const char __pyx_k_C[] = "C";
static const char __pyx_k_D[] = "D";
static const char __pyx_k_E[] = "E";
static const char __pyx_k_F[] = "F";
static const char __pyx_k_G[] = "G";
static const char __pyx_k_H[] = "H";
static const char __pyx_k_J[] = "J";
static const char __pyx_k_K[] = "K";
static const char __pyx_k_P[] = "P";
static const char __pyx_k_X[] = "X";
static const char __pyx_k_d[] = "d";
static const char __pyx_k_f[] = "f";
static const char __pyx_k_h[] = "h";
static const char __pyx_k_l[] = "l";
static const char __pyx_k_m[] = "m";
static const char __pyx_k_o[] = "o";
static const char __pyx_k_r[] = "r";
static const char __pyx_k_s[] = "s";
static const char __pyx_k_u[] = "u";
static const char __pyx_k_ED[] = "ED";
static const char __pyx_k_EL[] = "EL";
static const char __pyx_k_G0[] = "G0";
static const char __pyx_k_G1[] = "G1";
static const char __pyx_k_Hf[] = "Hf";
static const char __pyx_k__2[] = "*";
static const char __pyx_k__3[] = "~";
static const char __pyx_k__4[] = "`";
static const char __pyx_k__5[] = ";";
#if PY_MAJOR_VERSION >= 3
static const char __pyx_k__6[] = "(?:\033\\]|\302\235).*?(?:\033\\\\|[\007\302\234])";
#endif
#if PY_MAJOR_VERSION < 3
static const char __pyx_k__7[] = "(?:\033\\]|\235).*?(?:\033\\\\|[\007\234])";
#endif
static const char __pyx_k__8[] = "(";
static const char __pyx_k__9[] = ")|(";
static const char __pyx_k_re[] = "re";
static const char __pyx_k_CHA[] = "CHA";
static const char __pyx_k_CNL[] = "CNL";
static const char __pyx_k_COL[] = "COL";
static const char __pyx_k_CPL[] = "CPL";
static const char __pyx_k_CSI[] = "CSI";
static const char __pyx_k_CUB[] = "CUB";
static const char __pyx_k_CUD[] = "CUD";
static const char __pyx_k_CUF[] = "CUF";
static const char __pyx_k_CUP[] = "CUP";
static const char __pyx_k_CUU[] = "CUU";
static const char __pyx_k_DCH[] = "DCH";
static const char __pyx_k_ECH[] = "ECH";
static const char __pyx_k_ERR[] = "ERR";
static const char __pyx_k_G_2[] = "[\033]\\%([@G*])";
static const char __pyx_k_HPA[] = "HPA";
static const char __pyx_k_HVP[] = "HVP";
static const char __pyx_k_JKm[] = "JKm";
static const char __pyx_k_OSC[] = "OSC";
static const char __pyx_k_RCP[] = "RCP";
static const char __pyx_k_SCP[] = "SCP";
static const char __pyx_k_VPA[] = "VPA";
static const char __pyx_k__10[] = ")";
static const char __pyx_k__13[] = "?";
static const char __pyx_k__16[] = "============";
static const char __pyx_k__17[] = ",";
static const char __pyx_k_chr[] = "chr";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_h_2[] = "?h";
static const char __pyx_k_l_2[] = "?l";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_ABCD[] = "ABCD";
static const char __pyx_k_B0UK[] = "[\033]\\(([B0UK])";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_file[] = "file";
static const char __pyx_k_join[] = "join";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_span[] = "span";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_text[] = "text";
static const char __pyx_k_type[] = "type";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_delay[] = "delay";
static const char __pyx_k_linux[] = "linux";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_round[] = "round";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_state[] = "state";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_20_0_1[] = "[\033]\\[(20[0-1]~)";
static const char __pyx_k_B0UK_2[] = "[\033]\\)([B0UK])";
static const char __pyx_k_DECRST[] = "DECRST";
static const char __pyx_k_DECSET[] = "DECSET";
static const char __pyx_k_SINGLE[] = "SINGLE";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_dict_2[] = "_dict";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_groups[] = "groups";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_params[] = "params";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_xrange[] = "xrange";
static const char __pyx_k_Count_0[] = "Count:{0}";
static const char __pyx_k_DECSTBM[] = "DECSTBM";
static const char __pyx_k_command[] = "command";
static const char __pyx_k_compile[] = "compile";
static const char __pyx_k_genexpr[] = "genexpr";
static const char __pyx_k_CHAR_SET[] = "CHAR_SET";
static const char __pyx_k_cDEHMZ78[] = "[\033]([cDEHMZ78>=])";
static const char __pyx_k_d_a_zA_Z[] = "[\033]\\[((?:\\d|;|<|>|=|\\?)*)([a-zA-Z])\002?";
static const char __pyx_k_esc_type[] = "esc_type";
static const char __pyx_k_finditer[] = "finditer";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_set_mode[] = "set_mode";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_timestamp[] = "timestamp";
static const char __pyx_k_debug_mode[] = "debug_mode";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_reset_mode[] = "reset_mode";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_term_parser[] = "term_parser";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_use_setstate[] = "use_setstate";
static const char __pyx_k_Impliment_0_1[] = "Impliment: {0}-{1}";
static const char __pyx_k_Sequence_List[] = "Sequence List";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_2_6x_3_3_5f_0_1[] = "{2: 6x} {3:3.5f} : {0}({1})";
static const char __pyx_k_BRACKETED_PASTE[] = "BRACKETED_PASTE";
static const char __pyx_k_bracketed_paste[] = "bracketed paste";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_terminal_graphics[] = "terminal_graphics";
static const char __pyx_k_ttygif_tty_parser[] = "ttygif.tty.parser";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_2_6x_3_3_5f_text_0_1[] = "{2:6x} {3:3.5f} : text('{0},{1}')";
static const char __pyx_k_cursor_save_position[] = "cursor_save_position";
static const char __pyx_k_cursor_restore_position[] = "cursor_restore_position";
static const char __pyx_k_pyx_unpickle_term_parser[] = "__pyx_unpickle_term_parser";
static const char __pyx_k_term_parser___reduce_cython[] = "term_parser.__reduce_cython__";
static const char __pyx_k_term_parser___setstate_cython[] = "term_parser.__setstate_cython__";
static const char __pyx_k_5_05x_CMD_0_Name_3_Command_1_Pa[] = "{5:05x} CMD:  '{0}', Name:'{3}', Command:{1}, Params:{2}  Timestamp:{4}";
static const char __pyx_k_Incompatible_checksums_s_vs_0x35[] = "Incompatible checksums (%s vs 0x35256c6 = (bracketed_paste, current_sequence_position, debug_mode, extra_text, g, last_timestamp, no_codes, sequence, sequence_pos))";
static const char __pyx_k_term_parser_stream_2_sequence_lo[] = "term_parser.stream_2_sequence.<locals>.genexpr";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_kp_s_0;
static PyObject *__pyx_kp_s_20_0_1;
static PyObject *__pyx_kp_s_2_6x_3_3_5f_0_1;
static PyObject *__pyx_kp_s_2_6x_3_3_5f_text_0_1;
static PyObject *__pyx_kp_s_5_05x_CMD_0_Name_3_Command_1_Pa;
static PyObject *__pyx_n_s_A;
static PyObject *__pyx_n_s_ABCD;
static PyObject *__pyx_n_s_B;
static PyObject *__pyx_kp_s_B0UK;
static PyObject *__pyx_kp_s_B0UK_2;
static PyObject *__pyx_n_s_BRACKETED_PASTE;
static PyObject *__pyx_n_s_C;
static PyObject *__pyx_n_s_CHA;
static PyObject *__pyx_n_s_CHAR_SET;
static PyObject *__pyx_n_s_CNL;
static PyObject *__pyx_n_s_COL;
static PyObject *__pyx_n_s_CPL;
static PyObject *__pyx_n_s_CSI;
static PyObject *__pyx_n_s_CUB;
static PyObject *__pyx_n_s_CUD;
static PyObject *__pyx_n_s_CUF;
static PyObject *__pyx_n_s_CUP;
static PyObject *__pyx_n_s_CUU;
static PyObject *__pyx_kp_s_Count_0;
static PyObject *__pyx_n_s_D;
static PyObject *__pyx_n_s_DCH;
static PyObject *__pyx_n_s_DECRST;
static PyObject *__pyx_n_s_DECSET;
static PyObject *__pyx_n_s_DECSTBM;
static PyObject *__pyx_n_s_E;
static PyObject *__pyx_n_s_ECH;
static PyObject *__pyx_n_s_ED;
static PyObject *__pyx_n_s_EL;
static PyObject *__pyx_n_s_ERR;
static PyObject *__pyx_n_s_F;
static PyObject *__pyx_n_s_G;
static PyObject *__pyx_n_s_G0;
static PyObject *__pyx_n_s_G1;
static PyObject *__pyx_kp_s_G_2;
static PyObject *__pyx_n_s_H;
static PyObject *__pyx_n_s_HPA;
static PyObject *__pyx_n_s_HVP;
static PyObject *__pyx_n_s_Hf;
static PyObject *__pyx_kp_s_Impliment_0_1;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x35;
static PyObject *__pyx_n_s_J;
static PyObject *__pyx_n_s_JKm;
static PyObject *__pyx_n_s_K;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_n_s_OSC;
static PyObject *__pyx_n_s_P;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_RCP;
static PyObject *__pyx_n_s_SCP;
static PyObject *__pyx_n_s_SINGLE;
static PyObject *__pyx_kp_s_Sequence_List;
static PyObject *__pyx_n_s_VPA;
static PyObject *__pyx_n_s_X;
static PyObject *__pyx_kp_s__10;
static PyObject *__pyx_kp_s__13;
static PyObject *__pyx_kp_s__16;
static PyObject *__pyx_kp_s__17;
static PyObject *__pyx_kp_s__2;
static PyObject *__pyx_kp_s__3;
static PyObject *__pyx_kp_s__4;
static PyObject *__pyx_kp_s__5;
static PyObject *__pyx_kp_s__7;
static PyObject *__pyx_kp_s__8;
static PyObject *__pyx_kp_s__9;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_kp_s_bracketed_paste;
static PyObject *__pyx_kp_s_cDEHMZ78;
static PyObject *__pyx_n_s_chr;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_command;
static PyObject *__pyx_n_s_compile;
static PyObject *__pyx_n_s_cursor_restore_position;
static PyObject *__pyx_n_s_cursor_save_position;
static PyObject *__pyx_n_s_d;
static PyObject *__pyx_kp_s_d_a_zA_Z;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_debug_mode;
static PyObject *__pyx_n_s_delay;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_dict_2;
static PyObject *__pyx_n_s_end;
static PyObject *__pyx_n_s_esc_type;
static PyObject *__pyx_n_s_f;
static PyObject *__pyx_n_s_file;
static PyObject *__pyx_n_s_finditer;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_genexpr;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_groups;
static PyObject *__pyx_n_s_h;
static PyObject *__pyx_kp_s_h_2;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_join;
static PyObject *__pyx_n_s_l;
static PyObject *__pyx_kp_s_l_2;
static PyObject *__pyx_n_s_linux;
static PyObject *__pyx_n_s_m;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_o;
static PyObject *__pyx_n_s_params;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_print;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_term_parser;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_r;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_re;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_reset_mode;
static PyObject *__pyx_n_s_round;
static PyObject *__pyx_n_s_s;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_set_mode;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_span;
static PyObject *__pyx_n_s_split;
static PyObject *__pyx_n_s_state;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_term_parser;
static PyObject *__pyx_n_s_term_parser___reduce_cython;
static PyObject *__pyx_n_s_term_parser___setstate_cython;
static PyObject *__pyx_n_s_term_parser_stream_2_sequence_lo;
static PyObject *__pyx_n_s_terminal_graphics;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_text;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_timestamp;
static PyObject *__pyx_n_s_ttygif_tty_parser;
static PyObject *__pyx_n_s_type;
static PyObject *__pyx_n_s_u;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_use_setstate;
static PyObject *__pyx_n_s_xrange;
static int __pyx_pf_6ttygif_3tty_6parser_11term_parser___init__(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *__pyx_v_terminal_graphics, PyObject *__pyx_v_debug_mode); /* proto */
static PyObject *__pyx_pf_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_genexpr(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_3genexpr(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_6ttygif_3tty_6parser_11term_parser_2__reduce_cython__(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6ttygif_3tty_6parser_11term_parser_4__setstate_cython__(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6ttygif_3tty_6parser___pyx_unpickle_term_parser(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7cpython_5array_5array___getbuffer__(arrayobject *__pyx_v_self, Py_buffer *__pyx_v_info, CYTHON_UNUSED int __pyx_v_flags); /* proto */
static void __pyx_pf_7cpython_5array_5array_2__releasebuffer__(CYTHON_UNUSED arrayobject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static PyObject *__pyx_tp_new_6ttygif_3tty_6parser_term_parser(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_5;
static PyObject *__pyx_int_7;
static PyObject *__pyx_int_8;
static PyObject *__pyx_int_27;
static PyObject *__pyx_int_30;
static PyObject *__pyx_int_37;
static PyObject *__pyx_int_38;
static PyObject *__pyx_int_39;
static PyObject *__pyx_int_40;
static PyObject *__pyx_int_47;
static PyObject *__pyx_int_48;
static PyObject *__pyx_int_49;
static PyObject *__pyx_int_90;
static PyObject *__pyx_int_97;
static PyObject *__pyx_int_100;
static PyObject *__pyx_int_107;
static PyObject *__pyx_int_200;
static PyObject *__pyx_int_201;
static PyObject *__pyx_int_255;
static PyObject *__pyx_int_55727814;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_slice__11;
static PyObject *__pyx_slice__14;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_codeobj__18;
static PyObject *__pyx_codeobj__19;
static PyObject *__pyx_codeobj__20;
/* Late includes */

/* "ttygif/tty/parser.pyx":14
 * 
 * cdef class term_parser:
 *     def __init__(self,terminal_graphics terminal_graphics,debug_mode=None):             # <<<<<<<<<<<<<<
 *         self.debug_mode=debug_mode
 *         self.sequence=[]
 */

/* Python wrapper */
static int __pyx_pw_6ttygif_3tty_6parser_11term_parser_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6ttygif_3tty_6parser_11term_parser_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *__pyx_v_terminal_graphics = 0;
  PyObject *__pyx_v_debug_mode = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_terminal_graphics,&__pyx_n_s_debug_mode,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_terminal_graphics)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_debug_mode);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 14, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_terminal_graphics = ((struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *)values[0]);
    __pyx_v_debug_mode = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 14, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_terminal_graphics), __pyx_ptype_6ttygif_3tty_17terminal_graphics_terminal_graphics, 1, "terminal_graphics", 0))) __PYX_ERR(0, 14, __pyx_L1_error)
  __pyx_r = __pyx_pf_6ttygif_3tty_6parser_11term_parser___init__(((struct __pyx_obj_6ttygif_3tty_6parser_term_parser *)__pyx_v_self), __pyx_v_terminal_graphics, __pyx_v_debug_mode);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6ttygif_3tty_6parser_11term_parser___init__(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *__pyx_v_terminal_graphics, PyObject *__pyx_v_debug_mode) {
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_TraceCall("__init__", __pyx_f[0], 14, 0, __PYX_ERR(0, 14, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":15
 * cdef class term_parser:
 *     def __init__(self,terminal_graphics terminal_graphics,debug_mode=None):
 *         self.debug_mode=debug_mode             # <<<<<<<<<<<<<<
 *         self.sequence=[]
 *         self.sequence_pos=0
 */
  __Pyx_INCREF(__pyx_v_debug_mode);
  __Pyx_GIVEREF(__pyx_v_debug_mode);
  __Pyx_GOTREF(__pyx_v_self->debug_mode);
  __Pyx_DECREF(__pyx_v_self->debug_mode);
  __pyx_v_self->debug_mode = __pyx_v_debug_mode;

  /* "ttygif/tty/parser.pyx":16
 *     def __init__(self,terminal_graphics terminal_graphics,debug_mode=None):
 *         self.debug_mode=debug_mode
 *         self.sequence=[]             # <<<<<<<<<<<<<<
 *         self.sequence_pos=0
 *         self.last_timestamp=0
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->sequence);
  __Pyx_DECREF(__pyx_v_self->sequence);
  __pyx_v_self->sequence = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":17
 *         self.debug_mode=debug_mode
 *         self.sequence=[]
 *         self.sequence_pos=0             # <<<<<<<<<<<<<<
 *         self.last_timestamp=0
 *         self.extra_text=""
 */
  __pyx_v_self->sequence_pos = 0;

  /* "ttygif/tty/parser.pyx":18
 *         self.sequence=[]
 *         self.sequence_pos=0
 *         self.last_timestamp=0             # <<<<<<<<<<<<<<
 *         self.extra_text=""
 *         self.no_codes=None
 */
  __pyx_v_self->last_timestamp = 0.0;

  /* "ttygif/tty/parser.pyx":19
 *         self.sequence_pos=0
 *         self.last_timestamp=0
 *         self.extra_text=""             # <<<<<<<<<<<<<<
 *         self.no_codes=None
 *         self.bracketed_paste=None
 */
  __Pyx_INCREF(__pyx_kp_s_);
  __Pyx_GIVEREF(__pyx_kp_s_);
  __Pyx_GOTREF(__pyx_v_self->extra_text);
  __Pyx_DECREF(__pyx_v_self->extra_text);
  __pyx_v_self->extra_text = __pyx_kp_s_;

  /* "ttygif/tty/parser.pyx":20
 *         self.last_timestamp=0
 *         self.extra_text=""
 *         self.no_codes=None             # <<<<<<<<<<<<<<
 *         self.bracketed_paste=None
 *         self.g=terminal_graphics
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->no_codes);
  __Pyx_DECREF(__pyx_v_self->no_codes);
  __pyx_v_self->no_codes = Py_None;

  /* "ttygif/tty/parser.pyx":21
 *         self.extra_text=""
 *         self.no_codes=None
 *         self.bracketed_paste=None             # <<<<<<<<<<<<<<
 *         self.g=terminal_graphics
 *         self.current_sequence_position=0
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->bracketed_paste);
  __Pyx_DECREF(__pyx_v_self->bracketed_paste);
  __pyx_v_self->bracketed_paste = Py_None;

  /* "ttygif/tty/parser.pyx":22
 *         self.no_codes=None
 *         self.bracketed_paste=None
 *         self.g=terminal_graphics             # <<<<<<<<<<<<<<
 *         self.current_sequence_position=0
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_terminal_graphics));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_terminal_graphics));
  __Pyx_GOTREF(__pyx_v_self->g);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->g));
  __pyx_v_self->g = __pyx_v_terminal_graphics;

  /* "ttygif/tty/parser.pyx":23
 *         self.bracketed_paste=None
 *         self.g=terminal_graphics
 *         self.current_sequence_position=0             # <<<<<<<<<<<<<<
 * 
 *     cdef ascii_safe(self,text):
 */
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  __Pyx_GOTREF(__pyx_v_self->current_sequence_position);
  __Pyx_DECREF(__pyx_v_self->current_sequence_position);
  __pyx_v_self->current_sequence_position = __pyx_int_0;

  /* "ttygif/tty/parser.pyx":14
 * 
 * cdef class term_parser:
 *     def __init__(self,terminal_graphics terminal_graphics,debug_mode=None):             # <<<<<<<<<<<<<<
 *         self.debug_mode=debug_mode
 *         self.sequence=[]
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":25
 *         self.current_sequence_position=0
 * 
 *     cdef ascii_safe(self,text):             # <<<<<<<<<<<<<<
 *         return ''.join([i if ord(i) < 128 else '*' for i in text])
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_ascii_safe(CYTHON_UNUSED struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_text) {
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  long __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ascii_safe", 0);
  __Pyx_TraceCall("ascii_safe", __pyx_f[0], 25, 0, __PYX_ERR(0, 25, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":26
 * 
 *     cdef ascii_safe(self,text):
 *         return ''.join([i if ord(i) < 128 else '*' for i in text])             # <<<<<<<<<<<<<<
 * 
 *     cdef info(self,text):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_v_text)) || PyTuple_CheckExact(__pyx_v_text)) {
    __pyx_t_2 = __pyx_v_text; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_text); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 26, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 26, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 26, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 26, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 26, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 26, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 26, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_6 = __Pyx_PyObject_Ord(__pyx_v_i); if (unlikely(__pyx_t_6 == ((long)(long)(Py_UCS4)-1))) __PYX_ERR(0, 26, __pyx_L1_error)
    if (((__pyx_t_6 < 0x80) != 0)) {
      __Pyx_INCREF(__pyx_v_i);
      __pyx_t_5 = __pyx_v_i;
    } else {
      __Pyx_INCREF(__pyx_kp_s__2);
      __pyx_t_5 = __pyx_kp_s__2;
    }
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 26, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyString_Join(__pyx_kp_s_, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ttygif/tty/parser.pyx":25
 *         self.current_sequence_position=0
 * 
 *     cdef ascii_safe(self,text):             # <<<<<<<<<<<<<<
 *         return ''.join([i if ord(i) < 128 else '*' for i in text])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.ascii_safe", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":28
 *         return ''.join([i if ord(i) < 128 else '*' for i in text])
 * 
 *     cdef info(self,text):             # <<<<<<<<<<<<<<
 *         if self.debug_mode:
 *             print(self.ascii_safe(text))
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_info(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_text) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("info", 0);
  __Pyx_TraceCall("info", __pyx_f[0], 28, 0, __PYX_ERR(0, 28, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":29
 * 
 *     cdef info(self,text):
 *         if self.debug_mode:             # <<<<<<<<<<<<<<
 *             print(self.ascii_safe(text))
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_self->debug_mode); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 29, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "ttygif/tty/parser.pyx":30
 *     cdef info(self,text):
 *         if self.debug_mode:
 *             print(self.ascii_safe(text))             # <<<<<<<<<<<<<<
 * 
 *     cdef clear_sequence(self):
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->ascii_safe(__pyx_v_self, __pyx_v_text); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 30, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_PrintOne(0, __pyx_t_2) < 0) __PYX_ERR(0, 30, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "ttygif/tty/parser.pyx":29
 * 
 *     cdef info(self,text):
 *         if self.debug_mode:             # <<<<<<<<<<<<<<
 *             print(self.ascii_safe(text))
 * 
 */
  }

  /* "ttygif/tty/parser.pyx":28
 *         return ''.join([i if ord(i) < 128 else '*' for i in text])
 * 
 *     cdef info(self,text):             # <<<<<<<<<<<<<<
 *         if self.debug_mode:
 *             print(self.ascii_safe(text))
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.info", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":32
 *             print(self.ascii_safe(text))
 * 
 *     cdef clear_sequence(self):             # <<<<<<<<<<<<<<
 *         self.sequence=[]
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_clear_sequence(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clear_sequence", 0);
  __Pyx_TraceCall("clear_sequence", __pyx_f[0], 32, 0, __PYX_ERR(0, 32, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":33
 * 
 *     cdef clear_sequence(self):
 *         self.sequence=[]             # <<<<<<<<<<<<<<
 * 
 *     cdef rgb_to_palette(self,r,g,b):
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->sequence);
  __Pyx_DECREF(__pyx_v_self->sequence);
  __pyx_v_self->sequence = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":32
 *             print(self.ascii_safe(text))
 * 
 *     cdef clear_sequence(self):             # <<<<<<<<<<<<<<
 *         self.sequence=[]
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.clear_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":35
 *         self.sequence=[]
 * 
 *     cdef rgb_to_palette(self,r,g,b):             # <<<<<<<<<<<<<<
 *         last_distance=-1
 *         mappeded_color=-1
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_rgb_to_palette(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_r, PyObject *__pyx_v_g, PyObject *__pyx_v_b) {
  PyObject *__pyx_v_last_distance = NULL;
  PyObject *__pyx_v_mappeded_color = NULL;
  arrayobject *__pyx_v_palette = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_mr = NULL;
  PyObject *__pyx_v_mg = NULL;
  PyObject *__pyx_v_mb = NULL;
  PyObject *__pyx_v_color_distance = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("rgb_to_palette", 0);
  __Pyx_TraceCall("rgb_to_palette", __pyx_f[0], 35, 0, __PYX_ERR(0, 35, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":36
 * 
 *     cdef rgb_to_palette(self,r,g,b):
 *         last_distance=-1             # <<<<<<<<<<<<<<
 *         mappeded_color=-1
 *         palette=self.g.screen.palette
 */
  __Pyx_INCREF(__pyx_int_neg_1);
  __pyx_v_last_distance = __pyx_int_neg_1;

  /* "ttygif/tty/parser.pyx":37
 *     cdef rgb_to_palette(self,r,g,b):
 *         last_distance=-1
 *         mappeded_color=-1             # <<<<<<<<<<<<<<
 *         palette=self.g.screen.palette
 *         for i in xrange(0,len(palette),3):
 */
  __Pyx_INCREF(__pyx_int_neg_1);
  __pyx_v_mappeded_color = __pyx_int_neg_1;

  /* "ttygif/tty/parser.pyx":38
 *         last_distance=-1
 *         mappeded_color=-1
 *         palette=self.g.screen.palette             # <<<<<<<<<<<<<<
 *         for i in xrange(0,len(palette),3):
 *             mr=palette[i]
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->g->screen->palette);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_palette = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":39
 *         mappeded_color=-1
 *         palette=self.g.screen.palette
 *         for i in xrange(0,len(palette),3):             # <<<<<<<<<<<<<<
 *             mr=palette[i]
 *             mg=palette[i+1]
 */
  if (unlikely(((PyObject *)__pyx_v_palette) == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 39, __pyx_L1_error)
  }
  __pyx_t_2 = Py_SIZE(((PyObject *)__pyx_v_palette)); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 39, __pyx_L1_error)
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_int_0);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_int_3);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_xrange, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_3 = __pyx_t_1; __Pyx_INCREF(__pyx_t_3); __pyx_t_2 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 39, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 39, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 39, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 39, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_3);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 39, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":40
 *         palette=self.g.screen.palette
 *         for i in xrange(0,len(palette),3):
 *             mr=palette[i]             # <<<<<<<<<<<<<<
 *             mg=palette[i+1]
 *             mb=palette[i+2]
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_palette), __pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_mr, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":41
 *         for i in xrange(0,len(palette),3):
 *             mr=palette[i]
 *             mg=palette[i+1]             # <<<<<<<<<<<<<<
 *             mb=palette[i+2]
 * 
 */
    __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_i, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 41, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_palette), __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 41, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF_SET(__pyx_v_mg, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "ttygif/tty/parser.pyx":42
 *             mr=palette[i]
 *             mg=palette[i+1]
 *             mb=palette[i+2]             # <<<<<<<<<<<<<<
 * 
 *             color_distance=(r-mr)*(r-mr)+(g-mg)*(g-mg)+(b-mb)*(b-mb)
 */
    __pyx_t_5 = __Pyx_PyInt_AddObjC(__pyx_v_i, __pyx_int_2, 2, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 42, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_palette), __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 42, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_mb, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":44
 *             mb=palette[i+2]
 * 
 *             color_distance=(r-mr)*(r-mr)+(g-mg)*(g-mg)+(b-mb)*(b-mb)             # <<<<<<<<<<<<<<
 *             if last_distance==-1 or color_distance<last_distance:
 *                 last_distance=color_distance
 */
    __pyx_t_1 = PyNumber_Subtract(__pyx_v_r, __pyx_v_mr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = PyNumber_Subtract(__pyx_v_r, __pyx_v_mr); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyNumber_Multiply(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyNumber_Subtract(__pyx_v_g, __pyx_v_mg); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyNumber_Subtract(__pyx_v_g, __pyx_v_mg); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = PyNumber_Multiply(__pyx_t_5, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyNumber_Add(__pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyNumber_Subtract(__pyx_v_b, __pyx_v_mb); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PyNumber_Subtract(__pyx_v_b, __pyx_v_mb); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = PyNumber_Multiply(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyNumber_Add(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_color_distance, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "ttygif/tty/parser.pyx":45
 * 
 *             color_distance=(r-mr)*(r-mr)+(g-mg)*(g-mg)+(b-mb)*(b-mb)
 *             if last_distance==-1 or color_distance<last_distance:             # <<<<<<<<<<<<<<
 *                 last_distance=color_distance
 *                 mappeded_color=i
 */
    __pyx_t_6 = __Pyx_PyInt_EqObjC(__pyx_v_last_distance, __pyx_int_neg_1, -1L, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 45, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 45, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_6 = PyObject_RichCompare(__pyx_v_color_distance, __pyx_v_last_distance, Py_LT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 45, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 45, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_8 = __pyx_t_9;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_8) {

      /* "ttygif/tty/parser.pyx":46
 *             color_distance=(r-mr)*(r-mr)+(g-mg)*(g-mg)+(b-mb)*(b-mb)
 *             if last_distance==-1 or color_distance<last_distance:
 *                 last_distance=color_distance             # <<<<<<<<<<<<<<
 *                 mappeded_color=i
 *         if mappeded_color>255:
 */
      __Pyx_INCREF(__pyx_v_color_distance);
      __Pyx_DECREF_SET(__pyx_v_last_distance, __pyx_v_color_distance);

      /* "ttygif/tty/parser.pyx":47
 *             if last_distance==-1 or color_distance<last_distance:
 *                 last_distance=color_distance
 *                 mappeded_color=i             # <<<<<<<<<<<<<<
 *         if mappeded_color>255:
 *             print r,g,b,mappeded_color
 */
      __Pyx_INCREF(__pyx_v_i);
      __Pyx_DECREF_SET(__pyx_v_mappeded_color, __pyx_v_i);

      /* "ttygif/tty/parser.pyx":45
 * 
 *             color_distance=(r-mr)*(r-mr)+(g-mg)*(g-mg)+(b-mb)*(b-mb)
 *             if last_distance==-1 or color_distance<last_distance:             # <<<<<<<<<<<<<<
 *                 last_distance=color_distance
 *                 mappeded_color=i
 */
    }

    /* "ttygif/tty/parser.pyx":39
 *         mappeded_color=-1
 *         palette=self.g.screen.palette
 *         for i in xrange(0,len(palette),3):             # <<<<<<<<<<<<<<
 *             mr=palette[i]
 *             mg=palette[i+1]
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ttygif/tty/parser.pyx":48
 *                 last_distance=color_distance
 *                 mappeded_color=i
 *         if mappeded_color>255:             # <<<<<<<<<<<<<<
 *             print r,g,b,mappeded_color
 *         return mappeded_color
 */
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_mappeded_color, __pyx_int_255, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 48, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_8) {

    /* "ttygif/tty/parser.pyx":49
 *                 mappeded_color=i
 *         if mappeded_color>255:
 *             print r,g,b,mappeded_color             # <<<<<<<<<<<<<<
 *         return mappeded_color
 * 
 */
    __pyx_t_3 = PyTuple_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 49, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_r);
    __Pyx_GIVEREF(__pyx_v_r);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_r);
    __Pyx_INCREF(__pyx_v_g);
    __Pyx_GIVEREF(__pyx_v_g);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_g);
    __Pyx_INCREF(__pyx_v_b);
    __Pyx_GIVEREF(__pyx_v_b);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_b);
    __Pyx_INCREF(__pyx_v_mappeded_color);
    __Pyx_GIVEREF(__pyx_v_mappeded_color);
    PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_v_mappeded_color);
    if (__Pyx_Print(0, __pyx_t_3, 1) < 0) __PYX_ERR(0, 49, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "ttygif/tty/parser.pyx":48
 *                 last_distance=color_distance
 *                 mappeded_color=i
 *         if mappeded_color>255:             # <<<<<<<<<<<<<<
 *             print r,g,b,mappeded_color
 *         return mappeded_color
 */
  }

  /* "ttygif/tty/parser.pyx":50
 *         if mappeded_color>255:
 *             print r,g,b,mappeded_color
 *         return mappeded_color             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_mappeded_color);
  __pyx_r = __pyx_v_mappeded_color;
  goto __pyx_L0;

  /* "ttygif/tty/parser.pyx":35
 *         self.sequence=[]
 * 
 *     cdef rgb_to_palette(self,r,g,b):             # <<<<<<<<<<<<<<
 *         last_distance=-1
 *         mappeded_color=-1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.rgb_to_palette", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_last_distance);
  __Pyx_XDECREF(__pyx_v_mappeded_color);
  __Pyx_XDECREF((PyObject *)__pyx_v_palette);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_mr);
  __Pyx_XDECREF(__pyx_v_mg);
  __Pyx_XDECREF(__pyx_v_mb);
  __Pyx_XDECREF(__pyx_v_color_distance);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":53
 * 
 * 
 *     cdef remap_character(self,character):             # <<<<<<<<<<<<<<
 *     #"""maps a character to ascii"""
 *       #print character
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_remap_character(CYTHON_UNUSED struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_character) {
  int __pyx_v_c;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  long __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("remap_character", 0);
  __Pyx_TraceCall("remap_character", __pyx_f[0], 53, 0, __PYX_ERR(0, 53, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":57
 *       #print character
 *         #print character
 *         if character==None:             # <<<<<<<<<<<<<<
 *             return ""
 * 
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_character, Py_None, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 57, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":58
 *         #print character
 *         if character==None:
 *             return ""             # <<<<<<<<<<<<<<
 * 
 *         cdef int c=ord(character)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_s_);
    __pyx_r = __pyx_kp_s_;
    goto __pyx_L0;

    /* "ttygif/tty/parser.pyx":57
 *       #print character
 *         #print character
 *         if character==None:             # <<<<<<<<<<<<<<
 *             return ""
 * 
 */
  }

  /* "ttygif/tty/parser.pyx":60
 *             return ""
 * 
 *         cdef int c=ord(character)             # <<<<<<<<<<<<<<
 *         #character=character.encode('latin-1')
 *         if c>255:
 */
  __pyx_t_3 = __Pyx_PyObject_Ord(__pyx_v_character); if (unlikely(__pyx_t_3 == ((long)(long)(Py_UCS4)-1))) __PYX_ERR(0, 60, __pyx_L1_error)
  __pyx_v_c = __pyx_t_3;

  /* "ttygif/tty/parser.pyx":62
 *         cdef int c=ord(character)
 *         #character=character.encode('latin-1')
 *         if c>255:             # <<<<<<<<<<<<<<
 *             if c==8216:
 *                 c=39
 */
  __pyx_t_2 = ((__pyx_v_c > 0xFF) != 0);
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":63
 *         #character=character.encode('latin-1')
 *         if c>255:
 *             if c==8216:             # <<<<<<<<<<<<<<
 *                 c=39
 *             elif c==8217:
 */
    switch (__pyx_v_c) {
      case 0x2018:

      /* "ttygif/tty/parser.pyx":64
 *         if c>255:
 *             if c==8216:
 *                 c=39             # <<<<<<<<<<<<<<
 *             elif c==8217:
 *                 c=39
 */
      __pyx_v_c = 39;

      /* "ttygif/tty/parser.pyx":63
 *         #character=character.encode('latin-1')
 *         if c>255:
 *             if c==8216:             # <<<<<<<<<<<<<<
 *                 c=39
 *             elif c==8217:
 */
      break;
      case 0x2019:

      /* "ttygif/tty/parser.pyx":66
 *                 c=39
 *             elif c==8217:
 *                 c=39             # <<<<<<<<<<<<<<
 *             elif c==9472:
 *                 c=196
 */
      __pyx_v_c = 39;

      /* "ttygif/tty/parser.pyx":65
 *             if c==8216:
 *                 c=39
 *             elif c==8217:             # <<<<<<<<<<<<<<
 *                 c=39
 *             elif c==9472:
 */
      break;
      case 0x2500:

      /* "ttygif/tty/parser.pyx":68
 *                 c=39
 *             elif c==9472:
 *                 c=196             # <<<<<<<<<<<<<<
 *             elif c==9474:
 *                 c=179
 */
      __pyx_v_c = 0xC4;

      /* "ttygif/tty/parser.pyx":67
 *             elif c==8217:
 *                 c=39
 *             elif c==9472:             # <<<<<<<<<<<<<<
 *                 c=196
 *             elif c==9474:
 */
      break;
      case 0x2502:

      /* "ttygif/tty/parser.pyx":70
 *                 c=196
 *             elif c==9474:
 *                 c=179             # <<<<<<<<<<<<<<
 *             elif c==9484:
 *                 c=218
 */
      __pyx_v_c = 0xB3;

      /* "ttygif/tty/parser.pyx":69
 *             elif c==9472:
 *                 c=196
 *             elif c==9474:             # <<<<<<<<<<<<<<
 *                 c=179
 *             elif c==9484:
 */
      break;
      case 0x250C:

      /* "ttygif/tty/parser.pyx":72
 *                 c=179
 *             elif c==9484:
 *                 c=218             # <<<<<<<<<<<<<<
 *             elif c==9488:
 *                 c=191
 */
      __pyx_v_c = 0xDA;

      /* "ttygif/tty/parser.pyx":71
 *             elif c==9474:
 *                 c=179
 *             elif c==9484:             # <<<<<<<<<<<<<<
 *                 c=218
 *             elif c==9488:
 */
      break;
      case 0x2510:

      /* "ttygif/tty/parser.pyx":74
 *                 c=218
 *             elif c==9488:
 *                 c=191             # <<<<<<<<<<<<<<
 *             elif c==9492:
 *                 c=192
 */
      __pyx_v_c = 0xBF;

      /* "ttygif/tty/parser.pyx":73
 *             elif c==9484:
 *                 c=218
 *             elif c==9488:             # <<<<<<<<<<<<<<
 *                 c=191
 *             elif c==9492:
 */
      break;
      case 0x2514:

      /* "ttygif/tty/parser.pyx":76
 *                 c=191
 *             elif c==9492:
 *                 c=192             # <<<<<<<<<<<<<<
 *             elif c==9496:
 *                 c=217
 */
      __pyx_v_c = 0xC0;

      /* "ttygif/tty/parser.pyx":75
 *             elif c==9488:
 *                 c=191
 *             elif c==9492:             # <<<<<<<<<<<<<<
 *                 c=192
 *             elif c==9496:
 */
      break;
      case 0x2518:

      /* "ttygif/tty/parser.pyx":78
 *                 c=192
 *             elif c==9496:
 *                 c=217             # <<<<<<<<<<<<<<
 *             elif c==9600:
 *                 c=223
 */
      __pyx_v_c = 0xD9;

      /* "ttygif/tty/parser.pyx":77
 *             elif c==9492:
 *                 c=192
 *             elif c==9496:             # <<<<<<<<<<<<<<
 *                 c=217
 *             elif c==9600:
 */
      break;
      case 0x2580:

      /* "ttygif/tty/parser.pyx":80
 *                 c=217
 *             elif c==9600:
 *                 c=223             # <<<<<<<<<<<<<<
 *             elif c==9604:
 *                 c=220
 */
      __pyx_v_c = 0xDF;

      /* "ttygif/tty/parser.pyx":79
 *             elif c==9496:
 *                 c=217
 *             elif c==9600:             # <<<<<<<<<<<<<<
 *                 c=223
 *             elif c==9604:
 */
      break;
      case 0x2584:

      /* "ttygif/tty/parser.pyx":82
 *                 c=223
 *             elif c==9604:
 *                 c=220             # <<<<<<<<<<<<<<
 *             elif c==9608:
 *                 c=219
 */
      __pyx_v_c = 0xDC;

      /* "ttygif/tty/parser.pyx":81
 *             elif c==9600:
 *                 c=223
 *             elif c==9604:             # <<<<<<<<<<<<<<
 *                 c=220
 *             elif c==9608:
 */
      break;
      case 0x2588:

      /* "ttygif/tty/parser.pyx":84
 *                 c=220
 *             elif c==9608:
 *                 c=219             # <<<<<<<<<<<<<<
 *             elif c==9612:
 *                 c=221
 */
      __pyx_v_c = 0xDB;

      /* "ttygif/tty/parser.pyx":83
 *             elif c==9604:
 *                 c=220
 *             elif c==9608:             # <<<<<<<<<<<<<<
 *                 c=219
 *             elif c==9612:
 */
      break;
      case 0x258C:

      /* "ttygif/tty/parser.pyx":86
 *                 c=219
 *             elif c==9612:
 *                 c=221             # <<<<<<<<<<<<<<
 *             elif c==9616:
 *                 c=222
 */
      __pyx_v_c = 0xDD;

      /* "ttygif/tty/parser.pyx":85
 *             elif c==9608:
 *                 c=219
 *             elif c==9612:             # <<<<<<<<<<<<<<
 *                 c=221
 *             elif c==9616:
 */
      break;
      case 0x2590:

      /* "ttygif/tty/parser.pyx":88
 *                 c=221
 *             elif c==9616:
 *                 c=222             # <<<<<<<<<<<<<<
 *             elif c==9617:
 *                 c=176
 */
      __pyx_v_c = 0xDE;

      /* "ttygif/tty/parser.pyx":87
 *             elif c==9612:
 *                 c=221
 *             elif c==9616:             # <<<<<<<<<<<<<<
 *                 c=222
 *             elif c==9617:
 */
      break;
      case 0x2591:

      /* "ttygif/tty/parser.pyx":90
 *                 c=222
 *             elif c==9617:
 *                 c=176             # <<<<<<<<<<<<<<
 *             elif c==9618:
 *                 c=177
 */
      __pyx_v_c = 0xB0;

      /* "ttygif/tty/parser.pyx":89
 *             elif c==9616:
 *                 c=222
 *             elif c==9617:             # <<<<<<<<<<<<<<
 *                 c=176
 *             elif c==9618:
 */
      break;
      case 0x2592:

      /* "ttygif/tty/parser.pyx":92
 *                 c=176
 *             elif c==9618:
 *                 c=177             # <<<<<<<<<<<<<<
 *             elif c==10140:
 *                 c=26
 */
      __pyx_v_c = 0xB1;

      /* "ttygif/tty/parser.pyx":91
 *             elif c==9617:
 *                 c=176
 *             elif c==9618:             # <<<<<<<<<<<<<<
 *                 c=177
 *             elif c==10140:
 */
      break;
      case 0x279C:

      /* "ttygif/tty/parser.pyx":94
 *                 c=177
 *             elif c==10140:
 *                 c=26             # <<<<<<<<<<<<<<
 *             else:
 *                 #print ("Missing character: {0}     ".format(c))
 */
      __pyx_v_c = 26;

      /* "ttygif/tty/parser.pyx":93
 *             elif c==9618:
 *                 c=177
 *             elif c==10140:             # <<<<<<<<<<<<<<
 *                 c=26
 *             else:
 */
      break;
      default:

      /* "ttygif/tty/parser.pyx":97
 *             else:
 *                 #print ("Missing character: {0}     ".format(c))
 *                 c=ord('*')             # <<<<<<<<<<<<<<
 *         return chr(c)
 * 
 */
      __pyx_v_c = 42;
      break;
    }

    /* "ttygif/tty/parser.pyx":62
 *         cdef int c=ord(character)
 *         #character=character.encode('latin-1')
 *         if c>255:             # <<<<<<<<<<<<<<
 *             if c==8216:
 *                 c=39
 */
  }

  /* "ttygif/tty/parser.pyx":98
 *                 #print ("Missing character: {0}     ".format(c))
 *                 c=ord('*')
 *         return chr(c)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_c); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_chr, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "ttygif/tty/parser.pyx":53
 * 
 * 
 *     cdef remap_character(self,character):             # <<<<<<<<<<<<<<
 *     #"""maps a character to ascii"""
 *       #print character
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.remap_character", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":103
 * 
 * 
 *     cdef render_to_buffer(self):             # <<<<<<<<<<<<<<
 * 
 *         new_sequence_pos=self.sequence_pos #self.sequence_pos:
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_render_to_buffer(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self) {
  PyObject *__pyx_v_new_sequence_pos = NULL;
  PyObject *__pyx_v_event = NULL;
  PyObject *__pyx_v_params = NULL;
  PyObject *__pyx_v_command = NULL;
  PyObject *__pyx_v_esc_type = NULL;
  PyObject *__pyx_v_groups = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("render_to_buffer", 0);
  __Pyx_TraceCall("render_to_buffer", __pyx_f[0], 103, 0, __PYX_ERR(0, 103, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":105
 *     cdef render_to_buffer(self):
 * 
 *         new_sequence_pos=self.sequence_pos #self.sequence_pos:             # <<<<<<<<<<<<<<
 * 
 *         for event in self.sequence[self.sequence_pos:]:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->sequence_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_new_sequence_pos = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":107
 *         new_sequence_pos=self.sequence_pos #self.sequence_pos:
 * 
 *         for event in self.sequence[self.sequence_pos:]:             # <<<<<<<<<<<<<<
 *             self.current_sequence_position=new_sequence_pos
 *             new_sequence_pos+=1
 */
  __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_self->sequence, __pyx_v_self->sequence_pos, 0, NULL, NULL, NULL, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 107, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 107, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 107, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 107, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 107, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 107, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_event, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":108
 * 
 *         for event in self.sequence[self.sequence_pos:]:
 *             self.current_sequence_position=new_sequence_pos             # <<<<<<<<<<<<<<
 *             new_sequence_pos+=1
 *             if   event['type']=='text':
 */
    __Pyx_INCREF(__pyx_v_new_sequence_pos);
    __Pyx_GIVEREF(__pyx_v_new_sequence_pos);
    __Pyx_GOTREF(__pyx_v_self->current_sequence_position);
    __Pyx_DECREF(__pyx_v_self->current_sequence_position);
    __pyx_v_self->current_sequence_position = __pyx_v_new_sequence_pos;

    /* "ttygif/tty/parser.pyx":109
 *         for event in self.sequence[self.sequence_pos:]:
 *             self.current_sequence_position=new_sequence_pos
 *             new_sequence_pos+=1             # <<<<<<<<<<<<<<
 *             if   event['type']=='text':
 *                 #print event
 */
    __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_new_sequence_pos, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_new_sequence_pos, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":110
 *             self.current_sequence_position=new_sequence_pos
 *             new_sequence_pos+=1
 *             if   event['type']=='text':             # <<<<<<<<<<<<<<
 *                 #print event
 *                 self.cmd_render_text(event)
 */
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_t_1, __pyx_n_s_text, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 110, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_5) {

      /* "ttygif/tty/parser.pyx":112
 *             if   event['type']=='text':
 *                 #print event
 *                 self.cmd_render_text(event)             # <<<<<<<<<<<<<<
 *                 continue
 *             params   =event['params']
 */
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_render_text(__pyx_v_self, __pyx_v_event); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 112, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ttygif/tty/parser.pyx":113
 *                 #print event
 *                 self.cmd_render_text(event)
 *                 continue             # <<<<<<<<<<<<<<
 *             params   =event['params']
 *             command  =event['command']
 */
      goto __pyx_L3_continue;

      /* "ttygif/tty/parser.pyx":110
 *             self.current_sequence_position=new_sequence_pos
 *             new_sequence_pos+=1
 *             if   event['type']=='text':             # <<<<<<<<<<<<<<
 *                 #print event
 *                 self.cmd_render_text(event)
 */
    }

    /* "ttygif/tty/parser.pyx":114
 *                 self.cmd_render_text(event)
 *                 continue
 *             params   =event['params']             # <<<<<<<<<<<<<<
 *             command  =event['command']
 *             esc_type =event['esc_type']
 */
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_params, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":115
 *                 continue
 *             params   =event['params']
 *             command  =event['command']             # <<<<<<<<<<<<<<
 *             esc_type =event['esc_type']
 *             groups   =event['groups']
 */
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_command); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 115, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_command, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":116
 *             params   =event['params']
 *             command  =event['command']
 *             esc_type =event['esc_type']             # <<<<<<<<<<<<<<
 *             groups   =event['groups']
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_esc_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_esc_type, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":117
 *             command  =event['command']
 *             esc_type =event['esc_type']
 *             groups   =event['groups']             # <<<<<<<<<<<<<<
 * 
 *             if self.bracketed_paste:
 */
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_groups); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_groups, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":119
 *             groups   =event['groups']
 * 
 *             if self.bracketed_paste:             # <<<<<<<<<<<<<<
 *                 if  self.no_codes:
 *                     if esc_type=='CSI' and  command=='~':
 */
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_self->bracketed_paste); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 119, __pyx_L1_error)
    if (__pyx_t_5) {

      /* "ttygif/tty/parser.pyx":120
 * 
 *             if self.bracketed_paste:
 *                 if  self.no_codes:             # <<<<<<<<<<<<<<
 *                     if esc_type=='CSI' and  command=='~':
 *                         self.process_CSI(command,params)
 */
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_self->no_codes); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 120, __pyx_L1_error)
      if (__pyx_t_5) {

        /* "ttygif/tty/parser.pyx":121
 *             if self.bracketed_paste:
 *                 if  self.no_codes:
 *                     if esc_type=='CSI' and  command=='~':             # <<<<<<<<<<<<<<
 *                         self.process_CSI(command,params)
 *                         continue
 */
        __pyx_t_6 = (__Pyx_PyString_Equals(__pyx_v_esc_type, __pyx_n_s_CSI, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 121, __pyx_L1_error)
        if (__pyx_t_6) {
        } else {
          __pyx_t_5 = __pyx_t_6;
          goto __pyx_L9_bool_binop_done;
        }
        __pyx_t_6 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_kp_s__3, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 121, __pyx_L1_error)
        __pyx_t_5 = __pyx_t_6;
        __pyx_L9_bool_binop_done:;
        if (__pyx_t_5) {

          /* "ttygif/tty/parser.pyx":122
 *                 if  self.no_codes:
 *                     if esc_type=='CSI' and  command=='~':
 *                         self.process_CSI(command,params)             # <<<<<<<<<<<<<<
 *                         continue
 *                     else:
 */
          __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->process_CSI(__pyx_v_self, __pyx_v_command, __pyx_v_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "ttygif/tty/parser.pyx":123
 *                     if esc_type=='CSI' and  command=='~':
 *                         self.process_CSI(command,params)
 *                         continue             # <<<<<<<<<<<<<<
 *                     else:
 *                         self.cmd_render_text(event)
 */
          goto __pyx_L3_continue;

          /* "ttygif/tty/parser.pyx":121
 *             if self.bracketed_paste:
 *                 if  self.no_codes:
 *                     if esc_type=='CSI' and  command=='~':             # <<<<<<<<<<<<<<
 *                         self.process_CSI(command,params)
 *                         continue
 */
        }

        /* "ttygif/tty/parser.pyx":125
 *                         continue
 *                     else:
 *                         self.cmd_render_text(event)             # <<<<<<<<<<<<<<
 *                         continue
 * 
 */
        /*else*/ {
          __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_render_text(__pyx_v_self, __pyx_v_event); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 125, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "ttygif/tty/parser.pyx":126
 *                     else:
 *                         self.cmd_render_text(event)
 *                         continue             # <<<<<<<<<<<<<<
 * 
 * 
 */
          goto __pyx_L3_continue;
        }

        /* "ttygif/tty/parser.pyx":120
 * 
 *             if self.bracketed_paste:
 *                 if  self.no_codes:             # <<<<<<<<<<<<<<
 *                     if esc_type=='CSI' and  command=='~':
 *                         self.process_CSI(command,params)
 */
      }

      /* "ttygif/tty/parser.pyx":119
 *             groups   =event['groups']
 * 
 *             if self.bracketed_paste:             # <<<<<<<<<<<<<<
 *                 if  self.no_codes:
 *                     if esc_type=='CSI' and  command=='~':
 */
    }

    /* "ttygif/tty/parser.pyx":129
 * 
 * 
 *             if   esc_type=='OSC'      : self.procces_OSC(groups)             # <<<<<<<<<<<<<<
 *             elif esc_type=='SINGLE'   : self.process_SINGLE(groups[1])
 *             elif esc_type=='CHAR_SET' : self.process_CHAR_SET(groups[3])
 */
    __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_esc_type, __pyx_n_s_OSC, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 129, __pyx_L1_error)
    if (__pyx_t_5) {
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->procces_OSC(__pyx_v_self, __pyx_v_groups); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 129, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L11;
    }

    /* "ttygif/tty/parser.pyx":130
 * 
 *             if   esc_type=='OSC'      : self.procces_OSC(groups)
 *             elif esc_type=='SINGLE'   : self.process_SINGLE(groups[1])             # <<<<<<<<<<<<<<
 *             elif esc_type=='CHAR_SET' : self.process_CHAR_SET(groups[3])
 *             elif esc_type=='G0'       : self.process_G0(groups[5])
 */
    __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_esc_type, __pyx_n_s_SINGLE, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 130, __pyx_L1_error)
    if (__pyx_t_5) {
      __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_groups, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 130, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->process_SINGLE(__pyx_v_self, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 130, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L11;
    }

    /* "ttygif/tty/parser.pyx":131
 *             if   esc_type=='OSC'      : self.procces_OSC(groups)
 *             elif esc_type=='SINGLE'   : self.process_SINGLE(groups[1])
 *             elif esc_type=='CHAR_SET' : self.process_CHAR_SET(groups[3])             # <<<<<<<<<<<<<<
 *             elif esc_type=='G0'       : self.process_G0(groups[5])
 *             elif esc_type=='G1'       : self.process_G1(groups[7])
 */
    __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_esc_type, __pyx_n_s_CHAR_SET, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 131, __pyx_L1_error)
    if (__pyx_t_5) {
      __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_groups, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 131, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->process_CHAR_SET(__pyx_v_self, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 131, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L11;
    }

    /* "ttygif/tty/parser.pyx":132
 *             elif esc_type=='SINGLE'   : self.process_SINGLE(groups[1])
 *             elif esc_type=='CHAR_SET' : self.process_CHAR_SET(groups[3])
 *             elif esc_type=='G0'       : self.process_G0(groups[5])             # <<<<<<<<<<<<<<
 *             elif esc_type=='G1'       : self.process_G1(groups[7])
 *             elif esc_type=='CSI'      : self.process_CSI(command,params)
 */
    __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_esc_type, __pyx_n_s_G0, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 132, __pyx_L1_error)
    if (__pyx_t_5) {
      __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_groups, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->process_G0(__pyx_v_self, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 132, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L11;
    }

    /* "ttygif/tty/parser.pyx":133
 *             elif esc_type=='CHAR_SET' : self.process_CHAR_SET(groups[3])
 *             elif esc_type=='G0'       : self.process_G0(groups[5])
 *             elif esc_type=='G1'       : self.process_G1(groups[7])             # <<<<<<<<<<<<<<
 *             elif esc_type=='CSI'      : self.process_CSI(command,params)
 * 
 */
    __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_esc_type, __pyx_n_s_G1, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 133, __pyx_L1_error)
    if (__pyx_t_5) {
      __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_groups, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 133, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->process_G1(__pyx_v_self, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L11;
    }

    /* "ttygif/tty/parser.pyx":134
 *             elif esc_type=='G0'       : self.process_G0(groups[5])
 *             elif esc_type=='G1'       : self.process_G1(groups[7])
 *             elif esc_type=='CSI'      : self.process_CSI(command,params)             # <<<<<<<<<<<<<<
 * 
 *         self.sequence_pos=new_sequence_pos
 */
    __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_esc_type, __pyx_n_s_CSI, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 134, __pyx_L1_error)
    if (__pyx_t_5) {
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->process_CSI(__pyx_v_self, __pyx_v_command, __pyx_v_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L11:;

    /* "ttygif/tty/parser.pyx":107
 *         new_sequence_pos=self.sequence_pos #self.sequence_pos:
 * 
 *         for event in self.sequence[self.sequence_pos:]:             # <<<<<<<<<<<<<<
 *             self.current_sequence_position=new_sequence_pos
 *             new_sequence_pos+=1
 */
    __pyx_L3_continue:;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ttygif/tty/parser.pyx":136
 *             elif esc_type=='CSI'      : self.process_CSI(command,params)
 * 
 *         self.sequence_pos=new_sequence_pos             # <<<<<<<<<<<<<<
 * 
 *     # TODO STUBS
 */
  __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_v_new_sequence_pos); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 136, __pyx_L1_error)
  __pyx_v_self->sequence_pos = __pyx_t_8;

  /* "ttygif/tty/parser.pyx":103
 * 
 * 
 *     cdef render_to_buffer(self):             # <<<<<<<<<<<<<<
 * 
 *         new_sequence_pos=self.sequence_pos #self.sequence_pos:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.render_to_buffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_new_sequence_pos);
  __Pyx_XDECREF(__pyx_v_event);
  __Pyx_XDECREF(__pyx_v_params);
  __Pyx_XDECREF(__pyx_v_command);
  __Pyx_XDECREF(__pyx_v_esc_type);
  __Pyx_XDECREF(__pyx_v_groups);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":139
 * 
 *     # TODO STUBS
 *     cdef procces_OSC(self,groups):             # <<<<<<<<<<<<<<
 *         self.info(groups)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_procces_OSC(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_groups) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("procces_OSC", 0);
  __Pyx_TraceCall("procces_OSC", __pyx_f[0], 139, 0, __PYX_ERR(0, 139, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":140
 *     # TODO STUBS
 *     cdef procces_OSC(self,groups):
 *         self.info(groups)             # <<<<<<<<<<<<<<
 * 
 *     cdef process_SINGLE(self,groups):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->info(__pyx_v_self, __pyx_v_groups); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":139
 * 
 *     # TODO STUBS
 *     cdef procces_OSC(self,groups):             # <<<<<<<<<<<<<<
 *         self.info(groups)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.procces_OSC", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":142
 *         self.info(groups)
 * 
 *     cdef process_SINGLE(self,groups):             # <<<<<<<<<<<<<<
 *         self.info(groups)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_process_SINGLE(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_groups) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("process_SINGLE", 0);
  __Pyx_TraceCall("process_SINGLE", __pyx_f[0], 142, 0, __PYX_ERR(0, 142, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":143
 * 
 *     cdef process_SINGLE(self,groups):
 *         self.info(groups)             # <<<<<<<<<<<<<<
 * 
 *     cdef process_CHAR_SET(self,groups):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->info(__pyx_v_self, __pyx_v_groups); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":142
 *         self.info(groups)
 * 
 *     cdef process_SINGLE(self,groups):             # <<<<<<<<<<<<<<
 *         self.info(groups)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.process_SINGLE", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":145
 *         self.info(groups)
 * 
 *     cdef process_CHAR_SET(self,groups):             # <<<<<<<<<<<<<<
 *         self.info(groups)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_process_CHAR_SET(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_groups) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("process_CHAR_SET", 0);
  __Pyx_TraceCall("process_CHAR_SET", __pyx_f[0], 145, 0, __PYX_ERR(0, 145, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":146
 * 
 *     cdef process_CHAR_SET(self,groups):
 *         self.info(groups)             # <<<<<<<<<<<<<<
 * 
 *     cdef process_G0(self,groups):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->info(__pyx_v_self, __pyx_v_groups); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":145
 *         self.info(groups)
 * 
 *     cdef process_CHAR_SET(self,groups):             # <<<<<<<<<<<<<<
 *         self.info(groups)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.process_CHAR_SET", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":148
 *         self.info(groups)
 * 
 *     cdef process_G0(self,groups):             # <<<<<<<<<<<<<<
 *         self.info(groups)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_process_G0(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_groups) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("process_G0", 0);
  __Pyx_TraceCall("process_G0", __pyx_f[0], 148, 0, __PYX_ERR(0, 148, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":149
 * 
 *     cdef process_G0(self,groups):
 *         self.info(groups)             # <<<<<<<<<<<<<<
 * 
 *     cdef process_G1(self,groups):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->info(__pyx_v_self, __pyx_v_groups); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":148
 *         self.info(groups)
 * 
 *     cdef process_G0(self,groups):             # <<<<<<<<<<<<<<
 *         self.info(groups)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.process_G0", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":151
 *         self.info(groups)
 * 
 *     cdef process_G1(self,groups):             # <<<<<<<<<<<<<<
 *         self.info(groups)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_process_G1(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_groups) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("process_G1", 0);
  __Pyx_TraceCall("process_G1", __pyx_f[0], 151, 0, __PYX_ERR(0, 151, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":152
 * 
 *     cdef process_G1(self,groups):
 *         self.info(groups)             # <<<<<<<<<<<<<<
 * 
 *     cdef process_CSI(self,command,params):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->info(__pyx_v_self, __pyx_v_groups); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":151
 *         self.info(groups)
 * 
 *     cdef process_G1(self,groups):             # <<<<<<<<<<<<<<
 *         self.info(groups)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.process_G1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":154
 *         self.info(groups)
 * 
 *     cdef process_CSI(self,command,params):             # <<<<<<<<<<<<<<
 *         cdef int param_len=len(params)
 *         cdef int value1=0
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_process_CSI(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_command, PyObject *__pyx_v_params) {
  int __pyx_v_param_len;
  int __pyx_v_value1;
  int __pyx_v_value2;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("process_CSI", 0);
  __Pyx_TraceCall("process_CSI", __pyx_f[0], 154, 0, __PYX_ERR(0, 154, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":155
 * 
 *     cdef process_CSI(self,command,params):
 *         cdef int param_len=len(params)             # <<<<<<<<<<<<<<
 *         cdef int value1=0
 *         cdef int value2=0
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_params); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 155, __pyx_L1_error)
  __pyx_v_param_len = __pyx_t_1;

  /* "ttygif/tty/parser.pyx":156
 *     cdef process_CSI(self,command,params):
 *         cdef int param_len=len(params)
 *         cdef int value1=0             # <<<<<<<<<<<<<<
 *         cdef int value2=0
 * 
 */
  __pyx_v_value1 = 0;

  /* "ttygif/tty/parser.pyx":157
 *         cdef int param_len=len(params)
 *         cdef int value1=0
 *         cdef int value2=0             # <<<<<<<<<<<<<<
 * 
 *         # Defaults
 */
  __pyx_v_value2 = 0;

  /* "ttygif/tty/parser.pyx":160
 * 
 *         # Defaults
 *         if param_len>0:             # <<<<<<<<<<<<<<
 *             if isinstance(params[0],int):
 *                 value1=params[0]
 */
  __pyx_t_2 = ((__pyx_v_param_len > 0) != 0);
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":161
 *         # Defaults
 *         if param_len>0:
 *             if isinstance(params[0],int):             # <<<<<<<<<<<<<<
 *                 value1=params[0]
 *             else:
 */
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_params, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 161, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyInt_Check(__pyx_t_3); 
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_4 = (__pyx_t_2 != 0);
    if (__pyx_t_4) {

      /* "ttygif/tty/parser.pyx":162
 *         if param_len>0:
 *             if isinstance(params[0],int):
 *                 value1=params[0]             # <<<<<<<<<<<<<<
 *             else:
 *                 if command=='r':
 */
      __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_params, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 162, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 162, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_value1 = __pyx_t_5;

      /* "ttygif/tty/parser.pyx":161
 *         # Defaults
 *         if param_len>0:
 *             if isinstance(params[0],int):             # <<<<<<<<<<<<<<
 *                 value1=params[0]
 *             else:
 */
      goto __pyx_L4;
    }

    /* "ttygif/tty/parser.pyx":164
 *                 value1=params[0]
 *             else:
 *                 if command=='r':             # <<<<<<<<<<<<<<
 *                     value1=1
 * 
 */
    /*else*/ {
      __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_r, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 164, __pyx_L1_error)
      if (__pyx_t_4) {

        /* "ttygif/tty/parser.pyx":165
 *             else:
 *                 if command=='r':
 *                     value1=1             # <<<<<<<<<<<<<<
 * 
 *         if param_len>1:
 */
        __pyx_v_value1 = 1;

        /* "ttygif/tty/parser.pyx":164
 *                 value1=params[0]
 *             else:
 *                 if command=='r':             # <<<<<<<<<<<<<<
 *                     value1=1
 * 
 */
      }
    }
    __pyx_L4:;

    /* "ttygif/tty/parser.pyx":160
 * 
 *         # Defaults
 *         if param_len>0:             # <<<<<<<<<<<<<<
 *             if isinstance(params[0],int):
 *                 value1=params[0]
 */
  }

  /* "ttygif/tty/parser.pyx":167
 *                     value1=1
 * 
 *         if param_len>1:             # <<<<<<<<<<<<<<
 *             if isinstance(params[1],int):
 *                 value2=params[1]
 */
  __pyx_t_4 = ((__pyx_v_param_len > 1) != 0);
  if (__pyx_t_4) {

    /* "ttygif/tty/parser.pyx":168
 * 
 *         if param_len>1:
 *             if isinstance(params[1],int):             # <<<<<<<<<<<<<<
 *                 value2=params[1]
 *             else:
 */
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_params, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyInt_Check(__pyx_t_3); 
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_2 = (__pyx_t_4 != 0);
    if (__pyx_t_2) {

      /* "ttygif/tty/parser.pyx":169
 *         if param_len>1:
 *             if isinstance(params[1],int):
 *                 value2=params[1]             # <<<<<<<<<<<<<<
 *             else:
 *                 if command=='r':
 */
      __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_params, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 169, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 169, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_value2 = __pyx_t_5;

      /* "ttygif/tty/parser.pyx":168
 * 
 *         if param_len>1:
 *             if isinstance(params[1],int):             # <<<<<<<<<<<<<<
 *                 value2=params[1]
 *             else:
 */
      goto __pyx_L7;
    }

    /* "ttygif/tty/parser.pyx":171
 *                 value2=params[1]
 *             else:
 *                 if command=='r':             # <<<<<<<<<<<<<<
 *                     value2=self.g.state.height-1
 * 
 */
    /*else*/ {
      __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_r, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 171, __pyx_L1_error)
      if (__pyx_t_2) {

        /* "ttygif/tty/parser.pyx":172
 *             else:
 *                 if command=='r':
 *                     value2=self.g.state.height-1             # <<<<<<<<<<<<<<
 * 
 * 
 */
        __pyx_v_value2 = (__pyx_v_self->g->state->height - 1);

        /* "ttygif/tty/parser.pyx":171
 *                 value2=params[1]
 *             else:
 *                 if command=='r':             # <<<<<<<<<<<<<<
 *                     value2=self.g.state.height-1
 * 
 */
      }
    }
    __pyx_L7:;

    /* "ttygif/tty/parser.pyx":167
 *                     value1=1
 * 
 *         if param_len>1:             # <<<<<<<<<<<<<<
 *             if isinstance(params[1],int):
 *                 value2=params[1]
 */
  }

  /* "ttygif/tty/parser.pyx":178
 *         #print "\n"+command,value1,value1,params,self.g.state.cursor_x,self.g.state.cursor_y,self.g.state.width,self.g.state.height
 * 
 *         if   command=='A':  self.cmd_CUU(value1)             # <<<<<<<<<<<<<<
 *         elif command=='B':  self.cmd_CUD(value1)
 *         elif command=='C':  self.cmd_CUF(value1)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_A, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 178, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_value1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_CUU(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":179
 * 
 *         if   command=='A':  self.cmd_CUU(value1)
 *         elif command=='B':  self.cmd_CUD(value1)             # <<<<<<<<<<<<<<
 *         elif command=='C':  self.cmd_CUF(value1)
 *         elif command=='D':  self.cmd_CUB(value1)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_B, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 179, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_value1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_CUD(__pyx_v_self, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":180
 *         if   command=='A':  self.cmd_CUU(value1)
 *         elif command=='B':  self.cmd_CUD(value1)
 *         elif command=='C':  self.cmd_CUF(value1)             # <<<<<<<<<<<<<<
 *         elif command=='D':  self.cmd_CUB(value1)
 *         elif command=='E':  self.cmd_CNL(value1)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_C, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 180, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_value1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 180, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_CUF(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 180, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":181
 *         elif command=='B':  self.cmd_CUD(value1)
 *         elif command=='C':  self.cmd_CUF(value1)
 *         elif command=='D':  self.cmd_CUB(value1)             # <<<<<<<<<<<<<<
 *         elif command=='E':  self.cmd_CNL(value1)
 *         elif command=='F':  self.cmd_CPL(value1)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_D, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 181, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_value1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_CUB(__pyx_v_self, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":182
 *         elif command=='C':  self.cmd_CUF(value1)
 *         elif command=='D':  self.cmd_CUB(value1)
 *         elif command=='E':  self.cmd_CNL(value1)             # <<<<<<<<<<<<<<
 *         elif command=='F':  self.cmd_CPL(value1)
 *         elif command=='G':  self.cmd_CHA(value1-1)               # abs
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_E, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 182, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_value1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_CNL(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":183
 *         elif command=='D':  self.cmd_CUB(value1)
 *         elif command=='E':  self.cmd_CNL(value1)
 *         elif command=='F':  self.cmd_CPL(value1)             # <<<<<<<<<<<<<<
 *         elif command=='G':  self.cmd_CHA(value1-1)               # abs
 *         elif command=='H':  self.cmd_CUP(value2-1,value1-1)      # abs
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_F, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 183, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_value1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_CPL(__pyx_v_self, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":184
 *         elif command=='E':  self.cmd_CNL(value1)
 *         elif command=='F':  self.cmd_CPL(value1)
 *         elif command=='G':  self.cmd_CHA(value1-1)               # abs             # <<<<<<<<<<<<<<
 *         elif command=='H':  self.cmd_CUP(value2-1,value1-1)      # abs
 *         elif command=='J':  self.cmd_ED(value1)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_G, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 184, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = __Pyx_PyInt_From_long((__pyx_v_value1 - 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 184, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_CHA(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 184, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":185
 *         elif command=='F':  self.cmd_CPL(value1)
 *         elif command=='G':  self.cmd_CHA(value1-1)               # abs
 *         elif command=='H':  self.cmd_CUP(value2-1,value1-1)      # abs             # <<<<<<<<<<<<<<
 *         elif command=='J':  self.cmd_ED(value1)
 *         elif command=='K':  self.cmd_EL(value1)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_H, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 185, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_6 = __Pyx_PyInt_From_long((__pyx_v_value2 - 1)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = __Pyx_PyInt_From_long((__pyx_v_value1 - 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_CUP(__pyx_v_self, __pyx_t_6, __pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":186
 *         elif command=='G':  self.cmd_CHA(value1-1)               # abs
 *         elif command=='H':  self.cmd_CUP(value2-1,value1-1)      # abs
 *         elif command=='J':  self.cmd_ED(value1)             # <<<<<<<<<<<<<<
 *         elif command=='K':  self.cmd_EL(value1)
 *         elif command=='P':  self.cmd_DCH(value1)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_J, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 186, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_value1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_ED(__pyx_v_self, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":187
 *         elif command=='H':  self.cmd_CUP(value2-1,value1-1)      # abs
 *         elif command=='J':  self.cmd_ED(value1)
 *         elif command=='K':  self.cmd_EL(value1)             # <<<<<<<<<<<<<<
 *         elif command=='P':  self.cmd_DCH(value1)
 *         elif command=='X':  self.cmd_ECH(value1)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_K, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 187, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_value1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_EL(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":188
 *         elif command=='J':  self.cmd_ED(value1)
 *         elif command=='K':  self.cmd_EL(value1)
 *         elif command=='P':  self.cmd_DCH(value1)             # <<<<<<<<<<<<<<
 *         elif command=='X':  self.cmd_ECH(value1)
 *         elif command=='d':  self.cmd_VPA(value1-1)               # abs
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_P, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 188, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_value1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_DCH(__pyx_v_self, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":189
 *         elif command=='K':  self.cmd_EL(value1)
 *         elif command=='P':  self.cmd_DCH(value1)
 *         elif command=='X':  self.cmd_ECH(value1)             # <<<<<<<<<<<<<<
 *         elif command=='d':  self.cmd_VPA(value1-1)               # abs
 *         elif command=='`':  self.cmd_HPA(value1-1)               # abs
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_X, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 189, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_value1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_ECH(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":190
 *         elif command=='P':  self.cmd_DCH(value1)
 *         elif command=='X':  self.cmd_ECH(value1)
 *         elif command=='d':  self.cmd_VPA(value1-1)               # abs             # <<<<<<<<<<<<<<
 *         elif command=='`':  self.cmd_HPA(value1-1)               # abs
 *         elif command=='f':  self.cmd_HVP(value2-1,value1-1)      # abs
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_d, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 190, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_7 = __Pyx_PyInt_From_long((__pyx_v_value1 - 1)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_VPA(__pyx_v_self, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":191
 *         elif command=='X':  self.cmd_ECH(value1)
 *         elif command=='d':  self.cmd_VPA(value1-1)               # abs
 *         elif command=='`':  self.cmd_HPA(value1-1)               # abs             # <<<<<<<<<<<<<<
 *         elif command=='f':  self.cmd_HVP(value2-1,value1-1)      # abs
 *         elif command=='h':  self.cmd_set_mode(params)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_kp_s__4, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 191, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = __Pyx_PyInt_From_long((__pyx_v_value1 - 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_HPA(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":192
 *         elif command=='d':  self.cmd_VPA(value1-1)               # abs
 *         elif command=='`':  self.cmd_HPA(value1-1)               # abs
 *         elif command=='f':  self.cmd_HVP(value2-1,value1-1)      # abs             # <<<<<<<<<<<<<<
 *         elif command=='h':  self.cmd_set_mode(params)
 *         elif command=='l':  self.cmd_reset_mode(value1)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_f, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 192, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_7 = __Pyx_PyInt_From_long((__pyx_v_value2 - 1)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = __Pyx_PyInt_From_long((__pyx_v_value1 - 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_HVP(__pyx_v_self, __pyx_t_7, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":193
 *         elif command=='`':  self.cmd_HPA(value1-1)               # abs
 *         elif command=='f':  self.cmd_HVP(value2-1,value1-1)      # abs
 *         elif command=='h':  self.cmd_set_mode(params)             # <<<<<<<<<<<<<<
 *         elif command=='l':  self.cmd_reset_mode(value1)
 *         elif command=='m':  self.cmd_process_colors(params)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_h, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 193, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_set_mode(__pyx_v_self, __pyx_v_params); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":194
 *         elif command=='f':  self.cmd_HVP(value2-1,value1-1)      # abs
 *         elif command=='h':  self.cmd_set_mode(params)
 *         elif command=='l':  self.cmd_reset_mode(value1)             # <<<<<<<<<<<<<<
 *         elif command=='m':  self.cmd_process_colors(params)
 *         elif command=='r':  self.cmd_DECSTBM(value1-1,value2-1)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_l, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 194, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_value1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_reset_mode(__pyx_v_self, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":195
 *         elif command=='h':  self.cmd_set_mode(params)
 *         elif command=='l':  self.cmd_reset_mode(value1)
 *         elif command=='m':  self.cmd_process_colors(params)             # <<<<<<<<<<<<<<
 *         elif command=='r':  self.cmd_DECSTBM(value1-1,value2-1)
 *         elif command=='s':  self.cmd_SCP()
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_m, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 195, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_process_colors(__pyx_v_self, __pyx_v_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":196
 *         elif command=='l':  self.cmd_reset_mode(value1)
 *         elif command=='m':  self.cmd_process_colors(params)
 *         elif command=='r':  self.cmd_DECSTBM(value1-1,value2-1)             # <<<<<<<<<<<<<<
 *         elif command=='s':  self.cmd_SCP()
 *         elif command=='u':  self.cmd_RCP()
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_r, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 196, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_DECSTBM(__pyx_v_self, (__pyx_v_value1 - 1), (__pyx_v_value2 - 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 196, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":197
 *         elif command=='m':  self.cmd_process_colors(params)
 *         elif command=='r':  self.cmd_DECSTBM(value1-1,value2-1)
 *         elif command=='s':  self.cmd_SCP()             # <<<<<<<<<<<<<<
 *         elif command=='u':  self.cmd_RCP()
 *         elif command=='~':  self.cmd_BRACKETED_PASTE(value1)               # abs
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_s, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 197, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_SCP(__pyx_v_self); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":198
 *         elif command=='r':  self.cmd_DECSTBM(value1-1,value2-1)
 *         elif command=='s':  self.cmd_SCP()
 *         elif command=='u':  self.cmd_RCP()             # <<<<<<<<<<<<<<
 *         elif command=='~':  self.cmd_BRACKETED_PASTE(value1)               # abs
 *         elif command=='?h': self.cmd_DECSET(value1)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_n_s_u, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 198, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_RCP(__pyx_v_self); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":199
 *         elif command=='s':  self.cmd_SCP()
 *         elif command=='u':  self.cmd_RCP()
 *         elif command=='~':  self.cmd_BRACKETED_PASTE(value1)               # abs             # <<<<<<<<<<<<<<
 *         elif command=='?h': self.cmd_DECSET(value1)
 *         elif command=='?l': self.cmd_DECRST(value1)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_kp_s__3, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 199, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_value1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_BRACKETED_PASTE(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":200
 *         elif command=='u':  self.cmd_RCP()
 *         elif command=='~':  self.cmd_BRACKETED_PASTE(value1)               # abs
 *         elif command=='?h': self.cmd_DECSET(value1)             # <<<<<<<<<<<<<<
 *         elif command=='?l': self.cmd_DECRST(value1)
 * 
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_kp_s_h_2, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 200, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_DECSET(__pyx_v_self, __pyx_v_value1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 200, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":201
 *         elif command=='~':  self.cmd_BRACKETED_PASTE(value1)               # abs
 *         elif command=='?h': self.cmd_DECSET(value1)
 *         elif command=='?l': self.cmd_DECRST(value1)             # <<<<<<<<<<<<<<
 * 
 *         #elif command=='e':
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_command, __pyx_kp_s_l_2, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 201, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_DECRST(__pyx_v_self, __pyx_v_value1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L9;
  }

  /* "ttygif/tty/parser.pyx":208
 *         #    y+=value
 * 
 *         else: self.info("Impliment: {0}-{1}".format(command,params))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Impliment_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 208, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_5 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_command, __pyx_v_params};
      __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 208, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_6);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_command, __pyx_v_params};
      __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 208, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_6);
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 208, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_INCREF(__pyx_v_command);
      __Pyx_GIVEREF(__pyx_v_command);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_5, __pyx_v_command);
      __Pyx_INCREF(__pyx_v_params);
      __Pyx_GIVEREF(__pyx_v_params);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_5, __pyx_v_params);
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_8, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 208, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->info(__pyx_v_self, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 208, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L9:;

  /* "ttygif/tty/parser.pyx":154
 *         self.info(groups)
 * 
 *     cdef process_CSI(self,command,params):             # <<<<<<<<<<<<<<
 *         cdef int param_len=len(params)
 *         cdef int value1=0
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.process_CSI", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":211
 * 
 * 
 *     cdef cmd_DECSET(self,int code):             # <<<<<<<<<<<<<<
 *         #print "SET",parameters
 *         if    code==7:
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_DECSET(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, int __pyx_v_code) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_DECSET", 0);
  __Pyx_TraceCall("cmd_DECSET", __pyx_f[0], 211, 0, __PYX_ERR(0, 211, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":213
 *     cdef cmd_DECSET(self,int code):
 *         #print "SET",parameters
 *         if    code==7:             # <<<<<<<<<<<<<<
 *             self.g.state.autowrap_on()
 *         elif  code==25:
 */
  switch (__pyx_v_code) {
    case 7:

    /* "ttygif/tty/parser.pyx":214
 *         #print "SET",parameters
 *         if    code==7:
 *             self.g.state.autowrap_on()             # <<<<<<<<<<<<<<
 *         elif  code==25:
 *             self.g.state.show_cursor()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->autowrap_on(__pyx_v_self->g->state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":213
 *     cdef cmd_DECSET(self,int code):
 *         #print "SET",parameters
 *         if    code==7:             # <<<<<<<<<<<<<<
 *             self.g.state.autowrap_on()
 *         elif  code==25:
 */
    break;
    case 25:

    /* "ttygif/tty/parser.pyx":216
 *             self.g.state.autowrap_on()
 *         elif  code==25:
 *             self.g.state.show_cursor()             # <<<<<<<<<<<<<<
 *         elif  code==1049:
 *             self.g.alternate_screen_on()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->show_cursor(__pyx_v_self->g->state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":215
 *         if    code==7:
 *             self.g.state.autowrap_on()
 *         elif  code==25:             # <<<<<<<<<<<<<<
 *             self.g.state.show_cursor()
 *         elif  code==1049:
 */
    break;
    case 0x419:

    /* "ttygif/tty/parser.pyx":218
 *             self.g.state.show_cursor()
 *         elif  code==1049:
 *             self.g.alternate_screen_on()             # <<<<<<<<<<<<<<
 *         elif  code==2004:
 *             self.cmd_bracketed_paste_on()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->alternate_screen_on(__pyx_v_self->g); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 218, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":217
 *         elif  code==25:
 *             self.g.state.show_cursor()
 *         elif  code==1049:             # <<<<<<<<<<<<<<
 *             self.g.alternate_screen_on()
 *         elif  code==2004:
 */
    break;
    case 0x7D4:

    /* "ttygif/tty/parser.pyx":220
 *             self.g.alternate_screen_on()
 *         elif  code==2004:
 *             self.cmd_bracketed_paste_on()             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_DECRST(self,int code):
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_bracketed_paste_on(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":219
 *         elif  code==1049:
 *             self.g.alternate_screen_on()
 *         elif  code==2004:             # <<<<<<<<<<<<<<
 *             self.cmd_bracketed_paste_on()
 * 
 */
    break;
    default: break;
  }

  /* "ttygif/tty/parser.pyx":211
 * 
 * 
 *     cdef cmd_DECSET(self,int code):             # <<<<<<<<<<<<<<
 *         #print "SET",parameters
 *         if    code==7:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_DECSET", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":222
 *             self.cmd_bracketed_paste_on()
 * 
 *     cdef cmd_DECRST(self,int code):             # <<<<<<<<<<<<<<
 *         #print "RESET",parameters
 *         if  code==7:
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_DECRST(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, int __pyx_v_code) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_DECRST", 0);
  __Pyx_TraceCall("cmd_DECRST", __pyx_f[0], 222, 0, __PYX_ERR(0, 222, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":224
 *     cdef cmd_DECRST(self,int code):
 *         #print "RESET",parameters
 *         if  code==7:             # <<<<<<<<<<<<<<
 *             self.g.state.autowrap_off()
 *         elif code==25:
 */
  switch (__pyx_v_code) {
    case 7:

    /* "ttygif/tty/parser.pyx":225
 *         #print "RESET",parameters
 *         if  code==7:
 *             self.g.state.autowrap_off()             # <<<<<<<<<<<<<<
 *         elif code==25:
 *             self.g.state.hide_cursor()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->autowrap_off(__pyx_v_self->g->state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":224
 *     cdef cmd_DECRST(self,int code):
 *         #print "RESET",parameters
 *         if  code==7:             # <<<<<<<<<<<<<<
 *             self.g.state.autowrap_off()
 *         elif code==25:
 */
    break;
    case 25:

    /* "ttygif/tty/parser.pyx":227
 *             self.g.state.autowrap_off()
 *         elif code==25:
 *             self.g.state.hide_cursor()             # <<<<<<<<<<<<<<
 *         elif code==1049:
 *             self.g.alternate_screen_off()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->hide_cursor(__pyx_v_self->g->state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":226
 *         if  code==7:
 *             self.g.state.autowrap_off()
 *         elif code==25:             # <<<<<<<<<<<<<<
 *             self.g.state.hide_cursor()
 *         elif code==1049:
 */
    break;
    case 0x419:

    /* "ttygif/tty/parser.pyx":229
 *             self.g.state.hide_cursor()
 *         elif code==1049:
 *             self.g.alternate_screen_off()             # <<<<<<<<<<<<<<
 *         elif code==2004:
 *             self.cmd_bracketed_paste_off()
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->alternate_screen_off(__pyx_v_self->g); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":228
 *         elif code==25:
 *             self.g.state.hide_cursor()
 *         elif code==1049:             # <<<<<<<<<<<<<<
 *             self.g.alternate_screen_off()
 *         elif code==2004:
 */
    break;
    case 0x7D4:

    /* "ttygif/tty/parser.pyx":231
 *             self.g.alternate_screen_off()
 *         elif code==2004:
 *             self.cmd_bracketed_paste_off()             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_bracketed_paste_off(self):
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_bracketed_paste_off(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":230
 *         elif code==1049:
 *             self.g.alternate_screen_off()
 *         elif code==2004:             # <<<<<<<<<<<<<<
 *             self.cmd_bracketed_paste_off()
 * 
 */
    break;
    default: break;
  }

  /* "ttygif/tty/parser.pyx":222
 *             self.cmd_bracketed_paste_on()
 * 
 *     cdef cmd_DECRST(self,int code):             # <<<<<<<<<<<<<<
 *         #print "RESET",parameters
 *         if  code==7:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_DECRST", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":233
 *             self.cmd_bracketed_paste_off()
 * 
 *     cdef cmd_bracketed_paste_off(self):             # <<<<<<<<<<<<<<
 *         self.bracketed_paste=None
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_bracketed_paste_off(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_bracketed_paste_off", 0);
  __Pyx_TraceCall("cmd_bracketed_paste_off", __pyx_f[0], 233, 0, __PYX_ERR(0, 233, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":234
 * 
 *     cdef cmd_bracketed_paste_off(self):
 *         self.bracketed_paste=None             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_bracketed_paste_on(self):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->bracketed_paste);
  __Pyx_DECREF(__pyx_v_self->bracketed_paste);
  __pyx_v_self->bracketed_paste = Py_None;

  /* "ttygif/tty/parser.pyx":233
 *             self.cmd_bracketed_paste_off()
 * 
 *     cdef cmd_bracketed_paste_off(self):             # <<<<<<<<<<<<<<
 *         self.bracketed_paste=None
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_bracketed_paste_off", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":236
 *         self.bracketed_paste=None
 * 
 *     cdef cmd_bracketed_paste_on(self):             # <<<<<<<<<<<<<<
 *         self.bracketed_paste=True
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_bracketed_paste_on(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_bracketed_paste_on", 0);
  __Pyx_TraceCall("cmd_bracketed_paste_on", __pyx_f[0], 236, 0, __PYX_ERR(0, 236, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":237
 * 
 *     cdef cmd_bracketed_paste_on(self):
 *         self.bracketed_paste=True             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_BRACKETED_PASTE(self,value):
 */
  __Pyx_INCREF(Py_True);
  __Pyx_GIVEREF(Py_True);
  __Pyx_GOTREF(__pyx_v_self->bracketed_paste);
  __Pyx_DECREF(__pyx_v_self->bracketed_paste);
  __pyx_v_self->bracketed_paste = Py_True;

  /* "ttygif/tty/parser.pyx":236
 *         self.bracketed_paste=None
 * 
 *     cdef cmd_bracketed_paste_on(self):             # <<<<<<<<<<<<<<
 *         self.bracketed_paste=True
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_bracketed_paste_on", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":239
 *         self.bracketed_paste=True
 * 
 *     cdef cmd_BRACKETED_PASTE(self,value):             # <<<<<<<<<<<<<<
 *         #print "eh"
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_BRACKETED_PASTE(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_BRACKETED_PASTE", 0);
  __Pyx_TraceCall("cmd_BRACKETED_PASTE", __pyx_f[0], 239, 0, __PYX_ERR(0, 239, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":242
 *         #print "eh"
 * 
 *         if self.bracketed_paste:             # <<<<<<<<<<<<<<
 *             if value==200:
 *                 self.no_codes=True
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_self->bracketed_paste); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 242, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "ttygif/tty/parser.pyx":243
 * 
 *         if self.bracketed_paste:
 *             if value==200:             # <<<<<<<<<<<<<<
 *                 self.no_codes=True
 * 
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_value, __pyx_int_200, 0xC8, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 243, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 243, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_1) {

      /* "ttygif/tty/parser.pyx":244
 *         if self.bracketed_paste:
 *             if value==200:
 *                 self.no_codes=True             # <<<<<<<<<<<<<<
 * 
 *             if value==201:
 */
      __Pyx_INCREF(Py_True);
      __Pyx_GIVEREF(Py_True);
      __Pyx_GOTREF(__pyx_v_self->no_codes);
      __Pyx_DECREF(__pyx_v_self->no_codes);
      __pyx_v_self->no_codes = Py_True;

      /* "ttygif/tty/parser.pyx":243
 * 
 *         if self.bracketed_paste:
 *             if value==200:             # <<<<<<<<<<<<<<
 *                 self.no_codes=True
 * 
 */
    }

    /* "ttygif/tty/parser.pyx":246
 *                 self.no_codes=True
 * 
 *             if value==201:             # <<<<<<<<<<<<<<
 *                 self.no_codes=None
 * 
 */
    __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_v_value, __pyx_int_201, 0xC9, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 246, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 246, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_1) {

      /* "ttygif/tty/parser.pyx":247
 * 
 *             if value==201:
 *                 self.no_codes=None             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_set_mode(self,cmd):
 */
      __Pyx_INCREF(Py_None);
      __Pyx_GIVEREF(Py_None);
      __Pyx_GOTREF(__pyx_v_self->no_codes);
      __Pyx_DECREF(__pyx_v_self->no_codes);
      __pyx_v_self->no_codes = Py_None;

      /* "ttygif/tty/parser.pyx":246
 *                 self.no_codes=True
 * 
 *             if value==201:             # <<<<<<<<<<<<<<
 *                 self.no_codes=None
 * 
 */
    }

    /* "ttygif/tty/parser.pyx":242
 *         #print "eh"
 * 
 *         if self.bracketed_paste:             # <<<<<<<<<<<<<<
 *             if value==200:
 *                 self.no_codes=True
 */
  }

  /* "ttygif/tty/parser.pyx":239
 *         self.bracketed_paste=True
 * 
 *     cdef cmd_BRACKETED_PASTE(self,value):             # <<<<<<<<<<<<<<
 *         #print "eh"
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_BRACKETED_PASTE", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":249
 *                 self.no_codes=None
 * 
 *     cdef cmd_set_mode(self,cmd):             # <<<<<<<<<<<<<<
 *         if cmd==0:
 *             self.set_foreground(self.g.state.default_foreground)
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_set_mode(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_cmd) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_set_mode", 0);
  __Pyx_TraceCall("cmd_set_mode", __pyx_f[0], 249, 0, __PYX_ERR(0, 249, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":250
 * 
 *     cdef cmd_set_mode(self,cmd):
 *         if cmd==0:             # <<<<<<<<<<<<<<
 *             self.set_foreground(self.g.state.default_foreground)
 *             self.set_background(self.g.state.default_background)
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_cmd, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":251
 *     cdef cmd_set_mode(self,cmd):
 *         if cmd==0:
 *             self.set_foreground(self.g.state.default_foreground)             # <<<<<<<<<<<<<<
 *             self.set_background(self.g.state.default_background)
 *             self.g.state.bold=None
 */
    __pyx_t_1 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->g->state->default_foreground); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->set_foreground(__pyx_v_self, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "ttygif/tty/parser.pyx":252
 *         if cmd==0:
 *             self.set_foreground(self.g.state.default_foreground)
 *             self.set_background(self.g.state.default_background)             # <<<<<<<<<<<<<<
 *             self.g.state.bold=None
 *             self.g.state.reverse_video=None
 */
    __pyx_t_3 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->g->state->default_background); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->set_background(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":253
 *             self.set_foreground(self.g.state.default_foreground)
 *             self.set_background(self.g.state.default_background)
 *             self.g.state.bold=None             # <<<<<<<<<<<<<<
 *             self.g.state.reverse_video=None
 *         elif cmd==1:
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->g->state->bold);
    __Pyx_DECREF(__pyx_v_self->g->state->bold);
    __pyx_v_self->g->state->bold = Py_None;

    /* "ttygif/tty/parser.pyx":254
 *             self.set_background(self.g.state.default_background)
 *             self.g.state.bold=None
 *             self.g.state.reverse_video=None             # <<<<<<<<<<<<<<
 *         elif cmd==1:
 *             self.g.state.bold=True
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->g->state->reverse_video);
    __Pyx_DECREF(__pyx_v_self->g->state->reverse_video);
    __pyx_v_self->g->state->reverse_video = Py_None;

    /* "ttygif/tty/parser.pyx":250
 * 
 *     cdef cmd_set_mode(self,cmd):
 *         if cmd==0:             # <<<<<<<<<<<<<<
 *             self.set_foreground(self.g.state.default_foreground)
 *             self.set_background(self.g.state.default_background)
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":255
 *             self.g.state.bold=None
 *             self.g.state.reverse_video=None
 *         elif cmd==1:             # <<<<<<<<<<<<<<
 *             self.g.state.bold=True
 *         elif cmd==7:
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_cmd, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":256
 *             self.g.state.reverse_video=None
 *         elif cmd==1:
 *             self.g.state.bold=True             # <<<<<<<<<<<<<<
 *         elif cmd==7:
 *             self.g.state.reverse_video=True
 */
    __Pyx_INCREF(Py_True);
    __Pyx_GIVEREF(Py_True);
    __Pyx_GOTREF(__pyx_v_self->g->state->bold);
    __Pyx_DECREF(__pyx_v_self->g->state->bold);
    __pyx_v_self->g->state->bold = Py_True;

    /* "ttygif/tty/parser.pyx":255
 *             self.g.state.bold=None
 *             self.g.state.reverse_video=None
 *         elif cmd==1:             # <<<<<<<<<<<<<<
 *             self.g.state.bold=True
 *         elif cmd==7:
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":257
 *         elif cmd==1:
 *             self.g.state.bold=True
 *         elif cmd==7:             # <<<<<<<<<<<<<<
 *             self.g.state.reverse_video=True
 *         elif cmd==27:
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_cmd, __pyx_int_7, 7, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":258
 *             self.g.state.bold=True
 *         elif cmd==7:
 *             self.g.state.reverse_video=True             # <<<<<<<<<<<<<<
 *         elif cmd==27:
 *             self.g.state.reverse_video=None
 */
    __Pyx_INCREF(Py_True);
    __Pyx_GIVEREF(Py_True);
    __Pyx_GOTREF(__pyx_v_self->g->state->reverse_video);
    __Pyx_DECREF(__pyx_v_self->g->state->reverse_video);
    __pyx_v_self->g->state->reverse_video = Py_True;

    /* "ttygif/tty/parser.pyx":257
 *         elif cmd==1:
 *             self.g.state.bold=True
 *         elif cmd==7:             # <<<<<<<<<<<<<<
 *             self.g.state.reverse_video=True
 *         elif cmd==27:
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":259
 *         elif cmd==7:
 *             self.g.state.reverse_video=True
 *         elif cmd==27:             # <<<<<<<<<<<<<<
 *             self.g.state.reverse_video=None
 *         elif cmd>=30 and cmd<=37:
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_cmd, __pyx_int_27, 27, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":260
 *             self.g.state.reverse_video=True
 *         elif cmd==27:
 *             self.g.state.reverse_video=None             # <<<<<<<<<<<<<<
 *         elif cmd>=30 and cmd<=37:
 * 
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->g->state->reverse_video);
    __Pyx_DECREF(__pyx_v_self->g->state->reverse_video);
    __pyx_v_self->g->state->reverse_video = Py_None;

    /* "ttygif/tty/parser.pyx":259
 *         elif cmd==7:
 *             self.g.state.reverse_video=True
 *         elif cmd==27:             # <<<<<<<<<<<<<<
 *             self.g.state.reverse_video=None
 *         elif cmd>=30 and cmd<=37:
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":261
 *         elif cmd==27:
 *             self.g.state.reverse_video=None
 *         elif cmd>=30 and cmd<=37:             # <<<<<<<<<<<<<<
 * 
 *             if self.g.state.bold:
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_cmd, __pyx_int_30, Py_GE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 261, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_cmd, __pyx_int_37, Py_LE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 261, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":263
 *         elif cmd>=30 and cmd<=37:
 * 
 *             if self.g.state.bold:             # <<<<<<<<<<<<<<
 *                 self.set_foreground(self.g.state.foreground+8)
 *             else:
 */
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->g->state->bold); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 263, __pyx_L1_error)
    if (__pyx_t_2) {

      /* "ttygif/tty/parser.pyx":264
 * 
 *             if self.g.state.bold:
 *                 self.set_foreground(self.g.state.foreground+8)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.set_foreground(cmd-30)
 */
      __pyx_t_1 = __Pyx_PyInt_From_long((__pyx_v_self->g->state->foreground + 8)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 264, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->set_foreground(__pyx_v_self, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 264, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "ttygif/tty/parser.pyx":263
 *         elif cmd>=30 and cmd<=37:
 * 
 *             if self.g.state.bold:             # <<<<<<<<<<<<<<
 *                 self.set_foreground(self.g.state.foreground+8)
 *             else:
 */
      goto __pyx_L6;
    }

    /* "ttygif/tty/parser.pyx":266
 *                 self.set_foreground(self.g.state.foreground+8)
 *             else:
 *                 self.set_foreground(cmd-30)             # <<<<<<<<<<<<<<
 * 
 *         elif cmd==39:
 */
    /*else*/ {
      __pyx_t_3 = __Pyx_PyInt_SubtractObjC(__pyx_v_cmd, __pyx_int_30, 30, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 266, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->set_foreground(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 266, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L6:;

    /* "ttygif/tty/parser.pyx":261
 *         elif cmd==27:
 *             self.g.state.reverse_video=None
 *         elif cmd>=30 and cmd<=37:             # <<<<<<<<<<<<<<
 * 
 *             if self.g.state.bold:
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":268
 *                 self.set_foreground(cmd-30)
 * 
 *         elif cmd==39:             # <<<<<<<<<<<<<<
 *             self.set_foreground(self.g.state.default_foreground)
 *         elif cmd>=40 and cmd<=47:
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_cmd, __pyx_int_39, 39, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":269
 * 
 *         elif cmd==39:
 *             self.set_foreground(self.g.state.default_foreground)             # <<<<<<<<<<<<<<
 *         elif cmd>=40 and cmd<=47:
 *             if self.g.state.bold:
 */
    __pyx_t_1 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->g->state->default_foreground); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->set_foreground(__pyx_v_self, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "ttygif/tty/parser.pyx":268
 *                 self.set_foreground(cmd-30)
 * 
 *         elif cmd==39:             # <<<<<<<<<<<<<<
 *             self.set_foreground(self.g.state.default_foreground)
 *         elif cmd>=40 and cmd<=47:
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":270
 *         elif cmd==39:
 *             self.set_foreground(self.g.state.default_foreground)
 *         elif cmd>=40 and cmd<=47:             # <<<<<<<<<<<<<<
 *             if self.g.state.bold:
 *                 self.set_background(self.g.state.background+8)
 */
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_cmd, __pyx_int_40, Py_GE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 270, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_cmd, __pyx_int_47, Py_LE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 270, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_2 = __pyx_t_4;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":271
 *             self.set_foreground(self.g.state.default_foreground)
 *         elif cmd>=40 and cmd<=47:
 *             if self.g.state.bold:             # <<<<<<<<<<<<<<
 *                 self.set_background(self.g.state.background+8)
 *             else:
 */
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->g->state->bold); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 271, __pyx_L1_error)
    if (__pyx_t_2) {

      /* "ttygif/tty/parser.pyx":272
 *         elif cmd>=40 and cmd<=47:
 *             if self.g.state.bold:
 *                 self.set_background(self.g.state.background+8)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.set_background(cmd-40)
 */
      __pyx_t_3 = __Pyx_PyInt_From_long((__pyx_v_self->g->state->background + 8)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->set_background(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ttygif/tty/parser.pyx":271
 *             self.set_foreground(self.g.state.default_foreground)
 *         elif cmd>=40 and cmd<=47:
 *             if self.g.state.bold:             # <<<<<<<<<<<<<<
 *                 self.set_background(self.g.state.background+8)
 *             else:
 */
      goto __pyx_L9;
    }

    /* "ttygif/tty/parser.pyx":274
 *                 self.set_background(self.g.state.background+8)
 *             else:
 *                 self.set_background(cmd-40)             # <<<<<<<<<<<<<<
 *         elif cmd==49:
 *             self.set_background(self.g.state.default_background)
 */
    /*else*/ {
      __pyx_t_1 = __Pyx_PyInt_SubtractObjC(__pyx_v_cmd, __pyx_int_40, 40, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->set_background(__pyx_v_self, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 274, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_L9:;

    /* "ttygif/tty/parser.pyx":270
 *         elif cmd==39:
 *             self.set_foreground(self.g.state.default_foreground)
 *         elif cmd>=40 and cmd<=47:             # <<<<<<<<<<<<<<
 *             if self.g.state.bold:
 *                 self.set_background(self.g.state.background+8)
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":275
 *             else:
 *                 self.set_background(cmd-40)
 *         elif cmd==49:             # <<<<<<<<<<<<<<
 *             self.set_background(self.g.state.default_background)
 *         elif cmd>=90 and cmd<=97:
 */
  __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_v_cmd, __pyx_int_49, 49, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":276
 *                 self.set_background(cmd-40)
 *         elif cmd==49:
 *             self.set_background(self.g.state.default_background)             # <<<<<<<<<<<<<<
 *         elif cmd>=90 and cmd<=97:
 *             self.set_foreground(cmd-90+8)
 */
    __pyx_t_3 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->g->state->default_background); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 276, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->set_background(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 276, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":275
 *             else:
 *                 self.set_background(cmd-40)
 *         elif cmd==49:             # <<<<<<<<<<<<<<
 *             self.set_background(self.g.state.default_background)
 *         elif cmd>=90 and cmd<=97:
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":277
 *         elif cmd==49:
 *             self.set_background(self.g.state.default_background)
 *         elif cmd>=90 and cmd<=97:             # <<<<<<<<<<<<<<
 *             self.set_foreground(cmd-90+8)
 *         elif cmd>=100 and cmd<=107:
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_cmd, __pyx_int_90, Py_GE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 277, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L10_bool_binop_done;
  }
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_cmd, __pyx_int_97, Py_LE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 277, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __pyx_t_4;
  __pyx_L10_bool_binop_done:;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":278
 *             self.set_background(self.g.state.default_background)
 *         elif cmd>=90 and cmd<=97:
 *             self.set_foreground(cmd-90+8)             # <<<<<<<<<<<<<<
 *         elif cmd>=100 and cmd<=107:
 *             self.set_background(cmd-100+8)
 */
    __pyx_t_1 = __Pyx_PyInt_SubtractObjC(__pyx_v_cmd, __pyx_int_90, 90, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 278, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_8, 8, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 278, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->set_foreground(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 278, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":277
 *         elif cmd==49:
 *             self.set_background(self.g.state.default_background)
 *         elif cmd>=90 and cmd<=97:             # <<<<<<<<<<<<<<
 *             self.set_foreground(cmd-90+8)
 *         elif cmd>=100 and cmd<=107:
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":279
 *         elif cmd>=90 and cmd<=97:
 *             self.set_foreground(cmd-90+8)
 *         elif cmd>=100 and cmd<=107:             # <<<<<<<<<<<<<<
 *             self.set_background(cmd-100+8)
 * 
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_cmd, __pyx_int_100, Py_GE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 279, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 279, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_cmd, __pyx_int_107, Py_LE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 279, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 279, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __pyx_t_4;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":280
 *             self.set_foreground(cmd-90+8)
 *         elif cmd>=100 and cmd<=107:
 *             self.set_background(cmd-100+8)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_reset_mode(self,cmd):
 */
    __pyx_t_1 = __Pyx_PyInt_SubtractObjC(__pyx_v_cmd, __pyx_int_100, 0x64, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 280, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_8, 8, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 280, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->set_background(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 280, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":279
 *         elif cmd>=90 and cmd<=97:
 *             self.set_foreground(cmd-90+8)
 *         elif cmd>=100 and cmd<=107:             # <<<<<<<<<<<<<<
 *             self.set_background(cmd-100+8)
 * 
 */
  }
  __pyx_L3:;

  /* "ttygif/tty/parser.pyx":249
 *                 self.no_codes=None
 * 
 *     cdef cmd_set_mode(self,cmd):             # <<<<<<<<<<<<<<
 *         if cmd==0:
 *             self.set_foreground(self.g.state.default_foreground)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_set_mode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":282
 *             self.set_background(cmd-100+8)
 * 
 *     cdef cmd_reset_mode(self,cmd):             # <<<<<<<<<<<<<<
 *         if cmd==0:
 *             self.g.state.set_foreground(self.g.state.default_foreground)
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_reset_mode(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_cmd) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_reset_mode", 0);
  __Pyx_TraceCall("cmd_reset_mode", __pyx_f[0], 282, 0, __PYX_ERR(0, 282, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":283
 * 
 *     cdef cmd_reset_mode(self,cmd):
 *         if cmd==0:             # <<<<<<<<<<<<<<
 *             self.g.state.set_foreground(self.g.state.default_foreground)
 *             self.g.state.set_background(self.g.state.default_background)
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_cmd, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":284
 *     cdef cmd_reset_mode(self,cmd):
 *         if cmd==0:
 *             self.g.state.set_foreground(self.g.state.default_foreground)             # <<<<<<<<<<<<<<
 *             self.g.state.set_background(self.g.state.default_background)
 *             self.g.state.bold=None
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->set_foreground(__pyx_v_self->g->state, __pyx_v_self->g->state->default_foreground); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":285
 *         if cmd==0:
 *             self.g.state.set_foreground(self.g.state.default_foreground)
 *             self.g.state.set_background(self.g.state.default_background)             # <<<<<<<<<<<<<<
 *             self.g.state.bold=None
 *             self.g.state.reverse_video=None
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->set_background(__pyx_v_self->g->state, __pyx_v_self->g->state->default_background); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":286
 *             self.g.state.set_foreground(self.g.state.default_foreground)
 *             self.g.state.set_background(self.g.state.default_background)
 *             self.g.state.bold=None             # <<<<<<<<<<<<<<
 *             self.g.state.reverse_video=None
 *         elif cmd==1:
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->g->state->bold);
    __Pyx_DECREF(__pyx_v_self->g->state->bold);
    __pyx_v_self->g->state->bold = Py_None;

    /* "ttygif/tty/parser.pyx":287
 *             self.g.state.set_background(self.g.state.default_background)
 *             self.g.state.bold=None
 *             self.g.state.reverse_video=None             # <<<<<<<<<<<<<<
 *         elif cmd==1:
 *             self.g.state.bold=None
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->g->state->reverse_video);
    __Pyx_DECREF(__pyx_v_self->g->state->reverse_video);
    __pyx_v_self->g->state->reverse_video = Py_None;

    /* "ttygif/tty/parser.pyx":283
 * 
 *     cdef cmd_reset_mode(self,cmd):
 *         if cmd==0:             # <<<<<<<<<<<<<<
 *             self.g.state.set_foreground(self.g.state.default_foreground)
 *             self.g.state.set_background(self.g.state.default_background)
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":288
 *             self.g.state.bold=None
 *             self.g.state.reverse_video=None
 *         elif cmd==1:             # <<<<<<<<<<<<<<
 *             self.g.state.bold=None
 *         elif cmd==7:
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_cmd, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 288, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":289
 *             self.g.state.reverse_video=None
 *         elif cmd==1:
 *             self.g.state.bold=None             # <<<<<<<<<<<<<<
 *         elif cmd==7:
 *             self.g.state.reverse_video=None
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->g->state->bold);
    __Pyx_DECREF(__pyx_v_self->g->state->bold);
    __pyx_v_self->g->state->bold = Py_None;

    /* "ttygif/tty/parser.pyx":288
 *             self.g.state.bold=None
 *             self.g.state.reverse_video=None
 *         elif cmd==1:             # <<<<<<<<<<<<<<
 *             self.g.state.bold=None
 *         elif cmd==7:
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":290
 *         elif cmd==1:
 *             self.g.state.bold=None
 *         elif cmd==7:             # <<<<<<<<<<<<<<
 *             self.g.state.reverse_video=None
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_cmd, __pyx_int_7, 7, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 290, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 290, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":291
 *             self.g.state.bold=None
 *         elif cmd==7:
 *             self.g.state.reverse_video=None             # <<<<<<<<<<<<<<
 * 
 *     cdef set_foreground(self,color):
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->g->state->reverse_video);
    __Pyx_DECREF(__pyx_v_self->g->state->reverse_video);
    __pyx_v_self->g->state->reverse_video = Py_None;

    /* "ttygif/tty/parser.pyx":290
 *         elif cmd==1:
 *             self.g.state.bold=None
 *         elif cmd==7:             # <<<<<<<<<<<<<<
 *             self.g.state.reverse_video=None
 * 
 */
  }
  __pyx_L3:;

  /* "ttygif/tty/parser.pyx":282
 *             self.set_background(cmd-100+8)
 * 
 *     cdef cmd_reset_mode(self,cmd):             # <<<<<<<<<<<<<<
 *         if cmd==0:
 *             self.g.state.set_foreground(self.g.state.default_foreground)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_reset_mode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":293
 *             self.g.state.reverse_video=None
 * 
 *     cdef set_foreground(self,color):             # <<<<<<<<<<<<<<
 *         if color>=self.g.theme.colors:
 *             self.g.set_foreground(self.g.state.default_foreground)
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_set_foreground(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_color) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_foreground", 0);
  __Pyx_TraceCall("set_foreground", __pyx_f[0], 293, 0, __PYX_ERR(0, 293, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":294
 * 
 *     cdef set_foreground(self,color):
 *         if color>=self.g.theme.colors:             # <<<<<<<<<<<<<<
 *             self.g.set_foreground(self.g.state.default_foreground)
 *         else:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->g->theme->colors); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_color, __pyx_t_1, Py_GE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "ttygif/tty/parser.pyx":295
 *     cdef set_foreground(self,color):
 *         if color>=self.g.theme.colors:
 *             self.g.set_foreground(self.g.state.default_foreground)             # <<<<<<<<<<<<<<
 *         else:
 *             self.g.set_foreground(color)
 */
    __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->g->state->default_foreground); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->set_foreground(__pyx_v_self->g, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":294
 * 
 *     cdef set_foreground(self,color):
 *         if color>=self.g.theme.colors:             # <<<<<<<<<<<<<<
 *             self.g.set_foreground(self.g.state.default_foreground)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":297
 *             self.g.set_foreground(self.g.state.default_foreground)
 *         else:
 *             self.g.set_foreground(color)             # <<<<<<<<<<<<<<
 * 
 *     cdef set_background(self,color):
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->set_foreground(__pyx_v_self->g, __pyx_v_color); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "ttygif/tty/parser.pyx":293
 *             self.g.state.reverse_video=None
 * 
 *     cdef set_foreground(self,color):             # <<<<<<<<<<<<<<
 *         if color>=self.g.theme.colors:
 *             self.g.set_foreground(self.g.state.default_foreground)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.set_foreground", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":299
 *             self.g.set_foreground(color)
 * 
 *     cdef set_background(self,color):             # <<<<<<<<<<<<<<
 *         if color>=self.g.theme.colors:
 *             self.g.set_background(self.g.state.default_background)
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_set_background(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_color) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_background", 0);
  __Pyx_TraceCall("set_background", __pyx_f[0], 299, 0, __PYX_ERR(0, 299, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":300
 * 
 *     cdef set_background(self,color):
 *         if color>=self.g.theme.colors:             # <<<<<<<<<<<<<<
 *             self.g.set_background(self.g.state.default_background)
 *         else:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->g->theme->colors); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_color, __pyx_t_1, Py_GE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "ttygif/tty/parser.pyx":301
 *     cdef set_background(self,color):
 *         if color>=self.g.theme.colors:
 *             self.g.set_background(self.g.state.default_background)             # <<<<<<<<<<<<<<
 *         else:
 *             self.g.set_background(color)
 */
    __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->g->state->default_background); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 301, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->set_background(__pyx_v_self->g, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 301, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":300
 * 
 *     cdef set_background(self,color):
 *         if color>=self.g.theme.colors:             # <<<<<<<<<<<<<<
 *             self.g.set_background(self.g.state.default_background)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":303
 *             self.g.set_background(self.g.state.default_background)
 *         else:
 *             self.g.set_background(color)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_process_colors(self,params):
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->set_background(__pyx_v_self->g, __pyx_v_color); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 303, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "ttygif/tty/parser.pyx":299
 *             self.g.set_foreground(color)
 * 
 *     cdef set_background(self,color):             # <<<<<<<<<<<<<<
 *         if color>=self.g.theme.colors:
 *             self.g.set_background(self.g.state.default_background)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.set_background", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":305
 *             self.g.set_background(color)
 * 
 *     cdef cmd_process_colors(self,params):             # <<<<<<<<<<<<<<
 *         if 38 in params:
 *             if params[1]==2:
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_process_colors(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_params) {
  PyObject *__pyx_v_cmd = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_process_colors", 0);
  __Pyx_TraceCall("cmd_process_colors", __pyx_f[0], 305, 0, __PYX_ERR(0, 305, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":306
 * 
 *     cdef cmd_process_colors(self,params):
 *         if 38 in params:             # <<<<<<<<<<<<<<
 *             if params[1]==2:
 *                 self.g.foreground_from_rgb(params[2],params[3],params[4])
 */
  __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_int_38, __pyx_v_params, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 306, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":307
 *     cdef cmd_process_colors(self,params):
 *         if 38 in params:
 *             if params[1]==2:             # <<<<<<<<<<<<<<
 *                 self.g.foreground_from_rgb(params[2],params[3],params[4])
 *             if params[1]==5:
 */
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_params, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_t_3, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_2) {

      /* "ttygif/tty/parser.pyx":308
 *         if 38 in params:
 *             if params[1]==2:
 *                 self.g.foreground_from_rgb(params[2],params[3],params[4])             # <<<<<<<<<<<<<<
 *             if params[1]==5:
 *                     self.set_foreground(params[2])
 */
      __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_params, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 308, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_params, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 308, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_params, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 308, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->foreground_from_rgb(__pyx_v_self->g, __pyx_t_4, __pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 308, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "ttygif/tty/parser.pyx":307
 *     cdef cmd_process_colors(self,params):
 *         if 38 in params:
 *             if params[1]==2:             # <<<<<<<<<<<<<<
 *                 self.g.foreground_from_rgb(params[2],params[3],params[4])
 *             if params[1]==5:
 */
    }

    /* "ttygif/tty/parser.pyx":309
 *             if params[1]==2:
 *                 self.g.foreground_from_rgb(params[2],params[3],params[4])
 *             if params[1]==5:             # <<<<<<<<<<<<<<
 *                     self.set_foreground(params[2])
 *         elif 48 in params:
 */
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_params, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 309, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_PyInt_EqObjC(__pyx_t_6, __pyx_int_5, 5, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 309, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 309, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__pyx_t_2) {

      /* "ttygif/tty/parser.pyx":310
 *                 self.g.foreground_from_rgb(params[2],params[3],params[4])
 *             if params[1]==5:
 *                     self.set_foreground(params[2])             # <<<<<<<<<<<<<<
 *         elif 48 in params:
 *                 if params[1]==2:
 */
      __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_params, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 310, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->set_foreground(__pyx_v_self, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 310, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "ttygif/tty/parser.pyx":309
 *             if params[1]==2:
 *                 self.g.foreground_from_rgb(params[2],params[3],params[4])
 *             if params[1]==5:             # <<<<<<<<<<<<<<
 *                     self.set_foreground(params[2])
 *         elif 48 in params:
 */
    }

    /* "ttygif/tty/parser.pyx":306
 * 
 *     cdef cmd_process_colors(self,params):
 *         if 38 in params:             # <<<<<<<<<<<<<<
 *             if params[1]==2:
 *                 self.g.foreground_from_rgb(params[2],params[3],params[4])
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":311
 *             if params[1]==5:
 *                     self.set_foreground(params[2])
 *         elif 48 in params:             # <<<<<<<<<<<<<<
 *                 if params[1]==2:
 *                     self.g.background_from_rgb(params[2],params[3],params[4])
 */
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_int_48, __pyx_v_params, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 311, __pyx_L1_error)
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "ttygif/tty/parser.pyx":312
 *                     self.set_foreground(params[2])
 *         elif 48 in params:
 *                 if params[1]==2:             # <<<<<<<<<<<<<<
 *                     self.g.background_from_rgb(params[2],params[3],params[4])
 *                 if params[1]==5:
 */
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_params, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_PyInt_EqObjC(__pyx_t_6, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__pyx_t_1) {

      /* "ttygif/tty/parser.pyx":313
 *         elif 48 in params:
 *                 if params[1]==2:
 *                     self.g.background_from_rgb(params[2],params[3],params[4])             # <<<<<<<<<<<<<<
 *                 if params[1]==5:
 *                     self.set_background(params[2])
 */
      __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_params, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 313, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_params, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 313, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_params, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 313, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->background_from_rgb(__pyx_v_self->g, __pyx_t_5, __pyx_t_6, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 313, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "ttygif/tty/parser.pyx":312
 *                     self.set_foreground(params[2])
 *         elif 48 in params:
 *                 if params[1]==2:             # <<<<<<<<<<<<<<
 *                     self.g.background_from_rgb(params[2],params[3],params[4])
 *                 if params[1]==5:
 */
    }

    /* "ttygif/tty/parser.pyx":314
 *                 if params[1]==2:
 *                     self.g.background_from_rgb(params[2],params[3],params[4])
 *                 if params[1]==5:             # <<<<<<<<<<<<<<
 *                     self.set_background(params[2])
 *         else:
 */
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_params, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 314, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_t_4, __pyx_int_5, 5, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 314, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 314, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_1) {

      /* "ttygif/tty/parser.pyx":315
 *                     self.g.background_from_rgb(params[2],params[3],params[4])
 *                 if params[1]==5:
 *                     self.set_background(params[2])             # <<<<<<<<<<<<<<
 *         else:
 *             for cmd in params:
 */
      __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_params, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 315, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->set_background(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 315, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "ttygif/tty/parser.pyx":314
 *                 if params[1]==2:
 *                     self.g.background_from_rgb(params[2],params[3],params[4])
 *                 if params[1]==5:             # <<<<<<<<<<<<<<
 *                     self.set_background(params[2])
 *         else:
 */
    }

    /* "ttygif/tty/parser.pyx":311
 *             if params[1]==5:
 *                     self.set_foreground(params[2])
 *         elif 48 in params:             # <<<<<<<<<<<<<<
 *                 if params[1]==2:
 *                     self.g.background_from_rgb(params[2],params[3],params[4])
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":317
 *                     self.set_background(params[2])
 *         else:
 *             for cmd in params:             # <<<<<<<<<<<<<<
 *                 self.cmd_set_mode(cmd)
 * 
 */
  /*else*/ {
    if (likely(PyList_CheckExact(__pyx_v_params)) || PyTuple_CheckExact(__pyx_v_params)) {
      __pyx_t_4 = __pyx_v_params; __Pyx_INCREF(__pyx_t_4); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_7 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_v_params); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 317, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 317, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_7); __Pyx_INCREF(__pyx_t_3); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 317, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 317, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        } else {
          if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_7); __Pyx_INCREF(__pyx_t_3); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 317, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 317, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        }
      } else {
        __pyx_t_3 = __pyx_t_8(__pyx_t_4);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 317, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_XDECREF_SET(__pyx_v_cmd, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "ttygif/tty/parser.pyx":318
 *         else:
 *             for cmd in params:
 *                 self.cmd_set_mode(cmd)             # <<<<<<<<<<<<<<
 * 
 *     # cdef int NULL=0   #   Null character
 */
      __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->cmd_set_mode(__pyx_v_self, __pyx_v_cmd); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 318, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "ttygif/tty/parser.pyx":317
 *                     self.set_background(params[2])
 *         else:
 *             for cmd in params:             # <<<<<<<<<<<<<<
 *                 self.cmd_set_mode(cmd)
 * 
 */
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_L3:;

  /* "ttygif/tty/parser.pyx":305
 *             self.g.set_background(color)
 * 
 *     cdef cmd_process_colors(self,params):             # <<<<<<<<<<<<<<
 *         if 38 in params:
 *             if params[1]==2:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_process_colors", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cmd);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":351
 * 
 * 
 *     cdef cmd_render_text(self,event):             # <<<<<<<<<<<<<<
 *         #print event['data']
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_render_text(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_event) {
  int __pyx_v_BS;
  int __pyx_v_LF;
  int __pyx_v_CR;
  PyObject *__pyx_v_character = NULL;
  long __pyx_v_char_ord;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  long __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_render_text", 0);
  __Pyx_TraceCall("cmd_render_text", __pyx_f[0], 351, 0, __PYX_ERR(0, 351, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":354
 *         #print event['data']
 * 
 *         cdef int BS=8     # x Backspace             # <<<<<<<<<<<<<<
 *         cdef int LF=10     # x Line feed
 *         cdef int CR=13     # x Carriage return
 */
  __pyx_v_BS = 8;

  /* "ttygif/tty/parser.pyx":355
 * 
 *         cdef int BS=8     # x Backspace
 *         cdef int LF=10     # x Line feed             # <<<<<<<<<<<<<<
 *         cdef int CR=13     # x Carriage return
 * 
 */
  __pyx_v_LF = 10;

  /* "ttygif/tty/parser.pyx":356
 *         cdef int BS=8     # x Backspace
 *         cdef int LF=10     # x Line feed
 *         cdef int CR=13     # x Carriage return             # <<<<<<<<<<<<<<
 * 
 *         self.g.state.text_mode_on()
 */
  __pyx_v_CR = 13;

  /* "ttygif/tty/parser.pyx":358
 *         cdef int CR=13     # x Carriage return
 * 
 *         self.g.state.text_mode_on()             # <<<<<<<<<<<<<<
 *         for character in event['data']:
 *             char_ord=ord(character)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->text_mode_on(__pyx_v_self->g->state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":359
 * 
 *         self.g.state.text_mode_on()
 *         for character in event['data']:             # <<<<<<<<<<<<<<
 *             char_ord=ord(character)
 *             if char_ord<32 and self.no_codes==None:
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 359, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 359, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 359, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 359, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 359, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 359, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 359, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 359, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_character, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":360
 *         self.g.state.text_mode_on()
 *         for character in event['data']:
 *             char_ord=ord(character)             # <<<<<<<<<<<<<<
 *             if char_ord<32 and self.no_codes==None:
 *                 if  char_ord==BS:
 */
    __pyx_t_5 = __Pyx_PyObject_Ord(__pyx_v_character); if (unlikely(__pyx_t_5 == ((long)(long)(Py_UCS4)-1))) __PYX_ERR(0, 360, __pyx_L1_error)
    __pyx_v_char_ord = __pyx_t_5;

    /* "ttygif/tty/parser.pyx":361
 *         for character in event['data']:
 *             char_ord=ord(character)
 *             if char_ord<32 and self.no_codes==None:             # <<<<<<<<<<<<<<
 *                 if  char_ord==BS:
 *                     self.g.state.cursor_left(1)
 */
    __pyx_t_7 = ((__pyx_v_char_ord < 32) != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_self->no_codes, Py_None, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 361, __pyx_L1_error)
    __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 361, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = __pyx_t_7;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_6) {

      /* "ttygif/tty/parser.pyx":362
 *             char_ord=ord(character)
 *             if char_ord<32 and self.no_codes==None:
 *                 if  char_ord==BS:             # <<<<<<<<<<<<<<
 *                     self.g.state.cursor_left(1)
 *                 elif char_ord==LF:
 */
      __pyx_t_6 = ((__pyx_v_char_ord == __pyx_v_BS) != 0);
      if (__pyx_t_6) {

        /* "ttygif/tty/parser.pyx":363
 *             if char_ord<32 and self.no_codes==None:
 *                 if  char_ord==BS:
 *                     self.g.state.cursor_left(1)             # <<<<<<<<<<<<<<
 *                 elif char_ord==LF:
 *                     self.g.state.cursor_down(1)
 */
        __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_left(__pyx_v_self->g->state, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 363, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "ttygif/tty/parser.pyx":362
 *             char_ord=ord(character)
 *             if char_ord<32 and self.no_codes==None:
 *                 if  char_ord==BS:             # <<<<<<<<<<<<<<
 *                     self.g.state.cursor_left(1)
 *                 elif char_ord==LF:
 */
        goto __pyx_L8;
      }

      /* "ttygif/tty/parser.pyx":364
 *                 if  char_ord==BS:
 *                     self.g.state.cursor_left(1)
 *                 elif char_ord==LF:             # <<<<<<<<<<<<<<
 *                     self.g.state.cursor_down(1)
 *                     if self.g.state.mode=="linux":
 */
      __pyx_t_6 = ((__pyx_v_char_ord == __pyx_v_LF) != 0);
      if (__pyx_t_6) {

        /* "ttygif/tty/parser.pyx":365
 *                     self.g.state.cursor_left(1)
 *                 elif char_ord==LF:
 *                     self.g.state.cursor_down(1)             # <<<<<<<<<<<<<<
 *                     if self.g.state.mode=="linux":
 *                         self.g.state.cursor_absolute_x(0)
 */
        __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_down(__pyx_v_self->g->state, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 365, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "ttygif/tty/parser.pyx":366
 *                 elif char_ord==LF:
 *                     self.g.state.cursor_down(1)
 *                     if self.g.state.mode=="linux":             # <<<<<<<<<<<<<<
 *                         self.g.state.cursor_absolute_x(0)
 * 
 */
        __pyx_t_6 = (__Pyx_PyString_Equals(__pyx_v_self->g->state->mode, __pyx_n_s_linux, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 366, __pyx_L1_error)
        __pyx_t_7 = (__pyx_t_6 != 0);
        if (__pyx_t_7) {

          /* "ttygif/tty/parser.pyx":367
 *                     self.g.state.cursor_down(1)
 *                     if self.g.state.mode=="linux":
 *                         self.g.state.cursor_absolute_x(0)             # <<<<<<<<<<<<<<
 * 
 *                 elif char_ord==CR:
 */
          __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute_x(__pyx_v_self->g->state, __pyx_int_0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 367, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "ttygif/tty/parser.pyx":366
 *                 elif char_ord==LF:
 *                     self.g.state.cursor_down(1)
 *                     if self.g.state.mode=="linux":             # <<<<<<<<<<<<<<
 *                         self.g.state.cursor_absolute_x(0)
 * 
 */
        }

        /* "ttygif/tty/parser.pyx":364
 *                 if  char_ord==BS:
 *                     self.g.state.cursor_left(1)
 *                 elif char_ord==LF:             # <<<<<<<<<<<<<<
 *                     self.g.state.cursor_down(1)
 *                     if self.g.state.mode=="linux":
 */
        goto __pyx_L8;
      }

      /* "ttygif/tty/parser.pyx":369
 *                         self.g.state.cursor_absolute_x(0)
 * 
 *                 elif char_ord==CR:             # <<<<<<<<<<<<<<
 *                     self.g.state.cursor_absolute_x(0)
 *             else:
 */
      __pyx_t_7 = ((__pyx_v_char_ord == __pyx_v_CR) != 0);
      if (__pyx_t_7) {

        /* "ttygif/tty/parser.pyx":370
 * 
 *                 elif char_ord==CR:
 *                     self.g.state.cursor_absolute_x(0)             # <<<<<<<<<<<<<<
 *             else:
 *                 if self.g.state.pending_wrap:
 */
        __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute_x(__pyx_v_self->g->state, __pyx_int_0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 370, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "ttygif/tty/parser.pyx":369
 *                         self.g.state.cursor_absolute_x(0)
 * 
 *                 elif char_ord==CR:             # <<<<<<<<<<<<<<
 *                     self.g.state.cursor_absolute_x(0)
 *             else:
 */
      }
      __pyx_L8:;

      /* "ttygif/tty/parser.pyx":361
 *         for character in event['data']:
 *             char_ord=ord(character)
 *             if char_ord<32 and self.no_codes==None:             # <<<<<<<<<<<<<<
 *                 if  char_ord==BS:
 *                     self.g.state.cursor_left(1)
 */
      goto __pyx_L5;
    }

    /* "ttygif/tty/parser.pyx":372
 *                     self.g.state.cursor_absolute_x(0)
 *             else:
 *                 if self.g.state.pending_wrap:             # <<<<<<<<<<<<<<
 *                     self.g.state.cursor_right(1)
 *                 self.g.write(char_ord)
 */
    /*else*/ {
      __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_self->g->state->pending_wrap); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 372, __pyx_L1_error)
      if (__pyx_t_7) {

        /* "ttygif/tty/parser.pyx":373
 *             else:
 *                 if self.g.state.pending_wrap:
 *                     self.g.state.cursor_right(1)             # <<<<<<<<<<<<<<
 *                 self.g.write(char_ord)
 *                 self.g.state.cursor_right(1)
 */
        __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_right(__pyx_v_self->g->state, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "ttygif/tty/parser.pyx":372
 *                     self.g.state.cursor_absolute_x(0)
 *             else:
 *                 if self.g.state.pending_wrap:             # <<<<<<<<<<<<<<
 *                     self.g.state.cursor_right(1)
 *                 self.g.write(char_ord)
 */
      }

      /* "ttygif/tty/parser.pyx":374
 *                 if self.g.state.pending_wrap:
 *                     self.g.state.cursor_right(1)
 *                 self.g.write(char_ord)             # <<<<<<<<<<<<<<
 *                 self.g.state.cursor_right(1)
 *             while self.g.state.scroll!=0:
 */
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->write(__pyx_v_self->g, __pyx_v_char_ord); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 374, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ttygif/tty/parser.pyx":375
 *                     self.g.state.cursor_right(1)
 *                 self.g.write(char_ord)
 *                 self.g.state.cursor_right(1)             # <<<<<<<<<<<<<<
 *             while self.g.state.scroll!=0:
 *                 #print("Scroll at {0:005x}".format(self.current_sequence_position))
 */
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_right(__pyx_v_self->g->state, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 375, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L5:;

    /* "ttygif/tty/parser.pyx":376
 *                 self.g.write(char_ord)
 *                 self.g.state.cursor_right(1)
 *             while self.g.state.scroll!=0:             # <<<<<<<<<<<<<<
 *                 #print("Scroll at {0:005x}".format(self.current_sequence_position))
 *                 self.g.scroll_buffer()
 */
    while (1) {
      __pyx_t_7 = ((__pyx_v_self->g->state->scroll != 0) != 0);
      if (!__pyx_t_7) break;

      /* "ttygif/tty/parser.pyx":378
 *             while self.g.state.scroll!=0:
 *                 #print("Scroll at {0:005x}".format(self.current_sequence_position))
 *                 self.g.scroll_buffer()             # <<<<<<<<<<<<<<
 *         self.g.state.text_mode_off()
 * 
 */
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->scroll_buffer(__pyx_v_self->g); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 378, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }

    /* "ttygif/tty/parser.pyx":359
 * 
 *         self.g.state.text_mode_on()
 *         for character in event['data']:             # <<<<<<<<<<<<<<
 *             char_ord=ord(character)
 *             if char_ord<32 and self.no_codes==None:
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ttygif/tty/parser.pyx":379
 *                 #print("Scroll at {0:005x}".format(self.current_sequence_position))
 *                 self.g.scroll_buffer()
 *         self.g.state.text_mode_off()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->text_mode_off(__pyx_v_self->g->state); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ttygif/tty/parser.pyx":351
 * 
 * 
 *     cdef cmd_render_text(self,event):             # <<<<<<<<<<<<<<
 *         #print event['data']
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_render_text", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_character);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":382
 * 
 * 
 *     cdef cmd_DECSTBM(self,int top,int bottom):             # <<<<<<<<<<<<<<
 *         self.g.state.set_scroll_region(top,bottom)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_DECSTBM(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, int __pyx_v_top, int __pyx_v_bottom) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_DECSTBM", 0);
  __Pyx_TraceCall("cmd_DECSTBM", __pyx_f[0], 382, 0, __PYX_ERR(0, 382, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":383
 * 
 *     cdef cmd_DECSTBM(self,int top,int bottom):
 *         self.g.state.set_scroll_region(top,bottom)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_CUU(self,distance):
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_top); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_bottom); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->set_scroll_region(__pyx_v_self->g->state, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ttygif/tty/parser.pyx":382
 * 
 * 
 *     cdef cmd_DECSTBM(self,int top,int bottom):             # <<<<<<<<<<<<<<
 *         self.g.state.set_scroll_region(top,bottom)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_DECSTBM", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":385
 *         self.g.state.set_scroll_region(top,bottom)
 * 
 *     cdef cmd_CUU(self,distance):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_up(distance)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUU(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_CUU", 0);
  __Pyx_TraceCall("cmd_CUU", __pyx_f[0], 385, 0, __PYX_ERR(0, 385, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":386
 * 
 *     cdef cmd_CUU(self,distance):
 *         self.g.state.cursor_up(distance)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_CUD(self,distance):
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_distance); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 386, __pyx_L1_error)
  __pyx_t_2 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_up(__pyx_v_self->g->state, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ttygif/tty/parser.pyx":385
 *         self.g.state.set_scroll_region(top,bottom)
 * 
 *     cdef cmd_CUU(self,distance):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_up(distance)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_CUU", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":388
 *         self.g.state.cursor_up(distance)
 * 
 *     cdef cmd_CUD(self,distance):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_down(distance)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUD(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_CUD", 0);
  __Pyx_TraceCall("cmd_CUD", __pyx_f[0], 388, 0, __PYX_ERR(0, 388, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":389
 * 
 *     cdef cmd_CUD(self,distance):
 *         self.g.state.cursor_down(distance)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_CUB(self,distance):
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_distance); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 389, __pyx_L1_error)
  __pyx_t_2 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_down(__pyx_v_self->g->state, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ttygif/tty/parser.pyx":388
 *         self.g.state.cursor_up(distance)
 * 
 *     cdef cmd_CUD(self,distance):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_down(distance)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_CUD", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":391
 *         self.g.state.cursor_down(distance)
 * 
 *     cdef cmd_CUB(self,distance):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_left(distance)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUB(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_CUB", 0);
  __Pyx_TraceCall("cmd_CUB", __pyx_f[0], 391, 0, __PYX_ERR(0, 391, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":392
 * 
 *     cdef cmd_CUB(self,distance):
 *         self.g.state.cursor_left(distance)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_CUF(self,distance):
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_distance); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 392, __pyx_L1_error)
  __pyx_t_2 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_left(__pyx_v_self->g->state, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ttygif/tty/parser.pyx":391
 *         self.g.state.cursor_down(distance)
 * 
 *     cdef cmd_CUB(self,distance):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_left(distance)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_CUB", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":394
 *         self.g.state.cursor_left(distance)
 * 
 *     cdef cmd_CUF(self,distance):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_right(distance)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUF(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_CUF", 0);
  __Pyx_TraceCall("cmd_CUF", __pyx_f[0], 394, 0, __PYX_ERR(0, 394, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":395
 * 
 *     cdef cmd_CUF(self,distance):
 *         self.g.state.cursor_right(distance)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_CPL(self,distance):
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_distance); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 395, __pyx_L1_error)
  __pyx_t_2 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_right(__pyx_v_self->g->state, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ttygif/tty/parser.pyx":394
 *         self.g.state.cursor_left(distance)
 * 
 *     cdef cmd_CUF(self,distance):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_right(distance)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_CUF", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":397
 *         self.g.state.cursor_right(distance)
 * 
 *     cdef cmd_CPL(self,distance):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute_x(0)
 *         self.g.state.cursor_up(distance)
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CPL(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_CPL", 0);
  __Pyx_TraceCall("cmd_CPL", __pyx_f[0], 397, 0, __PYX_ERR(0, 397, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":398
 * 
 *     cdef cmd_CPL(self,distance):
 *         self.g.state.cursor_absolute_x(0)             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_up(distance)
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute_x(__pyx_v_self->g->state, __pyx_int_0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":399
 *     cdef cmd_CPL(self,distance):
 *         self.g.state.cursor_absolute_x(0)
 *         self.g.state.cursor_up(distance)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_CNL(self,distance):
 */
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_v_distance); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 399, __pyx_L1_error)
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_up(__pyx_v_self->g->state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 399, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":397
 *         self.g.state.cursor_right(distance)
 * 
 *     cdef cmd_CPL(self,distance):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute_x(0)
 *         self.g.state.cursor_up(distance)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_CPL", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":401
 *         self.g.state.cursor_up(distance)
 * 
 *     cdef cmd_CNL(self,distance):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute_x(0)
 *         self.g.state.cursor_up(distance)
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CNL(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_CNL", 0);
  __Pyx_TraceCall("cmd_CNL", __pyx_f[0], 401, 0, __PYX_ERR(0, 401, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":402
 * 
 *     cdef cmd_CNL(self,distance):
 *         self.g.state.cursor_absolute_x(0)             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_up(distance)
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute_x(__pyx_v_self->g->state, __pyx_int_0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":403
 *     cdef cmd_CNL(self,distance):
 *         self.g.state.cursor_absolute_x(0)
 *         self.g.state.cursor_up(distance)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_CHA(self,x):
 */
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_v_distance); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 403, __pyx_L1_error)
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_up(__pyx_v_self->g->state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":401
 *         self.g.state.cursor_up(distance)
 * 
 *     cdef cmd_CNL(self,distance):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute_x(0)
 *         self.g.state.cursor_up(distance)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_CNL", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":405
 *         self.g.state.cursor_up(distance)
 * 
 *     cdef cmd_CHA(self,x):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute_x(x)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CHA(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_x) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_CHA", 0);
  __Pyx_TraceCall("cmd_CHA", __pyx_f[0], 405, 0, __PYX_ERR(0, 405, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":406
 * 
 *     cdef cmd_CHA(self,x):
 *         self.g.state.cursor_absolute_x(x)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute_x(__pyx_v_self->g->state, __pyx_v_x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":405
 *         self.g.state.cursor_up(distance)
 * 
 *     cdef cmd_CHA(self,x):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute_x(x)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_CHA", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":409
 * 
 * 
 *     cdef cmd_CUP(self,x,y):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute(x,y)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUP(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_CUP", 0);
  __Pyx_TraceCall("cmd_CUP", __pyx_f[0], 409, 0, __PYX_ERR(0, 409, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":410
 * 
 *     cdef cmd_CUP(self,x,y):
 *         self.g.state.cursor_absolute(x,y)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_ED(self,mode):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute(__pyx_v_self->g->state, __pyx_v_x, __pyx_v_y); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 410, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":409
 * 
 * 
 *     cdef cmd_CUP(self,x,y):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute(x,y)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_CUP", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":412
 *         self.g.state.cursor_absolute(x,y)
 * 
 *     cdef cmd_ED(self,mode):             # <<<<<<<<<<<<<<
 *         if mode==0:
 *             cp=self.g.state.cursor_get_position()
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_ED(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_mode) {
  PyObject *__pyx_v_cp = NULL;
  long __pyx_v_x;
  long __pyx_v_y;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;
  long __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  uint16_t __pyx_t_9;
  uint16_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  uint16_t __pyx_t_12;
  uint16_t __pyx_t_13;
  uint8_t __pyx_t_14[3];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_ED", 0);
  __Pyx_TraceCall("cmd_ED", __pyx_f[0], 412, 0, __PYX_ERR(0, 412, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":413
 * 
 *     cdef cmd_ED(self,mode):
 *         if mode==0:             # <<<<<<<<<<<<<<
 *             cp=self.g.state.cursor_get_position()
 *             for x in xrange(0,self.g.state.cursor_x+1):
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_mode, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 413, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 413, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":414
 *     cdef cmd_ED(self,mode):
 *         if mode==0:
 *             cp=self.g.state.cursor_get_position()             # <<<<<<<<<<<<<<
 *             for x in xrange(0,self.g.state.cursor_x+1):
 *                 self.g.state.cursor_absolute_x(x)
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_get_position(__pyx_v_self->g->state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 414, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_cp = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":415
 *         if mode==0:
 *             cp=self.g.state.cursor_get_position()
 *             for x in xrange(0,self.g.state.cursor_x+1):             # <<<<<<<<<<<<<<
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 */
    __pyx_t_3 = (__pyx_v_self->g->state->cursor_x + 1);
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_x = __pyx_t_5;

      /* "ttygif/tty/parser.pyx":416
 *             cp=self.g.state.cursor_get_position()
 *             for x in xrange(0,self.g.state.cursor_x+1):
 *                 self.g.state.cursor_absolute_x(x)             # <<<<<<<<<<<<<<
 *                 self.g.write(0)
 *             for y in xrange(0,self.g.state.cursor_y):
 */
      __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 416, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute_x(__pyx_v_self->g->state, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 416, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "ttygif/tty/parser.pyx":417
 *             for x in xrange(0,self.g.state.cursor_x+1):
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)             # <<<<<<<<<<<<<<
 *             for y in xrange(0,self.g.state.cursor_y):
 *                 for x in xrange(0,self.g.state.width):
 */
      __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->write(__pyx_v_self->g, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 417, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }

    /* "ttygif/tty/parser.pyx":418
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 *             for y in xrange(0,self.g.state.cursor_y):             # <<<<<<<<<<<<<<
 *                 for x in xrange(0,self.g.state.width):
 *                     self.g.state.cursor_absolute(x,y)
 */
    __pyx_t_7 = __pyx_v_self->g->state->cursor_y;
    __pyx_t_8 = __pyx_t_7;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_8; __pyx_t_3+=1) {
      __pyx_v_y = __pyx_t_3;

      /* "ttygif/tty/parser.pyx":419
 *                 self.g.write(0)
 *             for y in xrange(0,self.g.state.cursor_y):
 *                 for x in xrange(0,self.g.state.width):             # <<<<<<<<<<<<<<
 *                     self.g.state.cursor_absolute(x,y)
 *                     self.g.write(0)
 */
      __pyx_t_9 = __pyx_v_self->g->state->width;
      __pyx_t_10 = __pyx_t_9;
      for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_10; __pyx_t_4+=1) {
        __pyx_v_x = __pyx_t_4;

        /* "ttygif/tty/parser.pyx":420
 *             for y in xrange(0,self.g.state.cursor_y):
 *                 for x in xrange(0,self.g.state.width):
 *                     self.g.state.cursor_absolute(x,y)             # <<<<<<<<<<<<<<
 *                     self.g.write(0)
 *             self.g.state.cursor_absolute(cp[0],cp[1])
 */
        __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_v_x); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 420, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_y); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 420, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_11 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute(__pyx_v_self->g->state, __pyx_t_6, __pyx_t_1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 420, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

        /* "ttygif/tty/parser.pyx":421
 *                 for x in xrange(0,self.g.state.width):
 *                     self.g.state.cursor_absolute(x,y)
 *                     self.g.write(0)             # <<<<<<<<<<<<<<
 *             self.g.state.cursor_absolute(cp[0],cp[1])
 *         if mode==1:
 */
        __pyx_t_11 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->write(__pyx_v_self->g, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 421, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
    }

    /* "ttygif/tty/parser.pyx":422
 *                     self.g.state.cursor_absolute(x,y)
 *                     self.g.write(0)
 *             self.g.state.cursor_absolute(cp[0],cp[1])             # <<<<<<<<<<<<<<
 *         if mode==1:
 *             cp=self.g.state.cursor_get_position()
 */
    __pyx_t_11 = __Pyx_GetItemInt(__pyx_v_cp, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 422, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_cp, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 422, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute(__pyx_v_self->g->state, __pyx_t_11, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 422, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "ttygif/tty/parser.pyx":413
 * 
 *     cdef cmd_ED(self,mode):
 *         if mode==0:             # <<<<<<<<<<<<<<
 *             cp=self.g.state.cursor_get_position()
 *             for x in xrange(0,self.g.state.cursor_x+1):
 */
  }

  /* "ttygif/tty/parser.pyx":423
 *                     self.g.write(0)
 *             self.g.state.cursor_absolute(cp[0],cp[1])
 *         if mode==1:             # <<<<<<<<<<<<<<
 *             cp=self.g.state.cursor_get_position()
 *             for x in xrange(self.g.state.cursor_x,self.g.state.width):
 */
  __pyx_t_6 = __Pyx_PyInt_EqObjC(__pyx_v_mode, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 423, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":424
 *             self.g.state.cursor_absolute(cp[0],cp[1])
 *         if mode==1:
 *             cp=self.g.state.cursor_get_position()             # <<<<<<<<<<<<<<
 *             for x in xrange(self.g.state.cursor_x,self.g.state.width):
 *                 self.g.state.cursor_absolute_x(x)
 */
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_get_position(__pyx_v_self->g->state); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 424, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_XDECREF_SET(__pyx_v_cp, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "ttygif/tty/parser.pyx":425
 *         if mode==1:
 *             cp=self.g.state.cursor_get_position()
 *             for x in xrange(self.g.state.cursor_x,self.g.state.width):             # <<<<<<<<<<<<<<
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 */
    __pyx_t_9 = __pyx_v_self->g->state->width;
    __pyx_t_10 = __pyx_t_9;
    for (__pyx_t_3 = __pyx_v_self->g->state->cursor_x; __pyx_t_3 < __pyx_t_10; __pyx_t_3+=1) {
      __pyx_v_x = __pyx_t_3;

      /* "ttygif/tty/parser.pyx":426
 *             cp=self.g.state.cursor_get_position()
 *             for x in xrange(self.g.state.cursor_x,self.g.state.width):
 *                 self.g.state.cursor_absolute_x(x)             # <<<<<<<<<<<<<<
 *                 self.g.write(0)
 * 
 */
      __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_v_x); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 426, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute_x(__pyx_v_self->g->state, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 426, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ttygif/tty/parser.pyx":427
 *             for x in xrange(self.g.state.cursor_x,self.g.state.width):
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)             # <<<<<<<<<<<<<<
 * 
 *             for y in xrange(self.g.state.cursor_y+1,self.g.state.height):
 */
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->write(__pyx_v_self->g, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }

    /* "ttygif/tty/parser.pyx":429
 *                 self.g.write(0)
 * 
 *             for y in xrange(self.g.state.cursor_y+1,self.g.state.height):             # <<<<<<<<<<<<<<
 *                 for x in xrange(0,self.g.state.width):
 *                     self.g.state.cursor_absolute(x,y)
 */
    __pyx_t_9 = __pyx_v_self->g->state->height;
    __pyx_t_10 = __pyx_t_9;
    for (__pyx_t_3 = (__pyx_v_self->g->state->cursor_y + 1); __pyx_t_3 < __pyx_t_10; __pyx_t_3+=1) {
      __pyx_v_y = __pyx_t_3;

      /* "ttygif/tty/parser.pyx":430
 * 
 *             for y in xrange(self.g.state.cursor_y+1,self.g.state.height):
 *                 for x in xrange(0,self.g.state.width):             # <<<<<<<<<<<<<<
 *                     self.g.state.cursor_absolute(x,y)
 *                     self.g.write(0)
 */
      __pyx_t_12 = __pyx_v_self->g->state->width;
      __pyx_t_13 = __pyx_t_12;
      for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_13; __pyx_t_4+=1) {
        __pyx_v_x = __pyx_t_4;

        /* "ttygif/tty/parser.pyx":431
 *             for y in xrange(self.g.state.cursor_y+1,self.g.state.height):
 *                 for x in xrange(0,self.g.state.width):
 *                     self.g.state.cursor_absolute(x,y)             # <<<<<<<<<<<<<<
 *                     self.g.write(0)
 * 
 */
        __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 431, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_v_y); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 431, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_11 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute(__pyx_v_self->g->state, __pyx_t_1, __pyx_t_6); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 431, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

        /* "ttygif/tty/parser.pyx":432
 *                 for x in xrange(0,self.g.state.width):
 *                     self.g.state.cursor_absolute(x,y)
 *                     self.g.write(0)             # <<<<<<<<<<<<<<
 * 
 *             self.g.state.cursor_absolute(cp[0],cp[1])
 */
        __pyx_t_11 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->write(__pyx_v_self->g, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 432, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
    }

    /* "ttygif/tty/parser.pyx":434
 *                     self.g.write(0)
 * 
 *             self.g.state.cursor_absolute(cp[0],cp[1])             # <<<<<<<<<<<<<<
 * 
 *         if mode==2:
 */
    __pyx_t_11 = __Pyx_GetItemInt(__pyx_v_cp, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_cp, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute(__pyx_v_self->g->state, __pyx_t_11, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":423
 *                     self.g.write(0)
 *             self.g.state.cursor_absolute(cp[0],cp[1])
 *         if mode==1:             # <<<<<<<<<<<<<<
 *             cp=self.g.state.cursor_get_position()
 *             for x in xrange(self.g.state.cursor_x,self.g.state.width):
 */
  }

  /* "ttygif/tty/parser.pyx":436
 *             self.g.state.cursor_absolute(cp[0],cp[1])
 * 
 *         if mode==2:             # <<<<<<<<<<<<<<
 *             self.g.screen.clear([self.g.state.foreground,self.g.state.background,0])
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_mode, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":437
 * 
 *         if mode==2:
 *             self.g.screen.clear([self.g.state.foreground,self.g.state.background,0])             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_EL(self,mode):
 */
    __pyx_t_14[0] = __pyx_v_self->g->state->foreground;
    __pyx_t_14[1] = __pyx_v_self->g->state->background;
    __pyx_t_14[2] = 0;
    __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_5image_image *)__pyx_v_self->g->screen->__pyx_vtab)->clear(__pyx_v_self->g->screen, __pyx_t_14); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":436
 *             self.g.state.cursor_absolute(cp[0],cp[1])
 * 
 *         if mode==2:             # <<<<<<<<<<<<<<
 *             self.g.screen.clear([self.g.state.foreground,self.g.state.background,0])
 * 
 */
  }

  /* "ttygif/tty/parser.pyx":412
 *         self.g.state.cursor_absolute(x,y)
 * 
 *     cdef cmd_ED(self,mode):             # <<<<<<<<<<<<<<
 *         if mode==0:
 *             cp=self.g.state.cursor_get_position()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_ED", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cp);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":439
 *             self.g.screen.clear([self.g.state.foreground,self.g.state.background,0])
 * 
 *     cdef cmd_EL(self,mode):             # <<<<<<<<<<<<<<
 *         cp=self.g.state.cursor_get_position()
 *         #print ( "DEL",mode,cp)
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_EL(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_mode) {
  PyObject *__pyx_v_cp = NULL;
  long __pyx_v_x;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  uint16_t __pyx_t_3;
  uint16_t __pyx_t_4;
  long __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  long __pyx_t_7;
  long __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_EL", 0);
  __Pyx_TraceCall("cmd_EL", __pyx_f[0], 439, 0, __PYX_ERR(0, 439, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":440
 * 
 *     cdef cmd_EL(self,mode):
 *         cp=self.g.state.cursor_get_position()             # <<<<<<<<<<<<<<
 *         #print ( "DEL",mode,cp)
 *         if mode==0:
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_get_position(__pyx_v_self->g->state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 440, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_cp = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":442
 *         cp=self.g.state.cursor_get_position()
 *         #print ( "DEL",mode,cp)
 *         if mode==0:             # <<<<<<<<<<<<<<
 *             for x in xrange(self.g.state.cursor_x,self.g.state.width):
 *                 self.g.state.cursor_absolute_x(x)
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_mode, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":443
 *         #print ( "DEL",mode,cp)
 *         if mode==0:
 *             for x in xrange(self.g.state.cursor_x,self.g.state.width):             # <<<<<<<<<<<<<<
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 */
    __pyx_t_3 = __pyx_v_self->g->state->width;
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_5 = __pyx_v_self->g->state->cursor_x; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_x = __pyx_t_5;

      /* "ttygif/tty/parser.pyx":444
 *         if mode==0:
 *             for x in xrange(self.g.state.cursor_x,self.g.state.width):
 *                 self.g.state.cursor_absolute_x(x)             # <<<<<<<<<<<<<<
 *                 self.g.write(0)
 *         elif mode==1:
 */
      __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 444, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute_x(__pyx_v_self->g->state, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 444, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "ttygif/tty/parser.pyx":445
 *             for x in xrange(self.g.state.cursor_x,self.g.state.width):
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)             # <<<<<<<<<<<<<<
 *         elif mode==1:
 *             for x in xrange(0,self.g.state.cursor_x+1):
 */
      __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->write(__pyx_v_self->g, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 445, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }

    /* "ttygif/tty/parser.pyx":442
 *         cp=self.g.state.cursor_get_position()
 *         #print ( "DEL",mode,cp)
 *         if mode==0:             # <<<<<<<<<<<<<<
 *             for x in xrange(self.g.state.cursor_x,self.g.state.width):
 *                 self.g.state.cursor_absolute_x(x)
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":446
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 *         elif mode==1:             # <<<<<<<<<<<<<<
 *             for x in xrange(0,self.g.state.cursor_x+1):
 *                 self.g.state.cursor_absolute_x(x)
 */
  __pyx_t_6 = __Pyx_PyInt_EqObjC(__pyx_v_mode, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":447
 *                 self.g.write(0)
 *         elif mode==1:
 *             for x in xrange(0,self.g.state.cursor_x+1):             # <<<<<<<<<<<<<<
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 */
    __pyx_t_5 = (__pyx_v_self->g->state->cursor_x + 1);
    __pyx_t_7 = __pyx_t_5;
    for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
      __pyx_v_x = __pyx_t_8;

      /* "ttygif/tty/parser.pyx":448
 *         elif mode==1:
 *             for x in xrange(0,self.g.state.cursor_x+1):
 *                 self.g.state.cursor_absolute_x(x)             # <<<<<<<<<<<<<<
 *                 self.g.write(0)
 *         elif mode==2:
 */
      __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_v_x); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 448, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute_x(__pyx_v_self->g->state, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 448, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ttygif/tty/parser.pyx":449
 *             for x in xrange(0,self.g.state.cursor_x+1):
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)             # <<<<<<<<<<<<<<
 *         elif mode==2:
 *             for x in xrange(0,self.g.state.width):
 */
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->write(__pyx_v_self->g, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }

    /* "ttygif/tty/parser.pyx":446
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 *         elif mode==1:             # <<<<<<<<<<<<<<
 *             for x in xrange(0,self.g.state.cursor_x+1):
 *                 self.g.state.cursor_absolute_x(x)
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":450
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 *         elif mode==2:             # <<<<<<<<<<<<<<
 *             for x in xrange(0,self.g.state.width):
 *                 self.g.state.cursor_absolute_x(x)
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_mode, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":451
 *                 self.g.write(0)
 *         elif mode==2:
 *             for x in xrange(0,self.g.state.width):             # <<<<<<<<<<<<<<
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 */
    __pyx_t_3 = __pyx_v_self->g->state->width;
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_x = __pyx_t_5;

      /* "ttygif/tty/parser.pyx":452
 *         elif mode==2:
 *             for x in xrange(0,self.g.state.width):
 *                 self.g.state.cursor_absolute_x(x)             # <<<<<<<<<<<<<<
 *                 self.g.write(0)
 *         self.g.state.cursor_absolute(cp[0],cp[1])
 */
      __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 452, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute_x(__pyx_v_self->g->state, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 452, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "ttygif/tty/parser.pyx":453
 *             for x in xrange(0,self.g.state.width):
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute(cp[0],cp[1])
 * 
 */
      __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->write(__pyx_v_self->g, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 453, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }

    /* "ttygif/tty/parser.pyx":450
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 *         elif mode==2:             # <<<<<<<<<<<<<<
 *             for x in xrange(0,self.g.state.width):
 *                 self.g.state.cursor_absolute_x(x)
 */
  }
  __pyx_L3:;

  /* "ttygif/tty/parser.pyx":454
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 *         self.g.state.cursor_absolute(cp[0],cp[1])             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_DCH(self,distance):
 */
  __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_cp, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_cp, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute(__pyx_v_self->g->state, __pyx_t_6, __pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "ttygif/tty/parser.pyx":439
 *             self.g.screen.clear([self.g.state.foreground,self.g.state.background,0])
 * 
 *     cdef cmd_EL(self,mode):             # <<<<<<<<<<<<<<
 *         cp=self.g.state.cursor_get_position()
 *         #print ( "DEL",mode,cp)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_EL", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cp);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":456
 *         self.g.state.cursor_absolute(cp[0],cp[1])
 * 
 *     cdef cmd_DCH(self,distance):             # <<<<<<<<<<<<<<
 *         cdef int x=self.g.state.cursor_x
 *         cdef int y=self.g.state.cursor_y
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_DCH(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance) {
  int __pyx_v_x;
  int __pyx_v_y;
  int __pyx_v_width;
  CYTHON_UNUSED PyObject *__pyx_v_temp = NULL;
  uint8_t __pyx_v_c[3];
  PyObject *__pyx_v_x2 = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  uint16_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  uint8_t __pyx_t_4[3];
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  uint8_t __pyx_t_9[3];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_DCH", 0);
  __Pyx_TraceCall("cmd_DCH", __pyx_f[0], 456, 0, __PYX_ERR(0, 456, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":457
 * 
 *     cdef cmd_DCH(self,distance):
 *         cdef int x=self.g.state.cursor_x             # <<<<<<<<<<<<<<
 *         cdef int y=self.g.state.cursor_y
 *         cdef int width=self.g.state.width
 */
  __pyx_t_1 = __pyx_v_self->g->state->cursor_x;
  __pyx_v_x = __pyx_t_1;

  /* "ttygif/tty/parser.pyx":458
 *     cdef cmd_DCH(self,distance):
 *         cdef int x=self.g.state.cursor_x
 *         cdef int y=self.g.state.cursor_y             # <<<<<<<<<<<<<<
 *         cdef int width=self.g.state.width
 *         temp=[]
 */
  __pyx_t_1 = __pyx_v_self->g->state->cursor_y;
  __pyx_v_y = __pyx_t_1;

  /* "ttygif/tty/parser.pyx":459
 *         cdef int x=self.g.state.cursor_x
 *         cdef int y=self.g.state.cursor_y
 *         cdef int width=self.g.state.width             # <<<<<<<<<<<<<<
 *         temp=[]
 *         cdef uint8_t[3] c=[0,0,0]
 */
  __pyx_t_2 = __pyx_v_self->g->state->width;
  __pyx_v_width = __pyx_t_2;

  /* "ttygif/tty/parser.pyx":460
 *         cdef int y=self.g.state.cursor_y
 *         cdef int width=self.g.state.width
 *         temp=[]             # <<<<<<<<<<<<<<
 *         cdef uint8_t[3] c=[0,0,0]
 *         #copy elements to buffer
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_temp = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ttygif/tty/parser.pyx":461
 *         cdef int width=self.g.state.width
 *         temp=[]
 *         cdef uint8_t[3] c=[0,0,0]             # <<<<<<<<<<<<<<
 *         #copy elements to buffer
 *         for x2 in xrange(x+distance,width):
 */
  __pyx_t_4[0] = 0;
  __pyx_t_4[1] = 0;
  __pyx_t_4[2] = 0;
  memcpy(&(__pyx_v_c[0]), __pyx_t_4, sizeof(__pyx_v_c[0]) * (3));

  /* "ttygif/tty/parser.pyx":463
 *         cdef uint8_t[3] c=[0,0,0]
 *         #copy elements to buffer
 *         for x2 in xrange(x+distance,width):             # <<<<<<<<<<<<<<
 *             self.g.screen.get_pixel_3byte(x2,y,c)
 *             self.g.screen.put_pixel_3byte(x2-distance,y,c)
 */
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = PyNumber_Add(__pyx_t_3, __pyx_v_distance); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_width); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_3);
  __pyx_t_5 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_xrange, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
    __pyx_t_6 = __pyx_t_3; __Pyx_INCREF(__pyx_t_6); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_6 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 463, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = Py_TYPE(__pyx_t_6)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 463, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_6))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_6)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyList_GET_ITEM(__pyx_t_6, __pyx_t_7); __Pyx_INCREF(__pyx_t_3); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 463, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_6, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 463, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_6)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_6, __pyx_t_7); __Pyx_INCREF(__pyx_t_3); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 463, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_6, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 463, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      }
    } else {
      __pyx_t_3 = __pyx_t_8(__pyx_t_6);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 463, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_x2, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ttygif/tty/parser.pyx":464
 *         #copy elements to buffer
 *         for x2 in xrange(x+distance,width):
 *             self.g.screen.get_pixel_3byte(x2,y,c)             # <<<<<<<<<<<<<<
 *             self.g.screen.put_pixel_3byte(x2-distance,y,c)
 * 
 */
    __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_x2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 464, __pyx_L1_error)
    ((struct __pyx_vtabstruct_6ttygif_3tty_5image_image *)__pyx_v_self->g->screen->__pyx_vtab)->get_pixel_3byte(__pyx_v_self->g->screen, __pyx_t_1, __pyx_v_y, __pyx_v_c);

    /* "ttygif/tty/parser.pyx":465
 *         for x2 in xrange(x+distance,width):
 *             self.g.screen.get_pixel_3byte(x2,y,c)
 *             self.g.screen.put_pixel_3byte(x2-distance,y,c)             # <<<<<<<<<<<<<<
 * 
 *         # clear the end of the line
 */
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_x2, __pyx_v_distance); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    ((struct __pyx_vtabstruct_6ttygif_3tty_5image_image *)__pyx_v_self->g->screen->__pyx_vtab)->put_pixel_3byte(__pyx_v_self->g->screen, __pyx_t_1, __pyx_v_y, __pyx_v_c);

    /* "ttygif/tty/parser.pyx":463
 *         cdef uint8_t[3] c=[0,0,0]
 *         #copy elements to buffer
 *         for x2 in xrange(x+distance,width):             # <<<<<<<<<<<<<<
 *             self.g.screen.get_pixel_3byte(x2,y,c)
 *             self.g.screen.put_pixel_3byte(x2-distance,y,c)
 */
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "ttygif/tty/parser.pyx":468
 * 
 *         # clear the end of the line
 *         for x2 in xrange(width-distance,width):             # <<<<<<<<<<<<<<
 *             c=[self.g.state.foreground,self.g.state.background,0]
 *             self.g.screen.put_pixel_3byte(x2,y,c)
 */
  __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_width); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_6, __pyx_v_distance); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_width); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_xrange, __pyx_t_5, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
    __pyx_t_5 = __pyx_t_6; __Pyx_INCREF(__pyx_t_5); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 468, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 468, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_5))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_5)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_7); __Pyx_INCREF(__pyx_t_6); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 468, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_5, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 468, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_7); __Pyx_INCREF(__pyx_t_6); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 468, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_5, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 468, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      }
    } else {
      __pyx_t_6 = __pyx_t_8(__pyx_t_5);
      if (unlikely(!__pyx_t_6)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 468, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_XDECREF_SET(__pyx_v_x2, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "ttygif/tty/parser.pyx":469
 *         # clear the end of the line
 *         for x2 in xrange(width-distance,width):
 *             c=[self.g.state.foreground,self.g.state.background,0]             # <<<<<<<<<<<<<<
 *             self.g.screen.put_pixel_3byte(x2,y,c)
 * 
 */
    __pyx_t_9[0] = __pyx_v_self->g->state->foreground;
    __pyx_t_9[1] = __pyx_v_self->g->state->background;
    __pyx_t_9[2] = 0;
    memcpy(&(__pyx_v_c[0]), __pyx_t_9, sizeof(__pyx_v_c[0]) * (3));

    /* "ttygif/tty/parser.pyx":470
 *         for x2 in xrange(width-distance,width):
 *             c=[self.g.state.foreground,self.g.state.background,0]
 *             self.g.screen.put_pixel_3byte(x2,y,c)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_ECH(self,distance):
 */
    __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_x2); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 470, __pyx_L1_error)
    ((struct __pyx_vtabstruct_6ttygif_3tty_5image_image *)__pyx_v_self->g->screen->__pyx_vtab)->put_pixel_3byte(__pyx_v_self->g->screen, __pyx_t_1, __pyx_v_y, __pyx_v_c);

    /* "ttygif/tty/parser.pyx":468
 * 
 *         # clear the end of the line
 *         for x2 in xrange(width-distance,width):             # <<<<<<<<<<<<<<
 *             c=[self.g.state.foreground,self.g.state.background,0]
 *             self.g.screen.put_pixel_3byte(x2,y,c)
 */
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "ttygif/tty/parser.pyx":456
 *         self.g.state.cursor_absolute(cp[0],cp[1])
 * 
 *     cdef cmd_DCH(self,distance):             # <<<<<<<<<<<<<<
 *         cdef int x=self.g.state.cursor_x
 *         cdef int y=self.g.state.cursor_y
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_DCH", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_temp);
  __Pyx_XDECREF(__pyx_v_x2);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":472
 *             self.g.screen.put_pixel_3byte(x2,y,c)
 * 
 *     cdef cmd_ECH(self,distance):             # <<<<<<<<<<<<<<
 *         cp=self.g.state.cursor_get_position()
 *         for x in xrange(self.g.state.cursor_x,self.g.state.cursor_x+distance):
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_ECH(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_distance) {
  PyObject *__pyx_v_cp = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_ECH", 0);
  __Pyx_TraceCall("cmd_ECH", __pyx_f[0], 472, 0, __PYX_ERR(0, 472, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":473
 * 
 *     cdef cmd_ECH(self,distance):
 *         cp=self.g.state.cursor_get_position()             # <<<<<<<<<<<<<<
 *         for x in xrange(self.g.state.cursor_x,self.g.state.cursor_x+distance):
 *                 self.g.state.cursor_absolute_x(x)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_get_position(__pyx_v_self->g->state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_cp = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":474
 *     cdef cmd_ECH(self,distance):
 *         cp=self.g.state.cursor_get_position()
 *         for x in xrange(self.g.state.cursor_x,self.g.state.cursor_x+distance):             # <<<<<<<<<<<<<<
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->g->state->cursor_x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->g->state->cursor_x); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_v_distance); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_xrange, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
    __pyx_t_2 = __pyx_t_3; __Pyx_INCREF(__pyx_t_2); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 474, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 474, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 474, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 474, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 474, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 474, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      }
    } else {
      __pyx_t_3 = __pyx_t_5(__pyx_t_2);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 474, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_x, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ttygif/tty/parser.pyx":475
 *         cp=self.g.state.cursor_get_position()
 *         for x in xrange(self.g.state.cursor_x,self.g.state.cursor_x+distance):
 *                 self.g.state.cursor_absolute_x(x)             # <<<<<<<<<<<<<<
 *                 self.g.write(0)
 *         self.g.state.cursor_absolute(cp[0],cp[1])
 */
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute_x(__pyx_v_self->g->state, __pyx_v_x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 475, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "ttygif/tty/parser.pyx":476
 *         for x in xrange(self.g.state.cursor_x,self.g.state.cursor_x+distance):
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute(cp[0],cp[1])
 * 
 */
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_v_self->g->__pyx_vtab)->write(__pyx_v_self->g, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "ttygif/tty/parser.pyx":474
 *     cdef cmd_ECH(self,distance):
 *         cp=self.g.state.cursor_get_position()
 *         for x in xrange(self.g.state.cursor_x,self.g.state.cursor_x+distance):             # <<<<<<<<<<<<<<
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ttygif/tty/parser.pyx":477
 *                 self.g.state.cursor_absolute_x(x)
 *                 self.g.write(0)
 *         self.g.state.cursor_absolute(cp[0],cp[1])             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_HVP(self,x,y):
 */
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_cp, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_cp, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute(__pyx_v_self->g->state, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":472
 *             self.g.screen.put_pixel_3byte(x2,y,c)
 * 
 *     cdef cmd_ECH(self,distance):             # <<<<<<<<<<<<<<
 *         cp=self.g.state.cursor_get_position()
 *         for x in xrange(self.g.state.cursor_x,self.g.state.cursor_x+distance):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_ECH", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cp);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":479
 *         self.g.state.cursor_absolute(cp[0],cp[1])
 * 
 *     cdef cmd_HVP(self,x,y):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute(x,y)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_HVP(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_HVP", 0);
  __Pyx_TraceCall("cmd_HVP", __pyx_f[0], 479, 0, __PYX_ERR(0, 479, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":480
 * 
 *     cdef cmd_HVP(self,x,y):
 *         self.g.state.cursor_absolute(x,y)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_HPA(self,x):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute(__pyx_v_self->g->state, __pyx_v_x, __pyx_v_y); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":479
 *         self.g.state.cursor_absolute(cp[0],cp[1])
 * 
 *     cdef cmd_HVP(self,x,y):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute(x,y)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_HVP", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":482
 *         self.g.state.cursor_absolute(x,y)
 * 
 *     cdef cmd_HPA(self,x):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute_x(x)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_HPA(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_x) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_HPA", 0);
  __Pyx_TraceCall("cmd_HPA", __pyx_f[0], 482, 0, __PYX_ERR(0, 482, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":483
 * 
 *     cdef cmd_HPA(self,x):
 *         self.g.state.cursor_absolute_x(x)             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_SCP(self):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute_x(__pyx_v_self->g->state, __pyx_v_x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":482
 *         self.g.state.cursor_absolute(x,y)
 * 
 *     cdef cmd_HPA(self,x):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute_x(x)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_HPA", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":485
 *         self.g.state.cursor_absolute_x(x)
 * 
 *     cdef cmd_SCP(self):             # <<<<<<<<<<<<<<
 *         self.g.cursor_save_position()
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_SCP(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_SCP", 0);
  __Pyx_TraceCall("cmd_SCP", __pyx_f[0], 485, 0, __PYX_ERR(0, 485, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":486
 * 
 *     cdef cmd_SCP(self):
 *         self.g.cursor_save_position()             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_RCP(self):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->g), __pyx_n_s_cursor_save_position); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":485
 *         self.g.state.cursor_absolute_x(x)
 * 
 *     cdef cmd_SCP(self):             # <<<<<<<<<<<<<<
 *         self.g.cursor_save_position()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_SCP", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":488
 *         self.g.cursor_save_position()
 * 
 *     cdef cmd_RCP(self):             # <<<<<<<<<<<<<<
 *         self.g.cursor_restore_position()
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_RCP(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_RCP", 0);
  __Pyx_TraceCall("cmd_RCP", __pyx_f[0], 488, 0, __PYX_ERR(0, 488, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":489
 * 
 *     cdef cmd_RCP(self):
 *         self.g.cursor_restore_position()             # <<<<<<<<<<<<<<
 * 
 *     cdef cmd_VPA(self,position):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->g), __pyx_n_s_cursor_restore_position); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":488
 *         self.g.cursor_save_position()
 * 
 *     cdef cmd_RCP(self):             # <<<<<<<<<<<<<<
 *         self.g.cursor_restore_position()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_RCP", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":491
 *         self.g.cursor_restore_position()
 * 
 *     cdef cmd_VPA(self,position):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute(0,position)
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_VPA(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_position) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cmd_VPA", 0);
  __Pyx_TraceCall("cmd_VPA", __pyx_f[0], 491, 0, __PYX_ERR(0, 491, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":492
 * 
 *     cdef cmd_VPA(self,position):
 *         self.g.state.cursor_absolute(0,position)             # <<<<<<<<<<<<<<
 * 
 *     cdef stream_2_sequence(self,text,timestamp,delay):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state *)__pyx_v_self->g->state->__pyx_vtab)->cursor_absolute(__pyx_v_self->g->state, __pyx_int_0, __pyx_v_position); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":491
 *         self.g.cursor_restore_position()
 * 
 *     cdef cmd_VPA(self,position):             # <<<<<<<<<<<<<<
 *         self.g.state.cursor_absolute(0,position)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.cmd_VPA", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "ttygif/tty/parser.pyx":547
 *                 command=groups[10]
 *                 if command in 'Hf':
 *                     params = tuple(int(p) if len(p) != 0 else 1 for p in paramstring.split(';'))             # <<<<<<<<<<<<<<
 *                     while len(params) < 2:
 *                         params = params + (1,)
 */

static PyObject *__pyx_pf_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr *)__pyx_tp_new_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr(__pyx_ptype_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 547, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *) __pyx_self;
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_outer_scope));
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_outer_scope);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_2generator, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_term_parser_stream_2_sequence_lo, __pyx_n_s_ttygif_tty_parser); if (unlikely(!gen)) __PYX_ERR(0, 547, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.stream_2_sequence.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr *__pyx_cur_scope = ((struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  Py_ssize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  __Pyx_TraceCall("genexpr", __pyx_f[0], 547, 0, __PYX_ERR(0, 547, __pyx_L1_error));
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_TraceReturn(Py_None, 0);
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 547, __pyx_L1_error)
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_paramstring)) { __Pyx_RaiseClosureNameError("paramstring"); __PYX_ERR(0, 547, __pyx_L1_error) }
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_paramstring, __pyx_n_s_split); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 547, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_s__5) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_s__5);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 547, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 547, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 547, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 547, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 547, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 547, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 547, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_5(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 547, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_p);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_p, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_6 = PyObject_Length(__pyx_cur_scope->__pyx_v_p); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 547, __pyx_L1_error)
    if (((__pyx_t_6 != 0) != 0)) {
      __pyx_t_3 = __Pyx_PyNumber_Int(__pyx_cur_scope->__pyx_v_p); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = __pyx_t_3;
      __pyx_t_3 = 0;
    } else {
      __Pyx_INCREF(__pyx_int_1);
      __pyx_t_1 = __pyx_int_1;
    }
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    __Pyx_XGIVEREF(__pyx_t_2);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_4;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_5;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_TraceReturn(__pyx_r, 0);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_2);
    __pyx_t_4 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_5 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 547, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_5generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "ttygif/tty/parser.pyx":557
 *                 else:
 *                     try:
 *                         params = tuple(int(p) for p in paramstring.split(';') if len(p) != 0)             # <<<<<<<<<<<<<<
 *                         if len(params) == 0:
 *                             if command in 'JKm':
 */

static PyObject *__pyx_pf_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_3genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr *)__pyx_tp_new_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr(__pyx_ptype_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 557, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *) __pyx_self;
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_outer_scope));
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_outer_scope);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_5generator1, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_term_parser_stream_2_sequence_lo, __pyx_n_s_ttygif_tty_parser); if (unlikely(!gen)) __PYX_ERR(0, 557, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.stream_2_sequence.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_5generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr *__pyx_cur_scope = ((struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  __Pyx_TraceCall("genexpr", __pyx_f[0], 557, 0, __PYX_ERR(0, 557, __pyx_L1_error));
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L7_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_TraceReturn(Py_None, 0);
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 557, __pyx_L1_error)
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_paramstring)) { __Pyx_RaiseClosureNameError("paramstring"); __PYX_ERR(0, 557, __pyx_L1_error) }
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_paramstring, __pyx_n_s_split); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_s__5) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_s__5);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 557, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 557, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 557, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 557, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 557, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 557, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_5(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 557, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_p);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_p, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_6 = PyObject_Length(__pyx_cur_scope->__pyx_v_p); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 557, __pyx_L1_error)
    __pyx_t_7 = ((__pyx_t_6 != 0) != 0);
    if (__pyx_t_7) {
      __pyx_t_1 = __Pyx_PyNumber_Int(__pyx_cur_scope->__pyx_v_p); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_r = __pyx_t_1;
      __pyx_t_1 = 0;
      __Pyx_XGIVEREF(__pyx_t_2);
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_4;
      __pyx_cur_scope->__pyx_t_2 = __pyx_t_5;
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_TraceReturn(__pyx_r, 0);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 1;
      return __pyx_r;
      __pyx_L7_resume_from_yield:;
      __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
      __pyx_cur_scope->__pyx_t_0 = 0;
      __Pyx_XGOTREF(__pyx_t_2);
      __pyx_t_4 = __pyx_cur_scope->__pyx_t_1;
      __pyx_t_5 = __pyx_cur_scope->__pyx_t_2;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 557, __pyx_L1_error)
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":494
 *         self.g.state.cursor_absolute(0,position)
 * 
 *     cdef stream_2_sequence(self,text,timestamp,delay):             # <<<<<<<<<<<<<<
 *         # patterns for filtering out commands from the stream
 *         ANSI_SINGLE   ='[\033]([cDEHMZ78>=])'
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_stream_2_sequence(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_text, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_delay) {
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *__pyx_cur_scope;
  PyObject *__pyx_v_ANSI_SINGLE = NULL;
  PyObject *__pyx_v_ANSI_CHAR_SET = NULL;
  PyObject *__pyx_v_ANSI_G0 = NULL;
  PyObject *__pyx_v_ANSI_G1 = NULL;
  PyObject *__pyx_v_ANSI_CSI_RE = NULL;
  PyObject *__pyx_v_BRACKET_PASTE = NULL;
  PyObject *__pyx_v_ANSI_OSC = NULL;
  PyObject *__pyx_v_ESC_SEQUENCES = NULL;
  PyObject *__pyx_v_ANSI_REGEX = NULL;
  PyObject *__pyx_v_ANSI = NULL;
  PyObject *__pyx_v_cursor = NULL;
  PyObject *__pyx_v_match = NULL;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_v_start = NULL;
  PyObject *__pyx_v_end = NULL;
  PyObject *__pyx_v_command = NULL;
  PyObject *__pyx_v_params = NULL;
  PyObject *__pyx_v_esc_type = NULL;
  PyObject *__pyx_v_groups = NULL;
  PyObject *__pyx_v_param_tokens = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_ex = NULL;
  PyObject *__pyx_gb_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_2generator = 0;
  PyObject *__pyx_v_part = NULL;
  PyObject *__pyx_gb_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_5generator1 = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  struct __pyx_opt_args_6ttygif_3tty_6parser_11term_parser_add_command_sequence __pyx_t_10;
  int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  PyObject *(*__pyx_t_13)(PyObject *);
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("stream_2_sequence", 0);
  __pyx_cur_scope = (struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *)__pyx_tp_new_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence(__pyx_ptype_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 494, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __Pyx_TraceCall("stream_2_sequence", __pyx_f[0], 494, 0, __PYX_ERR(0, 494, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":496
 *     cdef stream_2_sequence(self,text,timestamp,delay):
 *         # patterns for filtering out commands from the stream
 *         ANSI_SINGLE   ='[\033]([cDEHMZ78>=])'             # <<<<<<<<<<<<<<
 *         ANSI_CHAR_SET = '[\033]\\%([@G*])'
 *         ANSI_G0       = '[\033]\\(([B0UK])'
 */
  __Pyx_INCREF(__pyx_kp_s_cDEHMZ78);
  __pyx_v_ANSI_SINGLE = __pyx_kp_s_cDEHMZ78;

  /* "ttygif/tty/parser.pyx":497
 *         # patterns for filtering out commands from the stream
 *         ANSI_SINGLE   ='[\033]([cDEHMZ78>=])'
 *         ANSI_CHAR_SET = '[\033]\\%([@G*])'             # <<<<<<<<<<<<<<
 *         ANSI_G0       = '[\033]\\(([B0UK])'
 *         ANSI_G1       = '[\033]\\)([B0UK])'
 */
  __Pyx_INCREF(__pyx_kp_s_G_2);
  __pyx_v_ANSI_CHAR_SET = __pyx_kp_s_G_2;

  /* "ttygif/tty/parser.pyx":498
 *         ANSI_SINGLE   ='[\033]([cDEHMZ78>=])'
 *         ANSI_CHAR_SET = '[\033]\\%([@G*])'
 *         ANSI_G0       = '[\033]\\(([B0UK])'             # <<<<<<<<<<<<<<
 *         ANSI_G1       = '[\033]\\)([B0UK])'
 *         ANSI_CSI_RE   = '[\033]\\[((?:\\d|;|<|>|=|\?)*)([a-zA-Z])\002?'
 */
  __Pyx_INCREF(__pyx_kp_s_B0UK);
  __pyx_v_ANSI_G0 = __pyx_kp_s_B0UK;

  /* "ttygif/tty/parser.pyx":499
 *         ANSI_CHAR_SET = '[\033]\\%([@G*])'
 *         ANSI_G0       = '[\033]\\(([B0UK])'
 *         ANSI_G1       = '[\033]\\)([B0UK])'             # <<<<<<<<<<<<<<
 *         ANSI_CSI_RE   = '[\033]\\[((?:\\d|;|<|>|=|\?)*)([a-zA-Z])\002?'
 *         BRACKET_PASTE = '[\033]\\[(20[0-1]~)'
 */
  __Pyx_INCREF(__pyx_kp_s_B0UK_2);
  __pyx_v_ANSI_G1 = __pyx_kp_s_B0UK_2;

  /* "ttygif/tty/parser.pyx":500
 *         ANSI_G0       = '[\033]\\(([B0UK])'
 *         ANSI_G1       = '[\033]\\)([B0UK])'
 *         ANSI_CSI_RE   = '[\033]\\[((?:\\d|;|<|>|=|\?)*)([a-zA-Z])\002?'             # <<<<<<<<<<<<<<
 *         BRACKET_PASTE = '[\033]\\[(20[0-1]~)'
 *         # guessed on this one
 */
  __Pyx_INCREF(__pyx_kp_s_d_a_zA_Z);
  __pyx_v_ANSI_CSI_RE = __pyx_kp_s_d_a_zA_Z;

  /* "ttygif/tty/parser.pyx":501
 *         ANSI_G1       = '[\033]\\)([B0UK])'
 *         ANSI_CSI_RE   = '[\033]\\[((?:\\d|;|<|>|=|\?)*)([a-zA-Z])\002?'
 *         BRACKET_PASTE = '[\033]\\[(20[0-1]~)'             # <<<<<<<<<<<<<<
 *         # guessed on this one
 *         #ANSI_OSC_777_REGEX='[\0x1b|\033]\]777[;]([._:A-Za-z0-9\-\s]*)[;]([._:A-Za-z0-9\-\s]*)[;]([._:A-Za-z0-9\-\s]*)'
 */
  __Pyx_INCREF(__pyx_kp_s_20_0_1);
  __pyx_v_BRACKET_PASTE = __pyx_kp_s_20_0_1;

  /* "ttygif/tty/parser.pyx":504
 *         # guessed on this one
 *         #ANSI_OSC_777_REGEX='[\0x1b|\033]\]777[;]([._:A-Za-z0-9\-\s]*)[;]([._:A-Za-z0-9\-\s]*)[;]([._:A-Za-z0-9\-\s]*)'
 *         ANSI_OSC ='(?:\033\\]|\x9d).*?(?:\033\\\\|[\a\x9c])'             # <<<<<<<<<<<<<<
 *         ESC_SEQUENCES=[ANSI_SINGLE,ANSI_CHAR_SET,ANSI_G0,ANSI_G1,ANSI_CSI_RE,ANSI_OSC,BRACKET_PASTE]
 *         ANSI_REGEX="("+")|(".join(ESC_SEQUENCES)+")"
 */
  __Pyx_INCREF(__pyx_kp_s__7);
  __pyx_v_ANSI_OSC = __pyx_kp_s__7;

  /* "ttygif/tty/parser.pyx":505
 *         #ANSI_OSC_777_REGEX='[\0x1b|\033]\]777[;]([._:A-Za-z0-9\-\s]*)[;]([._:A-Za-z0-9\-\s]*)[;]([._:A-Za-z0-9\-\s]*)'
 *         ANSI_OSC ='(?:\033\\]|\x9d).*?(?:\033\\\\|[\a\x9c])'
 *         ESC_SEQUENCES=[ANSI_SINGLE,ANSI_CHAR_SET,ANSI_G0,ANSI_G1,ANSI_CSI_RE,ANSI_OSC,BRACKET_PASTE]             # <<<<<<<<<<<<<<
 *         ANSI_REGEX="("+")|(".join(ESC_SEQUENCES)+")"
 *         ANSI=re.compile(ANSI_REGEX)
 */
  __pyx_t_1 = PyList_New(7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_ANSI_SINGLE);
  __Pyx_GIVEREF(__pyx_v_ANSI_SINGLE);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_v_ANSI_SINGLE);
  __Pyx_INCREF(__pyx_v_ANSI_CHAR_SET);
  __Pyx_GIVEREF(__pyx_v_ANSI_CHAR_SET);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_v_ANSI_CHAR_SET);
  __Pyx_INCREF(__pyx_v_ANSI_G0);
  __Pyx_GIVEREF(__pyx_v_ANSI_G0);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_v_ANSI_G0);
  __Pyx_INCREF(__pyx_v_ANSI_G1);
  __Pyx_GIVEREF(__pyx_v_ANSI_G1);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_v_ANSI_G1);
  __Pyx_INCREF(__pyx_v_ANSI_CSI_RE);
  __Pyx_GIVEREF(__pyx_v_ANSI_CSI_RE);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_v_ANSI_CSI_RE);
  __Pyx_INCREF(__pyx_v_ANSI_OSC);
  __Pyx_GIVEREF(__pyx_v_ANSI_OSC);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_v_ANSI_OSC);
  __Pyx_INCREF(__pyx_v_BRACKET_PASTE);
  __Pyx_GIVEREF(__pyx_v_BRACKET_PASTE);
  PyList_SET_ITEM(__pyx_t_1, 6, __pyx_v_BRACKET_PASTE);
  __pyx_v_ESC_SEQUENCES = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":506
 *         ANSI_OSC ='(?:\033\\]|\x9d).*?(?:\033\\\\|[\a\x9c])'
 *         ESC_SEQUENCES=[ANSI_SINGLE,ANSI_CHAR_SET,ANSI_G0,ANSI_G1,ANSI_CSI_RE,ANSI_OSC,BRACKET_PASTE]
 *         ANSI_REGEX="("+")|(".join(ESC_SEQUENCES)+")"             # <<<<<<<<<<<<<<
 *         ANSI=re.compile(ANSI_REGEX)
 *         cursor=0
 */
  __pyx_t_1 = __Pyx_PyString_Join(__pyx_kp_s__9, __pyx_v_ESC_SEQUENCES); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Add(__pyx_kp_s__8, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_t_2, __pyx_kp_s__10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_ANSI_REGEX = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":507
 *         ESC_SEQUENCES=[ANSI_SINGLE,ANSI_CHAR_SET,ANSI_G0,ANSI_G1,ANSI_CSI_RE,ANSI_OSC,BRACKET_PASTE]
 *         ANSI_REGEX="("+")|(".join(ESC_SEQUENCES)+")"
 *         ANSI=re.compile(ANSI_REGEX)             # <<<<<<<<<<<<<<
 *         cursor=0
 *         for match in ANSI.finditer(text):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_re); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_compile); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_v_ANSI_REGEX) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_ANSI_REGEX);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_ANSI = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":508
 *         ANSI_REGEX="("+")|(".join(ESC_SEQUENCES)+")"
 *         ANSI=re.compile(ANSI_REGEX)
 *         cursor=0             # <<<<<<<<<<<<<<
 *         for match in ANSI.finditer(text):
 *             name=""
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_cursor = __pyx_int_0;

  /* "ttygif/tty/parser.pyx":509
 *         ANSI=re.compile(ANSI_REGEX)
 *         cursor=0
 *         for match in ANSI.finditer(text):             # <<<<<<<<<<<<<<
 *             name=""
 *             start, end = match.span()
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_ANSI, __pyx_n_s_finditer); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_v_text) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_text);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_3 = __pyx_t_1; __Pyx_INCREF(__pyx_t_3); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 509, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 509, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 509, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 509, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 509, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 509, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_5(__pyx_t_3);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 509, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_match, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":510
 *         cursor=0
 *         for match in ANSI.finditer(text):
 *             name=""             # <<<<<<<<<<<<<<
 *             start, end = match.span()
 *             self.add_text_sequence(text[cursor:start],timestamp,delay)
 */
    __Pyx_INCREF(__pyx_kp_s_);
    __Pyx_XDECREF_SET(__pyx_v_name, __pyx_kp_s_);

    /* "ttygif/tty/parser.pyx":511
 *         for match in ANSI.finditer(text):
 *             name=""
 *             start, end = match.span()             # <<<<<<<<<<<<<<
 *             self.add_text_sequence(text[cursor:start],timestamp,delay)
 *             cursor = end
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_match, __pyx_n_s_span); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 511, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 511, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 511, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 511, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 511, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 511, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_2 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_2)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      index = 1; __pyx_t_6 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 2) < 0) __PYX_ERR(0, 511, __pyx_L1_error)
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 511, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_start, __pyx_t_2);
    __pyx_t_2 = 0;
    __Pyx_XDECREF_SET(__pyx_v_end, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "ttygif/tty/parser.pyx":512
 *             name=""
 *             start, end = match.span()
 *             self.add_text_sequence(text[cursor:start],timestamp,delay)             # <<<<<<<<<<<<<<
 *             cursor = end
 *             command=None
 */
    __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_text, 0, 0, &__pyx_v_cursor, &__pyx_v_start, NULL, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 512, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->add_text_sequence(__pyx_v_self, __pyx_t_1, __pyx_v_timestamp, __pyx_v_delay); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 512, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "ttygif/tty/parser.pyx":513
 *             start, end = match.span()
 *             self.add_text_sequence(text[cursor:start],timestamp,delay)
 *             cursor = end             # <<<<<<<<<<<<<<
 *             command=None
 *             params=None
 */
    __Pyx_INCREF(__pyx_v_end);
    __Pyx_DECREF_SET(__pyx_v_cursor, __pyx_v_end);

    /* "ttygif/tty/parser.pyx":514
 *             self.add_text_sequence(text[cursor:start],timestamp,delay)
 *             cursor = end
 *             command=None             # <<<<<<<<<<<<<<
 *             params=None
 *             esc_type=None
 */
    __Pyx_INCREF(Py_None);
    __Pyx_XDECREF_SET(__pyx_v_command, Py_None);

    /* "ttygif/tty/parser.pyx":515
 *             cursor = end
 *             command=None
 *             params=None             # <<<<<<<<<<<<<<
 *             esc_type=None
 *             groups=match.groups()
 */
    __Pyx_INCREF(Py_None);
    __Pyx_XDECREF_SET(__pyx_v_params, Py_None);

    /* "ttygif/tty/parser.pyx":516
 *             command=None
 *             params=None
 *             esc_type=None             # <<<<<<<<<<<<<<
 *             groups=match.groups()
 *             #print groups
 */
    __Pyx_INCREF(Py_None);
    __Pyx_XDECREF_SET(__pyx_v_esc_type, Py_None);

    /* "ttygif/tty/parser.pyx":517
 *             params=None
 *             esc_type=None
 *             groups=match.groups()             # <<<<<<<<<<<<<<
 *             #print groups
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_match, __pyx_n_s_groups); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 517, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_6 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 517, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF_SET(__pyx_v_groups, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "ttygif/tty/parser.pyx":520
 *             #print groups
 * 
 *             if groups[0]:             # <<<<<<<<<<<<<<
 *                 esc_type='SINGLE'
 *                 command=groups[1]
 */
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_groups, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 520, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 520, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_9) {

      /* "ttygif/tty/parser.pyx":521
 * 
 *             if groups[0]:
 *                 esc_type='SINGLE'             # <<<<<<<<<<<<<<
 *                 command=groups[1]
 *             elif groups[2]:
 */
      __Pyx_INCREF(__pyx_n_s_SINGLE);
      __Pyx_DECREF_SET(__pyx_v_esc_type, __pyx_n_s_SINGLE);

      /* "ttygif/tty/parser.pyx":522
 *             if groups[0]:
 *                 esc_type='SINGLE'
 *                 command=groups[1]             # <<<<<<<<<<<<<<
 *             elif groups[2]:
 *                 esc_type='CHAR_SET'
 */
      __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_groups, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 522, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF_SET(__pyx_v_command, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "ttygif/tty/parser.pyx":520
 *             #print groups
 * 
 *             if groups[0]:             # <<<<<<<<<<<<<<
 *                 esc_type='SINGLE'
 *                 command=groups[1]
 */
      goto __pyx_L7;
    }

    /* "ttygif/tty/parser.pyx":523
 *                 esc_type='SINGLE'
 *                 command=groups[1]
 *             elif groups[2]:             # <<<<<<<<<<<<<<
 *                 esc_type='CHAR_SET'
 *                 command=groups[3]
 */
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_groups, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 523, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 523, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_9) {

      /* "ttygif/tty/parser.pyx":524
 *                 command=groups[1]
 *             elif groups[2]:
 *                 esc_type='CHAR_SET'             # <<<<<<<<<<<<<<
 *                 command=groups[3]
 *             elif groups[4]:
 */
      __Pyx_INCREF(__pyx_n_s_CHAR_SET);
      __Pyx_DECREF_SET(__pyx_v_esc_type, __pyx_n_s_CHAR_SET);

      /* "ttygif/tty/parser.pyx":525
 *             elif groups[2]:
 *                 esc_type='CHAR_SET'
 *                 command=groups[3]             # <<<<<<<<<<<<<<
 *             elif groups[4]:
 *                 esc_type='G0'
 */
      __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_groups, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 525, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF_SET(__pyx_v_command, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "ttygif/tty/parser.pyx":523
 *                 esc_type='SINGLE'
 *                 command=groups[1]
 *             elif groups[2]:             # <<<<<<<<<<<<<<
 *                 esc_type='CHAR_SET'
 *                 command=groups[3]
 */
      goto __pyx_L7;
    }

    /* "ttygif/tty/parser.pyx":526
 *                 esc_type='CHAR_SET'
 *                 command=groups[3]
 *             elif groups[4]:             # <<<<<<<<<<<<<<
 *                 esc_type='G0'
 *                 command=groups[5]
 */
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_groups, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 526, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 526, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_9) {

      /* "ttygif/tty/parser.pyx":527
 *                 command=groups[3]
 *             elif groups[4]:
 *                 esc_type='G0'             # <<<<<<<<<<<<<<
 *                 command=groups[5]
 *             elif groups[6]:
 */
      __Pyx_INCREF(__pyx_n_s_G0);
      __Pyx_DECREF_SET(__pyx_v_esc_type, __pyx_n_s_G0);

      /* "ttygif/tty/parser.pyx":528
 *             elif groups[4]:
 *                 esc_type='G0'
 *                 command=groups[5]             # <<<<<<<<<<<<<<
 *             elif groups[6]:
 *                 esc_type='G1'
 */
      __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_groups, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 528, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF_SET(__pyx_v_command, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "ttygif/tty/parser.pyx":526
 *                 esc_type='CHAR_SET'
 *                 command=groups[3]
 *             elif groups[4]:             # <<<<<<<<<<<<<<
 *                 esc_type='G0'
 *                 command=groups[5]
 */
      goto __pyx_L7;
    }

    /* "ttygif/tty/parser.pyx":529
 *                 esc_type='G0'
 *                 command=groups[5]
 *             elif groups[6]:             # <<<<<<<<<<<<<<
 *                 esc_type='G1'
 *                 command=groups[7]
 */
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_groups, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 529, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 529, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_9) {

      /* "ttygif/tty/parser.pyx":530
 *                 command=groups[5]
 *             elif groups[6]:
 *                 esc_type='G1'             # <<<<<<<<<<<<<<
 *                 command=groups[7]
 *             elif groups[11]:
 */
      __Pyx_INCREF(__pyx_n_s_G1);
      __Pyx_DECREF_SET(__pyx_v_esc_type, __pyx_n_s_G1);

      /* "ttygif/tty/parser.pyx":531
 *             elif groups[6]:
 *                 esc_type='G1'
 *                 command=groups[7]             # <<<<<<<<<<<<<<
 *             elif groups[11]:
 *                 esc_type='OSC'
 */
      __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_groups, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 531, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF_SET(__pyx_v_command, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "ttygif/tty/parser.pyx":529
 *                 esc_type='G0'
 *                 command=groups[5]
 *             elif groups[6]:             # <<<<<<<<<<<<<<
 *                 esc_type='G1'
 *                 command=groups[7]
 */
      goto __pyx_L7;
    }

    /* "ttygif/tty/parser.pyx":532
 *                 esc_type='G1'
 *                 command=groups[7]
 *             elif groups[11]:             # <<<<<<<<<<<<<<
 *                 esc_type='OSC'
 *                 command=groups[11]
 */
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_groups, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 532, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 532, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_9) {

      /* "ttygif/tty/parser.pyx":533
 *                 command=groups[7]
 *             elif groups[11]:
 *                 esc_type='OSC'             # <<<<<<<<<<<<<<
 *                 command=groups[11]
 *                 params=[groups[11]]
 */
      __Pyx_INCREF(__pyx_n_s_OSC);
      __Pyx_DECREF_SET(__pyx_v_esc_type, __pyx_n_s_OSC);

      /* "ttygif/tty/parser.pyx":534
 *             elif groups[11]:
 *                 esc_type='OSC'
 *                 command=groups[11]             # <<<<<<<<<<<<<<
 *                 params=[groups[11]]
 *             elif groups[12]:
 */
      __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_groups, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 534, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF_SET(__pyx_v_command, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "ttygif/tty/parser.pyx":535
 *                 esc_type='OSC'
 *                 command=groups[11]
 *                 params=[groups[11]]             # <<<<<<<<<<<<<<
 *             elif groups[12]:
 *                 esc_type='CSI'
 */
      __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_groups, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 535, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_6);
      PyList_SET_ITEM(__pyx_t_1, 0, __pyx_t_6);
      __pyx_t_6 = 0;
      __Pyx_DECREF_SET(__pyx_v_params, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "ttygif/tty/parser.pyx":532
 *                 esc_type='G1'
 *                 command=groups[7]
 *             elif groups[11]:             # <<<<<<<<<<<<<<
 *                 esc_type='OSC'
 *                 command=groups[11]
 */
      goto __pyx_L7;
    }

    /* "ttygif/tty/parser.pyx":536
 *                 command=groups[11]
 *                 params=[groups[11]]
 *             elif groups[12]:             # <<<<<<<<<<<<<<
 *                 esc_type='CSI'
 *                 command=groups[12][-1]
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_groups, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 536, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_9) {

      /* "ttygif/tty/parser.pyx":537
 *                 params=[groups[11]]
 *             elif groups[12]:
 *                 esc_type='CSI'             # <<<<<<<<<<<<<<
 *                 command=groups[12][-1]
 *                 params=[int(groups[13][0:-1])]
 */
      __Pyx_INCREF(__pyx_n_s_CSI);
      __Pyx_DECREF_SET(__pyx_v_esc_type, __pyx_n_s_CSI);

      /* "ttygif/tty/parser.pyx":538
 *             elif groups[12]:
 *                 esc_type='CSI'
 *                 command=groups[12][-1]             # <<<<<<<<<<<<<<
 *                 params=[int(groups[13][0:-1])]
 *                 name="bracketed paste"
 */
      __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_groups, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 538, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = __Pyx_GetItemInt(__pyx_t_1, -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 538, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF_SET(__pyx_v_command, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "ttygif/tty/parser.pyx":539
 *                 esc_type='CSI'
 *                 command=groups[12][-1]
 *                 params=[int(groups[13][0:-1])]             # <<<<<<<<<<<<<<
 *                 name="bracketed paste"
 *                 self.add_command_sequence(esc_type,command,params,groups,name,timestamp,delay,text)
 */
      __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_groups, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 539, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_t_6, 0, -1L, NULL, NULL, &__pyx_slice__11, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyNumber_Int(__pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 539, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_6);
      PyList_SET_ITEM(__pyx_t_1, 0, __pyx_t_6);
      __pyx_t_6 = 0;
      __Pyx_DECREF_SET(__pyx_v_params, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "ttygif/tty/parser.pyx":540
 *                 command=groups[12][-1]
 *                 params=[int(groups[13][0:-1])]
 *                 name="bracketed paste"             # <<<<<<<<<<<<<<
 *                 self.add_command_sequence(esc_type,command,params,groups,name,timestamp,delay,text)
 *             elif groups[8]:
 */
      __Pyx_INCREF(__pyx_kp_s_bracketed_paste);
      __Pyx_DECREF_SET(__pyx_v_name, __pyx_kp_s_bracketed_paste);

      /* "ttygif/tty/parser.pyx":541
 *                 params=[int(groups[13][0:-1])]
 *                 name="bracketed paste"
 *                 self.add_command_sequence(esc_type,command,params,groups,name,timestamp,delay,text)             # <<<<<<<<<<<<<<
 *             elif groups[8]:
 *                 esc_type='CSI'
 */
      __pyx_t_10.__pyx_n = 1;
      __pyx_t_10.text = __pyx_v_text;
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->add_command_sequence(__pyx_v_self, __pyx_v_esc_type, __pyx_v_command, __pyx_v_params, __pyx_v_groups, __pyx_v_name, __pyx_v_timestamp, __pyx_v_delay, &__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ttygif/tty/parser.pyx":536
 *                 command=groups[11]
 *                 params=[groups[11]]
 *             elif groups[12]:             # <<<<<<<<<<<<<<
 *                 esc_type='CSI'
 *                 command=groups[12][-1]
 */
      goto __pyx_L7;
    }

    /* "ttygif/tty/parser.pyx":542
 *                 name="bracketed paste"
 *                 self.add_command_sequence(esc_type,command,params,groups,name,timestamp,delay,text)
 *             elif groups[8]:             # <<<<<<<<<<<<<<
 *                 esc_type='CSI'
 *                 paramstring=groups[9]
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_groups, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 542, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 542, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_9) {

      /* "ttygif/tty/parser.pyx":543
 *                 self.add_command_sequence(esc_type,command,params,groups,name,timestamp,delay,text)
 *             elif groups[8]:
 *                 esc_type='CSI'             # <<<<<<<<<<<<<<
 *                 paramstring=groups[9]
 *                 command=groups[10]
 */
      __Pyx_INCREF(__pyx_n_s_CSI);
      __Pyx_DECREF_SET(__pyx_v_esc_type, __pyx_n_s_CSI);

      /* "ttygif/tty/parser.pyx":544
 *             elif groups[8]:
 *                 esc_type='CSI'
 *                 paramstring=groups[9]             # <<<<<<<<<<<<<<
 *                 command=groups[10]
 *                 if command in 'Hf':
 */
      __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_groups, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 544, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_paramstring);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_paramstring, __pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_1);
      __pyx_t_1 = 0;

      /* "ttygif/tty/parser.pyx":545
 *                 esc_type='CSI'
 *                 paramstring=groups[9]
 *                 command=groups[10]             # <<<<<<<<<<<<<<
 *                 if command in 'Hf':
 *                     params = tuple(int(p) if len(p) != 0 else 1 for p in paramstring.split(';'))
 */
      __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_groups, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_command, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "ttygif/tty/parser.pyx":546
 *                 paramstring=groups[9]
 *                 command=groups[10]
 *                 if command in 'Hf':             # <<<<<<<<<<<<<<
 *                     params = tuple(int(p) if len(p) != 0 else 1 for p in paramstring.split(';'))
 *                     while len(params) < 2:
 */
      __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_command, __pyx_n_s_Hf, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 546, __pyx_L1_error)
      __pyx_t_11 = (__pyx_t_9 != 0);
      if (__pyx_t_11) {

        /* "ttygif/tty/parser.pyx":547
 *                 command=groups[10]
 *                 if command in 'Hf':
 *                     params = tuple(int(p) if len(p) != 0 else 1 for p in paramstring.split(';'))             # <<<<<<<<<<<<<<
 *                     while len(params) < 2:
 *                         params = params + (1,)
 */
        __pyx_t_1 = __pyx_pf_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 547, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_6 = __Pyx_PySequence_Tuple(__pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 547, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF_SET(__pyx_v_params, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "ttygif/tty/parser.pyx":548
 *                 if command in 'Hf':
 *                     params = tuple(int(p) if len(p) != 0 else 1 for p in paramstring.split(';'))
 *                     while len(params) < 2:             # <<<<<<<<<<<<<<
 *                         params = params + (1,)
 *                 #        DEC Private Mode (DECSET/DECRST) sequences
 */
        while (1) {
          __pyx_t_12 = PyObject_Length(__pyx_v_params); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 548, __pyx_L1_error)
          __pyx_t_11 = ((__pyx_t_12 < 2) != 0);
          if (!__pyx_t_11) break;

          /* "ttygif/tty/parser.pyx":549
 *                     params = tuple(int(p) if len(p) != 0 else 1 for p in paramstring.split(';'))
 *                     while len(params) < 2:
 *                         params = params + (1,)             # <<<<<<<<<<<<<<
 *                 #        DEC Private Mode (DECSET/DECRST) sequences
 *                 elif paramstring and len(paramstring)>0 and paramstring[0]=='?':
 */
          __pyx_t_6 = PyNumber_Add(__pyx_v_params, __pyx_tuple__12); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 549, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF_SET(__pyx_v_params, __pyx_t_6);
          __pyx_t_6 = 0;
        }

        /* "ttygif/tty/parser.pyx":546
 *                 paramstring=groups[9]
 *                 command=groups[10]
 *                 if command in 'Hf':             # <<<<<<<<<<<<<<
 *                     params = tuple(int(p) if len(p) != 0 else 1 for p in paramstring.split(';'))
 *                     while len(params) < 2:
 */
        goto __pyx_L8;
      }

      /* "ttygif/tty/parser.pyx":551
 *                         params = params + (1,)
 *                 #        DEC Private Mode (DECSET/DECRST) sequences
 *                 elif paramstring and len(paramstring)>0 and paramstring[0]=='?':             # <<<<<<<<<<<<<<
 *                     command='?'+command
 *                     param_tokens=paramstring[1:].split(';')
 */
      __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_paramstring); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 551, __pyx_L1_error)
      if (__pyx_t_9) {
      } else {
        __pyx_t_11 = __pyx_t_9;
        goto __pyx_L11_bool_binop_done;
      }
      __pyx_t_6 = __pyx_cur_scope->__pyx_v_paramstring;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_12 = PyObject_Length(__pyx_t_6); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_9 = ((__pyx_t_12 > 0) != 0);
      if (__pyx_t_9) {
      } else {
        __pyx_t_11 = __pyx_t_9;
        goto __pyx_L11_bool_binop_done;
      }
      __pyx_t_6 = __Pyx_GetItemInt(__pyx_cur_scope->__pyx_v_paramstring, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = (__Pyx_PyString_Equals(__pyx_t_6, __pyx_kp_s__13, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_11 = __pyx_t_9;
      __pyx_L11_bool_binop_done:;
      if (__pyx_t_11) {

        /* "ttygif/tty/parser.pyx":552
 *                 #        DEC Private Mode (DECSET/DECRST) sequences
 *                 elif paramstring and len(paramstring)>0 and paramstring[0]=='?':
 *                     command='?'+command             # <<<<<<<<<<<<<<
 *                     param_tokens=paramstring[1:].split(';')
 *                     params=[int(part) for part in param_tokens]
 */
        __pyx_t_6 = PyNumber_Add(__pyx_kp_s__13, __pyx_v_command); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 552, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF_SET(__pyx_v_command, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "ttygif/tty/parser.pyx":553
 *                 elif paramstring and len(paramstring)>0 and paramstring[0]=='?':
 *                     command='?'+command
 *                     param_tokens=paramstring[1:].split(';')             # <<<<<<<<<<<<<<
 *                     params=[int(part) for part in param_tokens]
 *                 else:
 */
        __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_cur_scope->__pyx_v_paramstring, 1, 0, NULL, NULL, &__pyx_slice__14, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 553, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 553, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_2, function);
          }
        }
        __pyx_t_6 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_kp_s__5) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_s__5);
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 553, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF_SET(__pyx_v_param_tokens, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "ttygif/tty/parser.pyx":554
 *                     command='?'+command
 *                     param_tokens=paramstring[1:].split(';')
 *                     params=[int(part) for part in param_tokens]             # <<<<<<<<<<<<<<
 *                 else:
 *                     try:
 */
        __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 554, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (likely(PyList_CheckExact(__pyx_v_param_tokens)) || PyTuple_CheckExact(__pyx_v_param_tokens)) {
          __pyx_t_2 = __pyx_v_param_tokens; __Pyx_INCREF(__pyx_t_2); __pyx_t_12 = 0;
          __pyx_t_13 = NULL;
        } else {
          __pyx_t_12 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_param_tokens); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 554, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_13 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 554, __pyx_L1_error)
        }
        for (;;) {
          if (likely(!__pyx_t_13)) {
            if (likely(PyList_CheckExact(__pyx_t_2))) {
              if (__pyx_t_12 >= PyList_GET_SIZE(__pyx_t_2)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_12); __Pyx_INCREF(__pyx_t_1); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 554, __pyx_L1_error)
              #else
              __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 554, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_1);
              #endif
            } else {
              if (__pyx_t_12 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_12); __Pyx_INCREF(__pyx_t_1); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 554, __pyx_L1_error)
              #else
              __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 554, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_1);
              #endif
            }
          } else {
            __pyx_t_1 = __pyx_t_13(__pyx_t_2);
            if (unlikely(!__pyx_t_1)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 554, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_1);
          }
          __Pyx_XDECREF_SET(__pyx_v_part, __pyx_t_1);
          __pyx_t_1 = 0;
          __pyx_t_1 = __Pyx_PyNumber_Int(__pyx_v_part); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 554, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          if (unlikely(__Pyx_ListComp_Append(__pyx_t_6, (PyObject*)__pyx_t_1))) __PYX_ERR(0, 554, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF_SET(__pyx_v_params, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "ttygif/tty/parser.pyx":551
 *                         params = params + (1,)
 *                 #        DEC Private Mode (DECSET/DECRST) sequences
 *                 elif paramstring and len(paramstring)>0 and paramstring[0]=='?':             # <<<<<<<<<<<<<<
 *                     command='?'+command
 *                     param_tokens=paramstring[1:].split(';')
 */
        goto __pyx_L8;
      }

      /* "ttygif/tty/parser.pyx":556
 *                     params=[int(part) for part in param_tokens]
 *                 else:
 *                     try:             # <<<<<<<<<<<<<<
 *                         params = tuple(int(p) for p in paramstring.split(';') if len(p) != 0)
 *                         if len(params) == 0:
 */
      /*else*/ {
        {
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __Pyx_ExceptionSave(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_14);
          __Pyx_XGOTREF(__pyx_t_15);
          __Pyx_XGOTREF(__pyx_t_16);
          /*try:*/ {

            /* "ttygif/tty/parser.pyx":557
 *                 else:
 *                     try:
 *                         params = tuple(int(p) for p in paramstring.split(';') if len(p) != 0)             # <<<<<<<<<<<<<<
 *                         if len(params) == 0:
 *                             if command in 'JKm':
 */
            __pyx_t_6 = __pyx_pf_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_3genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 557, __pyx_L16_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_2 = __Pyx_PySequence_Tuple(__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 557, __pyx_L16_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_DECREF_SET(__pyx_v_params, __pyx_t_2);
            __pyx_t_2 = 0;

            /* "ttygif/tty/parser.pyx":558
 *                     try:
 *                         params = tuple(int(p) for p in paramstring.split(';') if len(p) != 0)
 *                         if len(params) == 0:             # <<<<<<<<<<<<<<
 *                             if command in 'JKm':
 *                                 params = (0,)
 */
            __pyx_t_12 = PyObject_Length(__pyx_v_params); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 558, __pyx_L16_error)
            __pyx_t_11 = ((__pyx_t_12 == 0) != 0);
            if (__pyx_t_11) {

              /* "ttygif/tty/parser.pyx":559
 *                         params = tuple(int(p) for p in paramstring.split(';') if len(p) != 0)
 *                         if len(params) == 0:
 *                             if command in 'JKm':             # <<<<<<<<<<<<<<
 *                                 params = (0,)
 *                             elif command in 'ABCD':
 */
              __pyx_t_11 = (__Pyx_PySequence_ContainsTF(__pyx_v_command, __pyx_n_s_JKm, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 559, __pyx_L16_error)
              __pyx_t_9 = (__pyx_t_11 != 0);
              if (__pyx_t_9) {

                /* "ttygif/tty/parser.pyx":560
 *                         if len(params) == 0:
 *                             if command in 'JKm':
 *                                 params = (0,)             # <<<<<<<<<<<<<<
 *                             elif command in 'ABCD':
 *                                 params = (1,)
 */
                __Pyx_INCREF(__pyx_tuple__15);
                __Pyx_DECREF_SET(__pyx_v_params, __pyx_tuple__15);

                /* "ttygif/tty/parser.pyx":559
 *                         params = tuple(int(p) for p in paramstring.split(';') if len(p) != 0)
 *                         if len(params) == 0:
 *                             if command in 'JKm':             # <<<<<<<<<<<<<<
 *                                 params = (0,)
 *                             elif command in 'ABCD':
 */
                goto __pyx_L25;
              }

              /* "ttygif/tty/parser.pyx":561
 *                             if command in 'JKm':
 *                                 params = (0,)
 *                             elif command in 'ABCD':             # <<<<<<<<<<<<<<
 *                                 params = (1,)
 *                     except Exception as ex:
 */
              __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_command, __pyx_n_s_ABCD, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 561, __pyx_L16_error)
              __pyx_t_11 = (__pyx_t_9 != 0);
              if (__pyx_t_11) {

                /* "ttygif/tty/parser.pyx":562
 *                                 params = (0,)
 *                             elif command in 'ABCD':
 *                                 params = (1,)             # <<<<<<<<<<<<<<
 *                     except Exception as ex:
 *                         continue
 */
                __Pyx_INCREF(__pyx_tuple__12);
                __Pyx_DECREF_SET(__pyx_v_params, __pyx_tuple__12);

                /* "ttygif/tty/parser.pyx":561
 *                             if command in 'JKm':
 *                                 params = (0,)
 *                             elif command in 'ABCD':             # <<<<<<<<<<<<<<
 *                                 params = (1,)
 *                     except Exception as ex:
 */
              }
              __pyx_L25:;

              /* "ttygif/tty/parser.pyx":558
 *                     try:
 *                         params = tuple(int(p) for p in paramstring.split(';') if len(p) != 0)
 *                         if len(params) == 0:             # <<<<<<<<<<<<<<
 *                             if command in 'JKm':
 *                                 params = (0,)
 */
            }

            /* "ttygif/tty/parser.pyx":556
 *                     params=[int(part) for part in param_tokens]
 *                 else:
 *                     try:             # <<<<<<<<<<<<<<
 *                         params = tuple(int(p) for p in paramstring.split(';') if len(p) != 0)
 *                         if len(params) == 0:
 */
          }
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          goto __pyx_L23_try_end;
          __pyx_L16_error:;
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

          /* "ttygif/tty/parser.pyx":563
 *                             elif command in 'ABCD':
 *                                 params = (1,)
 *                     except Exception as ex:             # <<<<<<<<<<<<<<
 *                         continue
 *                 self.add_command_sequence(esc_type,command,params,groups,name,timestamp,delay,text)
 */
          __pyx_t_17 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
          if (__pyx_t_17) {
            __Pyx_AddTraceback("ttygif.tty.parser.term_parser.stream_2_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
            if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_6, &__pyx_t_1) < 0) __PYX_ERR(0, 563, __pyx_L18_except_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_GOTREF(__pyx_t_6);
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_XDECREF_SET(__pyx_v_ex, __pyx_t_6);

            /* "ttygif/tty/parser.pyx":564
 *                                 params = (1,)
 *                     except Exception as ex:
 *                         continue             # <<<<<<<<<<<<<<
 *                 self.add_command_sequence(esc_type,command,params,groups,name,timestamp,delay,text)
 * 
 */
            goto __pyx_L27_except_continue;
            __pyx_L27_except_continue:;
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            goto __pyx_L22_try_continue;
          }
          goto __pyx_L18_except_error;
          __pyx_L18_except_error:;

          /* "ttygif/tty/parser.pyx":556
 *                     params=[int(part) for part in param_tokens]
 *                 else:
 *                     try:             # <<<<<<<<<<<<<<
 *                         params = tuple(int(p) for p in paramstring.split(';') if len(p) != 0)
 *                         if len(params) == 0:
 */
          __Pyx_XGIVEREF(__pyx_t_14);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
          goto __pyx_L1_error;
          __pyx_L22_try_continue:;
          __Pyx_XGIVEREF(__pyx_t_14);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
          goto __pyx_L3_continue;
          __pyx_L23_try_end:;
        }
      }
      __pyx_L8:;

      /* "ttygif/tty/parser.pyx":565
 *                     except Exception as ex:
 *                         continue
 *                 self.add_command_sequence(esc_type,command,params,groups,name,timestamp,delay,text)             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_10.__pyx_n = 1;
      __pyx_t_10.text = __pyx_v_text;
      __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->add_command_sequence(__pyx_v_self, __pyx_v_esc_type, __pyx_v_command, __pyx_v_params, __pyx_v_groups, __pyx_v_name, __pyx_v_timestamp, __pyx_v_delay, &__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 565, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ttygif/tty/parser.pyx":542
 *                 name="bracketed paste"
 *                 self.add_command_sequence(esc_type,command,params,groups,name,timestamp,delay,text)
 *             elif groups[8]:             # <<<<<<<<<<<<<<
 *                 esc_type='CSI'
 *                 paramstring=groups[9]
 */
    }
    __pyx_L7:;

    /* "ttygif/tty/parser.pyx":509
 *         ANSI=re.compile(ANSI_REGEX)
 *         cursor=0
 *         for match in ANSI.finditer(text):             # <<<<<<<<<<<<<<
 *             name=""
 *             start, end = match.span()
 */
    __pyx_L3_continue:;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ttygif/tty/parser.pyx":568
 * 
 * 
 *         if self.has_escape(text[cursor:]):             # <<<<<<<<<<<<<<
 *             self.extra_text=text[cursor:]
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_text, 0, 0, &__pyx_v_cursor, NULL, NULL, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->has_escape(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_11) {

    /* "ttygif/tty/parser.pyx":569
 * 
 *         if self.has_escape(text[cursor:]):
 *             self.extra_text=text[cursor:]             # <<<<<<<<<<<<<<
 * 
 *         else:
 */
    __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_text, 0, 0, &__pyx_v_cursor, NULL, NULL, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->extra_text);
    __Pyx_DECREF(__pyx_v_self->extra_text);
    __pyx_v_self->extra_text = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":568
 * 
 * 
 *         if self.has_escape(text[cursor:]):             # <<<<<<<<<<<<<<
 *             self.extra_text=text[cursor:]
 * 
 */
    goto __pyx_L28;
  }

  /* "ttygif/tty/parser.pyx":572
 * 
 *         else:
 *             self.extra_text=""             # <<<<<<<<<<<<<<
 *             self.add_text_sequence(text[cursor:],timestamp,0)
 * 
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_kp_s_);
    __Pyx_GIVEREF(__pyx_kp_s_);
    __Pyx_GOTREF(__pyx_v_self->extra_text);
    __Pyx_DECREF(__pyx_v_self->extra_text);
    __pyx_v_self->extra_text = __pyx_kp_s_;

    /* "ttygif/tty/parser.pyx":573
 *         else:
 *             self.extra_text=""
 *             self.add_text_sequence(text[cursor:],timestamp,0)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_text, 0, 0, &__pyx_v_cursor, NULL, NULL, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 573, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->add_text_sequence(__pyx_v_self, __pyx_t_1, __pyx_v_timestamp, __pyx_int_0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 573, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L28:;

  /* "ttygif/tty/parser.pyx":494
 *         self.g.state.cursor_absolute(0,position)
 * 
 *     cdef stream_2_sequence(self,text,timestamp,delay):             # <<<<<<<<<<<<<<
 *         # patterns for filtering out commands from the stream
 *         ANSI_SINGLE   ='[\033]([cDEHMZ78>=])'
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.stream_2_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ANSI_SINGLE);
  __Pyx_XDECREF(__pyx_v_ANSI_CHAR_SET);
  __Pyx_XDECREF(__pyx_v_ANSI_G0);
  __Pyx_XDECREF(__pyx_v_ANSI_G1);
  __Pyx_XDECREF(__pyx_v_ANSI_CSI_RE);
  __Pyx_XDECREF(__pyx_v_BRACKET_PASTE);
  __Pyx_XDECREF(__pyx_v_ANSI_OSC);
  __Pyx_XDECREF(__pyx_v_ESC_SEQUENCES);
  __Pyx_XDECREF(__pyx_v_ANSI_REGEX);
  __Pyx_XDECREF(__pyx_v_ANSI);
  __Pyx_XDECREF(__pyx_v_cursor);
  __Pyx_XDECREF(__pyx_v_match);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_start);
  __Pyx_XDECREF(__pyx_v_end);
  __Pyx_XDECREF(__pyx_v_command);
  __Pyx_XDECREF(__pyx_v_params);
  __Pyx_XDECREF(__pyx_v_esc_type);
  __Pyx_XDECREF(__pyx_v_groups);
  __Pyx_XDECREF(__pyx_v_param_tokens);
  __Pyx_XDECREF(__pyx_v_ex);
  __Pyx_XDECREF(__pyx_gb_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_2generator);
  __Pyx_XDECREF(__pyx_v_part);
  __Pyx_XDECREF(__pyx_gb_6ttygif_3tty_6parser_11term_parser_17stream_2_sequence_5generator1);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":576
 * 
 * 
 *     cdef last_frame(self):             # <<<<<<<<<<<<<<
 *         self.add_text_sequence(self.extra_text,self.last_timestamp,0)
 *         self.extra_text=""
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_last_frame(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("last_frame", 0);
  __Pyx_TraceCall("last_frame", __pyx_f[0], 576, 0, __PYX_ERR(0, 576, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":577
 * 
 *     cdef last_frame(self):
 *         self.add_text_sequence(self.extra_text,self.last_timestamp,0)             # <<<<<<<<<<<<<<
 *         self.extra_text=""
 * 
 */
  __pyx_t_1 = __pyx_v_self->extra_text;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->last_timestamp); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->add_text_sequence(__pyx_v_self, __pyx_t_1, __pyx_t_2, __pyx_int_0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ttygif/tty/parser.pyx":578
 *     cdef last_frame(self):
 *         self.add_text_sequence(self.extra_text,self.last_timestamp,0)
 *         self.extra_text=""             # <<<<<<<<<<<<<<
 * 
 *     cdef has_escape(self,text):
 */
  __Pyx_INCREF(__pyx_kp_s_);
  __Pyx_GIVEREF(__pyx_kp_s_);
  __Pyx_GOTREF(__pyx_v_self->extra_text);
  __Pyx_DECREF(__pyx_v_self->extra_text);
  __pyx_v_self->extra_text = __pyx_kp_s_;

  /* "ttygif/tty/parser.pyx":576
 * 
 * 
 *     cdef last_frame(self):             # <<<<<<<<<<<<<<
 *         self.add_text_sequence(self.extra_text,self.last_timestamp,0)
 *         self.extra_text=""
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.last_frame", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":580
 *         self.extra_text=""
 * 
 *     cdef has_escape(self,text):             # <<<<<<<<<<<<<<
 *         for i in text:
 *             if ord(i)==0x1B:
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_has_escape(CYTHON_UNUSED struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_text) {
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  long __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("has_escape", 0);
  __Pyx_TraceCall("has_escape", __pyx_f[0], 580, 0, __PYX_ERR(0, 580, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":581
 * 
 *     cdef has_escape(self,text):
 *         for i in text:             # <<<<<<<<<<<<<<
 *             if ord(i)==0x1B:
 *                 return True
 */
  if (likely(PyList_CheckExact(__pyx_v_text)) || PyTuple_CheckExact(__pyx_v_text)) {
    __pyx_t_1 = __pyx_v_text; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_text); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 581, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 581, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 581, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 581, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 581, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 581, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 581, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "ttygif/tty/parser.pyx":582
 *     cdef has_escape(self,text):
 *         for i in text:
 *             if ord(i)==0x1B:             # <<<<<<<<<<<<<<
 *                 return True
 *         return None
 */
    __pyx_t_5 = __Pyx_PyObject_Ord(__pyx_v_i); if (unlikely(__pyx_t_5 == ((long)(long)(Py_UCS4)-1))) __PYX_ERR(0, 582, __pyx_L1_error)
    __pyx_t_6 = ((__pyx_t_5 == 0x1B) != 0);
    if (__pyx_t_6) {

      /* "ttygif/tty/parser.pyx":583
 *         for i in text:
 *             if ord(i)==0x1B:
 *                 return True             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_True);
      __pyx_r = Py_True;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "ttygif/tty/parser.pyx":582
 *     cdef has_escape(self,text):
 *         for i in text:
 *             if ord(i)==0x1B:             # <<<<<<<<<<<<<<
 *                 return True
 *         return None
 */
    }

    /* "ttygif/tty/parser.pyx":581
 * 
 *     cdef has_escape(self,text):
 *         for i in text:             # <<<<<<<<<<<<<<
 *             if ord(i)==0x1B:
 *                 return True
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":584
 *             if ord(i)==0x1B:
 *                 return True
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     cdef add_event(self,event):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "ttygif/tty/parser.pyx":580
 *         self.extra_text=""
 * 
 *     cdef has_escape(self,text):             # <<<<<<<<<<<<<<
 *         for i in text:
 *             if ord(i)==0x1B:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.has_escape", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":586
 *         return None
 * 
 *     cdef add_event(self,event):             # <<<<<<<<<<<<<<
 *         timestamp=round(float(event[0]),3)
 *         event_type=event[1]
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_add_event(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_event) {
  PyObject *__pyx_v_timestamp = NULL;
  PyObject *__pyx_v_event_type = NULL;
  PyObject *__pyx_v_event_io = NULL;
  CYTHON_UNUSED long __pyx_v_delay;
  Py_ssize_t __pyx_v_length;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  double __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_event", 0);
  __Pyx_TraceCall("add_event", __pyx_f[0], 586, 0, __PYX_ERR(0, 586, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":587
 * 
 *     cdef add_event(self,event):
 *         timestamp=round(float(event[0]),3)             # <<<<<<<<<<<<<<
 *         event_type=event[1]
 *         event_io=event[2]
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_event, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyNumber_Float(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_3);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_round, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_timestamp = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "ttygif/tty/parser.pyx":588
 *     cdef add_event(self,event):
 *         timestamp=round(float(event[0]),3)
 *         event_type=event[1]             # <<<<<<<<<<<<<<
 *         event_io=event[2]
 *         if self.last_timestamp==0:
 */
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_event, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_event_type = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "ttygif/tty/parser.pyx":589
 *         timestamp=round(float(event[0]),3)
 *         event_type=event[1]
 *         event_io=event[2]             # <<<<<<<<<<<<<<
 *         if self.last_timestamp==0:
 *             delay=0
 */
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_event, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 589, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_event_io = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "ttygif/tty/parser.pyx":590
 *         event_type=event[1]
 *         event_io=event[2]
 *         if self.last_timestamp==0:             # <<<<<<<<<<<<<<
 *             delay=0
 *         else:
 */
  __pyx_t_3 = ((__pyx_v_self->last_timestamp == 0.0) != 0);
  if (__pyx_t_3) {

    /* "ttygif/tty/parser.pyx":591
 *         event_io=event[2]
 *         if self.last_timestamp==0:
 *             delay=0             # <<<<<<<<<<<<<<
 *         else:
 *             length=len(self.sequence)-1
 */
    __pyx_v_delay = 0;

    /* "ttygif/tty/parser.pyx":590
 *         event_type=event[1]
 *         event_io=event[2]
 *         if self.last_timestamp==0:             # <<<<<<<<<<<<<<
 *             delay=0
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ttygif/tty/parser.pyx":593
 *             delay=0
 *         else:
 *             length=len(self.sequence)-1             # <<<<<<<<<<<<<<
 *             if length>0:
 *                 self.sequence[length]['delay']=timestamp-self.last_timestamp
 */
  /*else*/ {
    __pyx_t_2 = __pyx_v_self->sequence;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_4 = PyObject_Length(__pyx_t_2); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 593, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_length = (__pyx_t_4 - 1);

    /* "ttygif/tty/parser.pyx":594
 *         else:
 *             length=len(self.sequence)-1
 *             if length>0:             # <<<<<<<<<<<<<<
 *                 self.sequence[length]['delay']=timestamp-self.last_timestamp
 *             else:
 */
    __pyx_t_3 = ((__pyx_v_length > 0) != 0);
    if (__pyx_t_3) {

      /* "ttygif/tty/parser.pyx":595
 *             length=len(self.sequence)-1
 *             if length>0:
 *                 self.sequence[length]['delay']=timestamp-self.last_timestamp             # <<<<<<<<<<<<<<
 *             else:
 *                 delay=0
 */
      __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->last_timestamp); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 595, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = PyNumber_Subtract(__pyx_v_timestamp, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 595, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_self->sequence, __pyx_v_length, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 595, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely(PyObject_SetItem(__pyx_t_2, __pyx_n_s_delay, __pyx_t_1) < 0)) __PYX_ERR(0, 595, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ttygif/tty/parser.pyx":594
 *         else:
 *             length=len(self.sequence)-1
 *             if length>0:             # <<<<<<<<<<<<<<
 *                 self.sequence[length]['delay']=timestamp-self.last_timestamp
 *             else:
 */
      goto __pyx_L4;
    }

    /* "ttygif/tty/parser.pyx":597
 *                 self.sequence[length]['delay']=timestamp-self.last_timestamp
 *             else:
 *                 delay=0             # <<<<<<<<<<<<<<
 *         if event_type=='o':
 *             self.stream_2_sequence(self.extra_text+event_io,timestamp,0)
 */
    /*else*/ {
      __pyx_v_delay = 0;
    }
    __pyx_L4:;
  }
  __pyx_L3:;

  /* "ttygif/tty/parser.pyx":598
 *             else:
 *                 delay=0
 *         if event_type=='o':             # <<<<<<<<<<<<<<
 *             self.stream_2_sequence(self.extra_text+event_io,timestamp,0)
 *             self.last_timestamp=timestamp
 */
  __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_v_event_type, __pyx_n_s_o, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 598, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "ttygif/tty/parser.pyx":599
 *                 delay=0
 *         if event_type=='o':
 *             self.stream_2_sequence(self.extra_text+event_io,timestamp,0)             # <<<<<<<<<<<<<<
 *             self.last_timestamp=timestamp
 * 
 */
    __pyx_t_1 = PyNumber_Add(__pyx_v_self->extra_text, __pyx_v_event_io); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 599, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->stream_2_sequence(__pyx_v_self, __pyx_t_1, __pyx_v_timestamp, __pyx_int_0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 599, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "ttygif/tty/parser.pyx":600
 *         if event_type=='o':
 *             self.stream_2_sequence(self.extra_text+event_io,timestamp,0)
 *             self.last_timestamp=timestamp             # <<<<<<<<<<<<<<
 * 
 *     cdef add_text_sequence(self,text,timestamp,delay):
 */
    __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_v_timestamp); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 600, __pyx_L1_error)
    __pyx_v_self->last_timestamp = __pyx_t_5;

    /* "ttygif/tty/parser.pyx":598
 *             else:
 *                 delay=0
 *         if event_type=='o':             # <<<<<<<<<<<<<<
 *             self.stream_2_sequence(self.extra_text+event_io,timestamp,0)
 *             self.last_timestamp=timestamp
 */
  }

  /* "ttygif/tty/parser.pyx":586
 *         return None
 * 
 *     cdef add_event(self,event):             # <<<<<<<<<<<<<<
 *         timestamp=round(float(event[0]),3)
 *         event_type=event[1]
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.add_event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_timestamp);
  __Pyx_XDECREF(__pyx_v_event_type);
  __Pyx_XDECREF(__pyx_v_event_io);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":602
 *             self.last_timestamp=timestamp
 * 
 *     cdef add_text_sequence(self,text,timestamp,delay):             # <<<<<<<<<<<<<<
 *         if len(text)==0:
 *             return
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_add_text_sequence(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_text, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_delay) {
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_text_sequence", 0);
  __Pyx_TraceCall("add_text_sequence", __pyx_f[0], 602, 0, __PYX_ERR(0, 602, __pyx_L1_error));
  __Pyx_INCREF(__pyx_v_text);

  /* "ttygif/tty/parser.pyx":603
 * 
 *     cdef add_text_sequence(self,text,timestamp,delay):
 *         if len(text)==0:             # <<<<<<<<<<<<<<
 *             return
 *         text=[self.remap_character(i) for i in text]
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_text); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 603, __pyx_L1_error)
  __pyx_t_2 = ((__pyx_t_1 == 0) != 0);
  if (__pyx_t_2) {

    /* "ttygif/tty/parser.pyx":604
 *     cdef add_text_sequence(self,text,timestamp,delay):
 *         if len(text)==0:
 *             return             # <<<<<<<<<<<<<<
 *         text=[self.remap_character(i) for i in text]
 *         self.sequence.append({'type':'text','command':'text','data':text,'timestamp':timestamp,'delay':delay})
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "ttygif/tty/parser.pyx":603
 * 
 *     cdef add_text_sequence(self,text,timestamp,delay):
 *         if len(text)==0:             # <<<<<<<<<<<<<<
 *             return
 *         text=[self.remap_character(i) for i in text]
 */
  }

  /* "ttygif/tty/parser.pyx":605
 *         if len(text)==0:
 *             return
 *         text=[self.remap_character(i) for i in text]             # <<<<<<<<<<<<<<
 *         self.sequence.append({'type':'text','command':'text','data':text,'timestamp':timestamp,'delay':delay})
 * 
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (likely(PyList_CheckExact(__pyx_v_text)) || PyTuple_CheckExact(__pyx_v_text)) {
    __pyx_t_4 = __pyx_v_text; __Pyx_INCREF(__pyx_t_4); __pyx_t_1 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_1 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_v_text); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 605, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_1 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_1); __Pyx_INCREF(__pyx_t_6); __pyx_t_1++; if (unlikely(0 < 0)) __PYX_ERR(0, 605, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_4, __pyx_t_1); __pyx_t_1++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 605, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      } else {
        if (__pyx_t_1 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_1); __Pyx_INCREF(__pyx_t_6); __pyx_t_1++; if (unlikely(0 < 0)) __PYX_ERR(0, 605, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_4, __pyx_t_1); __pyx_t_1++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 605, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      }
    } else {
      __pyx_t_6 = __pyx_t_5(__pyx_t_4);
      if (unlikely(!__pyx_t_6)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 605, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_6 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->remap_character(__pyx_v_self, __pyx_v_i); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_6))) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF_SET(__pyx_v_text, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "ttygif/tty/parser.pyx":606
 *             return
 *         text=[self.remap_character(i) for i in text]
 *         self.sequence.append({'type':'text','command':'text','data':text,'timestamp':timestamp,'delay':delay})             # <<<<<<<<<<<<<<
 * 
 *     cdef add_command_sequence(self,esc_type,command,params,groups,name,timestamp,delay,text=None):
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_type, __pyx_n_s_text) < 0) __PYX_ERR(0, 606, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_command, __pyx_n_s_text) < 0) __PYX_ERR(0, 606, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_data, __pyx_v_text) < 0) __PYX_ERR(0, 606, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_timestamp, __pyx_v_timestamp) < 0) __PYX_ERR(0, 606, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_delay, __pyx_v_delay) < 0) __PYX_ERR(0, 606, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyObject_Append(__pyx_v_self->sequence, __pyx_t_3); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 606, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ttygif/tty/parser.pyx":602
 *             self.last_timestamp=timestamp
 * 
 *     cdef add_text_sequence(self,text,timestamp,delay):             # <<<<<<<<<<<<<<
 *         if len(text)==0:
 *             return
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.add_text_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_text);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":608
 *         self.sequence.append({'type':'text','command':'text','data':text,'timestamp':timestamp,'delay':delay})
 * 
 *     cdef add_command_sequence(self,esc_type,command,params,groups,name,timestamp,delay,text=None):             # <<<<<<<<<<<<<<
 *         self.sequence.append({'type':'command','esc_type':esc_type,'command':command,'params':params,'groups':groups,'name':name,'timestamp':timestamp,'delay':delay,'data':text})
 * 
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_add_command_sequence(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_esc_type, PyObject *__pyx_v_command, PyObject *__pyx_v_params, PyObject *__pyx_v_groups, PyObject *__pyx_v_name, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_delay, struct __pyx_opt_args_6ttygif_3tty_6parser_11term_parser_add_command_sequence *__pyx_optional_args) {
  PyObject *__pyx_v_text = ((PyObject *)Py_None);
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_command_sequence", 0);
  __Pyx_TraceCall("add_command_sequence", __pyx_f[0], 608, 0, __PYX_ERR(0, 608, __pyx_L1_error));
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_text = __pyx_optional_args->text;
    }
  }

  /* "ttygif/tty/parser.pyx":609
 * 
 *     cdef add_command_sequence(self,esc_type,command,params,groups,name,timestamp,delay,text=None):
 *         self.sequence.append({'type':'command','esc_type':esc_type,'command':command,'params':params,'groups':groups,'name':name,'timestamp':timestamp,'delay':delay,'data':text})             # <<<<<<<<<<<<<<
 * 
 *     cdef debug_sequence(self):
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_type, __pyx_n_s_command) < 0) __PYX_ERR(0, 609, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_esc_type, __pyx_v_esc_type) < 0) __PYX_ERR(0, 609, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_command, __pyx_v_command) < 0) __PYX_ERR(0, 609, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_params, __pyx_v_params) < 0) __PYX_ERR(0, 609, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_groups, __pyx_v_groups) < 0) __PYX_ERR(0, 609, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_name, __pyx_v_name) < 0) __PYX_ERR(0, 609, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_timestamp, __pyx_v_timestamp) < 0) __PYX_ERR(0, 609, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_delay, __pyx_v_delay) < 0) __PYX_ERR(0, 609, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_data, __pyx_v_text) < 0) __PYX_ERR(0, 609, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_Append(__pyx_v_self->sequence, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 609, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":608
 *         self.sequence.append({'type':'text','command':'text','data':text,'timestamp':timestamp,'delay':delay})
 * 
 *     cdef add_command_sequence(self,esc_type,command,params,groups,name,timestamp,delay,text=None):             # <<<<<<<<<<<<<<
 *         self.sequence.append({'type':'command','esc_type':esc_type,'command':command,'params':params,'groups':groups,'name':name,'timestamp':timestamp,'delay':delay,'data':text})
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.add_command_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":611
 *         self.sequence.append({'type':'command','esc_type':esc_type,'command':command,'params':params,'groups':groups,'name':name,'timestamp':timestamp,'delay':delay,'data':text})
 * 
 *     cdef debug_sequence(self):             # <<<<<<<<<<<<<<
 *         print ("============")
 *         print ("Sequence List")
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_debug_sequence(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self) {
  int __pyx_v_i;
  PyObject *__pyx_v_item = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *(*__pyx_t_6)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("debug_sequence", 0);
  __Pyx_TraceCall("debug_sequence", __pyx_f[0], 611, 0, __PYX_ERR(0, 611, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":612
 * 
 *     cdef debug_sequence(self):
 *         print ("============")             # <<<<<<<<<<<<<<
 *         print ("Sequence List")
 *         print ("Count:{0}".format(len(self.sequence)))
 */
  if (__Pyx_PrintOne(0, __pyx_kp_s__16) < 0) __PYX_ERR(0, 612, __pyx_L1_error)

  /* "ttygif/tty/parser.pyx":613
 *     cdef debug_sequence(self):
 *         print ("============")
 *         print ("Sequence List")             # <<<<<<<<<<<<<<
 *         print ("Count:{0}".format(len(self.sequence)))
 *         cdef int i=0
 */
  if (__Pyx_PrintOne(0, __pyx_kp_s_Sequence_List) < 0) __PYX_ERR(0, 613, __pyx_L1_error)

  /* "ttygif/tty/parser.pyx":614
 *         print ("============")
 *         print ("Sequence List")
 *         print ("Count:{0}".format(len(self.sequence)))             # <<<<<<<<<<<<<<
 *         cdef int i=0
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Count_0, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_v_self->sequence;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = PyObject_Length(__pyx_t_3); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PrintOne(0, __pyx_t_1) < 0) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":615
 *         print ("Sequence List")
 *         print ("Count:{0}".format(len(self.sequence)))
 *         cdef int i=0             # <<<<<<<<<<<<<<
 * 
 *         for item in self.sequence:
 */
  __pyx_v_i = 0;

  /* "ttygif/tty/parser.pyx":617
 *         cdef int i=0
 * 
 *         for item in self.sequence:             # <<<<<<<<<<<<<<
 *             self.debug_event(item,i)
 *             i+=1
 */
  if (likely(PyList_CheckExact(__pyx_v_self->sequence)) || PyTuple_CheckExact(__pyx_v_self->sequence)) {
    __pyx_t_1 = __pyx_v_self->sequence; __Pyx_INCREF(__pyx_t_1); __pyx_t_4 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_self->sequence); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 617, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 617, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_2); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 617, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 617, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_2); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 617, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 617, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_6(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 617, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "ttygif/tty/parser.pyx":618
 * 
 *         for item in self.sequence:
 *             self.debug_event(item,i)             # <<<<<<<<<<<<<<
 *             i+=1
 * 
 */
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 618, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->debug_event(__pyx_v_self, __pyx_v_item, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 618, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "ttygif/tty/parser.pyx":619
 *         for item in self.sequence:
 *             self.debug_event(item,i)
 *             i+=1             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_i = (__pyx_v_i + 1);

    /* "ttygif/tty/parser.pyx":617
 *         cdef int i=0
 * 
 *         for item in self.sequence:             # <<<<<<<<<<<<<<
 *             self.debug_event(item,i)
 *             i+=1
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":611
 *         self.sequence.append({'type':'command','esc_type':esc_type,'command':command,'params':params,'groups':groups,'name':name,'timestamp':timestamp,'delay':delay,'data':text})
 * 
 *     cdef debug_sequence(self):             # <<<<<<<<<<<<<<
 *         print ("============")
 *         print ("Sequence List")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.debug_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ttygif/tty/parser.pyx":622
 * 
 * 
 *     cdef debug_event(self,event,index):             # <<<<<<<<<<<<<<
 *         commands=[
 *                     ['CSI','A' ,[1]   ,'CUU'             ],
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser_11term_parser_debug_event(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v_event, PyObject *__pyx_v_index) {
  PyObject *__pyx_v_commands = NULL;
  PyObject *__pyx_v_cmd = NULL;
  PyObject *__pyx_v_param = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  int __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  int __pyx_t_28;
  int __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  PyObject *(*__pyx_t_31)(PyObject *);
  int __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("debug_event", 0);
  __Pyx_TraceCall("debug_event", __pyx_f[0], 622, 0, __PYX_ERR(0, 622, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":624
 *     cdef debug_event(self,event,index):
 *         commands=[
 *                     ['CSI','A' ,[1]   ,'CUU'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','B' ,[1]   ,'CUD'             ],
 *                     ['CSI','C' ,[1]   ,'CUF'             ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 624, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_2 = PyList_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 624, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_A);
  __Pyx_GIVEREF(__pyx_n_s_A);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_A);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_CUU);
  __Pyx_GIVEREF(__pyx_n_s_CUU);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_s_CUU);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":625
 *         commands=[
 *                     ['CSI','A' ,[1]   ,'CUU'             ],
 *                     ['CSI','B' ,[1]   ,'CUD'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','C' ,[1]   ,'CUF'             ],
 *                     ['CSI','D' ,[1]   ,'CUB'             ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 625, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_3 = PyList_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 625, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_B);
  __Pyx_GIVEREF(__pyx_n_s_B);
  PyList_SET_ITEM(__pyx_t_3, 1, __pyx_n_s_B);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_CUD);
  __Pyx_GIVEREF(__pyx_n_s_CUD);
  PyList_SET_ITEM(__pyx_t_3, 3, __pyx_n_s_CUD);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":626
 *                     ['CSI','A' ,[1]   ,'CUU'             ],
 *                     ['CSI','B' ,[1]   ,'CUD'             ],
 *                     ['CSI','C' ,[1]   ,'CUF'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','D' ,[1]   ,'CUB'             ],
 *                     ['CSI','E' ,[1]   ,'CNL'             ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_4 = PyList_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_4, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_C);
  __Pyx_GIVEREF(__pyx_n_s_C);
  PyList_SET_ITEM(__pyx_t_4, 1, __pyx_n_s_C);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_4, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_CUF);
  __Pyx_GIVEREF(__pyx_n_s_CUF);
  PyList_SET_ITEM(__pyx_t_4, 3, __pyx_n_s_CUF);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":627
 *                     ['CSI','B' ,[1]   ,'CUD'             ],
 *                     ['CSI','C' ,[1]   ,'CUF'             ],
 *                     ['CSI','D' ,[1]   ,'CUB'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','E' ,[1]   ,'CNL'             ],
 *                     ['CSI','F' ,[1]   ,'CPL'             ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_5 = PyList_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_5, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_D);
  __Pyx_GIVEREF(__pyx_n_s_D);
  PyList_SET_ITEM(__pyx_t_5, 1, __pyx_n_s_D);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_5, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_CUB);
  __Pyx_GIVEREF(__pyx_n_s_CUB);
  PyList_SET_ITEM(__pyx_t_5, 3, __pyx_n_s_CUB);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":628
 *                     ['CSI','C' ,[1]   ,'CUF'             ],
 *                     ['CSI','D' ,[1]   ,'CUB'             ],
 *                     ['CSI','E' ,[1]   ,'CNL'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','F' ,[1]   ,'CPL'             ],
 *                     ['CSI','G' ,[3]   ,'CHA'             ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 628, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_6 = PyList_New(4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 628, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_6, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_E);
  __Pyx_GIVEREF(__pyx_n_s_E);
  PyList_SET_ITEM(__pyx_t_6, 1, __pyx_n_s_E);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_6, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_CNL);
  __Pyx_GIVEREF(__pyx_n_s_CNL);
  PyList_SET_ITEM(__pyx_t_6, 3, __pyx_n_s_CNL);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":629
 *                     ['CSI','D' ,[1]   ,'CUB'             ],
 *                     ['CSI','E' ,[1]   ,'CNL'             ],
 *                     ['CSI','F' ,[1]   ,'CPL'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','G' ,[3]   ,'CHA'             ],
 *                     ['CSI','H' ,[3,4] ,'CUP'             ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_7 = PyList_New(4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_7, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_F);
  __Pyx_GIVEREF(__pyx_n_s_F);
  PyList_SET_ITEM(__pyx_t_7, 1, __pyx_n_s_F);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_7, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_CPL);
  __Pyx_GIVEREF(__pyx_n_s_CPL);
  PyList_SET_ITEM(__pyx_t_7, 3, __pyx_n_s_CPL);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":630
 *                     ['CSI','E' ,[1]   ,'CNL'             ],
 *                     ['CSI','F' ,[1]   ,'CPL'             ],
 *                     ['CSI','G' ,[3]   ,'CHA'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','H' ,[3,4] ,'CUP'             ],
 *                     ['CSI','J' ,[1]   ,'ED'              ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 630, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_3);
  __pyx_t_8 = PyList_New(4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 630, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_8, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_G);
  __Pyx_GIVEREF(__pyx_n_s_G);
  PyList_SET_ITEM(__pyx_t_8, 1, __pyx_n_s_G);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_8, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_CHA);
  __Pyx_GIVEREF(__pyx_n_s_CHA);
  PyList_SET_ITEM(__pyx_t_8, 3, __pyx_n_s_CHA);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":631
 *                     ['CSI','F' ,[1]   ,'CPL'             ],
 *                     ['CSI','G' ,[3]   ,'CHA'             ],
 *                     ['CSI','H' ,[3,4] ,'CUP'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','J' ,[1]   ,'ED'              ],
 *                     ['CSI','K' ,[1]   ,'EL'              ],
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_3);
  __Pyx_INCREF(__pyx_int_4);
  __Pyx_GIVEREF(__pyx_int_4);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_int_4);
  __pyx_t_9 = PyList_New(4); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_9, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_H);
  __Pyx_GIVEREF(__pyx_n_s_H);
  PyList_SET_ITEM(__pyx_t_9, 1, __pyx_n_s_H);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_9, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_CUP);
  __Pyx_GIVEREF(__pyx_n_s_CUP);
  PyList_SET_ITEM(__pyx_t_9, 3, __pyx_n_s_CUP);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":632
 *                     ['CSI','G' ,[3]   ,'CHA'             ],
 *                     ['CSI','H' ,[3,4] ,'CUP'             ],
 *                     ['CSI','J' ,[1]   ,'ED'              ],             # <<<<<<<<<<<<<<
 *                     ['CSI','K' ,[1]   ,'EL'              ],
 *                     ['CSI','P' ,[1]   ,'DCH'             ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 632, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_10 = PyList_New(4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 632, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_10, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_J);
  __Pyx_GIVEREF(__pyx_n_s_J);
  PyList_SET_ITEM(__pyx_t_10, 1, __pyx_n_s_J);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_10, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_ED);
  __Pyx_GIVEREF(__pyx_n_s_ED);
  PyList_SET_ITEM(__pyx_t_10, 3, __pyx_n_s_ED);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":633
 *                     ['CSI','H' ,[3,4] ,'CUP'             ],
 *                     ['CSI','J' ,[1]   ,'ED'              ],
 *                     ['CSI','K' ,[1]   ,'EL'              ],             # <<<<<<<<<<<<<<
 *                     ['CSI','P' ,[1]   ,'DCH'             ],
 *                     ['CSI','X' ,[1]   ,'ECH'             ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_11 = PyList_New(4); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_11, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_K);
  __Pyx_GIVEREF(__pyx_n_s_K);
  PyList_SET_ITEM(__pyx_t_11, 1, __pyx_n_s_K);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_11, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_EL);
  __Pyx_GIVEREF(__pyx_n_s_EL);
  PyList_SET_ITEM(__pyx_t_11, 3, __pyx_n_s_EL);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":634
 *                     ['CSI','J' ,[1]   ,'ED'              ],
 *                     ['CSI','K' ,[1]   ,'EL'              ],
 *                     ['CSI','P' ,[1]   ,'DCH'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','X' ,[1]   ,'ECH'             ],
 *                     ['CSI','d' ,[3,]  ,'VPA'             ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_12 = PyList_New(4); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_12, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_P);
  __Pyx_GIVEREF(__pyx_n_s_P);
  PyList_SET_ITEM(__pyx_t_12, 1, __pyx_n_s_P);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_12, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_DCH);
  __Pyx_GIVEREF(__pyx_n_s_DCH);
  PyList_SET_ITEM(__pyx_t_12, 3, __pyx_n_s_DCH);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":635
 *                     ['CSI','K' ,[1]   ,'EL'              ],
 *                     ['CSI','P' ,[1]   ,'DCH'             ],
 *                     ['CSI','X' ,[1]   ,'ECH'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','d' ,[3,]  ,'VPA'             ],
 *                     ['CSI','f' ,[3,4] ,'HVP'             ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_13 = PyList_New(4); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_13, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_X);
  __Pyx_GIVEREF(__pyx_n_s_X);
  PyList_SET_ITEM(__pyx_t_13, 1, __pyx_n_s_X);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_13, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_ECH);
  __Pyx_GIVEREF(__pyx_n_s_ECH);
  PyList_SET_ITEM(__pyx_t_13, 3, __pyx_n_s_ECH);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":636
 *                     ['CSI','P' ,[1]   ,'DCH'             ],
 *                     ['CSI','X' ,[1]   ,'ECH'             ],
 *                     ['CSI','d' ,[3,]  ,'VPA'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','f' ,[3,4] ,'HVP'             ],
 *                     ['CSI','h' ,[1]   ,'set_mode'        ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 636, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_3);
  __pyx_t_14 = PyList_New(4); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 636, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_14, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_d);
  __Pyx_GIVEREF(__pyx_n_s_d);
  PyList_SET_ITEM(__pyx_t_14, 1, __pyx_n_s_d);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_14, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_VPA);
  __Pyx_GIVEREF(__pyx_n_s_VPA);
  PyList_SET_ITEM(__pyx_t_14, 3, __pyx_n_s_VPA);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":637
 *                     ['CSI','X' ,[1]   ,'ECH'             ],
 *                     ['CSI','d' ,[3,]  ,'VPA'             ],
 *                     ['CSI','f' ,[3,4] ,'HVP'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','h' ,[1]   ,'set_mode'        ],
 *                     ['CSI','l' ,[1]   ,'reset_mode'      ],
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_3);
  __Pyx_INCREF(__pyx_int_4);
  __Pyx_GIVEREF(__pyx_int_4);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_int_4);
  __pyx_t_15 = PyList_New(4); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_15, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_f);
  __Pyx_GIVEREF(__pyx_n_s_f);
  PyList_SET_ITEM(__pyx_t_15, 1, __pyx_n_s_f);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_15, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_HVP);
  __Pyx_GIVEREF(__pyx_n_s_HVP);
  PyList_SET_ITEM(__pyx_t_15, 3, __pyx_n_s_HVP);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":638
 *                     ['CSI','d' ,[3,]  ,'VPA'             ],
 *                     ['CSI','f' ,[3,4] ,'HVP'             ],
 *                     ['CSI','h' ,[1]   ,'set_mode'        ],             # <<<<<<<<<<<<<<
 *                     ['CSI','l' ,[1]   ,'reset_mode'      ],
 *                     ['CSI','m' ,[1]   ,'COL'  ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 638, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_16 = PyList_New(4); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 638, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_16, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_h);
  __Pyx_GIVEREF(__pyx_n_s_h);
  PyList_SET_ITEM(__pyx_t_16, 1, __pyx_n_s_h);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_16, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_set_mode);
  __Pyx_GIVEREF(__pyx_n_s_set_mode);
  PyList_SET_ITEM(__pyx_t_16, 3, __pyx_n_s_set_mode);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":639
 *                     ['CSI','f' ,[3,4] ,'HVP'             ],
 *                     ['CSI','h' ,[1]   ,'set_mode'        ],
 *                     ['CSI','l' ,[1]   ,'reset_mode'      ],             # <<<<<<<<<<<<<<
 *                     ['CSI','m' ,[1]   ,'COL'  ],
 *                     ['CSI','r' ,[3,4] ,'DECSTBM'         ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_17 = PyList_New(4); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_17, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_l);
  __Pyx_GIVEREF(__pyx_n_s_l);
  PyList_SET_ITEM(__pyx_t_17, 1, __pyx_n_s_l);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_17, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_reset_mode);
  __Pyx_GIVEREF(__pyx_n_s_reset_mode);
  PyList_SET_ITEM(__pyx_t_17, 3, __pyx_n_s_reset_mode);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":640
 *                     ['CSI','h' ,[1]   ,'set_mode'        ],
 *                     ['CSI','l' ,[1]   ,'reset_mode'      ],
 *                     ['CSI','m' ,[1]   ,'COL'  ],             # <<<<<<<<<<<<<<
 *                     ['CSI','r' ,[3,4] ,'DECSTBM'         ],
 *                     ['CSI','s' ,[0]   ,'SCP'             ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 640, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_18 = PyList_New(4); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 640, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_18);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_18, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_m);
  __Pyx_GIVEREF(__pyx_n_s_m);
  PyList_SET_ITEM(__pyx_t_18, 1, __pyx_n_s_m);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_18, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_COL);
  __Pyx_GIVEREF(__pyx_n_s_COL);
  PyList_SET_ITEM(__pyx_t_18, 3, __pyx_n_s_COL);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":641
 *                     ['CSI','l' ,[1]   ,'reset_mode'      ],
 *                     ['CSI','m' ,[1]   ,'COL'  ],
 *                     ['CSI','r' ,[3,4] ,'DECSTBM'         ],             # <<<<<<<<<<<<<<
 *                     ['CSI','s' ,[0]   ,'SCP'             ],
 *                     ['CSI','u' ,[0]   ,'RCP'             ],
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_3);
  __Pyx_INCREF(__pyx_int_4);
  __Pyx_GIVEREF(__pyx_int_4);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_int_4);
  __pyx_t_19 = PyList_New(4); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_19);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_19, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_r);
  __Pyx_GIVEREF(__pyx_n_s_r);
  PyList_SET_ITEM(__pyx_t_19, 1, __pyx_n_s_r);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_19, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_DECSTBM);
  __Pyx_GIVEREF(__pyx_n_s_DECSTBM);
  PyList_SET_ITEM(__pyx_t_19, 3, __pyx_n_s_DECSTBM);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":642
 *                     ['CSI','m' ,[1]   ,'COL'  ],
 *                     ['CSI','r' ,[3,4] ,'DECSTBM'         ],
 *                     ['CSI','s' ,[0]   ,'SCP'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','u' ,[0]   ,'RCP'             ],
 *                     ['CSI','`' ,[3]   ,'HPA'             ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 642, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_0);
  __pyx_t_20 = PyList_New(4); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 642, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_20);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_20, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_s);
  __Pyx_GIVEREF(__pyx_n_s_s);
  PyList_SET_ITEM(__pyx_t_20, 1, __pyx_n_s_s);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_20, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_SCP);
  __Pyx_GIVEREF(__pyx_n_s_SCP);
  PyList_SET_ITEM(__pyx_t_20, 3, __pyx_n_s_SCP);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":643
 *                     ['CSI','r' ,[3,4] ,'DECSTBM'         ],
 *                     ['CSI','s' ,[0]   ,'SCP'             ],
 *                     ['CSI','u' ,[0]   ,'RCP'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','`' ,[3]   ,'HPA'             ],
 *                     ['CSI','~' ,[1]   ,'BRACKETED_PASTE' ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 643, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_0);
  __pyx_t_21 = PyList_New(4); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 643, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_21);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_21, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_n_s_u);
  __Pyx_GIVEREF(__pyx_n_s_u);
  PyList_SET_ITEM(__pyx_t_21, 1, __pyx_n_s_u);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_21, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_RCP);
  __Pyx_GIVEREF(__pyx_n_s_RCP);
  PyList_SET_ITEM(__pyx_t_21, 3, __pyx_n_s_RCP);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":644
 *                     ['CSI','s' ,[0]   ,'SCP'             ],
 *                     ['CSI','u' ,[0]   ,'RCP'             ],
 *                     ['CSI','`' ,[3]   ,'HPA'             ],             # <<<<<<<<<<<<<<
 *                     ['CSI','~' ,[1]   ,'BRACKETED_PASTE' ],
 *                     ['CSI','?h',[1]   ,'DECSET'          ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 644, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_3);
  __pyx_t_22 = PyList_New(4); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 644, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_22);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_22, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_kp_s__4);
  __Pyx_GIVEREF(__pyx_kp_s__4);
  PyList_SET_ITEM(__pyx_t_22, 1, __pyx_kp_s__4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_22, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_HPA);
  __Pyx_GIVEREF(__pyx_n_s_HPA);
  PyList_SET_ITEM(__pyx_t_22, 3, __pyx_n_s_HPA);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":645
 *                     ['CSI','u' ,[0]   ,'RCP'             ],
 *                     ['CSI','`' ,[3]   ,'HPA'             ],
 *                     ['CSI','~' ,[1]   ,'BRACKETED_PASTE' ],             # <<<<<<<<<<<<<<
 *                     ['CSI','?h',[1]   ,'DECSET'          ],
 *                     ['CSI','?l',[1]   ,'DECRST'          ],
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_23 = PyList_New(4); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_23);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_23, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_kp_s__3);
  __Pyx_GIVEREF(__pyx_kp_s__3);
  PyList_SET_ITEM(__pyx_t_23, 1, __pyx_kp_s__3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_23, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_BRACKETED_PASTE);
  __Pyx_GIVEREF(__pyx_n_s_BRACKETED_PASTE);
  PyList_SET_ITEM(__pyx_t_23, 3, __pyx_n_s_BRACKETED_PASTE);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":646
 *                     ['CSI','`' ,[3]   ,'HPA'             ],
 *                     ['CSI','~' ,[1]   ,'BRACKETED_PASTE' ],
 *                     ['CSI','?h',[1]   ,'DECSET'          ],             # <<<<<<<<<<<<<<
 *                     ['CSI','?l',[1]   ,'DECRST'          ],
 *             ]
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 646, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_24 = PyList_New(4); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 646, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_24);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_24, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_kp_s_h_2);
  __Pyx_GIVEREF(__pyx_kp_s_h_2);
  PyList_SET_ITEM(__pyx_t_24, 1, __pyx_kp_s_h_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_24, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_DECSET);
  __Pyx_GIVEREF(__pyx_n_s_DECSET);
  PyList_SET_ITEM(__pyx_t_24, 3, __pyx_n_s_DECSET);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":647
 *                     ['CSI','~' ,[1]   ,'BRACKETED_PASTE' ],
 *                     ['CSI','?h',[1]   ,'DECSET'          ],
 *                     ['CSI','?l',[1]   ,'DECRST'          ],             # <<<<<<<<<<<<<<
 *             ]
 *         if event['type']=='text':
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
  __pyx_t_25 = PyList_New(4); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_25);
  __Pyx_INCREF(__pyx_n_s_CSI);
  __Pyx_GIVEREF(__pyx_n_s_CSI);
  PyList_SET_ITEM(__pyx_t_25, 0, __pyx_n_s_CSI);
  __Pyx_INCREF(__pyx_kp_s_l_2);
  __Pyx_GIVEREF(__pyx_kp_s_l_2);
  PyList_SET_ITEM(__pyx_t_25, 1, __pyx_kp_s_l_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_25, 2, __pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_DECRST);
  __Pyx_GIVEREF(__pyx_n_s_DECRST);
  PyList_SET_ITEM(__pyx_t_25, 3, __pyx_n_s_DECRST);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":623
 * 
 *     cdef debug_event(self,event,index):
 *         commands=[             # <<<<<<<<<<<<<<
 *                     ['CSI','A' ,[1]   ,'CUU'             ],
 *                     ['CSI','B' ,[1]   ,'CUD'             ],
 */
  __pyx_t_1 = PyList_New(24); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyList_SET_ITEM(__pyx_t_1, 6, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyList_SET_ITEM(__pyx_t_1, 7, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyList_SET_ITEM(__pyx_t_1, 8, __pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_11);
  PyList_SET_ITEM(__pyx_t_1, 9, __pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_12);
  PyList_SET_ITEM(__pyx_t_1, 10, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyList_SET_ITEM(__pyx_t_1, 11, __pyx_t_13);
  __Pyx_GIVEREF(__pyx_t_14);
  PyList_SET_ITEM(__pyx_t_1, 12, __pyx_t_14);
  __Pyx_GIVEREF(__pyx_t_15);
  PyList_SET_ITEM(__pyx_t_1, 13, __pyx_t_15);
  __Pyx_GIVEREF(__pyx_t_16);
  PyList_SET_ITEM(__pyx_t_1, 14, __pyx_t_16);
  __Pyx_GIVEREF(__pyx_t_17);
  PyList_SET_ITEM(__pyx_t_1, 15, __pyx_t_17);
  __Pyx_GIVEREF(__pyx_t_18);
  PyList_SET_ITEM(__pyx_t_1, 16, __pyx_t_18);
  __Pyx_GIVEREF(__pyx_t_19);
  PyList_SET_ITEM(__pyx_t_1, 17, __pyx_t_19);
  __Pyx_GIVEREF(__pyx_t_20);
  PyList_SET_ITEM(__pyx_t_1, 18, __pyx_t_20);
  __Pyx_GIVEREF(__pyx_t_21);
  PyList_SET_ITEM(__pyx_t_1, 19, __pyx_t_21);
  __Pyx_GIVEREF(__pyx_t_22);
  PyList_SET_ITEM(__pyx_t_1, 20, __pyx_t_22);
  __Pyx_GIVEREF(__pyx_t_23);
  PyList_SET_ITEM(__pyx_t_1, 21, __pyx_t_23);
  __Pyx_GIVEREF(__pyx_t_24);
  PyList_SET_ITEM(__pyx_t_1, 22, __pyx_t_24);
  __Pyx_GIVEREF(__pyx_t_25);
  PyList_SET_ITEM(__pyx_t_1, 23, __pyx_t_25);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_14 = 0;
  __pyx_t_15 = 0;
  __pyx_t_16 = 0;
  __pyx_t_17 = 0;
  __pyx_t_18 = 0;
  __pyx_t_19 = 0;
  __pyx_t_20 = 0;
  __pyx_t_21 = 0;
  __pyx_t_22 = 0;
  __pyx_t_23 = 0;
  __pyx_t_24 = 0;
  __pyx_t_25 = 0;
  __pyx_v_commands = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":649
 *                     ['CSI','?l',[1]   ,'DECRST'          ],
 *             ]
 *         if event['type']=='text':             # <<<<<<<<<<<<<<
 *             print("{2:6x} {3:3.5f} : text('{0},{1}')".format(self.ascii_safe(event['data']),len(event['data']),index, event['timestamp'] ) )
 *             return
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_26 = (__Pyx_PyString_Equals(__pyx_t_1, __pyx_n_s_text, Py_EQ)); if (unlikely(__pyx_t_26 < 0)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_26) {

    /* "ttygif/tty/parser.pyx":650
 *             ]
 *         if event['type']=='text':
 *             print("{2:6x} {3:3.5f} : text('{0},{1}')".format(self.ascii_safe(event['data']),len(event['data']),index, event['timestamp'] ) )             # <<<<<<<<<<<<<<
 *             return
 *         for cmd in commands:
 */
    __pyx_t_25 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_2_6x_3_3_5f_text_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_25);
    __pyx_t_24 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_data); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_24);
    __pyx_t_23 = ((struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser *)__pyx_v_self->__pyx_vtab)->ascii_safe(__pyx_v_self, __pyx_t_24); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_23);
    __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
    __pyx_t_24 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_data); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_24);
    __pyx_t_27 = PyObject_Length(__pyx_t_24); if (unlikely(__pyx_t_27 == ((Py_ssize_t)-1))) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
    __pyx_t_24 = PyInt_FromSsize_t(__pyx_t_27); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_24);
    __pyx_t_22 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_timestamp); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_22);
    __pyx_t_21 = NULL;
    __pyx_t_28 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_25))) {
      __pyx_t_21 = PyMethod_GET_SELF(__pyx_t_25);
      if (likely(__pyx_t_21)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_25);
        __Pyx_INCREF(__pyx_t_21);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_25, function);
        __pyx_t_28 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_25)) {
      PyObject *__pyx_temp[5] = {__pyx_t_21, __pyx_t_23, __pyx_t_24, __pyx_v_index, __pyx_t_22};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_25, __pyx_temp+1-__pyx_t_28, 4+__pyx_t_28); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 650, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
      __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
      __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_25)) {
      PyObject *__pyx_temp[5] = {__pyx_t_21, __pyx_t_23, __pyx_t_24, __pyx_v_index, __pyx_t_22};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_25, __pyx_temp+1-__pyx_t_28, 4+__pyx_t_28); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 650, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
      __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
      __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
    } else
    #endif
    {
      __pyx_t_20 = PyTuple_New(4+__pyx_t_28); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 650, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_20);
      if (__pyx_t_21) {
        __Pyx_GIVEREF(__pyx_t_21); PyTuple_SET_ITEM(__pyx_t_20, 0, __pyx_t_21); __pyx_t_21 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_23);
      PyTuple_SET_ITEM(__pyx_t_20, 0+__pyx_t_28, __pyx_t_23);
      __Pyx_GIVEREF(__pyx_t_24);
      PyTuple_SET_ITEM(__pyx_t_20, 1+__pyx_t_28, __pyx_t_24);
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_20, 2+__pyx_t_28, __pyx_v_index);
      __Pyx_GIVEREF(__pyx_t_22);
      PyTuple_SET_ITEM(__pyx_t_20, 3+__pyx_t_28, __pyx_t_22);
      __pyx_t_23 = 0;
      __pyx_t_24 = 0;
      __pyx_t_22 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_25, __pyx_t_20, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 650, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
    }
    __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
    if (__Pyx_PrintOne(0, __pyx_t_1) < 0) __PYX_ERR(0, 650, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ttygif/tty/parser.pyx":651
 *         if event['type']=='text':
 *             print("{2:6x} {3:3.5f} : text('{0},{1}')".format(self.ascii_safe(event['data']),len(event['data']),index, event['timestamp'] ) )
 *             return             # <<<<<<<<<<<<<<
 *         for cmd in commands:
 *             if cmd[1]==event['command'] and event['esc_type']==cmd[0]:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "ttygif/tty/parser.pyx":649
 *                     ['CSI','?l',[1]   ,'DECRST'          ],
 *             ]
 *         if event['type']=='text':             # <<<<<<<<<<<<<<
 *             print("{2:6x} {3:3.5f} : text('{0},{1}')".format(self.ascii_safe(event['data']),len(event['data']),index, event['timestamp'] ) )
 *             return
 */
  }

  /* "ttygif/tty/parser.pyx":652
 *             print("{2:6x} {3:3.5f} : text('{0},{1}')".format(self.ascii_safe(event['data']),len(event['data']),index, event['timestamp'] ) )
 *             return
 *         for cmd in commands:             # <<<<<<<<<<<<<<
 *             if cmd[1]==event['command'] and event['esc_type']==cmd[0]:
 *                 param=[]
 */
  __pyx_t_1 = __pyx_v_commands; __Pyx_INCREF(__pyx_t_1); __pyx_t_27 = 0;
  for (;;) {
    if (__pyx_t_27 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_25 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_27); __Pyx_INCREF(__pyx_t_25); __pyx_t_27++; if (unlikely(0 < 0)) __PYX_ERR(0, 652, __pyx_L1_error)
    #else
    __pyx_t_25 = PySequence_ITEM(__pyx_t_1, __pyx_t_27); __pyx_t_27++; if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 652, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_25);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_cmd, __pyx_t_25);
    __pyx_t_25 = 0;

    /* "ttygif/tty/parser.pyx":653
 *             return
 *         for cmd in commands:
 *             if cmd[1]==event['command'] and event['esc_type']==cmd[0]:             # <<<<<<<<<<<<<<
 *                 param=[]
 *                 for i in cmd[2]:
 */
    __pyx_t_25 = __Pyx_GetItemInt(__pyx_v_cmd, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 653, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_25);
    __pyx_t_20 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_command); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 653, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_20);
    __pyx_t_22 = PyObject_RichCompare(__pyx_t_25, __pyx_t_20, Py_EQ); __Pyx_XGOTREF(__pyx_t_22); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 653, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
    __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
    __pyx_t_29 = __Pyx_PyObject_IsTrue(__pyx_t_22); if (unlikely(__pyx_t_29 < 0)) __PYX_ERR(0, 653, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
    if (__pyx_t_29) {
    } else {
      __pyx_t_26 = __pyx_t_29;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_22 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_esc_type); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 653, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_22);
    __pyx_t_20 = __Pyx_GetItemInt(__pyx_v_cmd, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 653, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_20);
    __pyx_t_25 = PyObject_RichCompare(__pyx_t_22, __pyx_t_20, Py_EQ); __Pyx_XGOTREF(__pyx_t_25); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 653, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
    __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
    __pyx_t_29 = __Pyx_PyObject_IsTrue(__pyx_t_25); if (unlikely(__pyx_t_29 < 0)) __PYX_ERR(0, 653, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
    __pyx_t_26 = __pyx_t_29;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_26) {

      /* "ttygif/tty/parser.pyx":654
 *         for cmd in commands:
 *             if cmd[1]==event['command'] and event['esc_type']==cmd[0]:
 *                 param=[]             # <<<<<<<<<<<<<<
 *                 for i in cmd[2]:
 * 
 */
      __pyx_t_25 = PyList_New(0); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 654, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_25);
      __pyx_v_param = ((PyObject*)__pyx_t_25);
      __pyx_t_25 = 0;

      /* "ttygif/tty/parser.pyx":655
 *             if cmd[1]==event['command'] and event['esc_type']==cmd[0]:
 *                 param=[]
 *                 for i in cmd[2]:             # <<<<<<<<<<<<<<
 * 
 *                     if i==0: param.append( "" )
 */
      __pyx_t_25 = __Pyx_GetItemInt(__pyx_v_cmd, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 655, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_25);
      if (likely(PyList_CheckExact(__pyx_t_25)) || PyTuple_CheckExact(__pyx_t_25)) {
        __pyx_t_20 = __pyx_t_25; __Pyx_INCREF(__pyx_t_20); __pyx_t_30 = 0;
        __pyx_t_31 = NULL;
      } else {
        __pyx_t_30 = -1; __pyx_t_20 = PyObject_GetIter(__pyx_t_25); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 655, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_20);
        __pyx_t_31 = Py_TYPE(__pyx_t_20)->tp_iternext; if (unlikely(!__pyx_t_31)) __PYX_ERR(0, 655, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
      for (;;) {
        if (likely(!__pyx_t_31)) {
          if (likely(PyList_CheckExact(__pyx_t_20))) {
            if (__pyx_t_30 >= PyList_GET_SIZE(__pyx_t_20)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_25 = PyList_GET_ITEM(__pyx_t_20, __pyx_t_30); __Pyx_INCREF(__pyx_t_25); __pyx_t_30++; if (unlikely(0 < 0)) __PYX_ERR(0, 655, __pyx_L1_error)
            #else
            __pyx_t_25 = PySequence_ITEM(__pyx_t_20, __pyx_t_30); __pyx_t_30++; if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 655, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_25);
            #endif
          } else {
            if (__pyx_t_30 >= PyTuple_GET_SIZE(__pyx_t_20)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_25 = PyTuple_GET_ITEM(__pyx_t_20, __pyx_t_30); __Pyx_INCREF(__pyx_t_25); __pyx_t_30++; if (unlikely(0 < 0)) __PYX_ERR(0, 655, __pyx_L1_error)
            #else
            __pyx_t_25 = PySequence_ITEM(__pyx_t_20, __pyx_t_30); __pyx_t_30++; if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 655, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_25);
            #endif
          }
        } else {
          __pyx_t_25 = __pyx_t_31(__pyx_t_20);
          if (unlikely(!__pyx_t_25)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 655, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_25);
        }
        __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_25);
        __pyx_t_25 = 0;

        /* "ttygif/tty/parser.pyx":657
 *                 for i in cmd[2]:
 * 
 *                     if i==0: param.append( "" )             # <<<<<<<<<<<<<<
 * 
 *                     if i==1:
 */
        __pyx_t_25 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 657, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_25);
        __pyx_t_26 = __Pyx_PyObject_IsTrue(__pyx_t_25); if (unlikely(__pyx_t_26 < 0)) __PYX_ERR(0, 657, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
        if (__pyx_t_26) {
          __pyx_t_32 = __Pyx_PyList_Append(__pyx_v_param, __pyx_kp_s_); if (unlikely(__pyx_t_32 == ((int)-1))) __PYX_ERR(0, 657, __pyx_L1_error)
        }

        /* "ttygif/tty/parser.pyx":659
 *                     if i==0: param.append( "" )
 * 
 *                     if i==1:             # <<<<<<<<<<<<<<
 *                         if len(event['params'])>=1:
 *                             param.append( "{0}".format(event['params'][0])   )
 */
        __pyx_t_25 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 659, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_25);
        __pyx_t_26 = __Pyx_PyObject_IsTrue(__pyx_t_25); if (unlikely(__pyx_t_26 < 0)) __PYX_ERR(0, 659, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
        if (__pyx_t_26) {

          /* "ttygif/tty/parser.pyx":660
 * 
 *                     if i==1:
 *                         if len(event['params'])>=1:             # <<<<<<<<<<<<<<
 *                             param.append( "{0}".format(event['params'][0])   )
 *                         else:
 */
          __pyx_t_25 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_params); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 660, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_25);
          __pyx_t_33 = PyObject_Length(__pyx_t_25); if (unlikely(__pyx_t_33 == ((Py_ssize_t)-1))) __PYX_ERR(0, 660, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
          __pyx_t_26 = ((__pyx_t_33 >= 1) != 0);
          if (__pyx_t_26) {

            /* "ttygif/tty/parser.pyx":661
 *                     if i==1:
 *                         if len(event['params'])>=1:
 *                             param.append( "{0}".format(event['params'][0])   )             # <<<<<<<<<<<<<<
 *                         else:
 *                             param.append( "ERR" )
 */
            __pyx_t_22 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_0, __pyx_n_s_format); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 661, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_22);
            __pyx_t_24 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_params); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 661, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_24);
            __pyx_t_23 = __Pyx_GetItemInt(__pyx_t_24, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 661, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_23);
            __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
            __pyx_t_24 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_22))) {
              __pyx_t_24 = PyMethod_GET_SELF(__pyx_t_22);
              if (likely(__pyx_t_24)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_22);
                __Pyx_INCREF(__pyx_t_24);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_22, function);
              }
            }
            __pyx_t_25 = (__pyx_t_24) ? __Pyx_PyObject_Call2Args(__pyx_t_22, __pyx_t_24, __pyx_t_23) : __Pyx_PyObject_CallOneArg(__pyx_t_22, __pyx_t_23);
            __Pyx_XDECREF(__pyx_t_24); __pyx_t_24 = 0;
            __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
            if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 661, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_25);
            __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
            __pyx_t_32 = __Pyx_PyList_Append(__pyx_v_param, __pyx_t_25); if (unlikely(__pyx_t_32 == ((int)-1))) __PYX_ERR(0, 661, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;

            /* "ttygif/tty/parser.pyx":660
 * 
 *                     if i==1:
 *                         if len(event['params'])>=1:             # <<<<<<<<<<<<<<
 *                             param.append( "{0}".format(event['params'][0])   )
 *                         else:
 */
            goto __pyx_L13;
          }

          /* "ttygif/tty/parser.pyx":663
 *                             param.append( "{0}".format(event['params'][0])   )
 *                         else:
 *                             param.append( "ERR" )             # <<<<<<<<<<<<<<
 *                     if i==2:
 *                         if len(event['params'])>=2:
 */
          /*else*/ {
            __pyx_t_32 = __Pyx_PyList_Append(__pyx_v_param, __pyx_n_s_ERR); if (unlikely(__pyx_t_32 == ((int)-1))) __PYX_ERR(0, 663, __pyx_L1_error)
          }
          __pyx_L13:;

          /* "ttygif/tty/parser.pyx":659
 *                     if i==0: param.append( "" )
 * 
 *                     if i==1:             # <<<<<<<<<<<<<<
 *                         if len(event['params'])>=1:
 *                             param.append( "{0}".format(event['params'][0])   )
 */
        }

        /* "ttygif/tty/parser.pyx":664
 *                         else:
 *                             param.append( "ERR" )
 *                     if i==2:             # <<<<<<<<<<<<<<
 *                         if len(event['params'])>=2:
 *                             param.append( "{0}".format(event['params'][1])   )
 */
        __pyx_t_25 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 664, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_25);
        __pyx_t_26 = __Pyx_PyObject_IsTrue(__pyx_t_25); if (unlikely(__pyx_t_26 < 0)) __PYX_ERR(0, 664, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
        if (__pyx_t_26) {

          /* "ttygif/tty/parser.pyx":665
 *                             param.append( "ERR" )
 *                     if i==2:
 *                         if len(event['params'])>=2:             # <<<<<<<<<<<<<<
 *                             param.append( "{0}".format(event['params'][1])   )
 *                         else:
 */
          __pyx_t_25 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_params); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 665, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_25);
          __pyx_t_33 = PyObject_Length(__pyx_t_25); if (unlikely(__pyx_t_33 == ((Py_ssize_t)-1))) __PYX_ERR(0, 665, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
          __pyx_t_26 = ((__pyx_t_33 >= 2) != 0);
          if (__pyx_t_26) {

            /* "ttygif/tty/parser.pyx":666
 *                     if i==2:
 *                         if len(event['params'])>=2:
 *                             param.append( "{0}".format(event['params'][1])   )             # <<<<<<<<<<<<<<
 *                         else:
 *                             param.append( "ERR" )
 */
            __pyx_t_22 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_0, __pyx_n_s_format); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 666, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_22);
            __pyx_t_23 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_params); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 666, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_23);
            __pyx_t_24 = __Pyx_GetItemInt(__pyx_t_23, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 666, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_24);
            __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
            __pyx_t_23 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_22))) {
              __pyx_t_23 = PyMethod_GET_SELF(__pyx_t_22);
              if (likely(__pyx_t_23)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_22);
                __Pyx_INCREF(__pyx_t_23);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_22, function);
              }
            }
            __pyx_t_25 = (__pyx_t_23) ? __Pyx_PyObject_Call2Args(__pyx_t_22, __pyx_t_23, __pyx_t_24) : __Pyx_PyObject_CallOneArg(__pyx_t_22, __pyx_t_24);
            __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
            __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
            if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 666, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_25);
            __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
            __pyx_t_32 = __Pyx_PyList_Append(__pyx_v_param, __pyx_t_25); if (unlikely(__pyx_t_32 == ((int)-1))) __PYX_ERR(0, 666, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;

            /* "ttygif/tty/parser.pyx":665
 *                             param.append( "ERR" )
 *                     if i==2:
 *                         if len(event['params'])>=2:             # <<<<<<<<<<<<<<
 *                             param.append( "{0}".format(event['params'][1])   )
 *                         else:
 */
            goto __pyx_L15;
          }

          /* "ttygif/tty/parser.pyx":668
 *                             param.append( "{0}".format(event['params'][1])   )
 *                         else:
 *                             param.append( "ERR" )             # <<<<<<<<<<<<<<
 *                     if i==3:
 *                         if len(event['params'])>=1:
 */
          /*else*/ {
            __pyx_t_32 = __Pyx_PyList_Append(__pyx_v_param, __pyx_n_s_ERR); if (unlikely(__pyx_t_32 == ((int)-1))) __PYX_ERR(0, 668, __pyx_L1_error)
          }
          __pyx_L15:;

          /* "ttygif/tty/parser.pyx":664
 *                         else:
 *                             param.append( "ERR" )
 *                     if i==2:             # <<<<<<<<<<<<<<
 *                         if len(event['params'])>=2:
 *                             param.append( "{0}".format(event['params'][1])   )
 */
        }

        /* "ttygif/tty/parser.pyx":669
 *                         else:
 *                             param.append( "ERR" )
 *                     if i==3:             # <<<<<<<<<<<<<<
 *                         if len(event['params'])>=1:
 *                             param.append( "{0}".format(event['params'][0]-1) )
 */
        __pyx_t_25 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_3, 3, 0); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 669, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_25);
        __pyx_t_26 = __Pyx_PyObject_IsTrue(__pyx_t_25); if (unlikely(__pyx_t_26 < 0)) __PYX_ERR(0, 669, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
        if (__pyx_t_26) {

          /* "ttygif/tty/parser.pyx":670
 *                             param.append( "ERR" )
 *                     if i==3:
 *                         if len(event['params'])>=1:             # <<<<<<<<<<<<<<
 *                             param.append( "{0}".format(event['params'][0]-1) )
 *                         else:
 */
          __pyx_t_25 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_params); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 670, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_25);
          __pyx_t_33 = PyObject_Length(__pyx_t_25); if (unlikely(__pyx_t_33 == ((Py_ssize_t)-1))) __PYX_ERR(0, 670, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
          __pyx_t_26 = ((__pyx_t_33 >= 1) != 0);
          if (__pyx_t_26) {

            /* "ttygif/tty/parser.pyx":671
 *                     if i==3:
 *                         if len(event['params'])>=1:
 *                             param.append( "{0}".format(event['params'][0]-1) )             # <<<<<<<<<<<<<<
 *                         else:
 *                             param.append( "ERR" )
 */
            __pyx_t_22 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_0, __pyx_n_s_format); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 671, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_22);
            __pyx_t_24 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_params); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 671, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_24);
            __pyx_t_23 = __Pyx_GetItemInt(__pyx_t_24, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 671, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_23);
            __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
            __pyx_t_24 = __Pyx_PyInt_SubtractObjC(__pyx_t_23, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 671, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_24);
            __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
            __pyx_t_23 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_22))) {
              __pyx_t_23 = PyMethod_GET_SELF(__pyx_t_22);
              if (likely(__pyx_t_23)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_22);
                __Pyx_INCREF(__pyx_t_23);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_22, function);
              }
            }
            __pyx_t_25 = (__pyx_t_23) ? __Pyx_PyObject_Call2Args(__pyx_t_22, __pyx_t_23, __pyx_t_24) : __Pyx_PyObject_CallOneArg(__pyx_t_22, __pyx_t_24);
            __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
            __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
            if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 671, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_25);
            __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
            __pyx_t_32 = __Pyx_PyList_Append(__pyx_v_param, __pyx_t_25); if (unlikely(__pyx_t_32 == ((int)-1))) __PYX_ERR(0, 671, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;

            /* "ttygif/tty/parser.pyx":670
 *                             param.append( "ERR" )
 *                     if i==3:
 *                         if len(event['params'])>=1:             # <<<<<<<<<<<<<<
 *                             param.append( "{0}".format(event['params'][0]-1) )
 *                         else:
 */
            goto __pyx_L17;
          }

          /* "ttygif/tty/parser.pyx":673
 *                             param.append( "{0}".format(event['params'][0]-1) )
 *                         else:
 *                             param.append( "ERR" )             # <<<<<<<<<<<<<<
 *                     if i==4:
 *                         if len(event['params'])>=2:
 */
          /*else*/ {
            __pyx_t_32 = __Pyx_PyList_Append(__pyx_v_param, __pyx_n_s_ERR); if (unlikely(__pyx_t_32 == ((int)-1))) __PYX_ERR(0, 673, __pyx_L1_error)
          }
          __pyx_L17:;

          /* "ttygif/tty/parser.pyx":669
 *                         else:
 *                             param.append( "ERR" )
 *                     if i==3:             # <<<<<<<<<<<<<<
 *                         if len(event['params'])>=1:
 *                             param.append( "{0}".format(event['params'][0]-1) )
 */
        }

        /* "ttygif/tty/parser.pyx":674
 *                         else:
 *                             param.append( "ERR" )
 *                     if i==4:             # <<<<<<<<<<<<<<
 *                         if len(event['params'])>=2:
 *                             param.append( "{0}".format(event['params'][1]-1) )
 */
        __pyx_t_25 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_4, 4, 0); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 674, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_25);
        __pyx_t_26 = __Pyx_PyObject_IsTrue(__pyx_t_25); if (unlikely(__pyx_t_26 < 0)) __PYX_ERR(0, 674, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
        if (__pyx_t_26) {

          /* "ttygif/tty/parser.pyx":675
 *                             param.append( "ERR" )
 *                     if i==4:
 *                         if len(event['params'])>=2:             # <<<<<<<<<<<<<<
 *                             param.append( "{0}".format(event['params'][1]-1) )
 *                         else:
 */
          __pyx_t_25 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_params); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 675, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_25);
          __pyx_t_33 = PyObject_Length(__pyx_t_25); if (unlikely(__pyx_t_33 == ((Py_ssize_t)-1))) __PYX_ERR(0, 675, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
          __pyx_t_26 = ((__pyx_t_33 >= 2) != 0);
          if (__pyx_t_26) {

            /* "ttygif/tty/parser.pyx":676
 *                     if i==4:
 *                         if len(event['params'])>=2:
 *                             param.append( "{0}".format(event['params'][1]-1) )             # <<<<<<<<<<<<<<
 *                         else:
 *                             param.append( "ERR" )
 */
            __pyx_t_22 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_0, __pyx_n_s_format); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 676, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_22);
            __pyx_t_24 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_params); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 676, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_24);
            __pyx_t_23 = __Pyx_GetItemInt(__pyx_t_24, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 676, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_23);
            __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
            __pyx_t_24 = __Pyx_PyInt_SubtractObjC(__pyx_t_23, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 676, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_24);
            __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
            __pyx_t_23 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_22))) {
              __pyx_t_23 = PyMethod_GET_SELF(__pyx_t_22);
              if (likely(__pyx_t_23)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_22);
                __Pyx_INCREF(__pyx_t_23);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_22, function);
              }
            }
            __pyx_t_25 = (__pyx_t_23) ? __Pyx_PyObject_Call2Args(__pyx_t_22, __pyx_t_23, __pyx_t_24) : __Pyx_PyObject_CallOneArg(__pyx_t_22, __pyx_t_24);
            __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
            __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
            if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 676, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_25);
            __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
            __pyx_t_32 = __Pyx_PyList_Append(__pyx_v_param, __pyx_t_25); if (unlikely(__pyx_t_32 == ((int)-1))) __PYX_ERR(0, 676, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;

            /* "ttygif/tty/parser.pyx":675
 *                             param.append( "ERR" )
 *                     if i==4:
 *                         if len(event['params'])>=2:             # <<<<<<<<<<<<<<
 *                             param.append( "{0}".format(event['params'][1]-1) )
 *                         else:
 */
            goto __pyx_L19;
          }

          /* "ttygif/tty/parser.pyx":678
 *                             param.append( "{0}".format(event['params'][1]-1) )
 *                         else:
 *                             param.append( "ERR" )             # <<<<<<<<<<<<<<
 * 
 *                 print("{2: 6x} {3:3.5f} : {0}({1})".format(cmd[3],",".join(param),index,event['timestamp']))
 */
          /*else*/ {
            __pyx_t_32 = __Pyx_PyList_Append(__pyx_v_param, __pyx_n_s_ERR); if (unlikely(__pyx_t_32 == ((int)-1))) __PYX_ERR(0, 678, __pyx_L1_error)
          }
          __pyx_L19:;

          /* "ttygif/tty/parser.pyx":674
 *                         else:
 *                             param.append( "ERR" )
 *                     if i==4:             # <<<<<<<<<<<<<<
 *                         if len(event['params'])>=2:
 *                             param.append( "{0}".format(event['params'][1]-1) )
 */
        }

        /* "ttygif/tty/parser.pyx":655
 *             if cmd[1]==event['command'] and event['esc_type']==cmd[0]:
 *                 param=[]
 *                 for i in cmd[2]:             # <<<<<<<<<<<<<<
 * 
 *                     if i==0: param.append( "" )
 */
      }
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;

      /* "ttygif/tty/parser.pyx":680
 *                             param.append( "ERR" )
 * 
 *                 print("{2: 6x} {3:3.5f} : {0}({1})".format(cmd[3],",".join(param),index,event['timestamp']))             # <<<<<<<<<<<<<<
 *                 return
 * 
 */
      __pyx_t_25 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_2_6x_3_3_5f_0_1, __pyx_n_s_format); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 680, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_25);
      __pyx_t_22 = __Pyx_GetItemInt(__pyx_v_cmd, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 680, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_22);
      __pyx_t_24 = __Pyx_PyString_Join(__pyx_kp_s__17, __pyx_v_param); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 680, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_24);
      __pyx_t_23 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_timestamp); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 680, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_23);
      __pyx_t_21 = NULL;
      __pyx_t_28 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_25))) {
        __pyx_t_21 = PyMethod_GET_SELF(__pyx_t_25);
        if (likely(__pyx_t_21)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_25);
          __Pyx_INCREF(__pyx_t_21);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_25, function);
          __pyx_t_28 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_25)) {
        PyObject *__pyx_temp[5] = {__pyx_t_21, __pyx_t_22, __pyx_t_24, __pyx_v_index, __pyx_t_23};
        __pyx_t_20 = __Pyx_PyFunction_FastCall(__pyx_t_25, __pyx_temp+1-__pyx_t_28, 4+__pyx_t_28); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 680, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
        __Pyx_GOTREF(__pyx_t_20);
        __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
        __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
        __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_25)) {
        PyObject *__pyx_temp[5] = {__pyx_t_21, __pyx_t_22, __pyx_t_24, __pyx_v_index, __pyx_t_23};
        __pyx_t_20 = __Pyx_PyCFunction_FastCall(__pyx_t_25, __pyx_temp+1-__pyx_t_28, 4+__pyx_t_28); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 680, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
        __Pyx_GOTREF(__pyx_t_20);
        __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
        __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
        __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
      } else
      #endif
      {
        __pyx_t_19 = PyTuple_New(4+__pyx_t_28); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 680, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_19);
        if (__pyx_t_21) {
          __Pyx_GIVEREF(__pyx_t_21); PyTuple_SET_ITEM(__pyx_t_19, 0, __pyx_t_21); __pyx_t_21 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_22);
        PyTuple_SET_ITEM(__pyx_t_19, 0+__pyx_t_28, __pyx_t_22);
        __Pyx_GIVEREF(__pyx_t_24);
        PyTuple_SET_ITEM(__pyx_t_19, 1+__pyx_t_28, __pyx_t_24);
        __Pyx_INCREF(__pyx_v_index);
        __Pyx_GIVEREF(__pyx_v_index);
        PyTuple_SET_ITEM(__pyx_t_19, 2+__pyx_t_28, __pyx_v_index);
        __Pyx_GIVEREF(__pyx_t_23);
        PyTuple_SET_ITEM(__pyx_t_19, 3+__pyx_t_28, __pyx_t_23);
        __pyx_t_22 = 0;
        __pyx_t_24 = 0;
        __pyx_t_23 = 0;
        __pyx_t_20 = __Pyx_PyObject_Call(__pyx_t_25, __pyx_t_19, NULL); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 680, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_20);
        __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      }
      __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
      if (__Pyx_PrintOne(0, __pyx_t_20) < 0) __PYX_ERR(0, 680, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;

      /* "ttygif/tty/parser.pyx":681
 * 
 *                 print("{2: 6x} {3:3.5f} : {0}({1})".format(cmd[3],",".join(param),index,event['timestamp']))
 *                 return             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "ttygif/tty/parser.pyx":653
 *             return
 *         for cmd in commands:
 *             if cmd[1]==event['command'] and event['esc_type']==cmd[0]:             # <<<<<<<<<<<<<<
 *                 param=[]
 *                 for i in cmd[2]:
 */
    }

    /* "ttygif/tty/parser.pyx":652
 *             print("{2:6x} {3:3.5f} : text('{0},{1}')".format(self.ascii_safe(event['data']),len(event['data']),index, event['timestamp'] ) )
 *             return
 *         for cmd in commands:             # <<<<<<<<<<<<<<
 *             if cmd[1]==event['command'] and event['esc_type']==cmd[0]:
 *                 param=[]
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":684
 * 
 * 
 *         print("{5:05x} CMD:  '{0}', Name:'{3}', Command:{1}, Params:{2}  Timestamp:{4}".format(event['esc_type'],             # <<<<<<<<<<<<<<
 *                                             event['command'],
 *                                             event['params'],
 */
  __pyx_t_20 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_5_05x_CMD_0_Name_3_Command_1_Pa, __pyx_n_s_format); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 684, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_20);
  __pyx_t_25 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_esc_type); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 684, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_25);

  /* "ttygif/tty/parser.pyx":685
 * 
 *         print("{5:05x} CMD:  '{0}', Name:'{3}', Command:{1}, Params:{2}  Timestamp:{4}".format(event['esc_type'],
 *                                             event['command'],             # <<<<<<<<<<<<<<
 *                                             event['params'],
 *                                             event['name'],
 */
  __pyx_t_19 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_command); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 685, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_19);

  /* "ttygif/tty/parser.pyx":686
 *         print("{5:05x} CMD:  '{0}', Name:'{3}', Command:{1}, Params:{2}  Timestamp:{4}".format(event['esc_type'],
 *                                             event['command'],
 *                                             event['params'],             # <<<<<<<<<<<<<<
 *                                             event['name'],
 *                                             event['timestamp'],index))
 */
  __pyx_t_23 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_params); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 686, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_23);

  /* "ttygif/tty/parser.pyx":687
 *                                             event['command'],
 *                                             event['params'],
 *                                             event['name'],             # <<<<<<<<<<<<<<
 *                                             event['timestamp'],index))
 * 
 */
  __pyx_t_24 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_name); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 687, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_24);

  /* "ttygif/tty/parser.pyx":688
 *                                             event['params'],
 *                                             event['name'],
 *                                             event['timestamp'],index))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_22 = __Pyx_PyObject_Dict_GetItem(__pyx_v_event, __pyx_n_s_timestamp); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_22);
  __pyx_t_21 = NULL;
  __pyx_t_28 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_20))) {
    __pyx_t_21 = PyMethod_GET_SELF(__pyx_t_20);
    if (likely(__pyx_t_21)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_20);
      __Pyx_INCREF(__pyx_t_21);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_20, function);
      __pyx_t_28 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_20)) {
    PyObject *__pyx_temp[7] = {__pyx_t_21, __pyx_t_25, __pyx_t_19, __pyx_t_23, __pyx_t_24, __pyx_t_22, __pyx_v_index};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_20, __pyx_temp+1-__pyx_t_28, 6+__pyx_t_28); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 684, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
    __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
    __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
    __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_20)) {
    PyObject *__pyx_temp[7] = {__pyx_t_21, __pyx_t_25, __pyx_t_19, __pyx_t_23, __pyx_t_24, __pyx_t_22, __pyx_v_index};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_20, __pyx_temp+1-__pyx_t_28, 6+__pyx_t_28); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 684, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
    __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
    __Pyx_DECREF(__pyx_t_23); __pyx_t_23 = 0;
    __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
    __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
  } else
  #endif
  {
    __pyx_t_18 = PyTuple_New(6+__pyx_t_28); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 684, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_18);
    if (__pyx_t_21) {
      __Pyx_GIVEREF(__pyx_t_21); PyTuple_SET_ITEM(__pyx_t_18, 0, __pyx_t_21); __pyx_t_21 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_25);
    PyTuple_SET_ITEM(__pyx_t_18, 0+__pyx_t_28, __pyx_t_25);
    __Pyx_GIVEREF(__pyx_t_19);
    PyTuple_SET_ITEM(__pyx_t_18, 1+__pyx_t_28, __pyx_t_19);
    __Pyx_GIVEREF(__pyx_t_23);
    PyTuple_SET_ITEM(__pyx_t_18, 2+__pyx_t_28, __pyx_t_23);
    __Pyx_GIVEREF(__pyx_t_24);
    PyTuple_SET_ITEM(__pyx_t_18, 3+__pyx_t_28, __pyx_t_24);
    __Pyx_GIVEREF(__pyx_t_22);
    PyTuple_SET_ITEM(__pyx_t_18, 4+__pyx_t_28, __pyx_t_22);
    __Pyx_INCREF(__pyx_v_index);
    __Pyx_GIVEREF(__pyx_v_index);
    PyTuple_SET_ITEM(__pyx_t_18, 5+__pyx_t_28, __pyx_v_index);
    __pyx_t_25 = 0;
    __pyx_t_19 = 0;
    __pyx_t_23 = 0;
    __pyx_t_24 = 0;
    __pyx_t_22 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_20, __pyx_t_18, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 684, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
  }
  __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
  if (__Pyx_PrintOne(0, __pyx_t_1) < 0) __PYX_ERR(0, 684, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":622
 * 
 * 
 *     cdef debug_event(self,event,index):             # <<<<<<<<<<<<<<
 *         commands=[
 *                     ['CSI','A' ,[1]   ,'CUU'             ],
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_21);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_XDECREF(__pyx_t_23);
  __Pyx_XDECREF(__pyx_t_24);
  __Pyx_XDECREF(__pyx_t_25);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.debug_event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_commands);
  __Pyx_XDECREF(__pyx_v_cmd);
  __Pyx_XDECREF(__pyx_v_param);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ttygif_3tty_6parser_11term_parser_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_6ttygif_3tty_6parser_11term_parser_3__reduce_cython__ = {"__reduce_cython__", (PyCFunction)__pyx_pw_6ttygif_3tty_6parser_11term_parser_3__reduce_cython__, METH_NOARGS, 0};
static PyObject *__pyx_pw_6ttygif_3tty_6parser_11term_parser_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ttygif_3tty_6parser_11term_parser_2__reduce_cython__(((struct __pyx_obj_6ttygif_3tty_6parser_term_parser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ttygif_3tty_6parser_11term_parser_2__reduce_cython__(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_TraceFrameInit(__pyx_codeobj__18)
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);
  __Pyx_TraceCall("__reduce_cython__", __pyx_f[1], 1, 0, __PYX_ERR(1, 1, __pyx_L1_error));

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.bracketed_paste, self.current_sequence_position, self.debug_mode, self.extra_text, self.g, self.last_timestamp, self.no_codes, self.sequence, self.sequence_pos)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->last_timestamp); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->sequence_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(9); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->bracketed_paste);
  __Pyx_GIVEREF(__pyx_v_self->bracketed_paste);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->bracketed_paste);
  __Pyx_INCREF(__pyx_v_self->current_sequence_position);
  __Pyx_GIVEREF(__pyx_v_self->current_sequence_position);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_self->current_sequence_position);
  __Pyx_INCREF(__pyx_v_self->debug_mode);
  __Pyx_GIVEREF(__pyx_v_self->debug_mode);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_self->debug_mode);
  __Pyx_INCREF(__pyx_v_self->extra_text);
  __Pyx_GIVEREF(__pyx_v_self->extra_text);
  PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_v_self->extra_text);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->g));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->g));
  PyTuple_SET_ITEM(__pyx_t_3, 4, ((PyObject *)__pyx_v_self->g));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->no_codes);
  __Pyx_GIVEREF(__pyx_v_self->no_codes);
  PyTuple_SET_ITEM(__pyx_t_3, 6, __pyx_v_self->no_codes);
  __Pyx_INCREF(__pyx_v_self->sequence);
  __Pyx_GIVEREF(__pyx_v_self->sequence);
  PyTuple_SET_ITEM(__pyx_t_3, 7, __pyx_v_self->sequence);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 8, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.bracketed_paste, self.current_sequence_position, self.debug_mode, self.extra_text, self.g, self.last_timestamp, self.no_codes, self.sequence, self.sequence_pos)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self.bracketed_paste, self.current_sequence_position, self.debug_mode, self.extra_text, self.g, self.last_timestamp, self.no_codes, self.sequence, self.sequence_pos)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.bracketed_paste is not None or self.current_sequence_position is not None or self.debug_mode is not None or self.extra_text is not None or self.g is not None or self.no_codes is not None or self.sequence is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.bracketed_paste, self.current_sequence_position, self.debug_mode, self.extra_text, self.g, self.last_timestamp, self.no_codes, self.sequence, self.sequence_pos)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.bracketed_paste is not None or self.current_sequence_position is not None or self.debug_mode is not None or self.extra_text is not None or self.g is not None or self.no_codes is not None or self.sequence is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_term_parser, (type(self), 0x35256c6, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->bracketed_paste != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->current_sequence_position != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->debug_mode != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->extra_text != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (((PyObject *)__pyx_v_self->g) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->no_codes != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->sequence != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    __pyx_t_5 = __pyx_t_6;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.bracketed_paste is not None or self.current_sequence_position is not None or self.debug_mode is not None or self.extra_text is not None or self.g is not None or self.no_codes is not None or self.sequence is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_term_parser, (type(self), 0x35256c6, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self.bracketed_paste is not None or self.current_sequence_position is not None or self.debug_mode is not None or self.extra_text is not None or self.g is not None or self.no_codes is not None or self.sequence is not None
 *     if use_setstate:
 *         return __pyx_unpickle_term_parser, (type(self), 0x35256c6, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_term_parser, (type(self), 0x35256c6, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_term_parser); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55727814);
    __Pyx_GIVEREF(__pyx_int_55727814);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_55727814);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.bracketed_paste is not None or self.current_sequence_position is not None or self.debug_mode is not None or self.extra_text is not None or self.g is not None or self.no_codes is not None or self.sequence is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_term_parser, (type(self), 0x35256c6, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_term_parser, (type(self), 0x35256c6, None), state
 *     else:
 *         return __pyx_unpickle_term_parser, (type(self), 0x35256c6, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_term_parser__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_term_parser); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_55727814);
    __Pyx_GIVEREF(__pyx_int_55727814);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_55727814);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_term_parser, (type(self), 0x35256c6, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_term_parser__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ttygif_3tty_6parser_11term_parser_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyMethodDef __pyx_mdef_6ttygif_3tty_6parser_11term_parser_5__setstate_cython__ = {"__setstate_cython__", (PyCFunction)__pyx_pw_6ttygif_3tty_6parser_11term_parser_5__setstate_cython__, METH_O, 0};
static PyObject *__pyx_pw_6ttygif_3tty_6parser_11term_parser_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6ttygif_3tty_6parser_11term_parser_4__setstate_cython__(((struct __pyx_obj_6ttygif_3tty_6parser_term_parser *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ttygif_3tty_6parser_11term_parser_4__setstate_cython__(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_TraceFrameInit(__pyx_codeobj__19)
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);
  __Pyx_TraceCall("__setstate_cython__", __pyx_f[1], 16, 0, __PYX_ERR(1, 16, __pyx_L1_error));

  /* "(tree fragment)":17
 *         return __pyx_unpickle_term_parser, (type(self), 0x35256c6, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_term_parser__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_6ttygif_3tty_6parser___pyx_unpickle_term_parser__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_term_parser, (type(self), 0x35256c6, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_term_parser__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ttygif.tty.parser.term_parser.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_term_parser(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_6ttygif_3tty_6parser_1__pyx_unpickle_term_parser(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_6ttygif_3tty_6parser_1__pyx_unpickle_term_parser = {"__pyx_unpickle_term_parser", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_6ttygif_3tty_6parser_1__pyx_unpickle_term_parser, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6ttygif_3tty_6parser_1__pyx_unpickle_term_parser(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_term_parser (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_term_parser", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_term_parser", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_term_parser") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_term_parser", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ttygif.tty.parser.__pyx_unpickle_term_parser", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6ttygif_3tty_6parser___pyx_unpickle_term_parser(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6ttygif_3tty_6parser___pyx_unpickle_term_parser(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_TraceFrameInit(__pyx_codeobj__20)
  __Pyx_RefNannySetupContext("__pyx_unpickle_term_parser", 0);
  __Pyx_TraceCall("__pyx_unpickle_term_parser", __pyx_f[1], 1, 0, __PYX_ERR(1, 1, __pyx_L1_error));

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x35256c6:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x35256c6 = (bracketed_paste, current_sequence_position, debug_mode, extra_text, g, last_timestamp, no_codes, sequence, sequence_pos))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x35256c6) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x35256c6:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x35256c6 = (bracketed_paste, current_sequence_position, debug_mode, extra_text, g, last_timestamp, no_codes, sequence, sequence_pos))" % __pyx_checksum)
 *     __pyx_result = term_parser.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x35256c6:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x35256c6 = (bracketed_paste, current_sequence_position, debug_mode, extra_text, g, last_timestamp, no_codes, sequence, sequence_pos))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = term_parser.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x35, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x35256c6:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x35256c6 = (bracketed_paste, current_sequence_position, debug_mode, extra_text, g, last_timestamp, no_codes, sequence, sequence_pos))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x35256c6 = (bracketed_paste, current_sequence_position, debug_mode, extra_text, g, last_timestamp, no_codes, sequence, sequence_pos))" % __pyx_checksum)
 *     __pyx_result = term_parser.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_term_parser__set_state(<term_parser> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_6ttygif_3tty_6parser_term_parser), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x35256c6 = (bracketed_paste, current_sequence_position, debug_mode, extra_text, g, last_timestamp, no_codes, sequence, sequence_pos))" % __pyx_checksum)
 *     __pyx_result = term_parser.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_term_parser__set_state(<term_parser> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = term_parser.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_term_parser__set_state(<term_parser> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_term_parser__set_state(term_parser __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_6ttygif_3tty_6parser___pyx_unpickle_term_parser__set_state(((struct __pyx_obj_6ttygif_3tty_6parser_term_parser *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x35256c6 = (bracketed_paste, current_sequence_position, debug_mode, extra_text, g, last_timestamp, no_codes, sequence, sequence_pos))" % __pyx_checksum)
 *     __pyx_result = term_parser.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_term_parser__set_state(<term_parser> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_term_parser__set_state(<term_parser> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_term_parser__set_state(term_parser __pyx_result, tuple __pyx_state):
 *     __pyx_result.bracketed_paste = __pyx_state[0]; __pyx_result.current_sequence_position = __pyx_state[1]; __pyx_result.debug_mode = __pyx_state[2]; __pyx_result.extra_text = __pyx_state[3]; __pyx_result.g = __pyx_state[4]; __pyx_result.last_timestamp = __pyx_state[5]; __pyx_result.no_codes = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.sequence_pos = __pyx_state[8]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_term_parser(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ttygif.tty.parser.__pyx_unpickle_term_parser", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_term_parser__set_state(<term_parser> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_term_parser__set_state(term_parser __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.bracketed_paste = __pyx_state[0]; __pyx_result.current_sequence_position = __pyx_state[1]; __pyx_result.debug_mode = __pyx_state[2]; __pyx_result.extra_text = __pyx_state[3]; __pyx_result.g = __pyx_state[4]; __pyx_result.last_timestamp = __pyx_state[5]; __pyx_result.no_codes = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.sequence_pos = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_6ttygif_3tty_6parser___pyx_unpickle_term_parser__set_state(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_term_parser__set_state", 0);
  __Pyx_TraceCall("__pyx_unpickle_term_parser__set_state", __pyx_f[1], 11, 0, __PYX_ERR(1, 11, __pyx_L1_error));

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_term_parser__set_state(term_parser __pyx_result, tuple __pyx_state):
 *     __pyx_result.bracketed_paste = __pyx_state[0]; __pyx_result.current_sequence_position = __pyx_state[1]; __pyx_result.debug_mode = __pyx_state[2]; __pyx_result.extra_text = __pyx_state[3]; __pyx_result.g = __pyx_state[4]; __pyx_result.last_timestamp = __pyx_state[5]; __pyx_result.no_codes = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.sequence_pos = __pyx_state[8]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->bracketed_paste);
  __Pyx_DECREF(__pyx_v___pyx_result->bracketed_paste);
  __pyx_v___pyx_result->bracketed_paste = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->current_sequence_position);
  __Pyx_DECREF(__pyx_v___pyx_result->current_sequence_position);
  __pyx_v___pyx_result->current_sequence_position = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->debug_mode);
  __Pyx_DECREF(__pyx_v___pyx_result->debug_mode);
  __pyx_v___pyx_result->debug_mode = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->extra_text);
  __Pyx_DECREF(__pyx_v___pyx_result->extra_text);
  __pyx_v___pyx_result->extra_text = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_6ttygif_3tty_17terminal_graphics_terminal_graphics))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->g);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->g));
  __pyx_v___pyx_result->g = ((struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->last_timestamp = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->no_codes);
  __Pyx_DECREF(__pyx_v___pyx_result->no_codes);
  __pyx_v___pyx_result->no_codes = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->sequence);
  __Pyx_DECREF(__pyx_v___pyx_result->sequence);
  __pyx_v___pyx_result->sequence = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->sequence_pos = __pyx_t_3;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_term_parser__set_state(term_parser __pyx_result, tuple __pyx_state):
 *     __pyx_result.bracketed_paste = __pyx_state[0]; __pyx_result.current_sequence_position = __pyx_state[1]; __pyx_result.debug_mode = __pyx_state[2]; __pyx_result.extra_text = __pyx_state[3]; __pyx_result.g = __pyx_state[4]; __pyx_result.last_timestamp = __pyx_state[5]; __pyx_result.no_codes = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.sequence_pos = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 9) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_4 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_4 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":14
 *     __pyx_result.bracketed_paste = __pyx_state[0]; __pyx_result.current_sequence_position = __pyx_state[1]; __pyx_result.debug_mode = __pyx_state[2]; __pyx_result.extra_text = __pyx_state[3]; __pyx_result.g = __pyx_state[4]; __pyx_result.last_timestamp = __pyx_state[5]; __pyx_result.no_codes = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.sequence_pos = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[9])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_term_parser__set_state(term_parser __pyx_result, tuple __pyx_state):
 *     __pyx_result.bracketed_paste = __pyx_state[0]; __pyx_result.current_sequence_position = __pyx_state[1]; __pyx_result.debug_mode = __pyx_state[2]; __pyx_result.extra_text = __pyx_state[3]; __pyx_result.g = __pyx_state[4]; __pyx_result.last_timestamp = __pyx_state[5]; __pyx_result.no_codes = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.sequence_pos = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[9])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_term_parser__set_state(<term_parser> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_term_parser__set_state(term_parser __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.bracketed_paste = __pyx_state[0]; __pyx_result.current_sequence_position = __pyx_state[1]; __pyx_result.debug_mode = __pyx_state[2]; __pyx_result.extra_text = __pyx_state[3]; __pyx_result.g = __pyx_state[4]; __pyx_result.last_timestamp = __pyx_state[5]; __pyx_result.no_codes = __pyx_state[6]; __pyx_result.sequence = __pyx_state[7]; __pyx_result.sequence_pos = __pyx_state[8]
 *     if len(__pyx_state) > 9 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ttygif.tty.parser.__pyx_unpickle_term_parser__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "array.pxd":93
 *             __data_union data
 * 
 *         def __getbuffer__(self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fulfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_7cpython_5array_5array_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_7cpython_5array_5array_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7cpython_5array_5array___getbuffer__(((arrayobject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7cpython_5array_5array___getbuffer__(arrayobject *__pyx_v_self, Py_buffer *__pyx_v_info, CYTHON_UNUSED int __pyx_v_flags) {
  PyObject *__pyx_v_item_count = NULL;
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char *__pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  char __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  if (__pyx_v_info == NULL) {
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");
    return -1;
  }
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(__pyx_v_info->obj);
  __Pyx_TraceCall("__getbuffer__", __pyx_f[2], 93, 0, __PYX_ERR(2, 93, __pyx_L1_error));

  /* "array.pxd":98
 *             # In particular strided access is always provided regardless
 *             # of flags
 *             item_count = Py_SIZE(self)             # <<<<<<<<<<<<<<
 * 
 *             info.suboffsets = NULL
 */
  __pyx_t_1 = PyInt_FromSsize_t(Py_SIZE(((PyObject *)__pyx_v_self))); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_item_count = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "array.pxd":100
 *             item_count = Py_SIZE(self)
 * 
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.buf = self.data.as_chars
 *             info.readonly = 0
 */
  __pyx_v_info->suboffsets = NULL;

  /* "array.pxd":101
 * 
 *             info.suboffsets = NULL
 *             info.buf = self.data.as_chars             # <<<<<<<<<<<<<<
 *             info.readonly = 0
 *             info.ndim = 1
 */
  __pyx_t_2 = __pyx_v_self->data.as_chars;
  __pyx_v_info->buf = __pyx_t_2;

  /* "array.pxd":102
 *             info.suboffsets = NULL
 *             info.buf = self.data.as_chars
 *             info.readonly = 0             # <<<<<<<<<<<<<<
 *             info.ndim = 1
 *             info.itemsize = self.ob_descr.itemsize   # e.g. sizeof(float)
 */
  __pyx_v_info->readonly = 0;

  /* "array.pxd":103
 *             info.buf = self.data.as_chars
 *             info.readonly = 0
 *             info.ndim = 1             # <<<<<<<<<<<<<<
 *             info.itemsize = self.ob_descr.itemsize   # e.g. sizeof(float)
 *             info.len = info.itemsize * item_count
 */
  __pyx_v_info->ndim = 1;

  /* "array.pxd":104
 *             info.readonly = 0
 *             info.ndim = 1
 *             info.itemsize = self.ob_descr.itemsize   # e.g. sizeof(float)             # <<<<<<<<<<<<<<
 *             info.len = info.itemsize * item_count
 * 
 */
  __pyx_t_3 = __pyx_v_self->ob_descr->itemsize;
  __pyx_v_info->itemsize = __pyx_t_3;

  /* "array.pxd":105
 *             info.ndim = 1
 *             info.itemsize = self.ob_descr.itemsize   # e.g. sizeof(float)
 *             info.len = info.itemsize * item_count             # <<<<<<<<<<<<<<
 * 
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)
 */
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_info->itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyNumber_Multiply(__pyx_t_1, __pyx_v_item_count); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_t_4); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 105, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_info->len = __pyx_t_5;

  /* "array.pxd":107
 *             info.len = info.itemsize * item_count
 * 
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)             # <<<<<<<<<<<<<<
 *             if not info.shape:
 *                 raise MemoryError()
 */
  __pyx_v_info->shape = ((Py_ssize_t *)PyObject_Malloc(((sizeof(Py_ssize_t)) + 2)));

  /* "array.pxd":108
 * 
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)
 *             if not info.shape:             # <<<<<<<<<<<<<<
 *                 raise MemoryError()
 *             info.shape[0] = item_count      # constant regardless of resizing
 */
  __pyx_t_6 = ((!(__pyx_v_info->shape != 0)) != 0);
  if (unlikely(__pyx_t_6)) {

    /* "array.pxd":109
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)
 *             if not info.shape:
 *                 raise MemoryError()             # <<<<<<<<<<<<<<
 *             info.shape[0] = item_count      # constant regardless of resizing
 *             info.strides = &info.itemsize
 */
    PyErr_NoMemory(); __PYX_ERR(2, 109, __pyx_L1_error)

    /* "array.pxd":108
 * 
 *             info.shape = <Py_ssize_t*> PyObject_Malloc(sizeof(Py_ssize_t) + 2)
 *             if not info.shape:             # <<<<<<<<<<<<<<
 *                 raise MemoryError()
 *             info.shape[0] = item_count      # constant regardless of resizing
 */
  }

  /* "array.pxd":110
 *             if not info.shape:
 *                 raise MemoryError()
 *             info.shape[0] = item_count      # constant regardless of resizing             # <<<<<<<<<<<<<<
 *             info.strides = &info.itemsize
 * 
 */
  __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_v_item_count); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 110, __pyx_L1_error)
  (__pyx_v_info->shape[0]) = __pyx_t_5;

  /* "array.pxd":111
 *                 raise MemoryError()
 *             info.shape[0] = item_count      # constant regardless of resizing
 *             info.strides = &info.itemsize             # <<<<<<<<<<<<<<
 * 
 *             info.format = <char*> (info.shape + 1)
 */
  __pyx_v_info->strides = (&__pyx_v_info->itemsize);

  /* "array.pxd":113
 *             info.strides = &info.itemsize
 * 
 *             info.format = <char*> (info.shape + 1)             # <<<<<<<<<<<<<<
 *             info.format[0] = self.ob_descr.typecode
 *             info.format[1] = 0
 */
  __pyx_v_info->format = ((char *)(__pyx_v_info->shape + 1));

  /* "array.pxd":114
 * 
 *             info.format = <char*> (info.shape + 1)
 *             info.format[0] = self.ob_descr.typecode             # <<<<<<<<<<<<<<
 *             info.format[1] = 0
 *             info.obj = self
 */
  __pyx_t_7 = __pyx_v_self->ob_descr->typecode;
  (__pyx_v_info->format[0]) = __pyx_t_7;

  /* "array.pxd":115
 *             info.format = <char*> (info.shape + 1)
 *             info.format[0] = self.ob_descr.typecode
 *             info.format[1] = 0             # <<<<<<<<<<<<<<
 *             info.obj = self
 * 
 */
  (__pyx_v_info->format[1]) = 0;

  /* "array.pxd":116
 *             info.format[0] = self.ob_descr.typecode
 *             info.format[1] = 0
 *             info.obj = self             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(self, Py_buffer* info):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "array.pxd":93
 *             __data_union data
 * 
 *         def __getbuffer__(self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fulfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cpython.array.array.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  __pyx_L2:;
  __Pyx_XDECREF(__pyx_v_item_count);
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "array.pxd":118
 *             info.obj = self
 * 
 *         def __releasebuffer__(self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             PyObject_Free(info.shape)
 * 
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_7cpython_5array_5array_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_7cpython_5array_5array_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_7cpython_5array_5array_2__releasebuffer__(((arrayobject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_7cpython_5array_5array_2__releasebuffer__(CYTHON_UNUSED arrayobject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);
  __Pyx_TraceCall("__releasebuffer__", __pyx_f[2], 118, 0, __PYX_ERR(2, 118, __pyx_L1_error));

  /* "array.pxd":119
 * 
 *         def __releasebuffer__(self, Py_buffer* info):
 *             PyObject_Free(info.shape)             # <<<<<<<<<<<<<<
 * 
 *     array newarrayobject(PyTypeObject* type, Py_ssize_t size, arraydescr *descr)
 */
  PyObject_Free(__pyx_v_info->shape);

  /* "array.pxd":118
 *             info.obj = self
 * 
 *         def __releasebuffer__(self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             PyObject_Free(info.shape)
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("cpython.array.array.__releasebuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
}

/* "array.pxd":130
 * 
 * 
 * cdef inline array clone(array template, Py_ssize_t length, bint zero):             # <<<<<<<<<<<<<<
 *     """ fast creation of a new array, given a template array.
 *     type will be same as template.
 */

static CYTHON_INLINE arrayobject *__pyx_f_7cpython_5array_clone(arrayobject *__pyx_v_template, Py_ssize_t __pyx_v_length, int __pyx_v_zero) {
  arrayobject *__pyx_v_op = 0;
  arrayobject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  __Pyx_TraceCall("clone", __pyx_f[2], 130, 0, __PYX_ERR(2, 130, __pyx_L1_error));

  /* "array.pxd":134
 *     type will be same as template.
 *     if zero is true, new array will be initialized with zeroes."""
 *     cdef array op = newarrayobject(Py_TYPE(template), length, template.ob_descr)             # <<<<<<<<<<<<<<
 *     if zero and op is not None:
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)
 */
  __pyx_t_1 = ((PyObject *)newarrayobject(Py_TYPE(((PyObject *)__pyx_v_template)), __pyx_v_length, __pyx_v_template->ob_descr)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_op = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "array.pxd":135
 *     if zero is true, new array will be initialized with zeroes."""
 *     cdef array op = newarrayobject(Py_TYPE(template), length, template.ob_descr)
 *     if zero and op is not None:             # <<<<<<<<<<<<<<
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)
 *     return op
 */
  __pyx_t_3 = (__pyx_v_zero != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (((PyObject *)__pyx_v_op) != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "array.pxd":136
 *     cdef array op = newarrayobject(Py_TYPE(template), length, template.ob_descr)
 *     if zero and op is not None:
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)             # <<<<<<<<<<<<<<
 *     return op
 * 
 */
    (void)(memset(__pyx_v_op->data.as_chars, 0, (__pyx_v_length * __pyx_v_op->ob_descr->itemsize)));

    /* "array.pxd":135
 *     if zero is true, new array will be initialized with zeroes."""
 *     cdef array op = newarrayobject(Py_TYPE(template), length, template.ob_descr)
 *     if zero and op is not None:             # <<<<<<<<<<<<<<
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)
 *     return op
 */
  }

  /* "array.pxd":137
 *     if zero and op is not None:
 *         memset(op.data.as_chars, 0, length * op.ob_descr.itemsize)
 *     return op             # <<<<<<<<<<<<<<
 * 
 * cdef inline array copy(array self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_op));
  __pyx_r = __pyx_v_op;
  goto __pyx_L0;

  /* "array.pxd":130
 * 
 * 
 * cdef inline array clone(array template, Py_ssize_t length, bint zero):             # <<<<<<<<<<<<<<
 *     """ fast creation of a new array, given a template array.
 *     type will be same as template.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cpython.array.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_op);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "array.pxd":139
 *     return op
 * 
 * cdef inline array copy(array self):             # <<<<<<<<<<<<<<
 *     """ make a copy of an array. """
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)
 */

static CYTHON_INLINE arrayobject *__pyx_f_7cpython_5array_copy(arrayobject *__pyx_v_self) {
  arrayobject *__pyx_v_op = 0;
  arrayobject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);
  __Pyx_TraceCall("copy", __pyx_f[2], 139, 0, __PYX_ERR(2, 139, __pyx_L1_error));

  /* "array.pxd":141
 * cdef inline array copy(array self):
 *     """ make a copy of an array. """
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)             # <<<<<<<<<<<<<<
 *     memcpy(op.data.as_chars, self.data.as_chars, Py_SIZE(op) * op.ob_descr.itemsize)
 *     return op
 */
  __pyx_t_1 = ((PyObject *)newarrayobject(Py_TYPE(((PyObject *)__pyx_v_self)), Py_SIZE(((PyObject *)__pyx_v_self)), __pyx_v_self->ob_descr)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_op = ((arrayobject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "array.pxd":142
 *     """ make a copy of an array. """
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)
 *     memcpy(op.data.as_chars, self.data.as_chars, Py_SIZE(op) * op.ob_descr.itemsize)             # <<<<<<<<<<<<<<
 *     return op
 * 
 */
  (void)(memcpy(__pyx_v_op->data.as_chars, __pyx_v_self->data.as_chars, (Py_SIZE(((PyObject *)__pyx_v_op)) * __pyx_v_op->ob_descr->itemsize)));

  /* "array.pxd":143
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)
 *     memcpy(op.data.as_chars, self.data.as_chars, Py_SIZE(op) * op.ob_descr.itemsize)
 *     return op             # <<<<<<<<<<<<<<
 * 
 * cdef inline int extend_buffer(array self, char* stuff, Py_ssize_t n) except -1:
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_op));
  __pyx_r = __pyx_v_op;
  goto __pyx_L0;

  /* "array.pxd":139
 *     return op
 * 
 * cdef inline array copy(array self):             # <<<<<<<<<<<<<<
 *     """ make a copy of an array. """
 *     cdef array op = newarrayobject(Py_TYPE(self), Py_SIZE(self), self.ob_descr)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cpython.array.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_op);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "array.pxd":145
 *     return op
 * 
 * cdef inline int extend_buffer(array self, char* stuff, Py_ssize_t n) except -1:             # <<<<<<<<<<<<<<
 *     """ efficient appending of new stuff of same type
 *     (e.g. of same array type)
 */

static CYTHON_INLINE int __pyx_f_7cpython_5array_extend_buffer(arrayobject *__pyx_v_self, char *__pyx_v_stuff, Py_ssize_t __pyx_v_n) {
  Py_ssize_t __pyx_v_itemsize;
  Py_ssize_t __pyx_v_origsize;
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("extend_buffer", 0);
  __Pyx_TraceCall("extend_buffer", __pyx_f[2], 145, 0, __PYX_ERR(2, 145, __pyx_L1_error));

  /* "array.pxd":149
 *     (e.g. of same array type)
 *     n: number of elements (not number of bytes!) """
 *     cdef Py_ssize_t itemsize = self.ob_descr.itemsize             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t origsize = Py_SIZE(self)
 *     resize_smart(self, origsize + n)
 */
  __pyx_t_1 = __pyx_v_self->ob_descr->itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "array.pxd":150
 *     n: number of elements (not number of bytes!) """
 *     cdef Py_ssize_t itemsize = self.ob_descr.itemsize
 *     cdef Py_ssize_t origsize = Py_SIZE(self)             # <<<<<<<<<<<<<<
 *     resize_smart(self, origsize + n)
 *     memcpy(self.data.as_chars + origsize * itemsize, stuff, n * itemsize)
 */
  __pyx_v_origsize = Py_SIZE(((PyObject *)__pyx_v_self));

  /* "array.pxd":151
 *     cdef Py_ssize_t itemsize = self.ob_descr.itemsize
 *     cdef Py_ssize_t origsize = Py_SIZE(self)
 *     resize_smart(self, origsize + n)             # <<<<<<<<<<<<<<
 *     memcpy(self.data.as_chars + origsize * itemsize, stuff, n * itemsize)
 *     return 0
 */
  __pyx_t_1 = resize_smart(__pyx_v_self, (__pyx_v_origsize + __pyx_v_n)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 151, __pyx_L1_error)

  /* "array.pxd":152
 *     cdef Py_ssize_t origsize = Py_SIZE(self)
 *     resize_smart(self, origsize + n)
 *     memcpy(self.data.as_chars + origsize * itemsize, stuff, n * itemsize)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  (void)(memcpy((__pyx_v_self->data.as_chars + (__pyx_v_origsize * __pyx_v_itemsize)), __pyx_v_stuff, (__pyx_v_n * __pyx_v_itemsize)));

  /* "array.pxd":153
 *     resize_smart(self, origsize + n)
 *     memcpy(self.data.as_chars + origsize * itemsize, stuff, n * itemsize)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline int extend(array self, array other) except -1:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "array.pxd":145
 *     return op
 * 
 * cdef inline int extend_buffer(array self, char* stuff, Py_ssize_t n) except -1:             # <<<<<<<<<<<<<<
 *     """ efficient appending of new stuff of same type
 *     (e.g. of same array type)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cpython.array.extend_buffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "array.pxd":155
 *     return 0
 * 
 * cdef inline int extend(array self, array other) except -1:             # <<<<<<<<<<<<<<
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:
 */

static CYTHON_INLINE int __pyx_f_7cpython_5array_extend(arrayobject *__pyx_v_self, arrayobject *__pyx_v_other) {
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("extend", 0);
  __Pyx_TraceCall("extend", __pyx_f[2], 155, 0, __PYX_ERR(2, 155, __pyx_L1_error));

  /* "array.pxd":157
 * cdef inline int extend(array self, array other) except -1:
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:             # <<<<<<<<<<<<<<
 *         PyErr_BadArgument()
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 */
  __pyx_t_1 = ((__pyx_v_self->ob_descr->typecode != __pyx_v_other->ob_descr->typecode) != 0);
  if (__pyx_t_1) {

    /* "array.pxd":158
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:
 *         PyErr_BadArgument()             # <<<<<<<<<<<<<<
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 * 
 */
    __pyx_t_2 = PyErr_BadArgument(); if (unlikely(__pyx_t_2 == ((int)0))) __PYX_ERR(2, 158, __pyx_L1_error)

    /* "array.pxd":157
 * cdef inline int extend(array self, array other) except -1:
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:             # <<<<<<<<<<<<<<
 *         PyErr_BadArgument()
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 */
  }

  /* "array.pxd":159
 *     if self.ob_descr.typecode != other.ob_descr.typecode:
 *         PyErr_BadArgument()
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))             # <<<<<<<<<<<<<<
 * 
 * cdef inline void zero(array self):
 */
  __pyx_t_2 = __pyx_f_7cpython_5array_extend_buffer(__pyx_v_self, __pyx_v_other->data.as_chars, Py_SIZE(((PyObject *)__pyx_v_other))); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 159, __pyx_L1_error)
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "array.pxd":155
 *     return 0
 * 
 * cdef inline int extend(array self, array other) except -1:             # <<<<<<<<<<<<<<
 *     """ extend array with data from another array; types must match. """
 *     if self.ob_descr.typecode != other.ob_descr.typecode:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cpython.array.extend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "array.pxd":161
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 * 
 * cdef inline void zero(array self):             # <<<<<<<<<<<<<<
 *     """ set all elements of array to zero. """
 *     memset(self.data.as_chars, 0, Py_SIZE(self) * self.ob_descr.itemsize)
 */

static CYTHON_INLINE void __pyx_f_7cpython_5array_zero(arrayobject *__pyx_v_self) {
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("zero", 0);
  __Pyx_TraceCall("zero", __pyx_f[2], 161, 0, __PYX_ERR(2, 161, __pyx_L1_error));

  /* "array.pxd":163
 * cdef inline void zero(array self):
 *     """ set all elements of array to zero. """
 *     memset(self.data.as_chars, 0, Py_SIZE(self) * self.ob_descr.itemsize)             # <<<<<<<<<<<<<<
 */
  (void)(memset(__pyx_v_self->data.as_chars, 0, (Py_SIZE(((PyObject *)__pyx_v_self)) * __pyx_v_self->ob_descr->itemsize)));

  /* "array.pxd":161
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 * 
 * cdef inline void zero(array self):             # <<<<<<<<<<<<<<
 *     """ set all elements of array to zero. """
 *     memset(self.data.as_chars, 0, Py_SIZE(self) * self.ob_descr.itemsize)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("cpython.array.zero", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
}
static struct __pyx_vtabstruct_6ttygif_3tty_6parser_term_parser __pyx_vtable_6ttygif_3tty_6parser_term_parser;

static PyObject *__pyx_tp_new_6ttygif_3tty_6parser_term_parser(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6ttygif_3tty_6parser_term_parser *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6ttygif_3tty_6parser_term_parser *)o);
  p->__pyx_vtab = __pyx_vtabptr_6ttygif_3tty_6parser_term_parser;
  p->debug_mode = Py_None; Py_INCREF(Py_None);
  p->sequence = Py_None; Py_INCREF(Py_None);
  p->extra_text = Py_None; Py_INCREF(Py_None);
  p->no_codes = Py_None; Py_INCREF(Py_None);
  p->bracketed_paste = Py_None; Py_INCREF(Py_None);
  p->current_sequence_position = Py_None; Py_INCREF(Py_None);
  p->g = ((struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6ttygif_3tty_6parser_term_parser(PyObject *o) {
  struct __pyx_obj_6ttygif_3tty_6parser_term_parser *p = (struct __pyx_obj_6ttygif_3tty_6parser_term_parser *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->debug_mode);
  Py_CLEAR(p->sequence);
  Py_CLEAR(p->extra_text);
  Py_CLEAR(p->no_codes);
  Py_CLEAR(p->bracketed_paste);
  Py_CLEAR(p->current_sequence_position);
  Py_CLEAR(p->g);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_6ttygif_3tty_6parser_term_parser(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ttygif_3tty_6parser_term_parser *p = (struct __pyx_obj_6ttygif_3tty_6parser_term_parser *)o;
  if (p->debug_mode) {
    e = (*v)(p->debug_mode, a); if (e) return e;
  }
  if (p->sequence) {
    e = (*v)(p->sequence, a); if (e) return e;
  }
  if (p->extra_text) {
    e = (*v)(p->extra_text, a); if (e) return e;
  }
  if (p->no_codes) {
    e = (*v)(p->no_codes, a); if (e) return e;
  }
  if (p->bracketed_paste) {
    e = (*v)(p->bracketed_paste, a); if (e) return e;
  }
  if (p->current_sequence_position) {
    e = (*v)(p->current_sequence_position, a); if (e) return e;
  }
  if (p->g) {
    e = (*v)(((PyObject *)p->g), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ttygif_3tty_6parser_term_parser(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ttygif_3tty_6parser_term_parser *p = (struct __pyx_obj_6ttygif_3tty_6parser_term_parser *)o;
  tmp = ((PyObject*)p->debug_mode);
  p->debug_mode = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->sequence);
  p->sequence = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->extra_text);
  p->extra_text = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->no_codes);
  p->no_codes = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->bracketed_paste);
  p->bracketed_paste = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->current_sequence_position);
  p->current_sequence_position = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->g);
  p->g = ((struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6ttygif_3tty_6parser_term_parser[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_6ttygif_3tty_6parser_11term_parser_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_6ttygif_3tty_6parser_11term_parser_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6ttygif_3tty_6parser_term_parser = {
  PyVarObject_HEAD_INIT(0, 0)
  "ttygif.tty.parser.term_parser", /*tp_name*/
  sizeof(struct __pyx_obj_6ttygif_3tty_6parser_term_parser), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ttygif_3tty_6parser_term_parser, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6ttygif_3tty_6parser_term_parser, /*tp_traverse*/
  __pyx_tp_clear_6ttygif_3tty_6parser_term_parser, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6ttygif_3tty_6parser_term_parser, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_6ttygif_3tty_6parser_11term_parser_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ttygif_3tty_6parser_term_parser, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *__pyx_freelist_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence[8];
static int __pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence = 0;

static PyObject *__pyx_tp_new_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence)))) {
    o = (PyObject*)__pyx_freelist_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence[--__pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence];
    memset(o, 0, sizeof(struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence(PyObject *o) {
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *p = (struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_paramstring);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence)))) {
    __pyx_freelist_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence[__pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence++] = ((struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *p = (struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *)o;
  if (p->__pyx_v_paramstring) {
    e = (*v)(p->__pyx_v_paramstring, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *p = (struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence *)o;
  tmp = ((PyObject*)p->__pyx_v_paramstring);
  p->__pyx_v_paramstring = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence = {
  PyVarObject_HEAD_INIT(0, 0)
  "ttygif.tty.parser.__pyx_scope_struct__stream_2_sequence", /*tp_name*/
  sizeof(struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence, /*tp_traverse*/
  __pyx_tp_clear_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr *__pyx_freelist_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr[8];
static int __pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr = 0;

static PyObject *__pyx_tp_new_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr)))) {
    o = (PyObject*)__pyx_freelist_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr[--__pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr];
    memset(o, 0, sizeof(struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr(PyObject *o) {
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  Py_CLEAR(p->__pyx_v_p);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr)))) {
    __pyx_freelist_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr[__pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr++] = ((struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr *)o;
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;
  }
  if (p->__pyx_v_p) {
    e = (*v)(p->__pyx_v_p, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "ttygif.tty.parser.__pyx_scope_struct_1_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr *__pyx_freelist_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr[8];
static int __pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr = 0;

static PyObject *__pyx_tp_new_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr)))) {
    o = (PyObject*)__pyx_freelist_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr[--__pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr];
    memset(o, 0, sizeof(struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr(PyObject *o) {
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr *p = (struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  Py_CLEAR(p->__pyx_v_p);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr)))) {
    __pyx_freelist_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr[__pyx_freecount_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr++] = ((struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr *p = (struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr *)o;
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;
  }
  if (p->__pyx_v_p) {
    e = (*v)(p->__pyx_v_p, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "ttygif.tty.parser.__pyx_scope_struct_2_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_parser(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_parser},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "parser",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_kp_s_0, __pyx_k_0, sizeof(__pyx_k_0), 0, 0, 1, 0},
  {&__pyx_kp_s_20_0_1, __pyx_k_20_0_1, sizeof(__pyx_k_20_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_2_6x_3_3_5f_0_1, __pyx_k_2_6x_3_3_5f_0_1, sizeof(__pyx_k_2_6x_3_3_5f_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_2_6x_3_3_5f_text_0_1, __pyx_k_2_6x_3_3_5f_text_0_1, sizeof(__pyx_k_2_6x_3_3_5f_text_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_5_05x_CMD_0_Name_3_Command_1_Pa, __pyx_k_5_05x_CMD_0_Name_3_Command_1_Pa, sizeof(__pyx_k_5_05x_CMD_0_Name_3_Command_1_Pa), 0, 0, 1, 0},
  {&__pyx_n_s_A, __pyx_k_A, sizeof(__pyx_k_A), 0, 0, 1, 1},
  {&__pyx_n_s_ABCD, __pyx_k_ABCD, sizeof(__pyx_k_ABCD), 0, 0, 1, 1},
  {&__pyx_n_s_B, __pyx_k_B, sizeof(__pyx_k_B), 0, 0, 1, 1},
  {&__pyx_kp_s_B0UK, __pyx_k_B0UK, sizeof(__pyx_k_B0UK), 0, 0, 1, 0},
  {&__pyx_kp_s_B0UK_2, __pyx_k_B0UK_2, sizeof(__pyx_k_B0UK_2), 0, 0, 1, 0},
  {&__pyx_n_s_BRACKETED_PASTE, __pyx_k_BRACKETED_PASTE, sizeof(__pyx_k_BRACKETED_PASTE), 0, 0, 1, 1},
  {&__pyx_n_s_C, __pyx_k_C, sizeof(__pyx_k_C), 0, 0, 1, 1},
  {&__pyx_n_s_CHA, __pyx_k_CHA, sizeof(__pyx_k_CHA), 0, 0, 1, 1},
  {&__pyx_n_s_CHAR_SET, __pyx_k_CHAR_SET, sizeof(__pyx_k_CHAR_SET), 0, 0, 1, 1},
  {&__pyx_n_s_CNL, __pyx_k_CNL, sizeof(__pyx_k_CNL), 0, 0, 1, 1},
  {&__pyx_n_s_COL, __pyx_k_COL, sizeof(__pyx_k_COL), 0, 0, 1, 1},
  {&__pyx_n_s_CPL, __pyx_k_CPL, sizeof(__pyx_k_CPL), 0, 0, 1, 1},
  {&__pyx_n_s_CSI, __pyx_k_CSI, sizeof(__pyx_k_CSI), 0, 0, 1, 1},
  {&__pyx_n_s_CUB, __pyx_k_CUB, sizeof(__pyx_k_CUB), 0, 0, 1, 1},
  {&__pyx_n_s_CUD, __pyx_k_CUD, sizeof(__pyx_k_CUD), 0, 0, 1, 1},
  {&__pyx_n_s_CUF, __pyx_k_CUF, sizeof(__pyx_k_CUF), 0, 0, 1, 1},
  {&__pyx_n_s_CUP, __pyx_k_CUP, sizeof(__pyx_k_CUP), 0, 0, 1, 1},
  {&__pyx_n_s_CUU, __pyx_k_CUU, sizeof(__pyx_k_CUU), 0, 0, 1, 1},
  {&__pyx_kp_s_Count_0, __pyx_k_Count_0, sizeof(__pyx_k_Count_0), 0, 0, 1, 0},
  {&__pyx_n_s_D, __pyx_k_D, sizeof(__pyx_k_D), 0, 0, 1, 1},
  {&__pyx_n_s_DCH, __pyx_k_DCH, sizeof(__pyx_k_DCH), 0, 0, 1, 1},
  {&__pyx_n_s_DECRST, __pyx_k_DECRST, sizeof(__pyx_k_DECRST), 0, 0, 1, 1},
  {&__pyx_n_s_DECSET, __pyx_k_DECSET, sizeof(__pyx_k_DECSET), 0, 0, 1, 1},
  {&__pyx_n_s_DECSTBM, __pyx_k_DECSTBM, sizeof(__pyx_k_DECSTBM), 0, 0, 1, 1},
  {&__pyx_n_s_E, __pyx_k_E, sizeof(__pyx_k_E), 0, 0, 1, 1},
  {&__pyx_n_s_ECH, __pyx_k_ECH, sizeof(__pyx_k_ECH), 0, 0, 1, 1},
  {&__pyx_n_s_ED, __pyx_k_ED, sizeof(__pyx_k_ED), 0, 0, 1, 1},
  {&__pyx_n_s_EL, __pyx_k_EL, sizeof(__pyx_k_EL), 0, 0, 1, 1},
  {&__pyx_n_s_ERR, __pyx_k_ERR, sizeof(__pyx_k_ERR), 0, 0, 1, 1},
  {&__pyx_n_s_F, __pyx_k_F, sizeof(__pyx_k_F), 0, 0, 1, 1},
  {&__pyx_n_s_G, __pyx_k_G, sizeof(__pyx_k_G), 0, 0, 1, 1},
  {&__pyx_n_s_G0, __pyx_k_G0, sizeof(__pyx_k_G0), 0, 0, 1, 1},
  {&__pyx_n_s_G1, __pyx_k_G1, sizeof(__pyx_k_G1), 0, 0, 1, 1},
  {&__pyx_kp_s_G_2, __pyx_k_G_2, sizeof(__pyx_k_G_2), 0, 0, 1, 0},
  {&__pyx_n_s_H, __pyx_k_H, sizeof(__pyx_k_H), 0, 0, 1, 1},
  {&__pyx_n_s_HPA, __pyx_k_HPA, sizeof(__pyx_k_HPA), 0, 0, 1, 1},
  {&__pyx_n_s_HVP, __pyx_k_HVP, sizeof(__pyx_k_HVP), 0, 0, 1, 1},
  {&__pyx_n_s_Hf, __pyx_k_Hf, sizeof(__pyx_k_Hf), 0, 0, 1, 1},
  {&__pyx_kp_s_Impliment_0_1, __pyx_k_Impliment_0_1, sizeof(__pyx_k_Impliment_0_1), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x35, __pyx_k_Incompatible_checksums_s_vs_0x35, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x35), 0, 0, 1, 0},
  {&__pyx_n_s_J, __pyx_k_J, sizeof(__pyx_k_J), 0, 0, 1, 1},
  {&__pyx_n_s_JKm, __pyx_k_JKm, sizeof(__pyx_k_JKm), 0, 0, 1, 1},
  {&__pyx_n_s_K, __pyx_k_K, sizeof(__pyx_k_K), 0, 0, 1, 1},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_n_s_OSC, __pyx_k_OSC, sizeof(__pyx_k_OSC), 0, 0, 1, 1},
  {&__pyx_n_s_P, __pyx_k_P, sizeof(__pyx_k_P), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_RCP, __pyx_k_RCP, sizeof(__pyx_k_RCP), 0, 0, 1, 1},
  {&__pyx_n_s_SCP, __pyx_k_SCP, sizeof(__pyx_k_SCP), 0, 0, 1, 1},
  {&__pyx_n_s_SINGLE, __pyx_k_SINGLE, sizeof(__pyx_k_SINGLE), 0, 0, 1, 1},
  {&__pyx_kp_s_Sequence_List, __pyx_k_Sequence_List, sizeof(__pyx_k_Sequence_List), 0, 0, 1, 0},
  {&__pyx_n_s_VPA, __pyx_k_VPA, sizeof(__pyx_k_VPA), 0, 0, 1, 1},
  {&__pyx_n_s_X, __pyx_k_X, sizeof(__pyx_k_X), 0, 0, 1, 1},
  {&__pyx_kp_s__10, __pyx_k__10, sizeof(__pyx_k__10), 0, 0, 1, 0},
  {&__pyx_kp_s__13, __pyx_k__13, sizeof(__pyx_k__13), 0, 0, 1, 0},
  {&__pyx_kp_s__16, __pyx_k__16, sizeof(__pyx_k__16), 0, 0, 1, 0},
  {&__pyx_kp_s__17, __pyx_k__17, sizeof(__pyx_k__17), 0, 0, 1, 0},
  {&__pyx_kp_s__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 1, 0},
  {&__pyx_kp_s__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 0, 1, 0},
  {&__pyx_kp_s__4, __pyx_k__4, sizeof(__pyx_k__4), 0, 0, 1, 0},
  {&__pyx_kp_s__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 0, 1, 0},
  #if PY_MAJOR_VERSION >= 3
  {&__pyx_kp_s__7, __pyx_k__6, sizeof(__pyx_k__6), 0, 1, 0, 0},
  #else
  {&__pyx_kp_s__7, __pyx_k__7, sizeof(__pyx_k__7), 0, 0, 1, 0},
  #endif
  {&__pyx_kp_s__8, __pyx_k__8, sizeof(__pyx_k__8), 0, 0, 1, 0},
  {&__pyx_kp_s__9, __pyx_k__9, sizeof(__pyx_k__9), 0, 0, 1, 0},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_kp_s_bracketed_paste, __pyx_k_bracketed_paste, sizeof(__pyx_k_bracketed_paste), 0, 0, 1, 0},
  {&__pyx_kp_s_cDEHMZ78, __pyx_k_cDEHMZ78, sizeof(__pyx_k_cDEHMZ78), 0, 0, 1, 0},
  {&__pyx_n_s_chr, __pyx_k_chr, sizeof(__pyx_k_chr), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_command, __pyx_k_command, sizeof(__pyx_k_command), 0, 0, 1, 1},
  {&__pyx_n_s_compile, __pyx_k_compile, sizeof(__pyx_k_compile), 0, 0, 1, 1},
  {&__pyx_n_s_cursor_restore_position, __pyx_k_cursor_restore_position, sizeof(__pyx_k_cursor_restore_position), 0, 0, 1, 1},
  {&__pyx_n_s_cursor_save_position, __pyx_k_cursor_save_position, sizeof(__pyx_k_cursor_save_position), 0, 0, 1, 1},
  {&__pyx_n_s_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 0, 1, 1},
  {&__pyx_kp_s_d_a_zA_Z, __pyx_k_d_a_zA_Z, sizeof(__pyx_k_d_a_zA_Z), 0, 0, 1, 0},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_debug_mode, __pyx_k_debug_mode, sizeof(__pyx_k_debug_mode), 0, 0, 1, 1},
  {&__pyx_n_s_delay, __pyx_k_delay, sizeof(__pyx_k_delay), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_dict_2, __pyx_k_dict_2, sizeof(__pyx_k_dict_2), 0, 0, 1, 1},
  {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {&__pyx_n_s_esc_type, __pyx_k_esc_type, sizeof(__pyx_k_esc_type), 0, 0, 1, 1},
  {&__pyx_n_s_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 0, 1, 1},
  {&__pyx_n_s_file, __pyx_k_file, sizeof(__pyx_k_file), 0, 0, 1, 1},
  {&__pyx_n_s_finditer, __pyx_k_finditer, sizeof(__pyx_k_finditer), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_genexpr, __pyx_k_genexpr, sizeof(__pyx_k_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_groups, __pyx_k_groups, sizeof(__pyx_k_groups), 0, 0, 1, 1},
  {&__pyx_n_s_h, __pyx_k_h, sizeof(__pyx_k_h), 0, 0, 1, 1},
  {&__pyx_kp_s_h_2, __pyx_k_h_2, sizeof(__pyx_k_h_2), 0, 0, 1, 0},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_join, __pyx_k_join, sizeof(__pyx_k_join), 0, 0, 1, 1},
  {&__pyx_n_s_l, __pyx_k_l, sizeof(__pyx_k_l), 0, 0, 1, 1},
  {&__pyx_kp_s_l_2, __pyx_k_l_2, sizeof(__pyx_k_l_2), 0, 0, 1, 0},
  {&__pyx_n_s_linux, __pyx_k_linux, sizeof(__pyx_k_linux), 0, 0, 1, 1},
  {&__pyx_n_s_m, __pyx_k_m, sizeof(__pyx_k_m), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_o, __pyx_k_o, sizeof(__pyx_k_o), 0, 0, 1, 1},
  {&__pyx_n_s_params, __pyx_k_params, sizeof(__pyx_k_params), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_term_parser, __pyx_k_pyx_unpickle_term_parser, sizeof(__pyx_k_pyx_unpickle_term_parser), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_r, __pyx_k_r, sizeof(__pyx_k_r), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_re, __pyx_k_re, sizeof(__pyx_k_re), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_reset_mode, __pyx_k_reset_mode, sizeof(__pyx_k_reset_mode), 0, 0, 1, 1},
  {&__pyx_n_s_round, __pyx_k_round, sizeof(__pyx_k_round), 0, 0, 1, 1},
  {&__pyx_n_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_set_mode, __pyx_k_set_mode, sizeof(__pyx_k_set_mode), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_span, __pyx_k_span, sizeof(__pyx_k_span), 0, 0, 1, 1},
  {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
  {&__pyx_n_s_state, __pyx_k_state, sizeof(__pyx_k_state), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_term_parser, __pyx_k_term_parser, sizeof(__pyx_k_term_parser), 0, 0, 1, 1},
  {&__pyx_n_s_term_parser___reduce_cython, __pyx_k_term_parser___reduce_cython, sizeof(__pyx_k_term_parser___reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_term_parser___setstate_cython, __pyx_k_term_parser___setstate_cython, sizeof(__pyx_k_term_parser___setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_term_parser_stream_2_sequence_lo, __pyx_k_term_parser_stream_2_sequence_lo, sizeof(__pyx_k_term_parser_stream_2_sequence_lo), 0, 0, 1, 1},
  {&__pyx_n_s_terminal_graphics, __pyx_k_terminal_graphics, sizeof(__pyx_k_terminal_graphics), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_text, __pyx_k_text, sizeof(__pyx_k_text), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_timestamp, __pyx_k_timestamp, sizeof(__pyx_k_timestamp), 0, 0, 1, 1},
  {&__pyx_n_s_ttygif_tty_parser, __pyx_k_ttygif_tty_parser, sizeof(__pyx_k_ttygif_tty_parser), 0, 0, 1, 1},
  {&__pyx_n_s_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 0, 1, 1},
  {&__pyx_n_s_u, __pyx_k_u, sizeof(__pyx_k_u), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_use_setstate, __pyx_k_use_setstate, sizeof(__pyx_k_use_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_xrange, __pyx_k_xrange, sizeof(__pyx_k_xrange), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  #if PY_MAJOR_VERSION >= 3
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_xrange) __PYX_ERR(0, 39, __pyx_L1_error)
  #else
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_xrange); if (!__pyx_builtin_xrange) __PYX_ERR(0, 39, __pyx_L1_error)
  #endif
  __pyx_builtin_chr = __Pyx_GetBuiltinName(__pyx_n_s_chr); if (!__pyx_builtin_chr) __PYX_ERR(0, 98, __pyx_L1_error)
  __pyx_builtin_round = __Pyx_GetBuiltinName(__pyx_n_s_round); if (!__pyx_builtin_round) __PYX_ERR(0, 587, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(2, 109, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "ttygif/tty/parser.pyx":539
 *                 esc_type='CSI'
 *                 command=groups[12][-1]
 *                 params=[int(groups[13][0:-1])]             # <<<<<<<<<<<<<<
 *                 name="bracketed paste"
 *                 self.add_command_sequence(esc_type,command,params,groups,name,timestamp,delay,text)
 */
  __pyx_slice__11 = PySlice_New(__pyx_int_0, __pyx_int_neg_1, Py_None); if (unlikely(!__pyx_slice__11)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__11);
  __Pyx_GIVEREF(__pyx_slice__11);

  /* "ttygif/tty/parser.pyx":549
 *                     params = tuple(int(p) if len(p) != 0 else 1 for p in paramstring.split(';'))
 *                     while len(params) < 2:
 *                         params = params + (1,)             # <<<<<<<<<<<<<<
 *                 #        DEC Private Mode (DECSET/DECRST) sequences
 *                 elif paramstring and len(paramstring)>0 and paramstring[0]=='?':
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_int_1); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "ttygif/tty/parser.pyx":553
 *                 elif paramstring and len(paramstring)>0 and paramstring[0]=='?':
 *                     command='?'+command
 *                     param_tokens=paramstring[1:].split(';')             # <<<<<<<<<<<<<<
 *                     params=[int(part) for part in param_tokens]
 *                 else:
 */
  __pyx_slice__14 = PySlice_New(__pyx_int_1, Py_None, Py_None); if (unlikely(!__pyx_slice__14)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__14);
  __Pyx_GIVEREF(__pyx_slice__14);

  /* "ttygif/tty/parser.pyx":560
 *                         if len(params) == 0:
 *                             if command in 'JKm':
 *                                 params = (0,)             # <<<<<<<<<<<<<<
 *                             elif command in 'ABCD':
 *                                 params = (1,)
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_int_0); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_tuple__21 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_state, __pyx_n_s_dict_2, __pyx_n_s_use_setstate); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(1, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_reduce_cython, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(1, 1, __pyx_L1_error)

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_term_parser, (type(self), 0x35256c6, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_term_parser__set_state(self, __pyx_state)
 */
  __pyx_tuple__22 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_pyx_state); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);
  __pyx_codeobj__19 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__22, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_setstate_cython, 16, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__19)) __PYX_ERR(1, 16, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_term_parser(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__23 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__23, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_term_parser, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_5 = PyInt_FromLong(5); if (unlikely(!__pyx_int_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_7 = PyInt_FromLong(7); if (unlikely(!__pyx_int_7)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_8 = PyInt_FromLong(8); if (unlikely(!__pyx_int_8)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_27 = PyInt_FromLong(27); if (unlikely(!__pyx_int_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_30 = PyInt_FromLong(30); if (unlikely(!__pyx_int_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_37 = PyInt_FromLong(37); if (unlikely(!__pyx_int_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_38 = PyInt_FromLong(38); if (unlikely(!__pyx_int_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_39 = PyInt_FromLong(39); if (unlikely(!__pyx_int_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_40 = PyInt_FromLong(40); if (unlikely(!__pyx_int_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_47 = PyInt_FromLong(47); if (unlikely(!__pyx_int_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_48 = PyInt_FromLong(48); if (unlikely(!__pyx_int_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_49 = PyInt_FromLong(49); if (unlikely(!__pyx_int_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_90 = PyInt_FromLong(90); if (unlikely(!__pyx_int_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_97 = PyInt_FromLong(97); if (unlikely(!__pyx_int_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_100 = PyInt_FromLong(100); if (unlikely(!__pyx_int_100)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_107 = PyInt_FromLong(107); if (unlikely(!__pyx_int_107)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_200 = PyInt_FromLong(200); if (unlikely(!__pyx_int_200)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_201 = PyInt_FromLong(201); if (unlikely(!__pyx_int_201)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_255 = PyInt_FromLong(255); if (unlikely(!__pyx_int_255)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_55727814 = PyInt_FromLong(55727814L); if (unlikely(!__pyx_int_55727814)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_6ttygif_3tty_6parser_term_parser = &__pyx_vtable_6ttygif_3tty_6parser_term_parser;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.ascii_safe = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_ascii_safe;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.info = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_info;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.clear_sequence = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *))__pyx_f_6ttygif_3tty_6parser_11term_parser_clear_sequence;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.rgb_to_palette = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_rgb_to_palette;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.remap_character = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_remap_character;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.render_to_buffer = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *))__pyx_f_6ttygif_3tty_6parser_11term_parser_render_to_buffer;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.procces_OSC = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_procces_OSC;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.process_SINGLE = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_process_SINGLE;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.process_CHAR_SET = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_process_CHAR_SET;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.process_G0 = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_process_G0;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.process_G1 = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_process_G1;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.process_CSI = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_process_CSI;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_DECSET = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, int))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_DECSET;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_DECRST = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, int))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_DECRST;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_bracketed_paste_on = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_bracketed_paste_on;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_bracketed_paste_off = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_bracketed_paste_off;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_BRACKETED_PASTE = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_BRACKETED_PASTE;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_set_mode = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_set_mode;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_reset_mode = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_reset_mode;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.set_foreground = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_set_foreground;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.set_background = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_set_background;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_process_colors = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_process_colors;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_render_text = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_render_text;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_DECSTBM = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, int, int))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_DECSTBM;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_CUU = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUU;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_CUD = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUD;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_CUB = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUB;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_CUF = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUF;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_CPL = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CPL;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_CNL = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CNL;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_CHA = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CHA;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_CUP = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_CUP;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_ED = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_ED;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_EL = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_EL;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_ECH = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_ECH;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_DCH = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_DCH;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_HVP = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_HVP;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_HPA = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_HPA;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_SCP = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_SCP;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_RCP = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_RCP;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.cmd_VPA = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_cmd_VPA;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.stream_2_sequence = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_stream_2_sequence;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.last_frame = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *))__pyx_f_6ttygif_3tty_6parser_11term_parser_last_frame;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.has_escape = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_has_escape;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.add_event = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_add_event;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.add_text_sequence = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_add_text_sequence;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.add_command_sequence = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, struct __pyx_opt_args_6ttygif_3tty_6parser_11term_parser_add_command_sequence *__pyx_optional_args))__pyx_f_6ttygif_3tty_6parser_11term_parser_add_command_sequence;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.debug_sequence = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *))__pyx_f_6ttygif_3tty_6parser_11term_parser_debug_sequence;
  __pyx_vtable_6ttygif_3tty_6parser_term_parser.debug_event = (PyObject *(*)(struct __pyx_obj_6ttygif_3tty_6parser_term_parser *, PyObject *, PyObject *))__pyx_f_6ttygif_3tty_6parser_11term_parser_debug_event;
  if (PyType_Ready(&__pyx_type_6ttygif_3tty_6parser_term_parser) < 0) __PYX_ERR(0, 13, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ttygif_3tty_6parser_term_parser.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ttygif_3tty_6parser_term_parser.tp_dictoffset && __pyx_type_6ttygif_3tty_6parser_term_parser.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ttygif_3tty_6parser_term_parser.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_6ttygif_3tty_6parser_term_parser.tp_dict, __pyx_vtabptr_6ttygif_3tty_6parser_term_parser) < 0) __PYX_ERR(0, 13, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_term_parser, (PyObject *)&__pyx_type_6ttygif_3tty_6parser_term_parser) < 0) __PYX_ERR(0, 13, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_6ttygif_3tty_6parser_term_parser) < 0) __PYX_ERR(0, 13, __pyx_L1_error)
  __pyx_ptype_6ttygif_3tty_6parser_term_parser = &__pyx_type_6ttygif_3tty_6parser_term_parser;
  if (PyType_Ready(&__pyx_type_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence) < 0) __PYX_ERR(0, 494, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence.tp_dictoffset && __pyx_type_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence = &__pyx_type_6ttygif_3tty_6parser___pyx_scope_struct__stream_2_sequence;
  if (PyType_Ready(&__pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr) < 0) __PYX_ERR(0, 547, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr.tp_dictoffset && __pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr = &__pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_1_genexpr;
  if (PyType_Ready(&__pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr) < 0) __PYX_ERR(0, 557, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr.tp_dictoffset && __pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr = &__pyx_type_6ttygif_3tty_6parser___pyx_scope_struct_2_genexpr;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(4, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_7complex_complex) __PYX_ERR(5, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("array"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_5array_array = __Pyx_ImportType(__pyx_t_1, "array", "array", sizeof(arrayobject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_5array_array) __PYX_ERR(2, 58, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ttygif.tty.image"); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ttygif_3tty_5image_bounds = __Pyx_ImportType(__pyx_t_1, "ttygif.tty.image", "bounds", sizeof(struct __pyx_obj_6ttygif_3tty_5image_bounds), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ttygif_3tty_5image_bounds) __PYX_ERR(6, 10, __pyx_L1_error)
  __pyx_vtabptr_6ttygif_3tty_5image_bounds = (struct __pyx_vtabstruct_6ttygif_3tty_5image_bounds*)__Pyx_GetVtable(__pyx_ptype_6ttygif_3tty_5image_bounds->tp_dict); if (unlikely(!__pyx_vtabptr_6ttygif_3tty_5image_bounds)) __PYX_ERR(6, 10, __pyx_L1_error)
  __pyx_ptype_6ttygif_3tty_5image_point = __Pyx_ImportType(__pyx_t_1, "ttygif.tty.image", "point", sizeof(struct __pyx_obj_6ttygif_3tty_5image_point), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ttygif_3tty_5image_point) __PYX_ERR(6, 18, __pyx_L1_error)
  __pyx_vtabptr_6ttygif_3tty_5image_point = (struct __pyx_vtabstruct_6ttygif_3tty_5image_point*)__Pyx_GetVtable(__pyx_ptype_6ttygif_3tty_5image_point->tp_dict); if (unlikely(!__pyx_vtabptr_6ttygif_3tty_5image_point)) __PYX_ERR(6, 18, __pyx_L1_error)
  __pyx_ptype_6ttygif_3tty_5image_rect = __Pyx_ImportType(__pyx_t_1, "ttygif.tty.image", "rect", sizeof(struct __pyx_obj_6ttygif_3tty_5image_rect), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ttygif_3tty_5image_rect) __PYX_ERR(6, 24, __pyx_L1_error)
  __pyx_vtabptr_6ttygif_3tty_5image_rect = (struct __pyx_vtabstruct_6ttygif_3tty_5image_rect*)__Pyx_GetVtable(__pyx_ptype_6ttygif_3tty_5image_rect->tp_dict); if (unlikely(!__pyx_vtabptr_6ttygif_3tty_5image_rect)) __PYX_ERR(6, 24, __pyx_L1_error)
  __pyx_ptype_6ttygif_3tty_5image_image = __Pyx_ImportType(__pyx_t_1, "ttygif.tty.image", "image", sizeof(struct __pyx_obj_6ttygif_3tty_5image_image), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ttygif_3tty_5image_image) __PYX_ERR(6, 40, __pyx_L1_error)
  __pyx_vtabptr_6ttygif_3tty_5image_image = (struct __pyx_vtabstruct_6ttygif_3tty_5image_image*)__Pyx_GetVtable(__pyx_ptype_6ttygif_3tty_5image_image->tp_dict); if (unlikely(!__pyx_vtabptr_6ttygif_3tty_5image_image)) __PYX_ERR(6, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ttygif.tty.font"); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ttygif_3tty_4font_font = __Pyx_ImportType(__pyx_t_1, "ttygif.tty.font", "font", sizeof(struct __pyx_obj_6ttygif_3tty_4font_font), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ttygif_3tty_4font_font) __PYX_ERR(7, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ttygif.tty.theme"); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ttygif_3tty_5theme_layer = __Pyx_ImportType(__pyx_t_1, "ttygif.tty.theme", "layer", sizeof(struct __pyx_obj_6ttygif_3tty_5theme_layer), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ttygif_3tty_5theme_layer) __PYX_ERR(8, 7, __pyx_L1_error)
  __pyx_vtabptr_6ttygif_3tty_5theme_layer = (struct __pyx_vtabstruct_6ttygif_3tty_5theme_layer*)__Pyx_GetVtable(__pyx_ptype_6ttygif_3tty_5theme_layer->tp_dict); if (unlikely(!__pyx_vtabptr_6ttygif_3tty_5theme_layer)) __PYX_ERR(8, 7, __pyx_L1_error)
  __pyx_ptype_6ttygif_3tty_5theme_theme = __Pyx_ImportType(__pyx_t_1, "ttygif.tty.theme", "theme", sizeof(struct __pyx_obj_6ttygif_3tty_5theme_theme), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ttygif_3tty_5theme_theme) __PYX_ERR(8, 27, __pyx_L1_error)
  __pyx_vtabptr_6ttygif_3tty_5theme_theme = (struct __pyx_vtabstruct_6ttygif_3tty_5theme_theme*)__Pyx_GetVtable(__pyx_ptype_6ttygif_3tty_5theme_theme->tp_dict); if (unlikely(!__pyx_vtabptr_6ttygif_3tty_5theme_theme)) __PYX_ERR(8, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ttygif.tty.display_state"); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ttygif_3tty_13display_state_display_state = __Pyx_ImportType(__pyx_t_1, "ttygif.tty.display_state", "display_state", sizeof(struct __pyx_obj_6ttygif_3tty_13display_state_display_state), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ttygif_3tty_13display_state_display_state) __PYX_ERR(9, 6, __pyx_L1_error)
  __pyx_vtabptr_6ttygif_3tty_13display_state_display_state = (struct __pyx_vtabstruct_6ttygif_3tty_13display_state_display_state*)__Pyx_GetVtable(__pyx_ptype_6ttygif_3tty_13display_state_display_state->tp_dict); if (unlikely(!__pyx_vtabptr_6ttygif_3tty_13display_state_display_state)) __PYX_ERR(9, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ttygif.tty.terminal_graphics"); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6ttygif_3tty_17terminal_graphics_terminal_graphics = __Pyx_ImportType(__pyx_t_1, "ttygif.tty.terminal_graphics", "terminal_graphics", sizeof(struct __pyx_obj_6ttygif_3tty_17terminal_graphics_terminal_graphics), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6ttygif_3tty_17terminal_graphics_terminal_graphics) __PYX_ERR(10, 13, __pyx_L1_error)
  __pyx_vtabptr_6ttygif_3tty_17terminal_graphics_terminal_graphics = (struct __pyx_vtabstruct_6ttygif_3tty_17terminal_graphics_terminal_graphics*)__Pyx_GetVtable(__pyx_ptype_6ttygif_3tty_17terminal_graphics_terminal_graphics->tp_dict); if (unlikely(!__pyx_vtabptr_6ttygif_3tty_17terminal_graphics_terminal_graphics)) __PYX_ERR(10, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initparser(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initparser(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_parser(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_parser(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_parser(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  __Pyx_TraceDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'parser' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_parser(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("parser", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_ttygif__tty__parser) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name_2, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "ttygif.tty.parser")) {
      if (unlikely(PyDict_SetItemString(modules, "ttygif.tty.parser", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __Pyx_TraceCall("__Pyx_PyMODINIT_FUNC PyInit_parser(void)", __pyx_f[0], 1, 0, __PYX_ERR(0, 1, __pyx_L1_error));

  /* "ttygif/tty/parser.pyx":6
 * 
 * from .terminal_graphics cimport terminal_graphics
 * import re             # <<<<<<<<<<<<<<
 * from libc.stdint cimport uint32_t, int64_t,uint16_t,uint8_t,int32_t
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_re, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_re, __pyx_t_1) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_6ttygif_3tty_6parser_11term_parser_3__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_term_parser___reduce_cython, NULL, __pyx_n_s_ttygif_tty_parser, __pyx_d, ((PyObject *)__pyx_codeobj__18)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_6ttygif_3tty_6parser_term_parser->tp_dict, __pyx_n_s_reduce_cython, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_6ttygif_3tty_6parser_term_parser);

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_term_parser, (type(self), 0x35256c6, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_term_parser__set_state(self, __pyx_state)
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_6ttygif_3tty_6parser_11term_parser_5__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_term_parser___setstate_cython, NULL, __pyx_n_s_ttygif_tty_parser, __pyx_d, ((PyObject *)__pyx_codeobj__19)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_6ttygif_3tty_6parser_term_parser->tp_dict, __pyx_n_s_setstate_cython, __pyx_t_1) < 0) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_6ttygif_3tty_6parser_term_parser);

  /* "(tree fragment)":1
 * def __pyx_unpickle_term_parser(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_6ttygif_3tty_6parser_1__pyx_unpickle_term_parser, 0, __pyx_n_s_pyx_unpickle_term_parser, NULL, __pyx_n_s_ttygif_tty_parser, __pyx_d, ((PyObject *)__pyx_codeobj__20)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_term_parser, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ttygif/tty/parser.pyx":1
 * # cython: profile=True             # <<<<<<<<<<<<<<
 * # cython: binding=True
 * # cython: language_level=2
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "array.pxd":161
 *     return extend_buffer(self, other.data.as_chars, Py_SIZE(other))
 * 
 * cdef inline void zero(array self):             # <<<<<<<<<<<<<<
 *     """ set all elements of array to zero. """
 *     memset(self.data.as_chars, 0, Py_SIZE(self) * self.ob_descr.itemsize)
 */
  __Pyx_TraceReturn(Py_None, 0);

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init ttygif.tty.parser", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init ttygif.tty.parser");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* Profile */
#if CYTHON_PROFILE
static int __Pyx_TraceSetupAndCall(PyCodeObject** code,
                                   PyFrameObject** frame,
                                   PyThreadState* tstate,
                                   const char *funcname,
                                   const char *srcfile,
                                   int firstlineno) {
    PyObject *type, *value, *traceback;
    int retval;
    if (*frame == NULL || !CYTHON_PROFILE_REUSE_FRAME) {
        if (*code == NULL) {
            *code = __Pyx_createFrameCodeObject(funcname, srcfile, firstlineno);
            if (*code == NULL) return 0;
        }
        *frame = PyFrame_New(
            tstate,                          /*PyThreadState *tstate*/
            *code,                           /*PyCodeObject *code*/
            __pyx_d,                  /*PyObject *globals*/
            0                                /*PyObject *locals*/
        );
        if (*frame == NULL) return 0;
        if (CYTHON_TRACE && (*frame)->f_trace == NULL) {
            Py_INCREF(Py_None);
            (*frame)->f_trace = Py_None;
        }
#if PY_VERSION_HEX < 0x030400B1
    } else {
        (*frame)->f_tstate = tstate;
#endif
    }
    __Pyx_PyFrame_SetLineNumber(*frame, firstlineno);
    retval = 1;
    tstate->tracing++;
    __Pyx_SetTracing(tstate, 0);
    __Pyx_ErrFetchInState(tstate, &type, &value, &traceback);
    #if CYTHON_TRACE
    if (tstate->c_tracefunc)
        retval = tstate->c_tracefunc(tstate->c_traceobj, *frame, PyTrace_CALL, NULL) == 0;
    if (retval && tstate->c_profilefunc)
    #endif
        retval = tstate->c_profilefunc(tstate->c_profileobj, *frame, PyTrace_CALL, NULL) == 0;
    __Pyx_SetTracing(tstate, (tstate->c_profilefunc || (CYTHON_TRACE && tstate->c_tracefunc)));
    tstate->tracing--;
    if (retval) {
        __Pyx_ErrRestoreInState(tstate, type, value, traceback);
        return __Pyx_IsTracing(tstate, 0, 0) && retval;
    } else {
        Py_XDECREF(type);
        Py_XDECREF(value);
        Py_XDECREF(traceback);
        return -1;
    }
}
static PyCodeObject *__Pyx_createFrameCodeObject(const char *funcname, const char *srcfile, int firstlineno) {
    PyCodeObject *py_code = 0;
#if PY_MAJOR_VERSION >= 3
    py_code = PyCode_NewEmpty(srcfile, funcname, firstlineno);
    if (likely(py_code)) {
        py_code->co_flags |= CO_OPTIMIZED | CO_NEWLOCALS;
    }
#else
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    py_funcname = PyString_FromString(funcname);
    if (unlikely(!py_funcname)) goto bad;
    py_srcfile = PyString_FromString(srcfile);
    if (unlikely(!py_srcfile)) goto bad;
    py_code = PyCode_New(
        0,
        0,
        0,
        CO_OPTIMIZED | CO_NEWLOCALS,
        __pyx_empty_bytes,     /*PyObject *code,*/
        __pyx_empty_tuple,     /*PyObject *consts,*/
        __pyx_empty_tuple,     /*PyObject *names,*/
        __pyx_empty_tuple,     /*PyObject *varnames,*/
        __pyx_empty_tuple,     /*PyObject *freevars,*/
        __pyx_empty_tuple,     /*PyObject *cellvars,*/
        py_srcfile,       /*PyObject *filename,*/
        py_funcname,      /*PyObject *name,*/
        firstlineno,
        __pyx_empty_bytes      /*PyObject *lnotab*/
    );
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
#endif
    return py_code;
}
#endif

/* StringJoin */
#if !CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values) {
    return PyObject_CallMethodObjArgs(sep, __pyx_n_s_join, values, NULL);
}
#endif

/* UnicodeAsUCS4 */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject* x) {
   Py_ssize_t length;
   #if CYTHON_PEP393_ENABLED
   length = PyUnicode_GET_LENGTH(x);
   if (likely(length == 1)) {
       return PyUnicode_READ_CHAR(x, 0);
   }
   #else
   length = PyUnicode_GET_SIZE(x);
   if (likely(length == 1)) {
       return PyUnicode_AS_UNICODE(x)[0];
   }
   #if Py_UNICODE_SIZE == 2
   else if (PyUnicode_GET_SIZE(x) == 2) {
       Py_UCS4 high_val = PyUnicode_AS_UNICODE(x)[0];
       if (high_val >= 0xD800 && high_val <= 0xDBFF) {
           Py_UCS4 low_val = PyUnicode_AS_UNICODE(x)[1];
           if (low_val >= 0xDC00 && low_val <= 0xDFFF) {
               return 0x10000 + (((high_val & ((1<<10)-1)) << 10) | (low_val & ((1<<10)-1)));
           }
       }
   }
   #endif
   #endif
   PyErr_Format(PyExc_ValueError,
                "only single character unicode strings can be converted to Py_UCS4, "
                "got length %" CYTHON_FORMAT_SSIZE_T "d", length);
   return (Py_UCS4)-1;
}

/* object_ord */
static long __Pyx__PyObject_Ord(PyObject* c) {
    Py_ssize_t size;
    if (PyBytes_Check(c)) {
        size = PyBytes_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyBytes_AS_STRING(c)[0];
        }
#if PY_MAJOR_VERSION < 3
    } else if (PyUnicode_Check(c)) {
        return (long)__Pyx_PyUnicode_AsPy_UCS4(c);
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    } else if (PyByteArray_Check(c)) {
        size = PyByteArray_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyByteArray_AS_STRING(c)[0];
        }
#endif
    } else {
        PyErr_Format(PyExc_TypeError,
            "ord() expected string of length 1, but %.200s found", Py_TYPE(c)->tp_name);
        return (long)(Py_UCS4)-1;
    }
    PyErr_Format(PyExc_TypeError,
        "ord() expected a character, but string of length %zd found", size);
    return (long)(Py_UCS4)-1;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a - b);
            if (likely((x^a) >= 0 || (x^~b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
            }
        }
                x = a - b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla - llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("subtract", return NULL)
            result = ((double)a) - (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* None */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod1 */
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* append */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
#include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
#if PY_MAJOR_VERSION < 3
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
#endif
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* Print */
#if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static PyObject *__Pyx_GetStdout(void) {
    PyObject *f = PySys_GetObject((char *)"stdout");
    if (!f) {
        PyErr_SetString(PyExc_RuntimeError, "lost sys.stdout");
    }
    return f;
}
static int __Pyx_Print(PyObject* f, PyObject *arg_tuple, int newline) {
    int i;
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    for (i=0; i < PyTuple_GET_SIZE(arg_tuple); i++) {
        PyObject* v;
        if (PyFile_SoftSpace(f, 1)) {
            if (PyFile_WriteString(" ", f) < 0)
                goto error;
        }
        v = PyTuple_GET_ITEM(arg_tuple, i);
        if (PyFile_WriteObject(v, f, Py_PRINT_RAW) < 0)
            goto error;
        if (PyString_Check(v)) {
            char *s = PyString_AsString(v);
            Py_ssize_t len = PyString_Size(v);
            if (len > 0) {
                switch (s[len-1]) {
                    case ' ': break;
                    case '\f': case '\r': case '\n': case '\t': case '\v':
                        PyFile_SoftSpace(f, 0);
                        break;
                    default:  break;
                }
            }
        }
    }
    if (newline) {
        if (PyFile_WriteString("\n", f) < 0)
            goto error;
        PyFile_SoftSpace(f, 0);
    }
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
}
#else
static int __Pyx_Print(PyObject* stream, PyObject *arg_tuple, int newline) {
    PyObject* kwargs = 0;
    PyObject* result = 0;
    PyObject* end_string;
    if (unlikely(!__pyx_print)) {
        __pyx_print = PyObject_GetAttr(__pyx_b, __pyx_n_s_print);
        if (!__pyx_print)
            return -1;
    }
    if (stream) {
        kwargs = PyDict_New();
        if (unlikely(!kwargs))
            return -1;
        if (unlikely(PyDict_SetItem(kwargs, __pyx_n_s_file, stream) < 0))
            goto bad;
        if (!newline) {
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                goto bad;
            if (PyDict_SetItem(kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                goto bad;
            }
            Py_DECREF(end_string);
        }
    } else if (!newline) {
        if (unlikely(!__pyx_print_kwargs)) {
            __pyx_print_kwargs = PyDict_New();
            if (unlikely(!__pyx_print_kwargs))
                return -1;
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                return -1;
            if (PyDict_SetItem(__pyx_print_kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                return -1;
            }
            Py_DECREF(end_string);
        }
        kwargs = __pyx_print_kwargs;
    }
    result = PyObject_Call(__pyx_print, arg_tuple, kwargs);
    if (unlikely(kwargs) && (kwargs != __pyx_print_kwargs))
        Py_DECREF(kwargs);
    if (!result)
        return -1;
    Py_DECREF(result);
    return 0;
bad:
    if (kwargs != __pyx_print_kwargs)
        Py_XDECREF(kwargs);
    return -1;
}
#endif

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8_t(uint8_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint8_t neg_one = (uint8_t) -1, const_zero = (uint8_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint8_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint8_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint8_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint8_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint8_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint8_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16_t(uint16_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint16_t neg_one = (uint16_t) -1, const_zero = (uint16_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint16_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint16_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint16_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint16_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint16_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint16_t),
                                     little, !is_unsigned);
    }
}

/* PrintOne */
#if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static int __Pyx_PrintOne(PyObject* f, PyObject *o) {
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    if (PyFile_SoftSpace(f, 0)) {
        if (PyFile_WriteString(" ", f) < 0)
            goto error;
    }
    if (PyFile_WriteObject(o, f, Py_PRINT_RAW) < 0)
        goto error;
    if (PyFile_WriteString("\n", f) < 0)
        goto error;
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
    /* the line below is just to avoid C compiler
     * warnings about unused functions */
    return __Pyx_Print(f, NULL, 0);
}
#else
static int __Pyx_PrintOne(PyObject* stream, PyObject *o) {
    int res;
    PyObject* arg_tuple = PyTuple_Pack(1, o);
    if (unlikely(!arg_tuple))
        return -1;
    res = __Pyx_Print(stream, arg_tuple, 1);
    Py_DECREF(arg_tuple);
    return res;
}
#endif

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* CoroutineBase */
#include <structmember.h>
#include <frameobject.h>
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(CYTHON_UNUSED __pyx_CoroutineObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(CYTHON_UNUSED PyObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(CYTHON_UNUSED PyObject *gen, PyObject *value, CYTHON_UNUSED int closing) {
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_type) {
        #if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
        #else
        if (exc_state->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_state->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            Py_XINCREF(tstate->frame);
            assert(f->f_back == NULL);
            f->f_back = tstate->frame;
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    self->is_running = 1;
    retval = self->body((PyObject *) self, tstate, value);
    self->is_running = 0;
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *exc_tb = exc_state->exc_traceback;
    if (likely(exc_tb)) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
#endif
    }
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(CYTHON_UNUSED PyObject* gen, PyObject *retval) {
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
            _PyGen_SetStopIterationValue(result);
        }
        Py_CLEAR(result);
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = Py_TYPE(yf)->tp_iternext(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf, NULL);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, CYTHON_UNUSED PyObject *arg) {
    return __Pyx_Coroutine_Close(self);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = PyObject_CallObject(meth, args);
            } else {
                ret = PyObject_CallFunctionObjArgs(meth, typ, val, tb, NULL);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (Py_REFCNT(self) > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    __Pyx_SET_REFCNT(self, 1);
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(Py_REFCNT(self) > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = Py_REFCNT(self);
        _Py_NewReference(self);
        __Pyx_SET_REFCNT(self, refcnt);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(Py_TYPE(self)) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_name;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_qualname;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *frame = self->gi_frame;
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            __pyx_d,                 /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        if (unlikely(!frame))
            return NULL;
        self->gi_frame = frame;
    }
    Py_INCREF(frame);
    return frame;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
#ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {(char *) "gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     (char*) PyDoc_STR("Frame of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
